@@ -51,6 +51,7 @@ class ActionManager
 	static std::vector<Glib::RefPtr<Gtk::Action> > time_selection_sensitive_actions;
 	static std::vector<Glib::RefPtr<Gtk::Action> > line_selection_sensitive_actions;
 	static std::vector<Glib::RefPtr<Gtk::Action> > playlist_selection_sensitive_actions;
+	static std::vector<Glib::RefPtr<Gtk::Action> > mouse_edit_point_requires_canvas_actions;
 
 	static std::vector<Glib::RefPtr<Gtk::Action> > range_sensitive_actions;
 	static std::vector<Glib::RefPtr<Gtk::Action> > transport_sensitive_actions;
@@ -69,6 +70,7 @@ class ActionManager
 
 	static Gtk::Widget* get_widget (const char * name);
 	static Glib::RefPtr<Gtk::Action> get_action (const char* group, const char* name);
+	static Glib::RefPtr<Gtk::Action> get_action (const char* path);
 
 	static void add_action_group (Glib::RefPtr<Gtk::ActionGroup>);
 
@@ -99,6 +101,10 @@ class ActionManager
 				     std::vector<std::string>& keys, 
 				     std::vector<Gtk::AccelKey>& bindings);
 
+	static void get_all_actions (std::vector<std::string>& groups, 
+				     std::vector<std::string>& paths, 
+				     std::vector<Gtk::AccelKey>& bindings);
+
 	static void uncheck_toggleaction (const char * actionname);
 };
 
