@@ -22,6 +22,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <unistd.h>
+#include <time.h>
 #include <cerrno>
 #include <fstream>
 
@@ -35,7 +36,6 @@
 #include <pbd/error.h>
 #include <pbd/basename.h>
 #include <pbd/compose.h>
-#include <pbd/misc.h>
 #include <pbd/pathscanner.h>
 #include <pbd/failed_constructor.h>
 #include <pbd/enumwriter.h>
@@ -64,6 +64,8 @@
 #include <ardour/port.h>
 #include <ardour/audio_track.h>
 
+typedef uint64_t microseconds_t;
+
 #include "actions.h"
 #include "ardour_ui.h"
 #include "public_editor.h"
@@ -80,6 +82,8 @@
 #include "gui_thread.h"
 #include "theme_manager.h"
 #include "engine_dialog.h"
+#include "gain_meter.h"
+#include "route_time_axis.h"
 
 #include "i18n.h"
 
@@ -106,10 +110,6 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	  preroll_clock (X_("preroll"), false, X_("PreRollClock"), true, true),
 	  postroll_clock (X_("postroll"), false, X_("PostRollClock"), true, true),
 
-	  /* adjuster table */
-
-	  adjuster_table (3, 3),
-
 	  /* preroll stuff */
 
 	  preroll_button (_("pre\nroll")),
@@ -162,10 +162,6 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	_auto_display_errors = false;
 #endif
 
-	if (getenv ("ARDOUR_DEBUG_UPDATES")) {
-		gdk_window_set_debug_updates (true);
-	}
-	
 	about = 0;
 	splash = 0;
 
@@ -202,17 +198,9 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	_will_create_new_session_automatically = false;
 	session_loaded = false;
 	last_speed_displayed = -1.0f;
+	ignore_dual_punch = false;
 
-	keybindings_path = ARDOUR::find_config_file ("ardour.bindings");
-	/* all changes go to the user directory */
-	user_keybindings_path = get_user_ardour_path ();
-	user_keybindings_path += '/';
-	user_keybindings_path += "ardour.bindings";
-
-	can_save_keybindings = false;
-
-	last_configure_time.tv_sec = 0;
-	last_configure_time.tv_usec = 0;
+	last_configure_time= 0;
 
 	shuttle_grabbed = false;
 	shuttle_fract = 0.0;
@@ -221,12 +209,17 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 	shuttle_style_menu = 0;
 	shuttle_unit_menu = 0;
 
-	gettimeofday (&last_peak_grab, 0);
-	gettimeofday (&last_shuttle_request, 0);
+        // We do not have jack linked in yet so;
+        
+	last_shuttle_request = last_peak_grab = 0; //  get_microseconds();
 
 	ARDOUR::Diskstream::DiskOverrun.connect (mem_fun(*this, &ARDOUR_UI::disk_overrun_handler));
 	ARDOUR::Diskstream::DiskUnderrun.connect (mem_fun(*this, &ARDOUR_UI::disk_underrun_handler));
 
+	/* handle dialog requests */
+
+	ARDOUR::Session::Dialog.connect (mem_fun(*this, &ARDOUR_UI::session_dialog));
+
 	/* handle pending state with a dialog */
 
 	ARDOUR::Session::AskAboutPendingState.connect (mem_fun(*this, &ARDOUR_UI::pending_state_dialog));
@@ -246,6 +239,9 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 		Config->set_current_owner (ConfigVariableBase::Interface);
 		setup_profile ();
 
+		GainMeter::setup_slider_pix ();
+		RouteTimeAxisView::setup_slider_pix ();
+
 	} catch (failed_constructor& err) {
 		error << _("could not initialize Ardour.") << endmsg;
 		// pass it on up
@@ -256,6 +252,8 @@ ARDOUR_UI::ARDOUR_UI (int *argcp, char **argvp[])
 
 	keyboard = new Keyboard;
 
+	reset_dpi();
+
 	starting.connect (mem_fun(*this, &ARDOUR_UI::startup));
 	stopping.connect (mem_fun(*this, &ARDOUR_UI::shutdown));
 
@@ -338,7 +336,11 @@ ARDOUR_UI::post_engine ()
 
 	/* set default clock modes */
 
-	primary_clock.set_mode (AudioClock::SMPTE);
+	if (Profile->get_sae()) {
+		primary_clock.set_mode (AudioClock::MinSec);
+	}  else {
+		primary_clock.set_mode (AudioClock::SMPTE);
+	}
 	secondary_clock.set_mode (AudioClock::BBT);
 
 	/* start the time-of-day-clock */
@@ -404,21 +406,15 @@ ARDOUR_UI::pop_back_splash ()
 gint
 ARDOUR_UI::configure_timeout ()
 {
-	struct timeval now;
-	struct timeval diff;
-
-	if (last_configure_time.tv_sec == 0 && last_configure_time.tv_usec == 0) {
+	if (last_configure_time == 0) {
 		/* no configure events yet */
 		return TRUE;
 	}
 
-	gettimeofday (&now, 0);
-	timersub (&now, &last_configure_time, &diff);
-
 	/* force a gap of 0.5 seconds since the last configure event
 	 */
 
-	if (diff.tv_sec == 0 && diff.tv_usec < 500000) {
+	if (get_microseconds() - last_configure_time < 500000) {
 		return TRUE;
 	} else {
 		have_configure_timeout = false;
@@ -431,7 +427,7 @@ gboolean
 ARDOUR_UI::configure_handler (GdkEventConfigure* conf)
 {
 	if (have_configure_timeout) {
-		gettimeofday (&last_configure_time, 0);
+		last_configure_time = get_microseconds();
 	} else {
 		Glib::signal_timeout().connect (mem_fun(*this, &ARDOUR_UI::configure_timeout), 100);
 		have_configure_timeout = true;
@@ -530,12 +526,20 @@ ARDOUR_UI::save_ardour_state ()
 		Config->add_instant_xml (mnode, get_user_ardour_path());
 	}
 
-	save_keybindings ();
+	Keyboard::save_keybindings ();
 }
 
 gint
 ARDOUR_UI::autosave_session ()
 {
+	if (g_main_depth() > 1) {
+		/* inside a recursive main loop,
+		   give up because we may not be able to 
+		   take a lock.
+		*/
+		return 1;
+	}
+
         if (!Config->get_periodic_safety_backups())
                 return 1;
         
@@ -637,6 +641,7 @@ ARDOUR_UI::startup ()
 		return;
 	}
 	
+	BootMessage (_("Ardour is ready for use"));
 	show ();
 }
 
@@ -796,8 +801,6 @@ ARDOUR_UI::ask_about_saving_session (const string & what)
 	window.set_resizable (false);
 	window.show_all ();
 
-	save_the_session = 0;
-
 	window.set_keep_above (true);
 	window.present ();
 
@@ -1128,33 +1131,6 @@ ARDOUR_UI::open_recent_session ()
 	}
 }
 
-bool
-ARDOUR_UI::filter_ardour_session_dirs (const FileFilter::Info& info) 
-{
-	struct stat statbuf;
-
-	if (stat (info.filename.c_str(), &statbuf) != 0) {
-		return false;
-	}
-
-	if (!S_ISDIR(statbuf.st_mode)) {
-		return false;
-	}
-
-        // XXX Portability
-        
-	string session_file = info.filename;
-	session_file += '/';
-	session_file += Glib::path_get_basename (info.filename);
-	session_file += ".ardour";
-	
-	if (stat (session_file.c_str(), &statbuf) != 0) {
-		return false;
-	}
-
-	return S_ISREG (statbuf.st_mode);
-}
-
 bool
 ARDOUR_UI::check_audioengine ()
 {
@@ -1342,6 +1318,34 @@ ARDOUR_UI::transport_goto_zero ()
 	}
 }
 
+void
+ARDOUR_UI::transport_goto_wallclock ()
+{
+	if (session && editor) {
+
+		time_t now;
+		struct tm tmnow;
+		nframes64_t frames;
+		
+		time (&now);
+		localtime_r (&now, &tmnow);
+	
+		frames = tmnow.tm_hour * (60 * 60 * session->frame_rate());
+		frames += tmnow.tm_min * (60 * session->frame_rate());
+		frames += tmnow.tm_sec * session->frame_rate();
+
+		session->request_locate (frames);
+
+		/* force displayed area in editor to start no matter
+		   what "follow playhead" setting is.
+		*/
+		
+		if (editor) {
+			editor->reset_x_origin (frames - (editor->current_page_frames()/2));
+		}
+	}
+}
+
 void
 ARDOUR_UI::transport_goto_end ()
 {
@@ -1397,6 +1401,7 @@ ARDOUR_UI::remove_last_capture()
 void
 ARDOUR_UI::transport_record (bool roll)
 {
+	
 	if (session) {
 		switch (session->record_status()) {
 		case Session::Disabled:
@@ -1422,6 +1427,7 @@ ARDOUR_UI::transport_record (bool roll)
 			session->disable_record (false, true);
 		}
 	}
+	//cerr << "ARDOUR_UI::transport_record () called roll = " << roll << " session->record_status() = " << session->record_status() << endl;
 }
 
 void
@@ -1435,6 +1441,8 @@ ARDOUR_UI::transport_roll ()
 
 	rolling = session->transport_rolling ();
 
+	//cerr << "ARDOUR_UI::transport_roll () called session->record_status() = " << session->record_status() << endl;
+
 	if (session->get_play_loop()) {
 		session->request_play_loop (false);
 		auto_loop_button.set_visual_state (1);
@@ -1602,7 +1610,7 @@ ARDOUR_UI::engine_running ()
 	ActionManager::set_sensitive (ActionManager::jack_opposite_sensitive_actions, false);
 
 	Glib::RefPtr<Action> action;
-	char* action_name = 0;
+	const char* action_name = 0;
 
 	switch (engine->frames_per_cycle()) {
 	case 32:
@@ -1859,17 +1867,6 @@ ARDOUR_UI::save_state_canfail (string name)
 	return 0;
 }
 
-void
-ARDOUR_UI::restore_state (string name)
-{
-	if (session) {
-		if (name.length() == 0) {
-			name = session->name();
-		}
-		session->restore_state (name);
-	}
-}
-
 void
 ARDOUR_UI::primary_clock_value_changed ()
 {
@@ -1894,37 +1891,6 @@ ARDOUR_UI::secondary_clock_value_changed ()
 	}
 }
 
-void
-ARDOUR_UI::rec_enable_button_blink (bool onoff, AudioDiskstream *dstream, Widget *w)
-{
-	if (session && dstream && dstream->record_enabled()) {
-
-		Session::RecordState rs;
-		
-		rs = session->record_status ();
-
-		switch (rs) {
-		case Session::Disabled:
-		case Session::Enabled:
-			if (w->get_state() != STATE_SELECTED) {
-				w->set_state (STATE_SELECTED);
-			}
-			break;
-
-		case Session::Recording:
-			if (w->get_state() != STATE_ACTIVE) {
-				w->set_state (STATE_ACTIVE);
-			}
-			break;
-		}
-
-	} else {
-		if (w->get_state() != STATE_NORMAL) {
-			w->set_state (STATE_NORMAL);
-		}
-	}
-}
-
 void
 ARDOUR_UI::transport_rec_enable_blink (bool onoff) 
 {
@@ -2196,6 +2162,28 @@ ARDOUR_UI::loading_message (const std::string& msg)
 	flush_pending ();
 }
 	
+void
+ARDOUR_UI::idle_load (const Glib::ustring& path)
+{
+	if (session) {
+		if (Glib::file_test (path, Glib::FILE_TEST_IS_DIR)) {
+			/* /path/to/foo => /path/to/foo, foo */
+			load_session (path, basename_nosuffix (path));
+		} else {
+			/* /path/to/foo/foo.ardour => /path/to/foo, foo */
+			load_session (Glib::path_get_dirname (path), basename_nosuffix (path));
+		}
+	} else {
+		ARDOUR_COMMAND_LINE::session_name = path;
+		if (new_session_dialog) {
+			/* make it break out of Dialog::run() and
+			   start again.
+			 */
+			new_session_dialog->response (1);
+		}
+	}
+}
+
 bool
 ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be_new)
 {
@@ -2203,8 +2191,10 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 	Glib::ustring session_name;
 	Glib::ustring session_path;
 	Glib::ustring template_name;
+	int response;
 
-	int response = Gtk::RESPONSE_NONE;
+  begin:
+	response = Gtk::RESPONSE_NONE;
 
 	if (!ARDOUR_COMMAND_LINE::session_name.empty()) {
 
@@ -2247,6 +2237,13 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 		/* handle possible negative responses */
 
 		switch (response) {
+		case 1:
+			/* sent by idle_load, meaning restart the whole process again */
+			new_session_dialog->hide();
+			new_session_dialog->reset();
+			goto begin;
+			break;
+
 		case Gtk::RESPONSE_CANCEL:
 		case Gtk::RESPONSE_DELETE_EVENT:
 			if (!session) {
@@ -2263,10 +2260,13 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 		fontconfig_dialog();
 
 		if (!backend_audio_is_running) {
-			if (new_session_dialog->engine_control.setup_engine ()) {
-				new_session_dialog->hide ();
+			int ret = new_session_dialog->engine_control.setup_engine ();
+			if (ret < 0) {
 				return false;
-			} 
+			} else if (ret > 0) {
+				response = Gtk::RESPONSE_REJECT;
+				goto try_again;
+			}
 		}
 		
 		if (create_engine ()) {
@@ -2370,7 +2370,7 @@ ARDOUR_UI::get_session_parameters (bool backend_audio_is_running, bool should_be
 			}
 		}
 
-	} while (response == Gtk::RESPONSE_NONE);
+	} while (response == Gtk::RESPONSE_NONE || response == Gtk::RESPONSE_REJECT);
 
   done:
 	show();
@@ -2425,7 +2425,6 @@ ARDOUR_UI::load_session (const Glib::ustring& path, const Glib::ustring& snap_na
 	}
 
 	loading_message (_("Please wait while Ardour loads your session"));
-	disable_screen_updates ();
 
 	try {
 		new_session = new Session (*engine, path, snap_name, mix_template);
@@ -2497,7 +2496,6 @@ ARDOUR_UI::load_session (const Glib::ustring& path, const Glib::ustring& snap_na
 		session->set_clean ();
 	}
 
-	enable_screen_updates ();
 	flush_pending ();
 	retval = 0;
 
@@ -2763,8 +2761,11 @@ After cleanup, unused audio files will be moved to a \
 	}
 
 	if (session->cleanup_sources (rep)) {
+		editor->finish_cleanup ();
 		return;
 	}
+	
+	editor->finish_cleanup ();
 
 	checker.hide();
 	display_cleanup_results (rep, 
@@ -2778,8 +2779,6 @@ release an additional\n\
 %4 %5bytes of disk space.\n"
 					 ));
 
-
-
 }
 
 void
@@ -2855,6 +2854,8 @@ ARDOUR_UI::add_route (Gtk::Window* float_window)
 
 	/* XXX do something with name template */
 
+	cerr << "Adding with " << input_chan << " in and " << output_chan << "out\n";
+
 	if (track) {
 		session_add_audio_track (input_chan, output_chan, add_route_dialog->mode(), count);
 	} else {
@@ -2910,16 +2911,34 @@ ARDOUR_UI::keyboard_settings () const
 	return node;
 }
 
+void
+ARDOUR_UI::create_xrun_marker(nframes_t where)
+{
+	editor->mouse_add_new_marker (where, false, true);
+}
+
 void
 ARDOUR_UI::halt_on_xrun_message ()
 {
-	ENSURE_GUI_THREAD (mem_fun(*this, &ARDOUR_UI::halt_on_xrun_message));
-
 	MessageDialog msg (*editor,
 			   _("Recording was stopped because your system could not keep up."));
 	msg.run ();
 }
 
+void
+ARDOUR_UI::xrun_handler(nframes_t where)
+{
+	ENSURE_GUI_THREAD (bind(mem_fun(*this, &ARDOUR_UI::xrun_handler), where));
+
+	if (Config->get_create_xrun_marker() && session->actively_recording()) {
+		create_xrun_marker(where);
+	}
+
+	if (Config->get_stop_recording_on_xrun() && session->actively_recording()) {
+		halt_on_xrun_message ();
+	}
+}
+
 void
 ARDOUR_UI::disk_overrun_handler ()
 {
@@ -2963,6 +2982,24 @@ ARDOUR_UI::disk_speed_dialog_gone (int ignored_response, MessageDialog* msg)
 	delete msg;
 }
 
+void
+ARDOUR_UI::session_dialog (std::string msg)
+{
+	ENSURE_GUI_THREAD (bind (mem_fun(*this, &ARDOUR_UI::session_dialog), msg));
+	
+	MessageDialog* d;
+
+	if (editor) {
+		d = new MessageDialog (*editor, msg, false, MESSAGE_INFO, BUTTONS_OK, true);
+	} else {
+		d = new MessageDialog (msg, false, MESSAGE_INFO, BUTTONS_OK, true);
+	}
+
+	d->show_all ();
+	d->run ();
+	delete d;
+}	
+
 int
 ARDOUR_UI::pending_state_dialog ()
 {
@@ -3152,27 +3189,18 @@ ARDOUR_UI::record_state_changed ()
 	}
 }
 
-void
-ARDOUR_UI::set_keybindings_path (string path)
-{
-	keybindings_path = path;
-}
-
-void
-ARDOUR_UI::save_keybindings ()
-{
-	if (can_save_keybindings) {
-		AccelMap::save (user_keybindings_path);
-	} 
-}
-
 bool
 ARDOUR_UI::first_idle ()
 {
 	if (session) {
 		session->allow_auto_play (true);
 	}
-	can_save_keybindings = true;
+
+	if (editor) {
+		editor->first_idle();
+	}
+
+	Keyboard::set_can_save_keybindings (true);
 	return false;
 }
 
@@ -3222,7 +3250,7 @@ ARDOUR_UI::TransportControllable::set_value (float val)
 		return;
 	}
 
-	char *action = 0;
+	const char *action = 0;
 
 	switch (type) {
 	case Roll:
