@@ -58,6 +58,7 @@
 #include <gtkmm2ext/bindable_button.h>
 #include <ardour/ardour.h>
 #include <ardour/session.h>
+#include <ardour/types.h>
 
 #include "audio_clock.h"
 #include "ardour_dialog.h"
@@ -98,6 +99,7 @@ namespace ALSA {
 #define FRAME_NAME "BaseFrame"
 
 extern sigc::signal<void>  ColorsChanged;
+extern sigc::signal<void>  DPIReset;
 
 class ARDOUR_UI : public Gtkmm2ext::UI
 {
@@ -114,7 +116,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void show_about ();
 	void hide_about ();
 	
-	int load_session (const Glib::ustring & path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
+	void idle_load (const Glib::ustring& path);
+	int load_session (const Glib::ustring& path, const Glib::ustring& snapshot, Glib::ustring mix_template = Glib::ustring());
 	bool session_loaded;
 	int build_session (const Glib::ustring& path, const Glib::ustring& snapshot, 
 			   uint32_t ctl_chns, 
@@ -146,7 +149,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	int  save_state_canfail (string state_name = "");
 	void save_state (const string & state_name = "");
-	void restore_state (string state_name = "");
 
 	static double gain_to_slider_position (ARDOUR::gain_t g);
         static ARDOUR::gain_t slider_position_to_gain (double pos);
@@ -174,12 +176,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	static sigc::signal<void>      SuperRapidScreenUpdate;
 	static sigc::signal<void,nframes_t, bool, nframes_t> Clock;
 
-	/* this is a helper function to centralize the (complex) logic for
-	   blinking rec-enable buttons.
-	*/
-
-	void rec_enable_button_blink (bool onoff, ARDOUR::AudioDiskstream *, Gtk::Widget *w);
-
 	void name_io_setup (ARDOUR::AudioEngine&, string&, ARDOUR::IO& io, bool in);
 
 	static gint hide_and_quit (GdkEventAny *ev, ArdourDialog *);
@@ -193,6 +189,8 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void do_transport_locate (nframes_t position);
 	void halt_on_xrun_message ();
+	void xrun_handler (nframes_t);
+	void create_xrun_marker (nframes_t);
 
 	AudioClock primary_clock;
 	AudioClock secondary_clock;
@@ -225,9 +223,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void set_native_file_header_format (ARDOUR::HeaderFormat sf);
 	void set_native_file_data_format (ARDOUR::SampleFormat sf);
 
-	void set_keybindings_path (std::string path);
-	void save_keybindings ();
-
 	void setup_profile ();
 	void setup_theme ();
 
@@ -239,6 +234,9 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_clocking ();
 	void toggle_auto_play ();
 	void toggle_auto_input ();
+	void toggle_punch ();
+	void unset_dual_punch ();
+	bool ignore_dual_punch;
 	void toggle_punch_in ();
 	void toggle_punch_out ();
 	void toggle_auto_return ();
@@ -280,10 +278,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void                     goto_editor_window ();
 	void                     goto_mixer_window ();
-	
-	Gtk::Table               adjuster_table;
-	Gtk::Frame               adjuster_frame;
-	Gtk::Fixed               adjuster_base;
 
 	GlobalClickBox     *online_control_button;
 	vector<string>      online_control_strings;
@@ -294,9 +288,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::ToggleButton   preroll_button;
 	Gtk::ToggleButton   postroll_button;
 
-	Gtk::Table          transport_table;
-	Gtk::Table          option_table;
-
 	int  setup_windows ();
 	void setup_transport ();
 	void setup_clock ();
@@ -309,7 +300,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 
 	void finish();
 	int  ask_about_saving_session (const string & why);
-	int  save_the_session;
 
 	/* periodic safety backup, to be precise */
 	gint autosave_session();
@@ -347,9 +337,11 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void manage_window (Gtk::Window&);
 	
 	AudioClock   big_clock;
-	Gtk::Frame   big_clock_frame;
 	Gtk::Window* big_clock_window;
 
+	void float_big_clock (Gtk::Window* parent);
+	bool main_window_state_event_handler (GdkEventWindowState*, bool window_was_editor);
+
 	void update_transport_clocks (nframes_t pos);
 	void record_state_changed ();
 
@@ -572,6 +564,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void transport_goto_zero ();
 	void transport_goto_start ();
 	void transport_goto_end ();
+	void transport_goto_wallclock ();
 	void transport_stop ();
 	void transport_stop_and_forget_capture ();
 	void transport_record (bool roll);
@@ -591,10 +584,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void connect_dependents_to_session (ARDOUR::Session *);
 	void we_have_dependents ();
 	
-	std::string keybindings_path;
-	std::string user_keybindings_path;
-
-	void setup_keybindings ();
 	void setup_session_options ();
 	
 	guint32  last_key_press_time;
@@ -659,17 +648,18 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void flush_trash ();
 
 	bool have_configure_timeout;
-	struct timeval last_configure_time;
+	ARDOUR::microseconds_t last_configure_time;
 	gint configure_timeout ();
 
-	struct timeval last_peak_grab;
-	struct timeval last_shuttle_request;
+	ARDOUR::microseconds_t last_peak_grab;
+	ARDOUR::microseconds_t last_shuttle_request;
 
 	bool have_disk_speed_dialog_displayed;
 	void disk_speed_dialog_gone (int ignored_response, Gtk::MessageDialog*);
 	void disk_overrun_handler ();
 	void disk_underrun_handler ();
 
+	void session_dialog (std::string);
 	int pending_state_dialog ();
 	int sr_mismatch_dialog (nframes_t, nframes_t);
 	
@@ -681,9 +671,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	Gtk::MenuItem* jack_reconnect_item;
 	Gtk::Menu*     jack_bufsize_menu;
 
-	int make_session_clean ();
-	bool filter_ardour_session_dirs (const Gtk::FileFilter::Info&);
-
 	Glib::RefPtr<Gtk::ActionGroup> common_actions;
 
 	void editor_realized ();
@@ -712,6 +699,7 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_VerifyRemoveLastCapture();
 	void toggle_PeriodicSafetyBackups();
 	void toggle_StopRecordingOnXrun();
+	void toggle_CreateXrunMarker();
 	void toggle_StopTransportAtEndOfSession();
 	void toggle_GainReduceFastTransport();
 	void toggle_LatchedSolo();
@@ -723,6 +711,10 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_only_copy_imported_files ();
 	void toggle_ShowTrackMeters ();
 	void toggle_use_narrow_ms();
+	void toggle_NameNewMarkers ();
+	void toggle_rubberbanding_snaps_to_grid ();
+	void toggle_auto_analyse_audio ();
+	void toggle_TapeMachineMode();
 
 	void mtc_port_changed ();
 	void map_solo_model ();
@@ -745,7 +737,6 @@ class ARDOUR_UI : public Gtkmm2ext::UI
 	void toggle_control_protocol (ARDOUR::ControlProtocolInfo*);
 	void toggle_control_protocol_feedback (ARDOUR::ControlProtocolInfo*, const char* group_name, std::string action_name);
 
-	bool can_save_keybindings;
 	bool first_idle ();
 
 	void no_memory_warning ();
