@@ -315,6 +315,7 @@ ARDOUR_UI::setup_transport ()
 
 	primary_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::primary_clock_value_changed));
 	secondary_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::secondary_clock_value_changed));
+	big_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::big_clock_value_changed));
 
 	ARDOUR_UI::instance()->tooltips().set_tip (primary_clock, _("Primary clock"));
 	ARDOUR_UI::instance()->tooltips().set_tip (secondary_clock, _("secondary clock"));
@@ -397,7 +398,9 @@ ARDOUR_UI::setup_transport ()
 	transport_tearoff_hbox.pack_start (*svbox, false, false, 3);
 
 	transport_tearoff_hbox.pack_start (auto_loop_button, false, false);
-	transport_tearoff_hbox.pack_start (play_selection_button, false, false);
+	if (!Profile->get_sae()) {
+		transport_tearoff_hbox.pack_start (play_selection_button, false, false);
+	}
 	transport_tearoff_hbox.pack_start (roll_button, false, false);
 	transport_tearoff_hbox.pack_start (stop_button, false, false);
 	transport_tearoff_hbox.pack_start (rec_button, false, false, 6);
@@ -407,12 +410,16 @@ ARDOUR_UI::setup_transport ()
 	if (!ARDOUR::Profile->get_small_screen()) {
 		clock_box->pack_start (secondary_clock, false, false);
 	}
-	VBox* time_controls_box = manage (new VBox);
-	time_controls_box->pack_start (sync_option_combo, false, false);
-	time_controls_box->pack_start (time_master_button, false, false);
-	clock_box->pack_start (*time_controls_box, false, false, 1);
+
+	if (!Profile->get_sae()) {
+		VBox* time_controls_box = manage (new VBox);
+		time_controls_box->pack_start (sync_option_combo, false, false);
+		time_controls_box->pack_start (time_master_button, false, false);
+		clock_box->pack_start (*time_controls_box, false, false, 1);
+	}
+
 	transport_tearoff_hbox.pack_start (*clock_box, false, false, 0);
-	
+
 	HBox* toggle_box = manage(new HBox);
 	
 	VBox* punch_box = manage (new VBox);
@@ -442,6 +449,11 @@ ARDOUR_UI::setup_transport ()
 
 	transport_tearoff_hbox.pack_start (*toggle_box, false, false, 4);
 	transport_tearoff_hbox.pack_start (alert_box, false, false);
+
+	if (Profile->get_sae()) {
+		Image* img = manage (new Image ((::get_icon (X_("sae")))));
+		transport_tearoff_hbox.pack_end (*img, false, false, 6);
+	}
 }
 
 void
@@ -743,17 +755,13 @@ ARDOUR_UI::set_shuttle_fract (double f)
 void
 ARDOUR_UI::use_shuttle_fract (bool force)
 {
-	struct timeval now;
-	struct timeval diff;
+	microseconds_t now = get_microseconds();
 	
 	/* do not attempt to submit a motion-driven transport speed request
 	   more than once per process cycle.
 	 */
 
-	gettimeofday (&now, 0);
-	timersub (&now, &last_shuttle_request, &diff);
-
-	if (!force && (diff.tv_usec + (diff.tv_sec * 1000000)) < engine->usecs_per_cycle()) {
+	if (!force && (last_shuttle_request - now) < (microseconds_t) engine->usecs_per_cycle()) {
 		return;
 	}
 	
@@ -868,6 +876,7 @@ ARDOUR_UI::editor_realized ()
 	set_size_request_to_display_given_text (speed_display_box, _("-0.55"), 2, 2);
 	const guint32 FUDGE = 25; // Combo's are stupid - they steal space from the entry for the button
 	set_size_request_to_display_given_text (shuttle_style_button, _("sprung"), 2+FUDGE, 10);
+	reset_dpi();
 }
 
 void
