@@ -28,6 +28,7 @@
 #include <ardour/ardour.h>
 #include <ardour/session.h>
 #include <ardour/tempo.h>
+#include <ardour/profile.h>
 #include <sigc++/bind.h>
 
 #include "ardour_ui.h"
@@ -82,7 +83,8 @@ AudioClock::AudioClock (std::string clock_name, bool transient, std::string widg
 	key_entry_state = 0;
 	ops_menu = 0;
 	dragging = false;
-	
+	bbt_reference_time = -1;
+
 	if (with_info) {
 		frames_upper_info_label = manage (new Label);
 		frames_lower_info_label = manage (new Label);
@@ -500,15 +502,21 @@ AudioClock::set_frames (nframes_t when, bool force)
 			sprintf (buf, "%.3fK", rate/1000.0f);
 		}
 		
-		frames_upper_info_label->set_text (buf);
+		if (frames_upper_info_label->get_text() != buf) {
+			frames_upper_info_label->set_text (buf);
+		}
 		
 		float vid_pullup = Config->get_video_pullup();
 		
 		if (vid_pullup == 0.0) {
-			frames_lower_info_label->set_text(_("none"));
+			if (frames_lower_info_label->get_text () != _("none")) {
+				frames_lower_info_label->set_text(_("none"));
+			}
 		} else {
 			sprintf (buf, "%-6.4f", vid_pullup);
-			frames_lower_info_label->set_text (buf);
+			if (frames_lower_info_label->get_text() != buf) {
+				frames_lower_info_label->set_text (buf);
+			}
 		}
 	}
 }	
@@ -598,15 +606,24 @@ AudioClock::set_smpte (nframes_t when, bool force)
 			sprintf (buf, "%.2f", smpte_frames);
 		}
 		
-		smpte_upper_info_label->set_text (buf);
-		
-		if (session->smpte_drop_frames()) {
-			sprintf (buf, "DF");
-		} else {
-			sprintf (buf, "NDF");
+		if (smpte_upper_info_label->get_text() != buf) {
+			smpte_upper_info_label->set_text (buf);
 		}
 		
-		smpte_lower_info_label->set_text (buf);
+		if ((fabs(smpte_frames - 29.97) < 0.0001) || smpte_frames == 30) {
+			if (session->smpte_drop_frames()) {
+				sprintf (buf, "DF");
+			} else {
+				sprintf (buf, "NDF");
+			}
+		} else {
+			// there is no drop frame alternative
+			buf[0] = '\0';
+		}
+		
+		if (smpte_lower_info_label->get_text() != buf) {
+			smpte_lower_info_label->set_text (buf);
+		}
 	}
 }
 
@@ -617,6 +634,15 @@ AudioClock::set_bbt (nframes_t when, bool force)
 	BBT_Time bbt;
 
 	session->tempo_map().bbt_time (when, bbt);
+
+	/* handle a common case */
+
+	if (is_duration && when == 0) {
+		bbt.bars = 0;
+		bbt.beats = 0;
+
+	}
+
 	sprintf (buf, "%03" PRIu32, bbt.bars);
 	bars_label.set_text (buf);
 	sprintf (buf, "%02" PRIu32, bbt.beats);
@@ -625,11 +651,24 @@ AudioClock::set_bbt (nframes_t when, bool force)
 	ticks_label.set_text (buf);
 	
 	if (bbt_upper_info_label) {
-		TempoMap::Metric m (session->tempo_map().metric_at (when));
+		nframes64_t pos;
+
+		if (bbt_reference_time < 0) {
+			pos = when;
+		} else {
+			pos = bbt_reference_time;
+		}
+
+		TempoMap::Metric m (session->tempo_map().metric_at (pos));
+
 		sprintf (buf, "%-5.2f", m.tempo().beats_per_minute());
-		bbt_lower_info_label->set_text (buf);
+		if (bbt_lower_info_label->get_text() != buf) {
+			bbt_lower_info_label->set_text (buf);
+		}
 		sprintf (buf, "%g|%g", m.meter().beats_per_bar(), m.meter().note_divisor());
-		bbt_upper_info_label->set_text (buf);
+		if (bbt_upper_info_label->get_text() != buf) {
+			bbt_upper_info_label->set_text (buf);
+		}
 	}
 }
 
@@ -655,6 +694,32 @@ AudioClock::set_session (Session *s)
 	}
 }
 
+void
+AudioClock::focus ()
+{
+	switch (_mode) {
+	case SMPTE:
+		hours_ebox.grab_focus ();
+		break;
+
+	case BBT:
+		bars_ebox.grab_focus ();
+		break;
+
+	case MinSec:
+		ms_hours_ebox.grab_focus ();
+		break;
+
+	case Frames:
+		frames_ebox.grab_focus ();
+		break;
+
+	case Off:
+		break;
+	}
+}
+
+
 bool
 AudioClock::field_key_press_event (GdkEventKey *ev, Field field)
 {
@@ -763,13 +828,15 @@ AudioClock::field_key_release_event (GdkEventKey *ev, Field field)
 		break;
 
 	case GDK_Tab:
+	case GDK_Return:
+	case GDK_KP_Enter:
 		move_on = true;
 		break;
 
 	case GDK_Escape:
-	case GDK_Return:
-	case GDK_KP_Enter:
+		key_entry_state = 0;
 		clock_base.grab_focus ();
+		ChangeAborted();  /*  EMIT SIGNAL  */
 		return true;
 
 	default:
@@ -824,11 +891,12 @@ AudioClock::field_key_release_event (GdkEventKey *ev, Field field)
 			case Bars:
 			case Beats:
 			case Ticks:
-				// Bars or beats should never be 0
-				if (atoi(bars_label.get_text()) == 0) {
+				// Bars should never be, unless this clock is for a duration
+				if (atoi(bars_label.get_text()) == 0 && !is_duration) {
 					bars_label.set_text("001");
 				}
-				if (atoi(beats_label.get_text()) == 0) {
+				//  beats should never be 0, unless this clock is for a duration
+				if (atoi(beats_label.get_text()) == 0 && !is_duration) {
 					beats_label.set_text("01");
 				}
 				break;
@@ -894,6 +962,13 @@ AudioClock::field_key_release_event (GdkEventKey *ev, Field field)
 
 	}
 
+	//if user hit Enter, lose focus
+	switch (ev->keyval) {
+	case GDK_Return:
+	case GDK_KP_Enter:
+		clock_base.grab_focus ();
+	}
+
 	return true;
 }
 
@@ -1021,7 +1096,7 @@ AudioClock::field_button_release_event (GdkEventButton *ev, Field field)
 	if (dragging) {
 		gdk_pointer_ungrab (GDK_CURRENT_TIME);
 		dragging = false;
-		if (ev->y > drag_start_y+1 || ev->y < drag_start_y-1 || Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)){
+		if (ev->y > drag_start_y+1 || ev->y < drag_start_y-1 || Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)){
 			// we actually dragged so return without setting editing focus, or we shift clicked
 			return true;
 		}
@@ -1101,7 +1176,7 @@ AudioClock::field_button_press_event (GdkEventButton *ev, Field field)
 
 	switch (ev->button) {
 	case 1:
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			set (frames, true);
 			ValueChanged (); /* EMIT_SIGNAL */
 					}
@@ -1117,7 +1192,7 @@ AudioClock::field_button_press_event (GdkEventButton *ev, Field field)
 		break;
 
 	case 2:
-		if (Keyboard::modifier_state_equals (ev->state, Keyboard::Shift)) {
+		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			set (frames, true);
 			ValueChanged (); /* EMIT_SIGNAL */
 		}
@@ -1150,7 +1225,7 @@ AudioClock::field_button_scroll_event (GdkEventScroll *ev, Field field)
 	case GDK_SCROLL_UP:
 	       frames = get_frames (field);
 	       if (frames != 0) {
-		      if (Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+		      if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 			     frames *= 10;
 		      }
 		      set (current_time() + frames, true);
@@ -1161,7 +1236,7 @@ AudioClock::field_button_scroll_event (GdkEventScroll *ev, Field field)
 	case GDK_SCROLL_DOWN:
 	       frames = get_frames (field);
 	       if (frames != 0) {
-		      if (Keyboard::modifier_state_equals (ev->state, Keyboard::Control)) {
+		      if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 			     frames *= 10;
 		      }
 		      
@@ -1193,13 +1268,13 @@ AudioClock::field_motion_notify_event (GdkEventMotion *ev, Field field)
 	float pixel_frame_scale_factor = 0.2f;
 
 /*
-	if (Keyboard::modifier_state_equals (ev->state, Keyboard::Control))  {
+	if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier))  {
 		pixel_frame_scale_factor = 0.1f;
 	}
 
 
 	if (Keyboard::modifier_state_contains (ev->state, 
-					       Keyboard::Control|Keyboard::Alt)) {
+					       Keyboard::PrimaryModifier|Keyboard::SecondaryModifier)) {
 
 		pixel_frame_scale_factor = 0.025f;
 	}
@@ -1851,10 +1926,12 @@ AudioClock::build_ops_menu ()
 	MenuList& ops_items = ops_menu->items();
 	ops_menu->set_name ("ArdourContextMenu");
 	
-	ops_items.push_back (MenuElem (_("Timecode"), bind (mem_fun(*this, &AudioClock::set_mode), SMPTE)));
+	if (!Profile->get_sae()) {
+		ops_items.push_back (MenuElem (_("Timecode"), bind (mem_fun(*this, &AudioClock::set_mode), SMPTE)));
+	}
 	ops_items.push_back (MenuElem (_("Bars:Beats"), bind (mem_fun(*this, &AudioClock::set_mode), BBT)));
 	ops_items.push_back (MenuElem (_("Minutes:Seconds"), bind (mem_fun(*this, &AudioClock::set_mode), MinSec)));
-	ops_items.push_back (MenuElem (_("Audio Frames"), bind (mem_fun(*this, &AudioClock::set_mode), Frames)));
+	ops_items.push_back (MenuElem (_("Samples"), bind (mem_fun(*this, &AudioClock::set_mode), Frames)));
 	ops_items.push_back (MenuElem (_("Off"), bind (mem_fun(*this, &AudioClock::set_mode), Off)));
 }
 
@@ -1946,3 +2023,9 @@ AudioClock::set_size_requests ()
 		
 	}
 }
+
+void
+AudioClock::set_bbt_reference (nframes64_t pos)
+{
+	bbt_reference_time = pos;
+}
