@@ -77,9 +77,10 @@ AudioRegionView::AudioRegionView (ArdourCanvas::Group *parent, RouteTimeAxisView
 {
 }
 
+
 AudioRegionView::AudioRegionView (ArdourCanvas::Group *parent, RouteTimeAxisView &tv, boost::shared_ptr<AudioRegion> r, double spu, 
-				  Gdk::Color& basic_color, TimeAxisViewItem::Visibility visibility)
-	: RegionView (parent, tv, r, spu, basic_color, visibility)
+				  Gdk::Color& basic_color, bool recording, TimeAxisViewItem::Visibility visibility)
+	: RegionView (parent, tv, r, spu, basic_color, recording, visibility)
 	, sync_mark(0)
 	, zero_line(0)
 	, fade_in_shape(0)
@@ -116,6 +117,28 @@ AudioRegionView::AudioRegionView (const AudioRegionView& other)
 	init (c, true);
 }
 
+AudioRegionView::AudioRegionView (const AudioRegionView& other, boost::shared_ptr<AudioRegion> other_region)
+	: RegionView (other, boost::shared_ptr<Region> (other_region))
+	, zero_line(0)
+	, fade_in_shape(0)
+	, fade_out_shape(0)
+	, fade_in_handle(0)
+	, fade_out_handle(0)
+	, gain_line(0)
+	, _amplitude_above_axis(1.0)
+	, _flags(0)
+	, fade_color(0)
+
+{
+	Gdk::Color c;
+	int r,g,b,a;
+
+	UINT_TO_RGBA (other.fill_color, &r, &g, &b, &a);
+	c.set_rgb_p (r/255.0, g/255.0, b/255.0);
+	
+	init (c, true);
+}
+
 void
 AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 {
@@ -142,7 +165,7 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 	}
 
 	compute_colors (basic_color);
-
+	
 	create_waves ();
 
 	fade_in_shape = new ArdourCanvas::Polygon (*group);
@@ -192,7 +215,7 @@ AudioRegionView::init (Gdk::Color& basic_color, bool wfd)
 		gain_line->reset ();
 	}
 
-	set_height (trackview.height);
+	set_height (trackview.current_height());
 
 	region_muted ();
 	region_sync_changed ();
@@ -243,6 +266,7 @@ void
 AudioRegionView::region_changed (Change what_changed)
 {
 	ENSURE_GUI_THREAD (bind (mem_fun(*this, &AudioRegionView::region_changed), what_changed));
+	//cerr << "AudioRegionView::region_changed() called" << endl;
 
 	RegionView::region_changed(what_changed);
 
@@ -400,14 +424,16 @@ AudioRegionView::set_height (gdouble height)
 	uint32_t wcnt = waves.size();
 
 	// FIXME: ick
-	TimeAxisViewItem::set_height (height - 2);
+	height -= 2;
+	TimeAxisViewItem::set_height (height);
+	
 	
 	_height = height;
 
 	for (uint32_t n=0; n < wcnt; ++n) {
 		gdouble ht;
 
-		if ((height) <= NAME_HIGHLIGHT_THRESH) {
+		if ((height) < NAME_HIGHLIGHT_THRESH) {
 			ht = ((height-2*wcnt) / (double) wcnt);
 		} else {
 			ht = (((height-2*wcnt) - NAME_HIGHLIGHT_SIZE) / (double) wcnt);
@@ -420,7 +446,7 @@ AudioRegionView::set_height (gdouble height)
 	}
 
 	if (gain_line) {
-		if ((height/wcnt) < NAME_HIGHLIGHT_SIZE) {
+		if ((height/wcnt) < NAME_HIGHLIGHT_THRESH) {
 			gain_line->hide ();
 		} else {
 			if (_flags & EnvelopeVisible) {
@@ -514,7 +540,7 @@ AudioRegionView::reset_fade_in_shape_width (nframes_t width)
 
 	points = get_canvas_points ("fade in shape", npoints+3);
 
-	if (_height > NAME_HIGHLIGHT_THRESH) {
+	if (_height >= NAME_HIGHLIGHT_THRESH) {
 		h = _height - NAME_HIGHLIGHT_SIZE;
 	} else {
 		h = _height;
@@ -598,7 +624,7 @@ AudioRegionView::reset_fade_out_shape_width (nframes_t width)
 	float curve[npoints];
 	audio_region()->fade_out().get_vector (0, audio_region()->fade_out().back()->when, curve, npoints);
 
-	if (_height > NAME_HIGHLIGHT_THRESH) {
+	if (_height >= NAME_HIGHLIGHT_THRESH) {
 		h = _height - NAME_HIGHLIGHT_SIZE;
 	} else {
 		h = _height;
@@ -763,7 +789,7 @@ AudioRegionView::set_envelope_visible (bool yn)
 void
 AudioRegionView::create_waves ()
 {
-	//cerr << "AudioRegionView::create_waves() called on " << this << endl;//DEBUG
+	// cerr << "AudioRegionView::create_waves() called on " << this << endl;//DEBUG
 	RouteTimeAxisView& atv (*(dynamic_cast<RouteTimeAxisView*>(&trackview))); // ick
 
 	if (!atv.get_diskstream()) {
@@ -771,6 +797,8 @@ AudioRegionView::create_waves ()
 	}
 
 	uint32_t nchans = atv.get_diskstream()->n_channels();
+
+	// cerr << "creating waves for " << _region->name() << " with wfd = " << wait_for_data << " and channels = " << nchans << endl;
 	
 	/* in tmp_waves, set up null pointers for each channel so the vector is allocated */
 	for (uint32_t n = 0; n < nchans; ++n) {
@@ -785,16 +813,20 @@ AudioRegionView::create_waves ()
 		
 		wave_caches.push_back (WaveView::create_cache ());
 
+		// cerr << "\tchannel " << n << endl;
+
 		if (wait_for_data) {
 			if (audio_region()->source(n)->peaks_ready (bind (mem_fun(*this, &AudioRegionView::peaks_ready_handler), n), data_ready_connection)) {
+				// cerr << "\tData is ready\n";
 				create_one_wave (n, true);
 			} else {
+				// cerr << "\tdata is not ready\n";
 				// we'll get a PeaksReady signal from the source in the future
 				// and will call create_one_wave(n) then.
 			}
 			
 		} else {
-
+			// cerr << "\tdon't delay, display today!\n";
 			create_one_wave (n, true);
 		}
 
@@ -811,10 +843,10 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 	uint32_t nwaves = std::min (nchans, audio_region()->n_channels());
 	gdouble ht;
 
-	if (trackview.height < NAME_HIGHLIGHT_SIZE) {
-		ht = ((trackview.height) / (double) nchans);
+	if (trackview.current_height() < NAME_HIGHLIGHT_THRESH) {
+		ht = ((trackview.current_height()) / (double) nchans);
 	} else {
-		ht = ((trackview.height - NAME_HIGHLIGHT_SIZE) / (double) nchans);
+		ht = ((trackview.current_height() - NAME_HIGHLIGHT_SIZE) / (double) nchans);
 	}
 
 	gdouble yoff = which * ht;
@@ -833,8 +865,15 @@ AudioRegionView::create_one_wave (uint32_t which, bool direct)
 	wave->property_height() =  (double) ht;
 	wave->property_samples_per_unit() =  samples_per_unit;
 	wave->property_amplitude_above_axis() =  _amplitude_above_axis;
-	wave->property_wave_color() = _region->muted() ? UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA) : ARDOUR_UI::config()->canvasvar_WaveForm.get();
-	wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
+
+	if (_recregion) {
+		wave->property_wave_color() = _region->muted() ? UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_RecWaveForm.get(), MUTED_ALPHA) : ARDOUR_UI::config()->canvasvar_RecWaveForm.get();
+		wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_RecWaveFormFill.get();
+	} else {
+		wave->property_wave_color() = _region->muted() ? UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA) : ARDOUR_UI::config()->canvasvar_WaveForm.get();
+		wave->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
+	}
+
 	wave->property_clip_color() = ARDOUR_UI::config()->canvasvar_WaveFormClip.get();
 	wave->property_zero_color() = ARDOUR_UI::config()->canvasvar_ZeroLine.get();
 	wave->property_region_start() = _region->start();
@@ -918,7 +957,7 @@ AudioRegionView::add_gain_point_event (ArdourCanvas::Item *item, GdkEvent *ev)
 
 	/* compute vertical fractional position */
 
-	y = 1.0 - (y / (trackview.height - NAME_HIGHLIGHT_SIZE));
+	y = 1.0 - (y / (trackview.current_height() - NAME_HIGHLIGHT_SIZE));
 	
 	/* map using gain line */
 
@@ -1199,16 +1238,31 @@ AudioRegionView::set_frame_color ()
 			}
 		}
 	} else {
-		UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
-		frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+		if (_recregion) {
+			UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_RecordingRect.get(), &r, &g, &b, &a);
+			frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, a);
 
-		for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
-			if (_region->muted()) {
-				(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
-			} else {
-				(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
-				(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
+			for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+				if (_region->muted()) {
+					(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_RecWaveForm.get(), MUTED_ALPHA);
+				} else {
+					(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_RecWaveForm.get();
+					(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_RecWaveFormFill.get();
+				}
+			}
+		} else {
+			UINT_TO_RGBA(ARDOUR_UI::config()->canvasvar_FrameBase.get(), &r, &g, &b, &a);
+			frame->property_fill_color_rgba() = RGBA_TO_UINT(r, g, b, fill_opacity ? fill_opacity : a);
+
+			for (vector<ArdourCanvas::WaveView*>::iterator w = waves.begin(); w != waves.end(); ++w) {
+				if (_region->muted()) {
+					(*w)->property_wave_color() = UINT_RGBA_CHANGE_A(ARDOUR_UI::config()->canvasvar_WaveForm.get(), MUTED_ALPHA);
+				} else {
+					(*w)->property_wave_color() = ARDOUR_UI::config()->canvasvar_WaveForm.get();
+					(*w)->property_fill_color() = ARDOUR_UI::config()->canvasvar_WaveFormFill.get();
+				}
 			}
 		}
 	}
 }
+
