@@ -54,7 +54,16 @@ class AudioRegionView : public RegionView
 			 double initial_samples_per_unit,
 			 Gdk::Color& basic_color);
 
+	AudioRegionView (ArdourCanvas::Group *, 
+		     RouteTimeAxisView&,
+		     boost::shared_ptr<ARDOUR::AudioRegion>,
+		     double      samples_per_unit,
+		     Gdk::Color& basic_color,
+			 bool recording,
+		     TimeAxisViewItem::Visibility);
+
         AudioRegionView (const AudioRegionView& other);
+        AudioRegionView (const AudioRegionView& other, boost::shared_ptr<ARDOUR::AudioRegion>);
 
 	~AudioRegionView ();
 	
@@ -62,6 +71,8 @@ class AudioRegionView : public RegionView
 	
 	boost::shared_ptr<ARDOUR::AudioRegion> audio_region() const;
 	
+	void create_waves ();
+
 	void set_height (double);
 	void set_samples_per_unit (double);
 	
@@ -72,8 +83,8 @@ class AudioRegionView : public RegionView
 	
 	void set_envelope_visible (bool);
 	void set_waveform_visible (bool yn);
-	void set_waveform_shape (WaveformShape);
-	void set_waveform_scale (WaveformScale);
+	void set_waveform_shape (Editing::WaveformShape);
+	void set_waveform_scale (Editing::WaveformScale);
 	
 	bool waveform_rectified() const { return _flags & WaveformRectified; }
 	bool waveform_logscaled() const { return _flags & WaveformLogScaled; }
@@ -105,13 +116,6 @@ class AudioRegionView : public RegionView
        to the TimeAxisViewItem parent class
     */
     
-    AudioRegionView (ArdourCanvas::Group *, 
-		     RouteTimeAxisView&,
-		     boost::shared_ptr<ARDOUR::AudioRegion>,
-		     double      samples_per_unit,
-		     Gdk::Color& basic_color,
-		     TimeAxisViewItem::Visibility);
-    
     enum Flags {
 	    EnvelopeVisible = 0x1,
 	    WaveformVisible = 0x4,
@@ -132,7 +136,6 @@ class AudioRegionView : public RegionView
     void region_muted ();
     void region_scale_amplitude_changed ();
 
-    void create_waves ();
     void create_one_wave (uint32_t, bool);
     void manage_zero_line ();
     void peaks_ready_handler (uint32_t);
