@@ -129,7 +129,7 @@ AudioStreamView::set_amplitude_above_axis (gdouble app)
 }
 
 void
-AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait_for_waves)
+AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wait_for_waves, bool recording)
 {
 	AudioRegionView *region_view = 0;
 
@@ -161,8 +161,13 @@ AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wai
 
 	switch (_trackview.audio_track()->mode()) {
 	case Normal:
-		region_view = new AudioRegionView (canvas_group, _trackview, region, 
+		if (recording) {
+			region_view = new AudioRegionView (canvas_group, _trackview, region, 
+						   _samples_per_unit, region_color, recording, TimeAxisViewItem::Visibility(TimeAxisViewItem::ShowFrame | TimeAxisViewItem::HideFrameRight));
+		} else {
+			region_view = new AudioRegionView (canvas_group, _trackview, region, 
 						   _samples_per_unit, region_color);
+		}
 		break;
 	case Destructive:
 		region_view = new TapeAudioRegionView (canvas_group, _trackview, region, 
@@ -178,7 +183,6 @@ AudioStreamView::add_region_view_internal (boost::shared_ptr<Region> r, bool wai
 	region_view->set_amplitude_above_axis(_amplitude_above_axis);
 	region_views.push_front (region_view);
 
-	
 	/* if its the special single-sample length that we use for rec-regions, make it 
 	   insensitive to events 
 	*/
@@ -434,8 +438,9 @@ AudioStreamView::set_show_waveforms (bool yn)
 {
 	for (list<RegionView *>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
 		AudioRegionView* const arv = dynamic_cast<AudioRegionView*>(*i);
-		if (arv)
+		if (arv) {
 			arv->set_waveform_visible (yn);
+		}
 	}
 }
 
@@ -464,7 +469,7 @@ AudioStreamView::set_waveform_scale (WaveformScale scale)
 void
 AudioStreamView::setup_rec_box ()
 {
-	// cerr << _trackview.name() << " streamview SRB\n";
+	//cerr << _trackview.name() << " streamview SRB region_views.size() = " << region_views.size() << endl;
 
 	if (_trackview.session().transport_rolling()) {
 
@@ -473,7 +478,6 @@ AudioStreamView::setup_rec_box ()
 		if (!rec_active && 
 		    _trackview.session().record_status() == Session::Recording && 
 		    _trackview.get_diskstream()->record_enabled()) {
-
 			if (_trackview.audio_track()->mode() == Normal && use_rec_regions && rec_regions.size() == rec_rects.size()) {
 
 				/* add a new region, but don't bother if they set use_rec_regions mid-record */
@@ -513,16 +517,14 @@ AudioStreamView::setup_rec_box ()
 			
 			/* start a new rec box */
 
-			AudioTrack* at;
-
-			at = _trackview.audio_track(); /* we know what it is already */
+			boost::shared_ptr<AudioTrack> at = _trackview.audio_track ();
 			boost::shared_ptr<AudioDiskstream> ds = at->audio_diskstream();
 			nframes_t frame_pos = ds->current_capture_start ();
 			gdouble xstart = _trackview.editor.frame_to_pixel (frame_pos);
 			gdouble xend;
 			uint32_t fill_color;
 
-			switch (_trackview.audio_track()->mode()) {
+			switch (at->mode()) {
 			case Normal:
 				xend = xstart;
 				fill_color = ARDOUR_UI::config()->canvasvar_RecordingRect.get();
@@ -542,8 +544,9 @@ AudioStreamView::setup_rec_box ()
 			rec_rect->property_x1() = xstart;
 			rec_rect->property_y1() = 1.0;
 			rec_rect->property_x2() = xend;
-			rec_rect->property_y2() = (double) _trackview.height - 1;
-			rec_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_RecordingRect.get();
+			rec_rect->property_y2() = (double) _trackview.current_height() - 1;
+			rec_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeAxisFrame.get();
+			rec_rect->property_outline_what() = 0x1 | 0x2 | 0x4 | 0x8;
 			rec_rect->property_fill_color_rgba() = fill_color;
 			
 			RecBoxInfo recbox;
@@ -561,11 +564,9 @@ AudioStreamView::setup_rec_box ()
 		} else if (rec_active &&
 			   (_trackview.session().record_status() != Session::Recording ||
 			    !_trackview.get_diskstream()->record_enabled())) {
-
 			screen_update_connection.disconnect();
 			rec_active = false;
 			rec_updating = false;
-
 		}
 		
 	} else {
@@ -652,7 +653,6 @@ void
 AudioStreamView::update_rec_regions ()
 {
 	if (use_rec_regions) {
-
 		uint32_t n = 0;
 
 		for (list<boost::shared_ptr<Region> >::iterator iter = rec_regions.begin(); iter != rec_regions.end(); n++) {
@@ -690,7 +690,7 @@ AudioStreamView::update_rec_regions ()
 
 						if (origlen == 1) {
 							/* our special initial length */
-							add_region_view_internal (region, false);
+							add_region_view_internal (region, false, true);
 						}
 
 						/* also update rect */
@@ -715,7 +715,7 @@ AudioStreamView::update_rec_regions ()
 						
 						if (origlen == 1) {
 							/* our special initial length */
-							add_region_view_internal (region, false);
+							add_region_view_internal (region, false, true);
 						}
 						
 						/* also hide rect */
