@@ -23,7 +23,6 @@ CANVAS_VARIABLE(canvasvar_EnteredControlPointOutline, "entered control point out
 CANVAS_VARIABLE(canvasvar_EnteredControlPointSelected, "entered control point selected")
 CANVAS_VARIABLE(canvasvar_EnteredGainLine, "EnteredGainLine")
 CANVAS_VARIABLE(canvasvar_EnteredMarker, "EnteredMarker")
-CANVAS_VARIABLE(canvasvar_FrameBase, "region base")
 CANVAS_VARIABLE(canvasvar_FrameHandle, "FrameHandle")
 CANVAS_VARIABLE(canvasvar_GainLine, "GainLine")
 CANVAS_VARIABLE(canvasvar_GainLineInactive, "GainLineInactive")
@@ -60,7 +59,6 @@ CANVAS_VARIABLE(canvasvar_PunchLine, "PunchLine")
 CANVAS_VARIABLE(canvasvar_RangeDragBarRect, "RangeDragBarRect")
 CANVAS_VARIABLE(canvasvar_RangeDragRect, "RangeDragRect")
 CANVAS_VARIABLE(canvasvar_RangeMarkerBar, "rangemarker bar")
-CANVAS_VARIABLE(canvasvar_RecordingRect, "RecordingRect")
 CANVAS_VARIABLE(canvasvar_RedirectAutomationLine, "processor automation line")
 CANVAS_VARIABLE(canvasvar_RubberBandRect, "RubberBandRect")
 CANVAS_VARIABLE(canvasvar_SelectedCrossfadeEditorLine, "selected crossfade editor line")
@@ -83,8 +81,12 @@ CANVAS_VARIABLE(canvasvar_TrimHandleLocked, "TrimHandleLocked")
 CANVAS_VARIABLE(canvasvar_TrimHandle, "TrimHandle")
 CANVAS_VARIABLE(canvasvar_VerboseCanvasCursor, "verbose canvas cursor")
 CANVAS_VARIABLE(canvasvar_VestigialFrame, "VestigialFrame")
-CANVAS_VARIABLE(canvasvar_WaveFormClip, "clipped waveform")
+CANVAS_VARIABLE(canvasvar_FrameBase, "region base")
 CANVAS_VARIABLE(canvasvar_WaveFormFill, "waveform fill")
 CANVAS_VARIABLE(canvasvar_WaveForm, "waveform outline")
+CANVAS_VARIABLE(canvasvar_RecordingRect, "recorded region base")
+CANVAS_VARIABLE(canvasvar_RecWaveFormFill, "recorded waveform fill")
+CANVAS_VARIABLE(canvasvar_RecWaveForm, "recorded waveform outline")
+CANVAS_VARIABLE(canvasvar_WaveFormClip, "clipped waveform")
 CANVAS_VARIABLE(canvasvar_ZeroLine, "ZeroLine")
 CANVAS_VARIABLE(canvasvar_ZoomRect, "ZoomRect")
