@@ -59,6 +59,7 @@
 #include "editor_items.h"
 #include "region_selection.h"
 #include "canvas.h"
+#include "time_axis_view.h"
 #include "draginfo.h"
 
 namespace Gtkmm2ext {
@@ -129,10 +130,12 @@ class Editor : public PublicEditor
 
 	void             connect_to_session (ARDOUR::Session *);
 	ARDOUR::Session* current_session() const { return session; }
+	void             first_idle ();
+	virtual bool have_idled() const { return _have_idled; }
 
-	nframes_t leftmost_position() const { return leftmost_frame; }
-	nframes_t current_page_frames() const {
-		return (nframes_t) floor (canvas_width * frames_per_unit);
+	nframes64_t leftmost_position() const { return leftmost_frame; }
+	nframes64_t current_page_frames() const {
+		return (nframes64_t) floor (canvas_width * frames_per_unit);
 	}
 
 	void cycle_snap_mode ();
@@ -156,6 +159,8 @@ class Editor : public PublicEditor
 	void connect_to_image_compositor() ;
 	void scroll_timeaxis_to_imageframe_item(const TimeAxisViewItem* item) ;
 	TimeAxisView* get_named_time_axis(const std::string & name) ;
+	void foreach_time_axis_view (sigc::slot<void,TimeAxisView&>);
+	void add_to_idle_resize (TimeAxisView*, uint32_t);
 
 	void consider_auditioning (boost::shared_ptr<ARDOUR::Region>);
 	void hide_a_region (boost::shared_ptr<ARDOUR::Region>);
@@ -170,6 +175,8 @@ class Editor : public PublicEditor
 	void set_show_waveforms (bool yn);
 	bool show_waveforms() const { return _show_waveforms; }
 
+	void set_waveform_scale (Editing::WaveformScale);
+
 	void set_show_waveforms_recording (bool yn);
 	bool show_waveforms_recording() const { return _show_waveforms_recording; }
 	
@@ -178,17 +185,19 @@ class Editor : public PublicEditor
 	void new_region_from_selection ();
 	void separate_regions_between (const TimeSelection&);
 	void separate_region_from_selection ();
+	void separate_region_from_punch ();
+	void separate_region_from_loop ();
 	void separate_regions_using_location (ARDOUR::Location&);
 	void toggle_playback (bool with_abort);
 	void transition_to_rolling (bool forward);
 
 	/* undo related */
 
-	nframes_t unit_to_frame (double unit) const {
-		return (nframes_t) rint (unit * frames_per_unit);
+	nframes64_t unit_to_frame (double unit) const {
+		return (nframes64_t) rint (unit * frames_per_unit);
 	}
 	
-	double frame_to_unit (nframes_t frame) const {
+	double frame_to_unit (nframes64_t frame) const {
 		return rint ((double) frame / (double) frames_per_unit);
 	}
 
@@ -212,14 +221,14 @@ class Editor : public PublicEditor
 		*/
 
 		if (pixel >= 0) {
-			return (nframes_t) rint (pixel * frames_per_unit * GNOME_CANVAS(track_canvas.gobj())->pixels_per_unit);
+			return (nframes64_t) rint (pixel * frames_per_unit * GNOME_CANVAS(track_canvas->gobj())->pixels_per_unit);
 		} else {
 			return 0;
 		}
 	}
 
 	gulong frame_to_pixel (nframes64_t frame) const {
-		return (gulong) rint ((frame / (frames_per_unit *  GNOME_CANVAS(track_canvas.gobj())->pixels_per_unit)));
+		return (gulong) rint ((frame / (frames_per_unit *  GNOME_CANVAS(track_canvas->gobj())->pixels_per_unit)));
 	}
 
 	void flush_canvas ();
@@ -282,7 +291,7 @@ class Editor : public PublicEditor
 	void maybe_add_mixer_strip_width (XMLNode&);
 	void show_editor_mixer (bool yn);
 	void set_selected_mixer_strip (TimeAxisView&);
-	void hide_track_in_display (TimeAxisView& tv);
+	void hide_track_in_display (TimeAxisView& tv, bool temporary = false);
 	void show_track_in_display (TimeAxisView& tv);
 
 	/* nudge is initiated by transport controls owned by ARDOUR_UI */
@@ -347,13 +356,14 @@ class Editor : public PublicEditor
 
 	bool new_regionviews_display_gain () { return _new_regionviews_show_envelope; }
 	void prepare_for_cleanup ();
+	void finish_cleanup ();
 
 	void maximise_editing_space();
 	void restore_editing_space();
 
-	void reset_x_origin (nframes_t);
+	void reset_x_origin (nframes64_t);
 	void reset_zoom (double);
-	void reposition_and_zoom (nframes_t, double);
+	void reposition_and_zoom (nframes64_t, double);
 
 	nframes64_t get_preferred_edit_position (bool ignore_playhead = false);
 
@@ -364,11 +374,19 @@ class Editor : public PublicEditor
 
 	void show_rhythm_ferret();
 
+	void goto_visual_state (uint32_t);
+	void save_visual_state (uint32_t);
+
+	void queue_draw_resize_line (int at);
+	void start_resize_line_ops ();
+	void end_resize_line_ops ();
+
   protected:
 	void map_transport_state ();
-	void map_position_change (nframes_t);
+	void map_position_change (nframes64_t);
 
 	void on_realize();
+	bool on_expose_event (GdkEventExpose*);
 
   private:
 	
@@ -376,25 +394,41 @@ class Editor : public PublicEditor
 	bool                 constructed;
   
 	// to keep track of the playhead position for control_scroll
-	boost::optional<nframes_t> _control_scroll_target;
+	boost::optional<nframes64_t> _control_scroll_target;
 
 	PlaylistSelector* _playlist_selector;
 
+	typedef std::pair<TimeAxisView*,XMLNode*> TAVState;
+
 	struct VisualState {
-	    double    frames_per_unit;
-	    nframes_t leftmost_frame;
-	    Editing::ZoomFocus zoom_focus;
+	    double              y_position;
+	    double              frames_per_unit;
+	    nframes64_t         leftmost_frame;
+	    Editing::ZoomFocus  zoom_focus;
+	    bool                zoomed_to_region;
+	    std::list<TAVState> track_states;
 	};
 	
-	VisualState last_visual_state;
+	std::list<VisualState*> undo_visual_stack;
+	std::list<VisualState*> redo_visual_stack;
+	VisualState* current_visual_state (bool with_tracks = true);
+	void undo_visual_state ();
+	void redo_visual_state ();
+	void use_visual_state (VisualState&);
+	bool no_save_visual;
+	void swap_visual_state ();
+	
+	std::vector<VisualState*> visual_states;
+	sigc::connection visual_state_op_connection;
+	void start_visual_state_op (uint32_t n);
+	void cancel_visual_state_op (uint32_t n);
+	bool end_visual_state_op (uint32_t n);
 
-	nframes_t   leftmost_frame;
+	nframes64_t leftmost_frame;
 	double      frames_per_unit;
 	Editing::ZoomFocus zoom_focus;
 
-	void use_visual_state (const VisualState&);
 	void set_frames_per_unit (double);
-	void swap_visual_state ();
 	void post_zoom ();
 
 	Editing::MouseMode mouse_mode;
@@ -443,7 +477,7 @@ class Editor : public PublicEditor
 	    void hide();
 	    void show ();
 	    void set_name (const string&);
-	    void set_position (nframes_t start, nframes_t end = 0);
+	    void set_position (nframes64_t start, nframes64_t end = 0);
 	    void set_color_rgba (uint32_t);
 	};
 
@@ -456,7 +490,8 @@ class Editor : public PublicEditor
 
 	void hide_marker (ArdourCanvas::Item*, GdkEvent*);
 	void clear_marker_display ();
-	void mouse_add_new_marker (nframes_t where, bool is_cd=false);
+	void mouse_add_new_marker (nframes64_t where, bool is_cd=false, bool is_xrun=false);
+	bool choose_new_marker_name(string &name);
 	void update_cd_marker_display ();
 	void ensure_cd_marker_updated (LocationMarkers * lam, ARDOUR::Location * location);
 
@@ -511,12 +546,12 @@ class Editor : public PublicEditor
 	Gtk::Menu * track_edit_playlist_submenu;
 	Gtk::Menu * track_selection_edit_playlist_submenu;
 	
-	void popup_track_context_menu (int, int, ItemType, bool, nframes_t);
-	Gtk::Menu* build_track_context_menu (nframes_t);
-	Gtk::Menu* build_track_bus_context_menu (nframes_t);
-	Gtk::Menu* build_track_region_context_menu (nframes_t frame);
-	Gtk::Menu* build_track_crossfade_context_menu (nframes_t);
-	Gtk::Menu* build_track_selection_context_menu (nframes_t);
+	void popup_track_context_menu (int, int, ItemType, bool, nframes64_t);
+	Gtk::Menu* build_track_context_menu (nframes64_t);
+	Gtk::Menu* build_track_bus_context_menu (nframes64_t);
+	Gtk::Menu* build_track_region_context_menu (nframes64_t frame);
+	Gtk::Menu* build_track_crossfade_context_menu (nframes64_t);
+	Gtk::Menu* build_track_selection_context_menu (nframes64_t);
 	void add_dstream_context_items (Gtk::Menu_Helpers::MenuList&);
 	void add_bus_context_items (Gtk::Menu_Helpers::MenuList&);
 	void add_region_context_items (AudioStreamView*, boost::shared_ptr<ARDOUR::Region>, Gtk::Menu_Helpers::MenuList&);
@@ -531,10 +566,16 @@ class Editor : public PublicEditor
 	Gtk::VBox           global_vpacker;
 	Gtk::VBox           vpacker;
 
-	Gdk::Cursor*          current_canvas_cursor;
+	bool need_resize_line;
+	int  resize_line_y;
+	int  old_resize_line_y;
 
-	ArdourCanvas::CanvasAA track_canvas;
-	ArdourCanvas::CanvasAA time_canvas;
+	Gdk::Cursor*          current_canvas_cursor;
+	void set_canvas_cursor ();
+	Gdk::Cursor* which_grabber_cursor ();
+
+	ArdourCanvas::Canvas* track_canvas;
+	ArdourCanvas::Canvas* time_canvas;
 
 	ArdourCanvas::Text* first_action_message;
 	ArdourCanvas::Text* verbose_canvas_cursor;
@@ -568,7 +609,7 @@ class Editor : public PublicEditor
 	ArdourCanvas::Group*      transport_marker_group;
 	ArdourCanvas::Group*      cd_marker_group;
 	
-	enum {
+	enum RulerType {
 		ruler_metric_smpte = 0,
 		ruler_metric_bbt = 1,
 		ruler_metric_frames = 2,
@@ -583,7 +624,16 @@ class Editor : public PublicEditor
 	};
 
 	static GtkCustomMetric ruler_metrics[4];
-	bool                   ruler_shown[10];
+	Glib::RefPtr<Gtk::ToggleAction> ruler_timecode_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_bbt_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_samples_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_minsec_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_tempo_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_meter_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_marker_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_range_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_loop_punch_action;
+	Glib::RefPtr<Gtk::ToggleAction> ruler_cd_marker_action;
 	bool                   no_ruler_shown_update;
 	
 	gint ruler_button_press (GdkEventButton*);
@@ -598,8 +648,10 @@ class Editor : public PublicEditor
 	void update_just_smpte ();
 	void update_fixed_rulers ();
 	void update_tempo_based_rulers (); 
-	void popup_ruler_menu (nframes_t where = 0, ItemType type = RegionItem);
+	void popup_ruler_menu (nframes64_t where = 0, ItemType type = RegionItem);
 	void update_ruler_visibility ();
+	void set_ruler_visible (RulerType, bool);
+	void toggle_ruler_visibility (RulerType rt);
 	void ruler_toggled (int);
 	gint ruler_label_button_release (GdkEventButton*);
 	void store_ruler_visibility ();
@@ -664,13 +716,13 @@ class Editor : public PublicEditor
 	    Editor&               editor;
 	    ArdourCanvas::Points  points;
 	    ArdourCanvas::Line    canvas_item;
-	    nframes_t        current_frame;
+	    nframes64_t           current_frame;
 	    double		  length;
 
 	    Cursor (Editor&, bool (Editor::*)(GdkEvent*,ArdourCanvas::Item*));
 	    ~Cursor ();
 
-	    void set_position (nframes_t);
+	    void set_position (nframes64_t);
 	    void set_length (double units);
 	    void set_y_axis (double position);
 	};
@@ -705,14 +757,12 @@ class Editor : public PublicEditor
 	void    select_all_selectables_between (bool within);
 	void    select_range_between ();
 
-	boost::shared_ptr<ARDOUR::Region> find_next_region (nframes_t, ARDOUR::RegionPoint, int32_t dir, TrackViewList&, TimeAxisView ** = 0);
+	boost::shared_ptr<ARDOUR::Region> find_next_region (nframes64_t, ARDOUR::RegionPoint, int32_t dir, TrackViewList&, TimeAxisView ** = 0);
 	nframes64_t find_next_region_boundary (nframes64_t, int32_t dir, const TrackViewList&);
 
-	vector<nframes_t> region_boundary_cache;
+	vector<nframes64_t> region_boundary_cache;
 	void build_region_boundary_cache ();
 
-	Gtk::VBox           trackview_vpacker;
-
 	Gtk::HBox           top_hbox;
 	Gtk::HBox           bottom_hbox;
 	
@@ -738,7 +788,7 @@ class Editor : public PublicEditor
 	double canvas_width;
 	double canvas_height;
 	double full_canvas_height;
-	nframes_t last_canvas_frame;
+	nframes64_t last_canvas_frame;
 
 	bool track_canvas_map_handler (GdkEventAny*);
 	bool time_canvas_map_handler (GdkEventAny*);
@@ -753,7 +803,8 @@ class Editor : public PublicEditor
 	Gtk::HBox           edit_controls_hbox;
 
 	void control_scroll (float);
-	bool deferred_control_scroll (nframes_t);
+	void access_action (std::string,std::string);
+	bool deferred_control_scroll (nframes64_t);
 	sigc::connection control_scroll_connection;
 
 	void tie_vertical_scrolling ();
@@ -767,7 +818,7 @@ class Editor : public PublicEditor
 	    };
 
 	    Type pending;
-	    nframes_t time_origin;
+	    nframes64_t time_origin;
 	    double frames_per_unit;
 
 	    int idle_handler_id;
@@ -781,7 +832,7 @@ class Editor : public PublicEditor
 	static int _idle_visual_changer (void *arg);
 	int idle_visual_changer ();
 
-	void queue_visual_change (nframes_t);
+	void queue_visual_change (nframes64_t);
 	void queue_visual_change (double);
 
 	void end_location_changed (ARDOUR::Location*);
@@ -894,6 +945,7 @@ class Editor : public PublicEditor
 	static Gdk::Cursor* trimmer_cursor;
 	static Gdk::Cursor* selector_cursor;
 	static Gdk::Cursor* grabber_cursor;
+	static Gdk::Cursor* grabber_edit_point_cursor;
 	static Gdk::Cursor* zoom_cursor;
 	static Gdk::Cursor* time_fx_cursor;
 	static Gdk::Cursor* fader_cursor;
@@ -905,16 +957,16 @@ class Editor : public PublicEditor
 	static void build_cursors ();
 
 	sigc::connection scroll_connection;
-	nframes_t last_update_frame;
-	void center_screen (nframes_t);
-	void center_screen_internal (nframes_t, float);
+	nframes64_t last_update_frame;
+	void center_screen (nframes64_t);
+	void center_screen_internal (nframes64_t, float);
 	
 	void update_current_screen ();
 	
 	void session_going_away ();
 
-	nframes_t cut_buffer_start;
-	nframes_t cut_buffer_length;
+	nframes64_t cut_buffer_start;
+	nframes64_t cut_buffer_length;
 
 	bool typed_event (ArdourCanvas::Item*, GdkEvent*, ItemType);
 	bool button_press_handler (ArdourCanvas::Item*, GdkEvent*, ItemType);
@@ -927,24 +979,27 @@ class Editor : public PublicEditor
 
 	void register_actions ();
 
-	int ensure_cursor (nframes_t* pos);
+	int ensure_cursor (nframes64_t* pos);
 
 	void handle_new_audio_region (boost::weak_ptr<ARDOUR::AudioRegion>);
+	void handle_new_audio_regions (vector<boost::weak_ptr<ARDOUR::AudioRegion> >& );
 	void handle_audio_region_removed (boost::weak_ptr<ARDOUR::AudioRegion>);
 	void add_audio_region_to_region_display (boost::shared_ptr<ARDOUR::AudioRegion>);
+	void add_audio_regions_to_region_display (std::vector<boost::weak_ptr<ARDOUR::AudioRegion> > & );
 	void region_hidden (boost::shared_ptr<ARDOUR::Region>);
 	void redisplay_regions ();
+	bool no_region_list_redisplay;
 	void insert_into_tmp_audio_regionlist(boost::shared_ptr<ARDOUR::AudioRegion>);
 
 	list<boost::shared_ptr<ARDOUR::AudioRegion> > tmp_audio_region_list;
 
 	void cut_copy (Editing::CutCopyOp);
 	void cut_copy_points (Editing::CutCopyOp);
-	void cut_copy_regions (Editing::CutCopyOp);
+	void cut_copy_regions (Editing::CutCopyOp, RegionSelection&);
 	void cut_copy_ranges (Editing::CutCopyOp);
 
 	void mouse_paste ();
-	void paste_internal (nframes_t position, float times);
+	void paste_internal (nframes64_t position, float times);
 
 	/* EDITING OPERATIONS */
 	
@@ -958,21 +1013,21 @@ class Editor : public PublicEditor
 	void lower_region ();
 	void lower_region_to_bottom ();
 	void split_region ();
-	void split_region_at (nframes_t);
-	void split_regions_at (nframes_t, RegionSelection&);
+	void split_region_at (nframes64_t);
+	void split_regions_at (nframes64_t, RegionSelection&);
 	void split_region_at_transients ();
-	void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&);
+	void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&, bool can_ferret);
 	void crop_region_to_selection ();
-	void crop_region_to (nframes_t start, nframes_t end);
+	void crop_region_to (nframes64_t start, nframes64_t end);
 	void set_sync_point (nframes64_t, const RegionSelection&);
 	void set_region_sync_from_edit_point ();
 	void remove_region_sync();
-	void align_selection (ARDOUR::RegionPoint, nframes_t position, const RegionSelection&);
-	void align_selection_relative (ARDOUR::RegionPoint point, nframes_t position, const RegionSelection&);
-	void align_region (boost::shared_ptr<ARDOUR::Region>, ARDOUR::RegionPoint point, nframes_t position);
-	void align_region_internal (boost::shared_ptr<ARDOUR::Region>, ARDOUR::RegionPoint point, nframes_t position);
+	void align_selection (ARDOUR::RegionPoint, nframes64_t position, const RegionSelection&);
+	void align_selection_relative (ARDOUR::RegionPoint point, nframes64_t position, const RegionSelection&);
+	void align_region (boost::shared_ptr<ARDOUR::Region>, ARDOUR::RegionPoint point, nframes64_t position);
+	void align_region_internal (boost::shared_ptr<ARDOUR::Region>, ARDOUR::RegionPoint point, nframes64_t position);
+	void remove_region ();
 	void remove_clicked_region ();
-	void destroy_clicked_region ();
 	void edit_region ();
 	void rename_region ();
 	void duplicate_some_regions (RegionSelection&, float times);
@@ -988,6 +1043,9 @@ class Editor : public PublicEditor
 	void denormalize_region ();
 	void adjust_region_scale_amplitude (bool up);
 
+	void do_insert_time ();
+	void insert_time (nframes64_t pos, nframes64_t distance, Editing::InsertTimeOption opt, bool ignore_music_glue, bool markers_too);
+
 	void tab_to_transient (bool forward);
 
 	void use_region_as_bar ();
@@ -1029,13 +1087,13 @@ class Editor : public PublicEditor
 	void loop_location (ARDOUR::Location&);
 
 	void temporal_zoom_selection ();
-	void temporal_zoom_region ();
-	void toggle_zoom_region ();
+	void temporal_zoom_region (bool both_axes);
+	void toggle_zoom_region (bool both_axes);
 	bool zoomed_to_region;
 	void temporal_zoom_session ();
 	void temporal_zoom (gdouble scale);
-	void temporal_zoom_by_frame (nframes_t start, nframes_t end, const string & op);
-	void temporal_zoom_to_frame (bool coarser, nframes_t frame);
+	void temporal_zoom_by_frame (nframes64_t start, nframes64_t end, const string & op);
+	void temporal_zoom_to_frame (bool coarser, nframes64_t frame);
 
 	void amplitude_zoom (gdouble scale);
 	void amplitude_zoom_step (bool in);
@@ -1045,19 +1103,24 @@ class Editor : public PublicEditor
 
 	void add_external_audio_action (Editing::ImportMode);
 	void external_audio_dialog ();
+
+	int  check_whether_and_how_to_import(string, bool all_or_nothing = true);
 	bool check_multichannel_status (const std::vector<Glib::ustring>& paths);
 
 	SoundFileOmega* sfbrowser;
 	
 	void bring_in_external_audio (Editing::ImportMode mode,  nframes64_t& pos);
+
+	void _do_import (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, nframes64_t&);
 	void do_import (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, nframes64_t&);
+	bool idle_do_import (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality, nframes64_t&);
 
 	void _do_embed (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode,  nframes64_t&);
 	void do_embed (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode,  nframes64_t&);
 	bool idle_do_embed (vector<Glib::ustring> paths, Editing::ImportDisposition, Editing::ImportMode mode,  nframes64_t&);
 
 	int  import_sndfiles (vector<Glib::ustring> paths, Editing::ImportMode mode,  ARDOUR::SrcQuality, nframes64_t& pos,
-			      int target_regions, int target_tracks, boost::shared_ptr<ARDOUR::AudioTrack>&);
+			      int target_regions, int target_tracks, boost::shared_ptr<ARDOUR::AudioTrack>&, bool);
 	int  embed_sndfiles (vector<Glib::ustring> paths, bool multiple_files, bool& check_sample_rate, Editing::ImportMode mode, 
 			     nframes64_t& pos, int target_regions, int target_tracks, boost::shared_ptr<ARDOUR::AudioTrack>&);
 
@@ -1104,6 +1167,8 @@ class Editor : public PublicEditor
 	void center_edit_point ();
 	void edit_cursor_backward ();
 	void edit_cursor_forward ();
+	void playhead_forward_to_grid ();
+	void playhead_backward_to_grid ();
 	void playhead_backward ();
 	void playhead_forward ();
 	void scroll_playhead (bool forward);
@@ -1146,8 +1211,8 @@ class Editor : public PublicEditor
 	void set_loop_from_region (bool play);
 	void set_punch_from_edit_range ();
 
-	void set_loop_range (nframes_t start, nframes_t end, std::string cmd);
-	void set_punch_range (nframes_t start, nframes_t end, std::string cmd);
+	void set_loop_range (nframes64_t start, nframes64_t end, std::string cmd);
+	void set_punch_range (nframes64_t start, nframes64_t end, std::string cmd);
 
 	void add_location_from_playhead_cursor ();
 	bool select_new_marker;
@@ -1163,11 +1228,12 @@ class Editor : public PublicEditor
 	int scrubbing_direction;
 	int scrub_reversals;
 	int scrub_reverse_distance;
+	void scrub ();
 
 	void keyboard_selection_begin ();
 	void keyboard_selection_finish (bool add);
 	bool have_pending_keyboard_selection;
-	nframes_t pending_keyboard_selection_start;
+	nframes64_t pending_keyboard_selection_start;
 
 	boost::shared_ptr<ARDOUR::Region> select_region_for_operation (int dir, TimeAxisView **tv);
 	void extend_selection_to_end_of_region (bool next);
@@ -1187,6 +1253,8 @@ class Editor : public PublicEditor
 	void start_grab (GdkEvent*, Gdk::Cursor* cursor = 0);
 	bool end_grab (ArdourCanvas::Item*, GdkEvent*);
 	void swap_grab (ArdourCanvas::Item*, Gdk::Cursor* cursor, uint32_t time);
+	void break_drag ();
+	void finalize_drag ();
 
 	Gtk::Menu fade_context_menu;
 	void popup_fade_context_menu (int, int, ArdourCanvas::Item*, ItemType);
@@ -1254,11 +1322,11 @@ class Editor : public PublicEditor
 	void remove_gain_control_point (ArdourCanvas::Item*, GdkEvent*);
 	void remove_control_point (ArdourCanvas::Item*, GdkEvent*);
 
-	void mouse_brush_insert_region (RegionView*, nframes_t pos);
-	void brush (nframes_t);
+	void mouse_brush_insert_region (RegionView*, nframes64_t pos);
+	void brush (nframes64_t);
 
-	void show_verbose_time_cursor (nframes_t frame, double offset = 0, double xpos=-1, double ypos=-1);
-	void show_verbose_duration_cursor (nframes_t start, nframes_t end, double offset = 0, double xpos=-1, double ypos=-1);
+	void show_verbose_time_cursor (nframes64_t frame, double offset = 0, double xpos=-1, double ypos=-1);
+	void show_verbose_duration_cursor (nframes64_t start, nframes64_t end, double offset = 0, double xpos=-1, double ypos=-1);
 	double clamp_verbose_cursor_x (double);
 	double clamp_verbose_cursor_y (double);
 
@@ -1360,8 +1428,8 @@ public:
 
 	void new_tempo_section ();
 
-	void mouse_add_new_tempo_event (nframes_t where);
-	void mouse_add_new_meter_event (nframes_t where);
+	void mouse_add_new_tempo_event (nframes64_t where);
+	void mouse_add_new_meter_event (nframes64_t where);
 
 	void remove_tempo_marker (ArdourCanvas::Item*);
 	void remove_meter_marker (ArdourCanvas::Item*);
@@ -1387,6 +1455,8 @@ public:
 	void marker_menu_set_playhead ();
 	void marker_menu_set_from_playhead ();
 	void marker_menu_set_from_selection ();
+	void marker_menu_range_to_next ();
+	void marker_menu_export_range ();
 	void new_transport_marker_menu_set_loop ();
 	void new_transport_marker_menu_set_punch ();
 	void update_loop_range_view (bool visibility=false);
@@ -1420,12 +1490,6 @@ public:
 	void redisplay_tempo (bool immediate_redraw);
 	
 	void snap_to (nframes64_t& first, int32_t direction = 0, bool for_mark = false);
-	void snap_to (nframes_t& first, int32_t direction = 0, bool for_mark = false) {
-		/* XXX remove this function when everything moves to 64 bit frame counts */
-		nframes64_t first64 = first;
-		snap_to (first64, direction, for_mark);
-		first = (nframes_t) first64;
-	}
 
 	uint32_t bbt_beat_subdivision;
 
@@ -1518,6 +1582,7 @@ public:
 	void time_selection_changed ();
 	void track_selection_changed ();
 	void region_selection_changed ();
+	void sensitize_the_right_region_actions (bool have_selected_regions);
 	void point_selection_changed ();
 	void marker_selection_changed ();
 
@@ -1537,7 +1602,7 @@ public:
 	void region_selection_op (void (ARDOUR::Region::*pmf)(void*), void*);
 	void region_selection_op (void (ARDOUR::Region::*pmf)(bool), bool);
 
-	bool audio_region_selection_covers (nframes_t where);
+	bool audio_region_selection_covers (nframes64_t where);
 
 	/* transport range select process */
 	enum RangeMarkerOp {
@@ -1580,7 +1645,7 @@ public:
 	void drag_rubberband_select (ArdourCanvas::Item* item, GdkEvent* event);
 	void end_rubberband_select (ArdourCanvas::Item* item, GdkEvent* event);
 
-	bool select_all_within (nframes_t start, nframes_t end, gdouble topy, gdouble boty, const TrackViewList&, Selection::Operation op);
+	bool select_all_within (nframes64_t start, nframes64_t end, gdouble topy, gdouble boty, const TrackViewList&, Selection::Operation op);
 	
 	ArdourCanvas::SimpleRect   *rubberband_rect;
 	
@@ -1591,7 +1656,7 @@ public:
 	void end_mouse_zoom (ArdourCanvas::Item* item, GdkEvent* event);
 
 	ArdourCanvas::SimpleRect   *zoom_rect;
-	void reposition_zoom_rect (nframes_t start, nframes_t end);
+	void reposition_zoom_rect (nframes64_t start, nframes64_t end);
 	
 	/* diskstream/route display management */
 
@@ -1599,11 +1664,13 @@ public:
 	    RouteDisplayModelColumns() { 
 		    add (text);
 		    add (visible);
+		    add (temporary_visible);
 		    add (tv);
 		    add (route);
 	    }
 	    Gtk::TreeModelColumn<Glib::ustring>  text;
 	    Gtk::TreeModelColumn<bool>           visible;
+	    Gtk::TreeModelColumn<bool>           temporary_visible;
 	    Gtk::TreeModelColumn<TimeAxisView*>  tv;
 	    Gtk::TreeModelColumn<boost::shared_ptr<ARDOUR::Route> >  route;
 	};
@@ -1616,6 +1683,8 @@ public:
 	Gtk::ScrolledWindow                   route_list_scroller;
 	Gtk::Menu*                            route_list_menu;
 
+	void update_route_visibility ();
+
 	void sync_order_keys ();
 	bool ignore_route_order_sync;
 
@@ -1695,15 +1764,20 @@ public:
 
 	bool autoscroll_active;
 	int autoscroll_timeout_tag;
-	int autoscroll_direction;
+	int autoscroll_x;
+	int autoscroll_y;
+	int last_autoscroll_x;
+	int last_autoscroll_y;
 	uint32_t autoscroll_cnt;
-	nframes_t autoscroll_distance;
+	nframes64_t autoscroll_x_distance;
+	double autoscroll_y_distance;
      
 	static gint _autoscroll_canvas (void *);
 	bool autoscroll_canvas ();
-	void start_canvas_autoscroll (int direction);
+	void start_canvas_autoscroll (int x, int y);
 	void stop_canvas_autoscroll ();
-	void maybe_autoscroll (GdkEvent*);
+	void maybe_autoscroll (GdkEventMotion*);
+	bool allow_vertical_scroll;
 
 	/* trimming */
 	enum TrimOp {
@@ -1715,9 +1789,9 @@ public:
 	void start_trim (ArdourCanvas::Item*, GdkEvent*);
 	void point_trim (GdkEvent*);
 	void trim_motion_callback (ArdourCanvas::Item*, GdkEvent*);
-	void single_contents_trim (RegionView&, nframes_t, bool, bool, bool);
-	void single_start_trim (RegionView&, nframes_t, bool, bool);
-	void single_end_trim (RegionView&, nframes_t, bool, bool);
+	void single_contents_trim (RegionView&, nframes64_t, bool, bool, bool);
+	void single_start_trim (RegionView&, nframes64_t, bool, bool);
+	void single_end_trim (RegionView&, nframes64_t, bool, bool);
 
 	void trim_finished_callback (ArdourCanvas::Item*, GdkEvent*);
 	void thaw_region_after_trim (RegionView& rv);
@@ -1778,14 +1852,14 @@ public:
 	ExportDialog *export_dialog;
 	ExportDialog *export_range_markers_dialog;
 	
-	void export_range (nframes_t start, nframes_t end);
+	void export_range (nframes64_t start, nframes64_t end);
 	void export_range_markers ();
 
 	int  write_region_selection(RegionSelection&);
 	bool write_region (string path, boost::shared_ptr<ARDOUR::AudioRegion>);
 	void export_region ();
 	void bounce_region_selection ();
-	void bounce_range_selection ();
+	void bounce_range_selection (bool replace);
 	void external_edit_region ();
 
 	int write_audio_selection (TimeSelection&);
@@ -1801,12 +1875,6 @@ public:
 	void begin_reversible_command (string cmd_name);
 	void commit_reversible_command ();
 
-	/* visual history */
-
-	UndoHistory visual_history;
-	UndoTransaction current_visual_command;
-
-
 	void update_title ();	
 	void update_title_s (const string & snapshot_name);
 
@@ -1984,7 +2052,7 @@ public:
 	Gtk::VBox        nudge_vbox;
 	AudioClock       nudge_clock;
 
-	nframes_t get_nudge_distance (nframes_t pos, nframes_t& next);
+	nframes64_t get_nudge_distance (nframes64_t pos, nframes64_t& next);
 
 	bool nudge_forward_release (GdkEventButton*);
 	bool nudge_backward_release (GdkEventButton*);
@@ -2002,7 +2070,7 @@ public:
 	/* tracking step changes of track height */
 
 	TimeAxisView* current_stepping_trackview;
-	struct timeval last_track_height_step_timestamp;
+	ARDOUR::microseconds_t last_track_height_step_timestamp;
 	gint track_height_step_timeout();
 	sigc::connection step_timeout;
 
@@ -2012,13 +2080,15 @@ public:
 
 	void ensure_entered_track_selected (bool op_acts_on_objects = false);
 	bool clear_entered_track;
-	gint left_track_canvas (GdkEventCrossing*);
+	bool left_track_canvas (GdkEventCrossing*);
+	bool entered_track_canvas (GdkEventCrossing*);
 	void set_entered_track (TimeAxisView*);
 	void set_entered_regionview (RegionView*);
 	gint left_automation_track ();
 
 	bool _new_regionviews_show_envelope;
 
+	void reset_canvas_action_sensitivity (bool);
 	void toggle_gain_envelope_visibility ();
 	void toggle_gain_envelope_active ();
 	void reset_region_gain_envelopes ();
@@ -2040,7 +2110,7 @@ public:
 
 	Gtk::ComboBoxText edit_point_selector;
 
-	void set_edit_point_preference (Editing::EditPoint ep);
+	void set_edit_point_preference (Editing::EditPoint ep, bool force = false);
 	void cycle_edit_point (bool with_marker);
 	void set_edit_point ();
 	void edit_point_selection_done ();
@@ -2057,7 +2127,7 @@ public:
 	void get_regions_at (RegionSelection&, nframes64_t where, const TrackSelection& ts) const;
 	void get_regions_after (RegionSelection&, nframes64_t where, const TrackSelection& ts) const;
 	
-	void get_regions_for_action (RegionSelection&, bool allowed_entered_regionview = true);
+	void get_regions_for_action (RegionSelection&, bool allowed_entered_regionview = false);
 
 	sigc::connection fast_screen_update_connection;
 	gint start_updating ();
@@ -2068,7 +2138,29 @@ public:
 	void select_next_route ();
 	void select_prev_route ();
 
+	void snap_to_internal (nframes64_t& first, int32_t direction = 0, bool for_mark = false);
+
 	RhythmFerret* rhythm_ferret;
+
+	void fit_tracks ();
+	void set_track_height (uint32_t h);
+	void set_track_height_largest ();
+	void set_track_height_large ();
+	void set_track_height_larger ();
+	void set_track_height_normal ();
+	void set_track_height_smaller ();
+	void set_track_height_small ();
+
+	void remove_tracks ();
+	void toggle_tracks_active ();
+	void waveform_scale_chosen (Editing::WaveformScale);
+
+	bool _have_idled;
+	int resize_idle_id;
+	int32_t resize_idle_target;
+	bool idle_resize();
+	friend gboolean _idle_resize (gpointer);
+	std::vector<TimeAxisView*> pending_resizes;
 };
 
 #endif /* __ardour_editor_h__ */
