@@ -54,7 +54,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 {
 	int x, y;
 	double wx, wy;
-	nframes_t xdelta;
+	nframes64_t xdelta;
 	int direction = ev->direction;
 
   retry:
@@ -67,8 +67,8 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 			   if we're in mid zoom, so we have to get the damn mouse 
 			   pointer again
 			*/
-			track_canvas.get_pointer (x, y);
-			track_canvas.window_to_world (x, y, wx, wy);
+			track_canvas->get_pointer (x, y);
+			track_canvas->window_to_world (x, y, wx, wy);
 			wx += horizontal_adjustment.get_value();
 			wy += vertical_adjustment.get_value();
 			
@@ -77,7 +77,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 			event.button.x = wx;
 			event.button.y = wy;
 			
-			nframes_t where = event_frame (&event, 0, 0);
+			nframes64_t where = event_frame (&event, 0, 0);
 			temporal_zoom_to_frame (false, where);
 			return true;
 		} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::SecondaryModifier)) {
@@ -90,7 +90,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 					return false;
 				}
 			}
-			gettimeofday (&last_track_height_step_timestamp, 0);
+			last_track_height_step_timestamp = get_microseconds();
 			current_stepping_trackview->step_height (true);
 			return true;
 		} else {
@@ -102,8 +102,8 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 	case GDK_SCROLL_DOWN:
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
 			//if (ev->state == GDK_CONTROL_MASK) {
-			track_canvas.get_pointer (x, y);
-			track_canvas.window_to_world (x, y, wx, wy);
+			track_canvas->get_pointer (x, y);
+			track_canvas->window_to_world (x, y, wx, wy);
 			wx += horizontal_adjustment.get_value();
 			wy += vertical_adjustment.get_value();
 			
@@ -112,7 +112,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 			event.button.x = wx;
 			event.button.y = wy;
 			
-			nframes_t where = event_frame (&event, 0, 0);
+			nframes64_t where = event_frame (&event, 0, 0);
 			temporal_zoom_to_frame (true, where);
 			return true;
 		} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::SecondaryModifier)) {
@@ -125,7 +125,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 					return false;
 				}
 			}
-			gettimeofday (&last_track_height_step_timestamp, 0);
+			last_track_height_step_timestamp = get_microseconds();
 			current_stepping_trackview->step_height (false);
 			return true;
 		} else {
@@ -163,7 +163,7 @@ Editor::track_canvas_scroll (GdkEventScroll* ev)
 bool
 Editor::track_canvas_scroll_event (GdkEventScroll *event)
 {
-	track_canvas.grab_focus();
+	track_canvas->grab_focus();
 	track_canvas_scroll (event);
 	return false;
 }
@@ -171,7 +171,7 @@ Editor::track_canvas_scroll_event (GdkEventScroll *event)
 bool
 Editor::time_canvas_scroll (GdkEventScroll* ev)
 {
-	nframes_t xdelta;
+	nframes64_t xdelta;
 	int direction = ev->direction;
 
 	switch (direction) {
@@ -212,7 +212,7 @@ Editor::time_canvas_scroll (GdkEventScroll* ev)
 bool
 Editor::time_canvas_scroll_event (GdkEventScroll *event)
 {
-	time_canvas.grab_focus();
+	time_canvas->grab_focus();
 	time_canvas_scroll (event);
 	return false;
 }
@@ -221,7 +221,7 @@ bool
 Editor::track_canvas_button_press_event (GdkEventButton *event)
 {
 	selection->clear ();
-	track_canvas.grab_focus();
+	track_canvas->grab_focus();
 	return false;
 }
 
@@ -239,7 +239,7 @@ Editor::track_canvas_motion_notify_event (GdkEventMotion *event)
 {
 	int x, y;
 	/* keep those motion events coming */
-	track_canvas.get_pointer (x, y);
+	track_canvas->get_pointer (x, y);
 	return false;
 }
 
