@@ -173,7 +173,7 @@ Editor::update_current_screen ()
 {
 	if (session && session->engine().running()) {
 
-		nframes_t frame;
+		nframes64_t frame;
 
 		frame = session->audible_frame();
 
@@ -235,11 +235,7 @@ Editor::update_current_screen ()
 		} else {
 			
 			if (frame != last_update_frame) {
-				if (frame < leftmost_frame || frame > leftmost_frame + current_page_frames()) {
-					playhead_cursor->canvas_item.hide();
-				} else {
-					playhead_cursor->set_position (frame);
-				}
+				playhead_cursor->set_position (frame);
 			}
 		}
 
@@ -287,6 +283,8 @@ Editor::current_mixer_strip_hidden ()
 void
 Editor::session_going_away ()
 {
+	_have_idled = false;
+	
 	for (vector<sigc::connection>::iterator i = session_connections.begin(); i != session_connections.end(); ++i) {
 		(*i).disconnect ();
 	}
@@ -338,7 +336,6 @@ Editor::session_going_away ()
 	editor_mixer_button.set_active(false);
 	editor_mixer_button.set_sensitive(false);
 	/* clear tempo/meter rulers */
-
 	remove_metric_marks ();
 	hide_measures ();
 	clear_marker_display ();
