@@ -61,7 +61,7 @@ Editor::handle_new_route (Session::RouteList& routes)
 			continue;
 		}
 		
-		tv = new AudioTimeAxisView (*this, *session, route, track_canvas);
+		tv = new AudioTimeAxisView (*this, *session, route, *track_canvas);
 		//cerr << "Editor::handle_new_route() called on " << route->name() << endl;//DEBUG
 		row = *(route_display_model->append ());
 		
@@ -77,6 +77,7 @@ Editor::handle_new_route (Session::RouteList& routes)
 			if (atv->route()->order_key(N_("editor")) == -1) {
 				atv->route()->set_order_key (N_("editor"), route_display_model->children().size()-1);
 			}
+			atv->effective_gain_display ();
 		}
 
 		tv->set_old_order_key (route_display_model->children().size() - 1);
@@ -103,6 +104,14 @@ Editor::handle_gui_changes (const string & what, void *src)
 	ENSURE_GUI_THREAD(bind (mem_fun(*this, &Editor::handle_gui_changes), what, src));
 	
 	if (what == "track_height") {
+		/* make tracks change height while it happens, instead 
+		   of on first-idle
+		*/
+		track_canvas->update_now ();
+		redisplay_route_list ();
+	}
+
+	if (what == "visible_tracks") {
 		redisplay_route_list ();
 	}
 }
@@ -116,6 +125,10 @@ Editor::remove_route (TimeAxisView *tv)
 	TreeModel::Children rows = route_display_model->children();
 	TreeModel::Children::iterator ri;
 
+	if (tv == entered_track) {
+		entered_track = 0;
+	}
+
 	/* Decrement old order keys for tracks `above' the one that is being removed */
 	for (ri = rows.begin(); ri != rows.end(); ++ri) {
 		TimeAxisView* v = (*ri)[route_display_columns.tv];
@@ -163,11 +176,27 @@ Editor::route_name_changed (TimeAxisView *tv)
 			break;
 		}
 	} 
-
 }
 
 void
-Editor::hide_track_in_display (TimeAxisView& tv)
+Editor::update_route_visibility ()
+{
+	TreeModel::Children rows = route_display_model->children();
+	TreeModel::Children::iterator i;
+	
+	no_route_list_redisplay = true;
+
+	for (i = rows.begin(); i != rows.end(); ++i) {
+		TimeAxisView *tv = (*i)[route_display_columns.tv];
+		(*i)[route_display_columns.visible] = tv->marked_for_display ();
+	}
+
+	no_route_list_redisplay = false;
+	redisplay_route_list ();
+}
+
+void
+Editor::hide_track_in_display (TimeAxisView& tv, bool temponly)
 {
 	TreeModel::Children rows = route_display_model->children();
 	TreeModel::Children::iterator i;
@@ -175,6 +204,9 @@ Editor::hide_track_in_display (TimeAxisView& tv)
 	for (i = rows.begin(); i != rows.end(); ++i) {
 		if ((*i)[route_display_columns.tv] == &tv) { 
 			(*i)[route_display_columns.visible] = false;
+			// if (temponly) {
+			tv.set_marked_for_display (false);
+			// }
 			break;
 		}
 	}
@@ -548,7 +580,6 @@ Editor::route_list_delete (const Gtk::TreeModel::Path& path)
 	redisplay_route_list ();
 }
 
-
 void  
 Editor::route_list_display_drag_data_received (const RefPtr<Gdk::DragContext>& context,
 						int x, int y, 
@@ -565,3 +596,11 @@ Editor::route_list_display_drag_data_received (const RefPtr<Gdk::DragContext>& c
 	cerr << "some other kind of drag\n";
 	context->drag_finish (true, false, time);
 }
+
+void
+Editor::foreach_time_axis_view (sigc::slot<void,TimeAxisView&> theslot)
+{
+	for (TrackViewList::iterator i = track_views.begin(); i != track_views.end(); ++i) {
+		theslot (**i);
+	}
+}
