@@ -222,7 +222,7 @@ Editor::draw_measures ()
 		return;
 	}
 
-	track_canvas.get_scroll_region (x1, y1, x2, y2);
+	track_canvas->get_scroll_region (x1, y1, x2, y2);
 	y2 = TimeAxisView::hLargest*5000; // five thousand largest tracks should be enough.. :)
 
 	/* get the first bar spacing */
@@ -232,7 +232,7 @@ Editor::draw_measures ()
 	bars = (*i).bar - (*current_bbt_points->begin()).bar;
 	beats = current_bbt_points->size() - bars;
 
-	beat_density =  (beats * 10.0f) / track_canvas.get_width ();
+	beat_density =  (beats * 10.0f) / track_canvas->get_width ();
 
 	if (beat_density > 4.0f) {
 		/* if the lines are too close together, they become useless
@@ -260,7 +260,7 @@ Editor::draw_measures ()
 				}
 			}
 
-			xpos = frame_to_unit ((*i).frame);
+			xpos = frame_to_unit ((nframes64_t) (*i).frame);
 			line = get_time_line ();
 			line->property_x1() = xpos;
 			line->property_x2() = xpos;
@@ -283,7 +283,7 @@ Editor::draw_measures ()
 }
 
 void
-Editor::mouse_add_new_tempo_event (nframes_t frame)
+Editor::mouse_add_new_tempo_event (nframes64_t frame)
 {
 	if (session == 0) {
 		return;
@@ -293,7 +293,8 @@ Editor::mouse_add_new_tempo_event (nframes_t frame)
 	TempoDialog tempo_dialog (map, frame, _("add"));
 	
 	tempo_dialog.set_position (Gtk::WIN_POS_MOUSE);
-	tempo_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &tempo_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
+	//this causes compiz to display no border.
+	//tempo_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &tempo_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
 
 	ensure_float (tempo_dialog);
 
@@ -308,22 +309,23 @@ Editor::mouse_add_new_tempo_event (nframes_t frame)
 	BBT_Time requested;
 	
 	bpm = tempo_dialog.get_bpm ();
+	double nt = tempo_dialog.get_note_type();
 	bpm = max (0.01, bpm);
 	
 	tempo_dialog.get_bbt_time (requested);
 	
 	begin_reversible_command (_("add tempo mark"));
         XMLNode &before = map.get_state();
-	map.add_tempo (Tempo (bpm), requested);
+	map.add_tempo (Tempo (bpm,nt), requested);
         XMLNode &after = map.get_state();
 	session->add_command(new MementoCommand<TempoMap>(map, &before, &after));
 	commit_reversible_command ();
 	
-	map.dump (cerr);
+	//map.dump (cerr);
 }
 
 void
-Editor::mouse_add_new_meter_event (nframes_t frame)
+Editor::mouse_add_new_meter_event (nframes64_t frame)
 {
 	if (session == 0) {
 		return;
@@ -334,7 +336,9 @@ Editor::mouse_add_new_meter_event (nframes_t frame)
 	MeterDialog meter_dialog (map, frame, _("add"));
 
 	meter_dialog.set_position (Gtk::WIN_POS_MOUSE);
-	meter_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &meter_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
+
+	//this causes compiz to display no border.. 
+	//meter_dialog.signal_realize().connect (bind (sigc::ptr_fun (set_decoration), &meter_dialog, Gdk::WMDecoration (Gdk::DECOR_BORDER|Gdk::DECOR_RESIZEH)));
 
 	ensure_float (meter_dialog);
 	
@@ -350,16 +354,16 @@ Editor::mouse_add_new_meter_event (nframes_t frame)
 	
 	double note_type = meter_dialog.get_note_type ();
 	BBT_Time requested;
-	
+
 	meter_dialog.get_bbt_time (requested);
-	
+
 	begin_reversible_command (_("add meter mark"));
         XMLNode &before = map.get_state();
 	map.add_meter (Meter (bpb, note_type), requested);
 	session->add_command(new MementoCommand<TempoMap>(map, &before, &map.get_state()));
 	commit_reversible_command ();
 	
-	map.dump (cerr);
+	//map.dump (cerr);
 }
 
 void
@@ -429,14 +433,19 @@ Editor::edit_tempo_section (TempoSection* section)
 	}
 
 	double bpm = tempo_dialog.get_bpm ();
+	double nt = tempo_dialog.get_note_type ();
 	BBT_Time when;
 	tempo_dialog.get_bbt_time(when);
 	bpm = max (0.01, bpm);
 	
+	cerr << "Editing tempo section to be at " << when << endl;
+	session->tempo_map().dump (cerr);
 	begin_reversible_command (_("replace tempo mark"));
         XMLNode &before = session->tempo_map().get_state();
-	session->tempo_map().replace_tempo (*section, Tempo (bpm));
+	session->tempo_map().replace_tempo (*section, Tempo (bpm,nt));
+	session->tempo_map().dump (cerr);
 	session->tempo_map().move_tempo (*section, when);
+	session->tempo_map().dump (cerr);
         XMLNode &after = session->tempo_map().get_state();
 	session->add_command (new MementoCommand<TempoMap>(session->tempo_map(), &before, &after));
 	commit_reversible_command ();
