@@ -128,7 +128,7 @@ ExportDialog::ExportDialog(PublicEditor& e)
 	set_wmclass (X_("ardour_export"), "Ardour");
 	set_name ("ExportWindow");
 	add_events (Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK);
-
+	
 	spec.running = false;
 
 	file_entry.set_name ("ExportFileNameEntry");
@@ -142,7 +142,7 @@ ExportDialog::ExportDialog(PublicEditor& e)
 	master_selector.append_column_editable(_("Left"), exp_cols.left);
 	master_selector.append_column_editable(_("Right"), exp_cols.right);
 	master_selector.get_column(0)->set_min_width(100);
-	
+
 	master_selector.get_column(1)->set_min_width(40);
 	master_selector.get_column(1)->set_sizing(Gtk::TREE_VIEW_COLUMN_AUTOSIZE);
 	master_selector.get_column(2)->set_min_width(40);
@@ -974,6 +974,8 @@ ExportDialog::do_export ()
 	progress_connection = Glib::signal_timeout().connect (mem_fun(*this, &ExportDialog::progress_timeout), 100);
 	cancel_label.set_text (_("Stop Export"));
 
+	session->pre_export ();
+
 	export_audio_data();
 	
   	progress_connection.disconnect ();
@@ -1016,18 +1018,13 @@ ExportDialog::start_export ()
 	*/
 	
 	if (file_entry.get_text().length() == 0) {
-		string dir = session->export_dir();
-		string::size_type last_slash;
-		
-		if ((last_slash = dir.find_last_of ('/')) != string::npos && last_slash != 0) {
-			dir = dir.substr (0, last_slash+1);
-		}
+		Glib::ustring export_path = session->export_dir();
 
 		if (!wants_dir()) {
-			dir = dir + "export.wav";
+			export_path = Glib::build_filename (export_path, "export.wav");
 		}
 		
-		file_entry.set_text (dir);
+		file_entry.set_text (export_path);
 	}
 	
 	progress_bar.set_fraction (0);
@@ -1233,7 +1230,7 @@ ExportDialog::initSpec(string &filepath)
 {
 	spec.path = filepath;
 	spec.progress = 0;
-	spec.running = true;
+	spec.running = false;
 	spec.stop = false;
 	spec.port_map.clear();
 	
