@@ -53,43 +53,51 @@ using namespace Gtk;
 using namespace sigc;
 using namespace std;
 
-sigc::signal<void> GainMeter::ResetAllPeakDisplays;
-sigc::signal<void,RouteGroup*> GainMeter::ResetGroupPeakDisplays;
-Glib::RefPtr<Gdk::Pixbuf> GainMeter::slider;
-Glib::RefPtr<Gdk::Pixbuf> GainMeter::rail;
-map<string,Glib::RefPtr<Gdk::Pixmap> > GainMeter::metric_pixmaps;
+sigc::signal<void> GainMeterBase::ResetAllPeakDisplays;
+sigc::signal<void,RouteGroup*> GainMeterBase::ResetGroupPeakDisplays;
 
-int
+map<string,Glib::RefPtr<Gdk::Pixmap> > GainMeter::metric_pixmaps;
+Glib::RefPtr<Gdk::Pixbuf> GainMeter::slider;
+
+
+void
 GainMeter::setup_slider_pix ()
 {
-	slider = ::get_icon ("fader_belt");
-	return 0;
+	if ((slider = ::get_icon ("fader_belt")) == 0) {
+		throw failed_constructor();
+	}
 }
 
-GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
+GainMeterBase::GainMeterBase (boost::shared_ptr<IO> io, Session& s, 
+			      const Glib::RefPtr<Gdk::Pixbuf>& pix,
+			      bool horizontal)
 	: _io (io),
 	  _session (s),
-	  gain_slider (0),
 	  // 0.781787 is the value needed for gain to be set to 0.
 	  gain_adjustment (0.781787, 0.0, 1.0, 0.01, 0.1),
 	  gain_automation_style_button (""),
-	  gain_automation_state_button (""),
-	  regular_meter_width(5)
+	  gain_automation_state_button ("")
 	
 {
-	if (slider == 0) {
-		setup_slider_pix ();
-	}
-
 	ignore_toggle = false;
 	meter_menu = 0;
 	next_release_selects = false;
 	style_changed = true;
+	_width = Wide;
 
-	gain_slider = manage (new VSliderController (slider,
-						     &gain_adjustment,
-						     _io->gain_control(),
-						     false));
+	if (horizontal) {
+		gain_slider = manage (new HSliderController (pix,
+							     &gain_adjustment,
+							     _io->gain_control(),
+							     false));
+	} else {
+		gain_slider = manage (new VSliderController (pix,
+							     &gain_adjustment,
+							     _io->gain_control(),
+							     false));
+	}
+
+	level_meter = new LevelMeter(_io, _session);
 
 	gain_slider->signal_button_press_event().connect (mem_fun(*this, &GainMeter::start_gain_touch));
 	gain_slider->signal_button_release_event().connect (mem_fun(*this, &GainMeter::end_gain_touch));
@@ -102,10 +110,6 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	gain_display.signal_focus_in_event().connect (mem_fun (*this, &GainMeter::gain_focused), false);
 	gain_display.signal_focus_out_event().connect (mem_fun (*this, &GainMeter::gain_focused), false);
 
-	gain_display_box.set_homogeneous (true);
-	gain_display_box.set_spacing (2);
-	gain_display_box.pack_start (gain_display, true, true);
-
 	peak_display.set_name ("MixerStripPeakDisplay");
 //	peak_display.set_has_frame (false);
 //	peak_display.set_editable (false);
@@ -114,11 +118,639 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	peak_display.set_label (_("-inf"));
 	peak_display.unset_flags (Gtk::CAN_FOCUS);
 
+	gain_automation_style_button.set_name ("MixerAutomationModeButton");
+	gain_automation_state_button.set_name ("MixerAutomationPlaybackButton");
+
+	ARDOUR_UI::instance()->tooltips().set_tip (gain_automation_state_button, _("Fader automation mode"));
+	ARDOUR_UI::instance()->tooltips().set_tip (gain_automation_style_button, _("Fader automation type"));
+
+	gain_automation_style_button.unset_flags (Gtk::CAN_FOCUS);
+	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
+
+	gain_automation_state_button.set_size_request(15, 15);
+	gain_automation_style_button.set_size_request(15, 15);
+
+	boost::shared_ptr<Route> r;
+
+	if ((r = boost::dynamic_pointer_cast<Route> (_io)) != 0) {
+
+		if (!r->hidden()) {
+
+			using namespace Menu_Helpers;
+			
+			gain_astate_menu.items().push_back (MenuElem (_("Manual"), 
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Off)));
+			gain_astate_menu.items().push_back (MenuElem (_("Play"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Play)));
+			gain_astate_menu.items().push_back (MenuElem (_("Write"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Write)));
+			gain_astate_menu.items().push_back (MenuElem (_("Touch"),
+								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Touch)));
+			
+			gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
+			gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
+			
+			gain_astate_menu.set_name ("ArdourContextMenu");
+			gain_astyle_menu.set_name ("ArdourContextMenu");
+			
+			gain_automation_style_button.signal_button_press_event().connect (mem_fun(*this, &GainMeterBase::gain_automation_style_button_event), false);
+			gain_automation_state_button.signal_button_press_event().connect (mem_fun(*this, &GainMeterBase::gain_automation_state_button_event), false);
+			
+			r->gain_automation_curve().automation_state_changed.connect (mem_fun(*this, &GainMeterBase::gain_automation_state_changed));
+			r->gain_automation_curve().automation_style_changed.connect (mem_fun(*this, &GainMeterBase::gain_automation_style_changed));
+			
+			gain_automation_state_changed ();
+		}
+	}
+
+	_io->gain_changed.connect (mem_fun(*this, &GainMeterBase::gain_changed));
+
+	gain_adjustment.signal_value_changed().connect (mem_fun(*this, &GainMeterBase::gain_adjusted));
+	peak_display.signal_button_release_event().connect (mem_fun(*this, &GainMeterBase::peak_button_release), false);
+	gain_display.signal_key_press_event().connect (mem_fun(*this, &GainMeterBase::gain_key_press), false);
+
+	gain_changed (0);
+	show_gain ();
+
+	update_gain_sensitive ();
+	
+	ResetAllPeakDisplays.connect (mem_fun(*this, &GainMeterBase::reset_peak_display));
+	ResetGroupPeakDisplays.connect (mem_fun(*this, &GainMeterBase::reset_group_peak_display));
+
+	UI::instance()->theme_changed.connect (mem_fun(*this, &GainMeterBase::on_theme_changed));
+	ColorsChanged.connect (bind(mem_fun (*this, &GainMeterBase::color_handler), false));
+	DPIReset.connect (bind(mem_fun (*this, &GainMeterBase::color_handler), true));
+}
+
+GainMeterBase::~GainMeterBase ()
+{
+	if (meter_menu) {
+		delete meter_menu;
+	}
+
+	if (level_meter) {
+		delete level_meter;
+	}
+}
+
+void
+GainMeterBase::hide_all_meters ()
+{
+	level_meter->hide_meters();
+}
+
+void
+GainMeter::hide_all_meters ()
+{
+	bool remove_metric_area = false;
+
+	GainMeterBase::hide_all_meters ();
+
+	if (remove_metric_area) {
+		if (meter_metric_area.get_parent()) {
+			level_meter->remove (meter_metric_area);
+		}
+	}
+}
+
+void
+GainMeterBase::setup_meters (int len)
+{
+	level_meter->setup_meters(len, 5);
+}
+
+void 
+GainMeter::setup_meters (int len)
+{
+	if (!meter_metric_area.get_parent()) {
+		level_meter->pack_end (meter_metric_area, false, false);
+		meter_metric_area.show_all ();
+	}
+	GainMeterBase::setup_meters (len);
+}
+
+bool
+GainMeterBase::gain_key_press (GdkEventKey* ev)
+{
+	if (key_is_legal_for_numeric_entry (ev->keyval)) {
+		/* drop through to normal handling */
+		return false;
+	}
+	/* illegal key for gain entry */
+	return true;
+}
+
+bool
+GainMeterBase::peak_button_release (GdkEventButton* ev)
+{
+	/* reset peak label */
+
+	if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier|Keyboard::TertiaryModifier)) {
+		ResetAllPeakDisplays ();
+	} else if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
+		boost::shared_ptr<Route> r;
+
+		if ((r = boost::dynamic_pointer_cast<Route> (_io)) != 0) {
+			ResetGroupPeakDisplays (r->mix_group());
+		}
+	} else {
+		reset_peak_display ();
+	}
+
+	return true;
+}
+
+void
+GainMeterBase::reset_peak_display ()
+{
+	boost::shared_ptr<Route> r;
+
+	if ((r = boost::dynamic_pointer_cast<Route> (_io)) != 0) {
+		r->reset_max_peak_meters();
+	}
+
+	level_meter->clear_meters();
+	max_peak = -INFINITY;
+	peak_display.set_label (_("-Inf"));
+	peak_display.set_name ("MixerStripPeakDisplay");
+}
+
+void
+GainMeterBase::reset_group_peak_display (RouteGroup* group)
+{
+	boost::shared_ptr<Route> r;
+	
+	if ((r = boost::dynamic_pointer_cast<Route> (_io)) != 0) {
+		if (group == r->mix_group()) {
+			reset_peak_display ();
+		}
+	}
+}
+
+void
+GainMeterBase::popup_meter_menu (GdkEventButton *ev)
+{
+	using namespace Menu_Helpers;
+
+	if (meter_menu == 0) {
+		meter_menu = new Gtk::Menu;
+		MenuList& items = meter_menu->items();
+
+		items.push_back (MenuElem ("-inf .. +0dBFS"));
+		items.push_back (MenuElem ("-10dB .. +0dBFS"));
+		items.push_back (MenuElem ("-4 .. +0dBFS"));
+		items.push_back (SeparatorElem());
+		items.push_back (MenuElem ("-inf .. -2dBFS"));
+		items.push_back (MenuElem ("-10dB .. -2dBFS"));
+		items.push_back (MenuElem ("-4 .. -2dBFS"));
+	}
+
+	meter_menu->popup (1, ev->time);
+}
+
+bool
+GainMeterBase::gain_focused (GdkEventFocus* ev)
+{
+	if (ev->in) {
+		gain_display.select_region (0, -1);
+	} else {
+		gain_display.select_region (0, 0);
+	}
+	return false;
+}
+
+void
+GainMeterBase::gain_activated ()
+{
+	float f;
+
+	if (sscanf (gain_display.get_text().c_str(), "%f", &f) == 1) {
+
+		/* clamp to displayable values */
+
+		f = min (f, 6.0f);
+
+		_io->set_gain (dB_to_coefficient(f), this);
+
+		if (gain_display.has_focus()) {
+			PublicEditor::instance().reset_focus();
+		}
+	}
+}
+
+void
+GainMeterBase::show_gain ()
+{
+	char buf[32];
+
+	float v = gain_adjustment.get_value();
+	
+	if (v == 0.0) {
+		strcpy (buf, _("-inf"));
+	} else {
+		snprintf (buf, 32, "%.1f", coefficient_to_dB (slider_position_to_gain (v)));
+	}
+	
+	gain_display.set_text (buf);
+}
+
+void
+GainMeterBase::gain_adjusted ()
+{
+	if (!ignore_toggle) {
+		_io->set_gain (slider_position_to_gain (gain_adjustment.get_value()), this);
+	}
+	show_gain ();
+}
+
+void
+GainMeterBase::effective_gain_display ()
+{
+	gfloat value = gain_to_slider_position (_io->effective_gain());
+
+	if (gain_adjustment.get_value() != value) {
+		ignore_toggle = true; 
+		gain_adjustment.set_value (value);
+		ignore_toggle = false;
+	}
+}
+
+void
+GainMeterBase::gain_changed (void *src)
+{
+	Gtkmm2ext::UI::instance()->call_slot (mem_fun(*this, &GainMeterBase::effective_gain_display));
+}
+
+void
+GainMeterBase::set_meter_strip_name (const char * name)
+{
+	meter_metric_area.set_name (name);
+	style_changed = true;
+}
+
+void
+GainMeterBase::set_fader_name (const char * name)
+{
+	gain_slider->set_name (name);
+}
+
+void
+GainMeterBase::update_gain_sensitive ()
+{
+	static_cast<Gtkmm2ext::SliderController*>(gain_slider)->set_sensitive (!(_io->gain_automation_state() & Play));
+}
+
+
+static MeterPoint
+next_meter_point (MeterPoint mp)
+{
+	switch (mp) {
+	case MeterInput:
+		return MeterPreFader;
+		break;
+		
+	case MeterPreFader:
+		return MeterPostFader;
+		break;
+		
+	case MeterPostFader:
+		return MeterInput;
+		break;
+	}
+	/*NOTREACHED*/
+	return MeterInput;
+}
+
+gint
+GainMeterBase::meter_press(GdkEventButton* ev)
+{
+	boost::shared_ptr<Route> _route;
+
+	wait_for_release = false;
+	
+	if ((_route = boost::dynamic_pointer_cast<Route>(_io)) == 0) {
+		return FALSE;
+	}
+
+	if (!ignore_toggle) {
+
+		if (Keyboard::is_context_menu_event (ev)) {
+			
+			// no menu at this time.
+
+		} else {
+
+			if (ev->button == 2) {
+
+				// Primary-button2 click is the midi binding click
+				// button2-click is "momentary"
+				
+				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
+					wait_for_release = true;
+					old_meter_point = _route->meter_point ();
+				}
+			}
+
+			if (ev->button == 1 || ev->button == 2) {
+
+				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
+
+					/* Primary+Tertiary-click applies change to all routes */
+
+					_session.begin_reversible_command (_("meter point change"));
+                                        Session::GlobalMeteringStateCommand *cmd = new Session::GlobalMeteringStateCommand (_session, this);
+					_session.foreach_route (this, &GainMeterBase::set_meter_point, next_meter_point (_route->meter_point()));
+                                        cmd->mark();
+					_session.add_command (cmd);
+					_session.commit_reversible_command ();
+					
+					
+				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
+
+					/* Primary-click: solo mix group.
+					   NOTE: Primary-button2 is MIDI learn.
+					*/
+					
+					if (ev->button == 1) {
+						_session.begin_reversible_command (_("meter point change"));
+						Session::GlobalMeteringStateCommand *cmd = new Session::GlobalMeteringStateCommand (_session, this);
+						set_mix_group_meter_point (*_route, next_meter_point (_route->meter_point()));
+						cmd->mark();
+						_session.add_command (cmd);
+						_session.commit_reversible_command ();
+					}
+					
+				} else {
+					
+					/* click: change just this route */
+
+					// XXX no undo yet
+					
+					_route->set_meter_point (next_meter_point (_route->meter_point()), this);
+				}
+			}
+		}
+	}
+
+	return true;
+
+}
+
+gint
+GainMeterBase::meter_release(GdkEventButton* ev)
+{
+	if(!ignore_toggle){
+		if (wait_for_release){
+			wait_for_release = false;
+			
+			boost::shared_ptr<Route> r;
+			
+			if ((r = boost::dynamic_pointer_cast<Route>(_io)) != 0) {
+				set_meter_point (*r, old_meter_point);
+			}
+		}
+	}
+
+	return true;
+}
+
+void
+GainMeterBase::set_meter_point (Route& route, MeterPoint mp)
+{
+	route.set_meter_point (mp, this);
+}
+
+void
+GainMeterBase::set_mix_group_meter_point (Route& route, MeterPoint mp)
+{
+	RouteGroup* mix_group;
+
+	if((mix_group = route.mix_group()) != 0){
+		mix_group->apply (&Route::set_meter_point, mp, this);
+	} else {
+		route.set_meter_point (mp, this);
+	}
+}
+
+void
+GainMeterBase::meter_point_clicked ()
+{
+	boost::shared_ptr<Route> r;
+
+	if ((r = boost::dynamic_pointer_cast<Route> (_io)) != 0) {
+		/* WHAT? */
+	}
+}
+
+gint
+GainMeterBase::start_gain_touch (GdkEventButton* ev)
+{
+	_io->start_gain_touch ();
+	return FALSE;
+}
+
+gint
+GainMeterBase::end_gain_touch (GdkEventButton* ev)
+{
+	_io->end_gain_touch ();
+	return FALSE;
+}
+
+gint
+GainMeterBase::gain_automation_state_button_event (GdkEventButton *ev)
+{
+	if (ev->type == GDK_BUTTON_RELEASE) {
+		return TRUE;
+	}
+	
+	switch (ev->button) {
+		case 1:
+			gain_astate_menu.popup (1, ev->time);
+			break;
+		default:
+			break;
+	}
+
+	return TRUE;
+}
+
+gint
+GainMeterBase::gain_automation_style_button_event (GdkEventButton *ev)
+{
+	if (ev->type == GDK_BUTTON_RELEASE) {
+		return TRUE;
+	}
+
+	switch (ev->button) {
+	case 1:
+		gain_astyle_menu.popup (1, ev->time);
+		break;
+	default:
+		break;
+	}
+	return TRUE;
+}
+
+string
+GainMeterBase::astate_string (AutoState state)
+{
+	return _astate_string (state, false);
+}
+
+string
+GainMeterBase::short_astate_string (AutoState state)
+{
+	return _astate_string (state, true);
+}
+
+string
+GainMeterBase::_astate_string (AutoState state, bool shrt)
+{
+	string sstr;
+
+	switch (state) {
+	case Off:
+		sstr = (shrt ? "M" : _("M"));
+		break;
+	case Play:
+		sstr = (shrt ? "P" : _("P"));
+		break;
+	case Touch:
+		sstr = (shrt ? "T" : _("T"));
+		break;
+	case Write:
+		sstr = (shrt ? "W" : _("W"));
+		break;
+	}
+
+	return sstr;
+}
+
+string
+GainMeterBase::astyle_string (AutoStyle style)
+{
+	return _astyle_string (style, false);
+}
+
+string
+GainMeterBase::short_astyle_string (AutoStyle style)
+{
+	return _astyle_string (style, true);
+}
+
+string
+GainMeterBase::_astyle_string (AutoStyle style, bool shrt)
+{
+	if (style & Trim) {
+		return _("Trim");
+	} else {
+	        /* XXX it might different in different languages */
+
+		return (shrt ? _("Abs") : _("Abs"));
+	}
+}
+
+void
+GainMeterBase::gain_automation_style_changed ()
+{
+	switch (_width) {
+	case Wide:
+	        gain_automation_style_button.set_label (astyle_string(_io->gain_automation_curve().automation_style()));
+		break;
+	case Narrow:
+		gain_automation_style_button.set_label  (short_astyle_string(_io->gain_automation_curve().automation_style()));
+		break;
+	}
+}
+
+void
+GainMeterBase::gain_automation_state_changed ()
+{
+	ENSURE_GUI_THREAD(mem_fun(*this, &GainMeterBase::gain_automation_state_changed));
+	
+	bool x;
+
+	switch (_width) {
+	case Wide:
+		gain_automation_state_button.set_label (astate_string(_io->gain_automation_curve().automation_state()));
+		break;
+	case Narrow:
+		gain_automation_state_button.set_label (short_astate_string(_io->gain_automation_curve().automation_state()));
+		break;
+	}
+
+	x = (_io->gain_automation_state() != Off);
+	
+	if (gain_automation_state_button.get_active() != x) {
+		ignore_toggle = true;
+		gain_automation_state_button.set_active (x);
+		ignore_toggle = false;
+	}
+
+	update_gain_sensitive ();
+	
+	/* start watching automation so that things move */
+	
+	gain_watching.disconnect();
+
+	if (x) {
+		gain_watching = ARDOUR_UI::RapidScreenUpdate.connect (mem_fun (*this, &GainMeterBase::effective_gain_display));
+	}
+}
+
+void
+GainMeterBase::update_meters()
+{
+	char buf[32];
+	float mpeak = level_meter->update_meters();
+
+	if (mpeak > max_peak) {
+		max_peak = mpeak;
+		if (mpeak <= -200.0f) {
+			peak_display.set_label (_("-inf"));
+		} else {
+			snprintf (buf, sizeof(buf), "%.1f", mpeak);
+			peak_display.set_label (buf);
+		}
+
+		if (mpeak >= 0.0f) {
+			peak_display.set_name ("MixerStripPeakDisplayPeak");
+		}
+	}
+}
+
+void GainMeterBase::color_handler(bool dpi)
+{
+	color_changed = true;
+	dpi_changed = (dpi) ? true : false;
+	setup_meters();
+}
+
+void
+GainMeterBase::set_width (Width w, int len)
+{
+	_width = w;
+	level_meter->setup_meters (len);
+}
+
+
+void
+GainMeterBase::on_theme_changed()
+{
+	style_changed = true;
+}
+
+GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
+	: GainMeterBase (io, s, slider, false)
+{
+
+	gain_display_box.set_homogeneous (true);
+	gain_display_box.set_spacing (2);
+	gain_display_box.pack_start (gain_display, true, true);
+
 	meter_metric_area.set_name ("AudioTrackMetrics");
 	set_size_request_to_display_given_text (meter_metric_area, "-50", 0, 0);
 
-	meter_packer.set_spacing (2);
-
 	gain_automation_style_button.set_name ("MixerAutomationModeButton");
 	gain_automation_state_button.set_name ("MixerAutomationPlaybackButton");
 
@@ -141,45 +773,20 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	hbox.set_spacing (2);
 	hbox.pack_start (*fader_vbox, true, true);
 
-	Route* r;
-
-	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
+	boost::shared_ptr<Route> r;
 
+	if ((r = boost::dynamic_pointer_cast<Route> (_io)) != 0) {
+		
 		/* 
 		   if we have a non-hidden route (ie. we're not the click or the auditioner), 
 		   pack some route-dependent stuff.
 		*/
 
 		gain_display_box.pack_end (peak_display, true, true);
-		hbox.pack_end (meter_packer, true, true);
+		hbox.pack_end (*level_meter, true, true);
 
 		if (!r->hidden()) {
-
-			using namespace Menu_Helpers;
-			
-			gain_astate_menu.items().push_back (MenuElem (_("Manual"), 
-								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Off)));
-			gain_astate_menu.items().push_back (MenuElem (_("Play"),
-								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Play)));
-			gain_astate_menu.items().push_back (MenuElem (_("Write"),
-								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Write)));
-			gain_astate_menu.items().push_back (MenuElem (_("Touch"),
-								      bind (mem_fun (*_io, &IO::set_gain_automation_state), (AutoState) Touch)));
-			
-			gain_astyle_menu.items().push_back (MenuElem (_("Trim")));
-			gain_astyle_menu.items().push_back (MenuElem (_("Abs")));
-			
-			gain_astate_menu.set_name ("ArdourContextMenu");
-			gain_astyle_menu.set_name ("ArdourContextMenu");
-			
-			gain_automation_style_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_style_button_event), false);
-			gain_automation_state_button.signal_button_press_event().connect (mem_fun(*this, &GainMeter::gain_automation_state_button_event), false);
-			
-			r->gain_automation_curve().automation_state_changed.connect (mem_fun(*this, &GainMeter::gain_automation_state_changed));
-			r->gain_automation_curve().automation_style_changed.connect (mem_fun(*this, &GainMeter::gain_automation_style_changed));
-			
 			fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
-			gain_automation_state_changed ();
 		}
 	}
 
@@ -188,41 +795,21 @@ GainMeter::GainMeter (boost::shared_ptr<IO> io, Session& s)
 	pack_start (gain_display_box, Gtk::PACK_SHRINK);
 	pack_start (hbox, Gtk::PACK_SHRINK);
 
-	_io->gain_changed.connect (mem_fun(*this, &GainMeter::gain_changed));
-
 	meter_metric_area.signal_expose_event().connect (mem_fun(*this, &GainMeter::meter_metrics_expose));
-	gain_adjustment.signal_value_changed().connect (mem_fun(*this, &GainMeter::gain_adjusted));
-	peak_display.signal_button_release_event().connect (mem_fun(*this, &GainMeter::peak_button_release), false);
-	gain_display.signal_key_press_event().connect (mem_fun(*this, &GainMeter::gain_key_press), false);
-
-	Config->ParameterChanged.connect (mem_fun (*this, &GainMeter::parameter_changed));
-
-	gain_changed (0);
-	show_gain ();
-
-	update_gain_sensitive ();
-	
-	ResetAllPeakDisplays.connect (mem_fun(*this, &GainMeter::reset_peak_display));
-	ResetGroupPeakDisplays.connect (mem_fun(*this, &GainMeter::reset_group_peak_display));
-
-	UI::instance()->theme_changed.connect (mem_fun(*this, &GainMeter::on_theme_changed));
-	ColorsChanged.connect (mem_fun (*this, &GainMeter::color_handler));
-	//hide_all();
 }
 
-void
-GainMeter::set_width (Width w, int len)
+
+int
+GainMeter::get_gm_width ()
 {
-	_width = w;
-	setup_meters (len);
+	Gtk::Requisition sz;
+	hbox.size_request (sz);
+	return sz.width;
 }
 
 Glib::RefPtr<Gdk::Pixmap>
 GainMeter::render_metrics (Gtk::Widget& w)
 {
-	//cerr << "name: " << w.get_name();//DEBUG
-	//cerr << " bg_red = " << w.get_style()->get_bg(Gtk::STATE_NORMAL).get_red();//DEBUG
-	//cerr << " fg_red = " << w.get_style()->get_fg(Gtk::STATE_NORMAL).get_red();//DEBUG
 	Glib::RefPtr<Gdk::Window> win (w.get_window());
 	Glib::RefPtr<Gdk::GC> fg_gc (w.get_style()->get_fg_gc (Gtk::STATE_NORMAL));
 	Glib::RefPtr<Gdk::GC> bg_gc (w.get_style()->get_bg_gc (Gtk::STATE_NORMAL));
@@ -265,14 +852,10 @@ gint
 GainMeter::meter_metrics_expose (GdkEventExpose *ev)
 {
 	static Glib::RefPtr<Gtk::Style> meter_style;
-	//cerr << "GainMeter::meter_metrics_expose() called on " << meter_metric_area.get_name();//DEBUG
-	//cerr << " style_changed = " << style_changed;//DEBUG
 	if (style_changed) {
 		meter_style = meter_metric_area.get_style();
-	//	cerr << " get_style() called ";//DEBUG
 	}
 	Glib::RefPtr<Gdk::Window> win (meter_metric_area.get_window());
-	//Glib::RefPtr<Gdk::GC> fg_gc (meter_style->get_fg_gc (Gtk::STATE_NORMAL));
 	Glib::RefPtr<Gdk::GC> bg_gc (meter_style->get_bg_gc (Gtk::STATE_INSENSITIVE));
 	GdkRectangle base_rect;
 	GdkRectangle draw_rect;
@@ -288,685 +871,15 @@ GainMeter::meter_metrics_expose (GdkEventExpose *ev)
 	Glib::RefPtr<Gdk::Pixmap> pixmap;
 	std::map<string,Glib::RefPtr<Gdk::Pixmap> >::iterator i = metric_pixmaps.find (meter_metric_area.get_name());
 
-	if (i == metric_pixmaps.end() || style_changed) {
+	if (i == metric_pixmaps.end() || style_changed || dpi_changed) {
 		pixmap = render_metrics (meter_metric_area);
 	} else {
 		pixmap = i->second;
-	//	cerr << " re-used " << i->first;//DEBUG
 	}
 
 	gdk_rectangle_intersect (&ev->area, &base_rect, &draw_rect);
-	//win->draw_rectangle (bg_gc, false, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
 	win->draw_drawable (bg_gc, pixmap, draw_rect.x, draw_rect.y, draw_rect.x, draw_rect.y, draw_rect.width, draw_rect.height);
-	//cerr << endl;//DEBUG
 	style_changed = false;
 	return true;
 }
 
-void
-GainMeter::on_theme_changed()
-{
-	style_changed = true;
-}
-
-GainMeter::~GainMeter ()
-{
-	if (meter_menu) {
-		delete meter_menu;
-	}
-
-	for (vector<MeterInfo>::iterator i = meters.begin(); i != meters.end(); i++) {
-		if ((*i).meter) {
-			delete (*i).meter;
-		}
-	}
-}
-
-void
-GainMeter::update_meters ()
-{
-	vector<MeterInfo>::iterator i;
-	uint32_t n;
-	float peak, mpeak;
-	char buf[32];
-	
-	for (n = 0, i = meters.begin(); i != meters.end(); ++i, ++n) {
-		if ((*i).packed) {
-			peak = _io->peak_input_power (n);
-			
-			(*i).meter->set (log_meter (peak));
-
-			mpeak = _io->max_peak_power(n);
-			
-			if (mpeak > max_peak) {
-				max_peak = mpeak;
-				/* set peak display */
-				if (max_peak <= -200.0f) {
-					peak_display.set_label (_("-inf"));
-				} else {
-					snprintf (buf, sizeof(buf), "%.1f", max_peak);
-					peak_display.set_label (buf);
-				}
-
-				if (max_peak >= 0.0f) {
-					peak_display.set_name ("MixerStripPeakDisplayPeak");
-				}
-			}
-		}
-	}
-}
-
-void
-GainMeter::parameter_changed(const char* parameter_name)
-{
-#define PARAM_IS(x) (!strcmp (parameter_name, (x)))
-
-	ENSURE_GUI_THREAD (bind (mem_fun(*this, &GainMeter::parameter_changed), parameter_name));
-
-	if (PARAM_IS ("meter-hold")) {
-	
-		vector<MeterInfo>::iterator i;
-		uint32_t n;
-		
-		for (n = 0, i = meters.begin(); i != meters.end(); ++i, ++n) {
-			
-			(*i).meter->set_hold_count ((uint32_t) floor(Config->get_meter_hold()));
-		}
-	}
-
-#undef PARAM_IS
-}
-
-void
-GainMeter::hide_all_meters ()
-{
-	bool remove_metric_area = false;
-
-	for (vector<MeterInfo>::iterator i = meters.begin(); i != meters.end(); ++i) {
-		if ((*i).packed) {
-			remove_metric_area = true;
-			meter_packer.remove (*((*i).meter));
-			(*i).packed = false;
-		}
-	}
-
-	if (remove_metric_area) {
-		if (meter_metric_area.get_parent()) {
-			meter_packer.remove (meter_metric_area);
-		}
-	}
-}
-
-void
-GainMeter::setup_meters (int len)
-{
-	uint32_t nmeters = _io->n_outputs();
-	guint16 width;
-
-	hide_all_meters ();
-
-	Route* r;
-
-	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
-
-		switch (r->meter_point()) {
-		case MeterPreFader:
-		case MeterInput:
-			nmeters = r->n_inputs();
-			break;
-		case MeterPostFader:
-			nmeters = r->n_outputs();
-			break;
-		}
-
-	} else {
-
-		nmeters = _io->n_outputs();
-
-	}
-
-	if (nmeters == 0) {
-		return;
-	}
-
-	if (nmeters <= 2) {
-		width = regular_meter_width;
-	} else {
-		width = thin_meter_width;
-	}
-
-	while (meters.size() < nmeters) {
-		meters.push_back (MeterInfo());
-	}
-
-	/* pack them backwards */
-
-	meter_packer.pack_end (meter_metric_area, false, false);
-	meter_metric_area.show_all ();
-
-	int b = ARDOUR_UI::config()->canvasvar_MeterColorBase.get();
-	int m = ARDOUR_UI::config()->canvasvar_MeterColorMid.get();
-	int t = ARDOUR_UI::config()->canvasvar_MeterColorTop.get();
-	int c = ARDOUR_UI::config()->canvasvar_MeterColorClip.get();
-
-	//cerr << "GainMeter::setup_meters() called color_changed = " << color_changed << " colors: " << hex << b << " " << m << " " << t << " " << c << endl;//DEBUG
-
-	for (int32_t n = nmeters-1; nmeters && n >= 0 ; --n) {
-		if (meters[n].width != width || meters[n].length != len || color_changed) {
-			delete meters[n].meter;
-			meters[n].meter = new FastMeter ((uint32_t) floor (Config->get_meter_hold()), width, FastMeter::Vertical, len, b, m, t, c);
-			//cerr << "GainMeter::setup_meters() w:l = " << width << ":" << len << endl;//DEBUG
-			meters[n].width = width;
-			meters[n].length = len;
-			meters[n].meter->add_events (Gdk::BUTTON_RELEASE_MASK);
-			meters[n].meter->signal_button_release_event().connect (bind (mem_fun(*this, &GainMeter::meter_button_release), n));
-		}
-
-		meter_packer.pack_end (*meters[n].meter, false, false);
-		meters[n].meter->show_all ();
-		meters[n].packed = true;
-	}
-	color_changed = false;
-}	
-
-int
-GainMeter::get_gm_width ()
-{
-	Gtk::Requisition sz;
-	hbox.size_request (sz);
-	return sz.width;
-}
-
-bool
-GainMeter::gain_key_press (GdkEventKey* ev)
-{
-	if (key_is_legal_for_numeric_entry (ev->keyval)) {
-		/* drop through to normal handling */
-		return false;
-	}
-	/* illegal key for gain entry */
-	return true;
-}
-
-bool
-GainMeter::peak_button_release (GdkEventButton* ev)
-{
-	/* reset peak label */
-
-	if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier|Keyboard::TertiaryModifier)) {
-		ResetAllPeakDisplays ();
-	} else if (ev->button == 1 && Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
-		Route* r;
-		if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
-			ResetGroupPeakDisplays (r->mix_group());
-		}
-	} else {
-		reset_peak_display ();
-	}
-
-	return true;
-}
-
-void
-GainMeter::reset_peak_display ()
-{
-	Route * r;
-	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
-		r->reset_max_peak_meters();
-	}
-
-	max_peak = -INFINITY;
-	peak_display.set_label (_("-Inf"));
-	peak_display.set_name ("MixerStripPeakDisplay");
-}
-
-void
-GainMeter::reset_group_peak_display (RouteGroup* group)
-{
-	Route* r;
-	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
-		if (group == r->mix_group()) {
-			reset_peak_display ();
-		}
-	}
-}
-
-gint
-GainMeter::meter_button_release (GdkEventButton* ev, uint32_t which)
-{
-	switch (ev->button) {
-	case 1:
-		meters[which].meter->clear();
-		max_peak = minus_infinity();
-		peak_display.set_label (_("-inf"));
-		peak_display.set_name ("MixerStripPeakDisplay");
-		break;
-
-	case 3:
-		// popup_meter_menu (ev);
-		break;
-	};
-
-	return TRUE;
-}
-
-void
-GainMeter::popup_meter_menu (GdkEventButton *ev)
-{
-	using namespace Menu_Helpers;
-
-	if (meter_menu == 0) {
-		meter_menu = new Gtk::Menu;
-		MenuList& items = meter_menu->items();
-
-		items.push_back (MenuElem ("-inf .. +0dBFS"));
-		items.push_back (MenuElem ("-10dB .. +0dBFS"));
-		items.push_back (MenuElem ("-4 .. +0dBFS"));
-		items.push_back (SeparatorElem());
-		items.push_back (MenuElem ("-inf .. -2dBFS"));
-		items.push_back (MenuElem ("-10dB .. -2dBFS"));
-		items.push_back (MenuElem ("-4 .. -2dBFS"));
-	}
-
-	meter_menu->popup (1, ev->time);
-}
-
-bool
-GainMeter::gain_focused (GdkEventFocus* ev)
-{
-	if (ev->in) {
-		gain_display.select_region (0, -1);
-	} else {
-		gain_display.select_region (0, 0);
-	}
-	return false;
-}
-
-void
-GainMeter::gain_activated ()
-{
-	float f;
-
-	if (sscanf (gain_display.get_text().c_str(), "%f", &f) == 1) {
-
-		/* clamp to displayable values */
-
-		f = min (f, 6.0f);
-
-		_io->set_gain (dB_to_coefficient(f), this);
-
-		if (gain_display.has_focus()) {
-			PublicEditor::instance().reset_focus();
-		}
-	}
-}
-
-void
-GainMeter::show_gain ()
-{
-	char buf[32];
-
-	float v = gain_adjustment.get_value();
-	
-	if (v == 0.0) {
-		strcpy (buf, _("-inf"));
-	} else {
-		snprintf (buf, 32, "%.1f", coefficient_to_dB (slider_position_to_gain (v)));
-	}
-	
-	gain_display.set_text (buf);
-}
-
-void
-GainMeter::gain_adjusted ()
-{
-	if (!ignore_toggle) {
-		_io->set_gain (slider_position_to_gain (gain_adjustment.get_value()), this);
-	}
-	show_gain ();
-}
-
-void
-GainMeter::effective_gain_display ()
-{
-	gfloat value = gain_to_slider_position (_io->effective_gain());
-
-	if (gain_adjustment.get_value() != value) {
-		ignore_toggle = true; 
-		gain_adjustment.set_value (value);
-		ignore_toggle = false;
-	}
-}
-
-void
-GainMeter::gain_changed (void *src)
-{
-	Gtkmm2ext::UI::instance()->call_slot (mem_fun(*this, &GainMeter::effective_gain_display));
-}
-
-void
-GainMeter::set_meter_strip_name (const char * name)
-{
-	meter_metric_area.set_name (name);
-	style_changed = true;
-}
-
-void
-GainMeter::set_fader_name (const char * name)
-{
-	gain_slider->set_name (name);
-}
-
-void
-GainMeter::update_gain_sensitive ()
-{
-	static_cast<Gtkmm2ext::SliderController*>(gain_slider)->set_sensitive (!(_io->gain_automation_state() & Play));
-}
-
-
-static MeterPoint
-next_meter_point (MeterPoint mp)
-{
-	switch (mp) {
-	case MeterInput:
-		return MeterPreFader;
-		break;
-		
-	case MeterPreFader:
-		return MeterPostFader;
-		break;
-		
-	case MeterPostFader:
-		return MeterInput;
-		break;
-	}
-	/*NOTREACHED*/
-	return MeterInput;
-}
-
-gint
-GainMeter::meter_press(GdkEventButton* ev)
-{
-	Route* _route;
-
-	wait_for_release = false;
-
-	if ((_route = dynamic_cast<Route*>(_io.get())) == 0) {
-		return FALSE;
-	}
-
-	if (!ignore_toggle) {
-
-		if (Keyboard::is_context_menu_event (ev)) {
-			
-			// no menu at this time.
-
-		} else {
-
-			if (ev->button == 2) {
-
-				// Primary-button2 click is the midi binding click
-				// button2-click is "momentary"
-				
-				if (!Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier))) {
-					wait_for_release = true;
-					old_meter_point = _route->meter_point ();
-				}
-			}
-
-			if (ev->button == 1 || ev->button == 2) {
-
-				if (Keyboard::modifier_state_equals (ev->state, Keyboard::ModifierMask (Keyboard::PrimaryModifier|Keyboard::TertiaryModifier))) {
-
-					/* Primary+Tertiary-click applies change to all routes */
-
-					_session.begin_reversible_command (_("meter point change"));
-                                        Session::GlobalMeteringStateCommand *cmd = new Session::GlobalMeteringStateCommand (_session, this);
-					_session.foreach_route (this, &GainMeter::set_meter_point, next_meter_point (_route->meter_point()));
-                                        cmd->mark();
-					_session.add_command (cmd);
-					_session.commit_reversible_command ();
-					
-					
-				} else if (Keyboard::modifier_state_equals (ev->state, Keyboard::PrimaryModifier)) {
-
-					/* Primary-click: solo mix group.
-					   NOTE: Primary-button2 is MIDI learn.
-					*/
-					
-					if (ev->button == 1) {
-						_session.begin_reversible_command (_("meter point change"));
-						Session::GlobalMeteringStateCommand *cmd = new Session::GlobalMeteringStateCommand (_session, this);
-						set_mix_group_meter_point (*_route, next_meter_point (_route->meter_point()));
-						cmd->mark();
-						_session.add_command (cmd);
-						_session.commit_reversible_command ();
-					}
-					
-				} else {
-					
-					/* click: change just this route */
-
-					// XXX no undo yet
-					
-					_route->set_meter_point (next_meter_point (_route->meter_point()), this);
-				}
-			}
-		}
-	}
-
-	return true;
-
-}
-
-gint
-GainMeter::meter_release(GdkEventButton* ev)
-{
-
-	if(!ignore_toggle){
-		if (wait_for_release){
-			wait_for_release = false;
-			set_meter_point (*(dynamic_cast<Route*>(_io.get())), old_meter_point);
-		}
-	}
-	return true;
-}
-
-void
-GainMeter::set_meter_point (Route& route, MeterPoint mp)
-{
-	route.set_meter_point (mp, this);
-}
-
-void
-GainMeter::set_mix_group_meter_point (Route& route, MeterPoint mp)
-{
-	RouteGroup* mix_group;
-
-	if((mix_group = route.mix_group()) != 0){
-		mix_group->apply (&Route::set_meter_point, mp, this);
-	} else {
-		route.set_meter_point (mp, this);
-	}
-}
-
-void
-GainMeter::meter_point_clicked ()
-{
-	Route* r;
-
-	if ((r = dynamic_cast<Route*> (_io.get())) != 0) {
-
-	}
-}
-
-gint
-GainMeter::start_gain_touch (GdkEventButton* ev)
-{
-	_io->start_gain_touch ();
-	return FALSE;
-}
-
-gint
-GainMeter::end_gain_touch (GdkEventButton* ev)
-{
-	_io->end_gain_touch ();
-	return FALSE;
-}
-
-gint
-GainMeter::gain_automation_state_button_event (GdkEventButton *ev)
-{
-	if (ev->type == GDK_BUTTON_RELEASE) {
-		return TRUE;
-	}
-	
-	switch (ev->button) {
-		case 1:
-			gain_astate_menu.popup (1, ev->time);
-			break;
-		default:
-			break;
-	}
-
-	return TRUE;
-}
-
-gint
-GainMeter::gain_automation_style_button_event (GdkEventButton *ev)
-{
-	if (ev->type == GDK_BUTTON_RELEASE) {
-		return TRUE;
-	}
-
-	switch (ev->button) {
-	case 1:
-		gain_astyle_menu.popup (1, ev->time);
-		break;
-	default:
-		break;
-	}
-	return TRUE;
-}
-
-string
-GainMeter::astate_string (AutoState state)
-{
-	return _astate_string (state, false);
-}
-
-string
-GainMeter::short_astate_string (AutoState state)
-{
-	return _astate_string (state, true);
-}
-
-string
-GainMeter::_astate_string (AutoState state, bool shrt)
-{
-	string sstr;
-
-	switch (state) {
-	case Off:
-		sstr = (shrt ? "M" : _("M"));
-		break;
-	case Play:
-		sstr = (shrt ? "P" : _("P"));
-		break;
-	case Touch:
-		sstr = (shrt ? "T" : _("T"));
-		break;
-	case Write:
-		sstr = (shrt ? "W" : _("W"));
-		break;
-	}
-
-	return sstr;
-}
-
-string
-GainMeter::astyle_string (AutoStyle style)
-{
-	return _astyle_string (style, false);
-}
-
-string
-GainMeter::short_astyle_string (AutoStyle style)
-{
-	return _astyle_string (style, true);
-}
-
-string
-GainMeter::_astyle_string (AutoStyle style, bool shrt)
-{
-	if (style & Trim) {
-		return _("Trim");
-	} else {
-	        /* XXX it might different in different languages */
-
-		return (shrt ? _("Abs") : _("Abs"));
-	}
-}
-
-void
-GainMeter::gain_automation_style_changed ()
-{
-  // Route* _route = dynamic_cast<Route*>(&_io);
-	switch (_width) {
-	case Wide:
-	        gain_automation_style_button.set_label (astyle_string(_io->gain_automation_curve().automation_style()));
-		break;
-	case Narrow:
-		gain_automation_style_button.set_label  (short_astyle_string(_io->gain_automation_curve().automation_style()));
-		break;
-	}
-}
-
-void
-GainMeter::gain_automation_state_changed ()
-{
-	ENSURE_GUI_THREAD(mem_fun(*this, &GainMeter::gain_automation_state_changed));
-	//Route* _route = dynamic_cast<Route*>(&_io);
-	
-	bool x;
-
-	switch (_width) {
-	case Wide:
-		gain_automation_state_button.set_label (astate_string(_io->gain_automation_curve().automation_state()));
-		break;
-	case Narrow:
-		gain_automation_state_button.set_label (short_astate_string(_io->gain_automation_curve().automation_state()));
-		break;
-	}
-
-	x = (_io->gain_automation_state() != Off);
-	
-	if (gain_automation_state_button.get_active() != x) {
-		ignore_toggle = true;
-		gain_automation_state_button.set_active (x);
-		ignore_toggle = false;
-	}
-
-	update_gain_sensitive ();
-	
-	/* start watching automation so that things move */
-	
-	gain_watching.disconnect();
-
-	if (x) {
-		gain_watching = ARDOUR_UI::RapidScreenUpdate.connect (mem_fun (*this, &GainMeter::effective_gain_display));
-	}
-}
-
-void GainMeter::clear_meters ()
-{
-	for (vector<MeterInfo>::iterator i = meters.begin(); i < meters.end(); i++) {
-		(*i).meter->clear();
-	}
-}
-
-void GainMeter::color_handler()
-{
-	color_changed = true;
-	setup_meters();
-}
