@@ -57,17 +57,7 @@ TextReceiver text_receiver ("ardour");
 extern int curvetest (string);
 
 static ARDOUR_UI  *ui = 0;
-static char* localedir = LOCALEDIR;
-
-gint
-show_ui_callback (void *arg)
-{
- 	ARDOUR_UI * ui = (ARDOUR_UI *) arg;
-
-	ui->hide_splash();
-	
-	return FALSE;
-}
+static const char* localedir = LOCALEDIR;
 
 #ifdef __APPLE__
 
@@ -90,6 +80,16 @@ fixup_bundle_environment ()
 	Glib::ustring exec_path (execpath);
 	Glib::ustring dir_path = Glib::path_get_dirname (exec_path);
 	Glib::ustring path;
+	const char *cstr = getenv ("PATH");
+
+	/* ensure that we find any bundled executables (e.g. JACK) */
+
+	path = dir_path;
+	if (cstr) {
+		path += ':';
+		path += cstr;
+	}
+	setenv ("PATH", path.c_str(), 1);
 
 	path = dir_path;
 	path += "/../Resources";
@@ -113,6 +113,46 @@ fixup_bundle_environment ()
 	setenv ("ARDOUR_CONFIG_PATH", path.c_str(), 1);
 	setenv ("ARDOUR_DATA_PATH", path.c_str(), 1);
 
+	cstr = getenv ("LADSPA_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Plugins";
+	
+	setenv ("LADSPA_PATH", path.c_str(), 1);
+
+	cstr = getenv ("VAMP_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Frameworks";
+	
+	setenv ("VAMP_PATH", path.c_str(), 1);
+
+	cstr = getenv ("ARDOUR_CONTROL_SURFACE_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Surfaces";
+	
+	setenv ("ARDOUR_CONTROL_SURFACE_PATH", path.c_str(), 1);
+
+	cstr = getenv ("LV2_PATH");
+	if (cstr) {
+		path = cstr;
+		path += ':';
+	}
+	path = dir_path;
+	path += "/../Plugins";
+	
+	setenv ("LV2_PATH", path.c_str(), 1);
+
 	path = dir_path;
 	path += "/../Frameworks/clearlooks";
 
@@ -123,22 +163,33 @@ fixup_bundle_environment ()
 	
 	localedir = strdup (path.c_str());
 
-	/* write a pango.rc file and tell pango to use it */
+	/* write a pango.rc file and tell pango to use it. we'd love
+	   to put this into the Ardour.app bundle and leave it there,
+	   but the user may not have write permission. so ... 
 
-	path = dir_path;
-	path += "/../Resources/pango.rc";
+	   we also have to make sure that the user ardour directory
+	   actually exists ...
+	*/
 
-	std::ofstream pangorc (path.c_str());
-	if (!pangorc) {
-		error << string_compose (_("cannot open pango.rc file %1") , path) << endmsg;
+	if (g_mkdir_with_parents (ARDOUR::get_user_ardour_path().c_str(), 0755) < 0) {
+		error << string_compose (_("cannot create user ardour folder %1 (%2)"), ARDOUR::get_user_ardour_path(), strerror (errno))
+		      << endmsg;
 	} else {
-		pangorc << "[Pango]\nModuleFiles=";
-		Glib::ustring mpath = dir_path;
-		mpath += "/../Resources/pango.modules";
-		pangorc << mpath << endl;
+
+		path = Glib::build_filename (ARDOUR::get_user_ardour_path(), "pango.rc");
 		
-		pangorc.close ();
-		setenv ("PANGO_RC_FILE", path.c_str(), 1);
+		std::ofstream pangorc (path.c_str());
+		if (!pangorc) {
+			error << string_compose (_("cannot open pango.rc file %1") , path) << endmsg;
+		} else {
+			pangorc << "[Pango]\nModuleFiles=";
+			Glib::ustring mpath = dir_path;
+			mpath += "/../Resources/pango.modules";
+			pangorc << mpath << endl;
+			
+			pangorc.close ();
+			setenv ("PANGO_RC_FILE", path.c_str(), 1);
+		}
 	}
 
 	// gettext charset aliases
@@ -183,7 +234,7 @@ int main (int argc, char* argv[])
 #endif
 {
 	vector<Glib::ustring> null_file_list;
-
+	
 #ifdef __APPLE__
 	fixup_bundle_environment ();
 #endif
@@ -231,7 +282,7 @@ int main (int argc, char* argv[])
 	}
 
 	if (no_splash) {
-		cerr << _("Copyright (C) 1999-2007 Paul Davis") << endl
+		cerr << _("Copyright (C) 1999-2008 Paul Davis") << endl
 		     << _("Some portions Copyright (C) Steve Harris, Ari Johnson, Brett Viren, Joel Baker") << endl
 		     << endl
 		     << _("Ardour comes with ABSOLUTELY NO WARRANTY") << endl
@@ -252,17 +303,6 @@ int main (int argc, char* argv[])
 		exit (1);
 	}
 
-	if (!no_splash) {
-		ui->show_splash ();
-		if (session_name.length()) {  
-			g_timeout_add (4000, show_ui_callback, ui);
-		}
-	}
-
-	if (!keybindings_path.empty()) {
-		ui->set_keybindings_path (keybindings_path);
-	}
-
 	ui->run (text_receiver);
 	ui = 0;
 
