@@ -54,9 +54,17 @@ class NewSessionDialog : public ArdourDialog
 {
 public:
 		
+	enum Pages {
+		NewPage = 0x1,
+		OpenPage = 0x2,
+		EnginePage = 0x4
+	};
+
 	NewSessionDialog();
 	~NewSessionDialog ();
 
+	int run ();
+
 	void set_session_name(const Glib::ustring& name);
 	void set_session_folder(const Glib::ustring& folder);
 
@@ -84,6 +92,8 @@ public:
 
 	bool connect_outs_to_master() const;
 	bool connect_outs_to_physical() const ;
+	Pages which_page () const;
+
 	int get_current_page();
 	void set_current_page (int);
 	void reset_recent();
@@ -93,6 +103,7 @@ public:
 
 	EngineControl engine_control;
 	void set_have_engine (bool yn);
+	void set_existing_session (bool yn);
 
 protected:
 
@@ -165,6 +176,9 @@ protected:
 	Gtk::Notebook* m_notebook;
 
  private:
+
+	Pages page_set;
+
 	struct RecentSessionModelColumns : public Gtk::TreeModel::ColumnRecord {
 	    RecentSessionModelColumns() { 
 		    add (visible_name);
