@@ -16,9 +16,12 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 */
+#include <pango/pangoft2.h> // for fontmap resolution control for GnomeCanvas
+#include <pango/pangocairo.h> // for fontmap resolution control for GnomeCanvas
 
 #include <pbd/whitespace.h>
 
+#include <ardour/ardour.h>
 #include <ardour/session.h>
 #include <ardour/audioengine.h>
 #include <ardour/configuration.h>
@@ -43,6 +46,7 @@
 #include "option_editor.h"
 #include "midi_port_dialog.h"
 #include "gui_thread.h"
+#include "utils.h"
 
 #include "i18n.h"
 
@@ -100,7 +104,7 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 
 	  /* kbd/mouse */
 
-	  keyboard_mouse_table (3, 4),
+	  keyboard_mouse_table (4, 4),
 	  delete_button_adjustment (3, 1, 5),
 	  delete_button_spin (delete_button_adjustment),
 	  edit_button_adjustment (3, 1, 5),
@@ -109,6 +113,7 @@ OptionEditor::OptionEditor (ARDOUR_UI& uip, PublicEditor& ed, Mixer_UI& mixui)
 {
 	using namespace Notebook_Helpers;
 
+	first_click_setup = true;
 	click_io_selector = 0;
 	auditioner_io_selector = 0;
 	session = 0;
@@ -249,12 +254,40 @@ OptionEditor::add_session_paths ()
 	session_raid_entry.set_text(session->raid_path());
 }
 
+static void
+font_scale_changed (Gtk::Adjustment* adj)
+{
+	Config->set_font_scale((long)floor (adj->get_value() * 1024));
+	reset_dpi();
+}
+
 void
 OptionEditor::setup_misc_options ()
 {
 	Gtk::HBox* hbox;
-	
-	Label* label = manage (new Label (_("Short crossfade length (msecs)")));
+	Label* label;
+
+#ifndef GTKOSX
+	/* font scaling does nothing with GDK/Quartz */
+
+	Gtk::Adjustment* dpi_adj = new Gtk::Adjustment ((double)Config->get_font_scale() / 1024, 50, 250, 1, 10);
+	Gtk::HScale * dpi_range = new Gtk::HScale (*dpi_adj);
+
+	label = manage (new Label (_("Font Scaling")));
+	label->set_name ("OptionsLabel");
+
+	dpi_range->set_update_policy (Gtk::UPDATE_DISCONTINUOUS);
+	dpi_adj->signal_value_changed().connect (bind (sigc::ptr_fun (font_scale_changed), dpi_adj));
+
+	hbox = manage (new HBox);
+	hbox->set_border_width (5);
+	hbox->set_spacing (10);
+	hbox->pack_start (*label, false, false);
+	hbox->pack_start (*dpi_range, true, true);
+	misc_packer.pack_start (*hbox, false, false);
+#endif
+
+	label = manage (new Label (_("Short crossfade length (msecs)")));
 	label->set_name ("OptionsLabel");
 	
 	hbox = manage (new HBox);
@@ -962,8 +995,8 @@ void
 OptionEditor::clear_click_editor ()
 {
 	if (click_io_selector) {
-		click_packer.remove (*click_io_selector);
-		click_packer.remove (*click_gpm);
+		click_hpacker.remove (*click_io_selector);
+		click_hpacker.remove (*click_gpm);
 		delete click_io_selector;
 		delete click_gpm;
 		click_io_selector = 0;
@@ -975,51 +1008,58 @@ void
 OptionEditor::setup_click_editor ()
 {
 	Label* label;
-	HBox* hpacker = manage (new HBox);
+
+	if (first_click_setup) {
+		
+		click_path_entry.set_name ("OptionsEntry");
+		click_emphasis_path_entry.set_name ("OptionsEntry");
+		
+		click_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_sound_changed));
+		click_emphasis_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_emphasis_sound_changed));
+		
+		click_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_sound_changed));
+		click_emphasis_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_emphasis_sound_changed));
+		
+		click_browse_button.set_name ("EditorGTKButton");
+		click_emphasis_browse_button.set_name ("EditorGTKButton");
+
+		click_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_browse_clicked));
+		click_emphasis_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_emphasis_browse_clicked));
+
+		click_packer.set_border_width (12);
+		click_packer.set_spacing (5);
+
+		click_table.set_col_spacings (10);
+		
+		label = manage(new Label(_("Click audio file")));
+		label->set_name ("OptionsLabel");
+		click_table.attach (*label, 0, 1, 0, 1, FILL|EXPAND, FILL);
+		click_table.attach (click_path_entry, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, FILL);
+		click_table.attach (click_browse_button, 2, 3, 0, 1, FILL|EXPAND, FILL);
+		
+		label = manage(new Label(_("Click emphasis audiofile")));
+		label->set_name ("OptionsLabel");
+		click_table.attach (*label, 0, 1, 1, 2, FILL|EXPAND, FILL);
+		click_table.attach (click_emphasis_path_entry, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, FILL);
+		click_table.attach (click_emphasis_browse_button, 2, 3, 1, 2, FILL|EXPAND, FILL);
+
+		click_packer.pack_start (click_table, false, false);
+		click_packer.pack_start (click_hpacker, false, false);
+	
+
+		click_hpacker.set_spacing (10);
+
+		first_click_setup = false;
+	}
 
 	click_path_entry.set_sensitive (true);
 	click_emphasis_path_entry.set_sensitive (true);
 
-	click_path_entry.set_name ("OptionsEntry");
-	click_emphasis_path_entry.set_name ("OptionsEntry");
-	
-	click_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_sound_changed));
-	click_emphasis_path_entry.signal_activate().connect (mem_fun(*this, &OptionEditor::click_emphasis_sound_changed));
-
-	click_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_sound_changed));
-	click_emphasis_path_entry.signal_focus_out_event().connect (bind (mem_fun(*this, &OptionEditor::focus_out_event_handler), &OptionEditor::click_emphasis_sound_changed));
-
-	click_browse_button.set_name ("EditorGTKButton");
-	click_emphasis_browse_button.set_name ("EditorGTKButton");
-	click_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_browse_clicked));
-	click_emphasis_browse_button.signal_clicked().connect (mem_fun(*this, &OptionEditor::click_emphasis_browse_clicked));
-
-	click_packer.set_border_width (12);
-	click_packer.set_spacing (5);
-
 	click_io_selector = new IOSelector (*session, session->click_io(), false);
 	click_gpm = new GainMeter (session->click_io(), *session);
 
-	click_table.set_col_spacings (10);
-	
-	label = manage(new Label(_("Click audio file")));
-	label->set_name ("OptionsLabel");
-	click_table.attach (*label, 0, 1, 0, 1, FILL|EXPAND, FILL);
-	click_table.attach (click_path_entry, 1, 2, 0, 1, Gtk::FILL|Gtk::EXPAND, FILL);
-	click_table.attach (click_browse_button, 2, 3, 0, 1, FILL|EXPAND, FILL);
-	
-	label = manage(new Label(_("Click emphasis audiofile")));
-	label->set_name ("OptionsLabel");
-	click_table.attach (*label, 0, 1, 1, 2, FILL|EXPAND, FILL);
-	click_table.attach (click_emphasis_path_entry, 1, 2, 1, 2, Gtk::FILL|Gtk::EXPAND, FILL);
-	click_table.attach (click_emphasis_browse_button, 2, 3, 1, 2, FILL|EXPAND, FILL);
-
-	hpacker->set_spacing (10);
-	hpacker->pack_start (*click_io_selector, false, false);
-	hpacker->pack_start (*click_gpm, false, false);
-
-	click_packer.pack_start (click_table, false, false);
-	click_packer.pack_start (*hpacker, false, false);
+	click_hpacker.pack_start (*click_io_selector, false, false);
+	click_hpacker.pack_start (*click_gpm, false, false);
 
 	click_packer.show_all ();
 }
@@ -1078,6 +1118,23 @@ static const struct {
     const char *name;
     guint   modifier;
 } modifiers[] = {
+
+#ifdef GTKOSX 
+
+	/* Command = Meta
+	   Option/Alt = Mod1
+	*/
+
+	{ "Shift", GDK_SHIFT_MASK },
+	{ "Command", GDK_META_MASK },
+	{ "Control", GDK_CONTROL_MASK },
+	{ "Option", GDK_MOD1_MASK },
+	{ "Command-Shift", GDK_MOD1_MASK|GDK_SHIFT_MASK },
+	{ "Command-Option", GDK_MOD1_MASK|GDK_MOD5_MASK },
+	{ "Shift-Option", GDK_SHIFT_MASK|GDK_MOD5_MASK },
+	{ "Shift-Command-Option", GDK_MOD5_MASK|GDK_SHIFT_MASK|GDK_MOD1_MASK },
+
+#else
 	{ "Shift", GDK_SHIFT_MASK },
 	{ "Control", GDK_CONTROL_MASK },
 	{ "Alt (Mod1)", GDK_MOD1_MASK },
@@ -1089,6 +1146,7 @@ static const struct {
 	{ "Mod3", GDK_MOD3_MASK },
 	{ "Mod4", GDK_MOD4_MASK },
 	{ "Mod5", GDK_MOD5_MASK },
+#endif
 	{ 0, 0 }
 };
 
@@ -1178,6 +1236,39 @@ OptionEditor::setup_keyboard_options ()
 	
 	keyboard_mouse_table.attach (*label, 0, 1, 2, 3, Gtk::FILL|Gtk::EXPAND, FILL);
 	keyboard_mouse_table.attach (snap_modifier_combo, 1, 2, 2, 3, Gtk::FILL|Gtk::EXPAND, FILL);
+
+	vector<string> strs;
+	
+	for (std::map<std::string,std::string>::iterator bf = Keyboard::binding_files.begin(); bf != Keyboard::binding_files.end(); ++bf) {
+		strs.push_back (bf->first);
+	}
+	
+	set_popdown_strings (keyboard_layout_selector, strs);
+	keyboard_layout_selector.set_active_text (Keyboard::current_binding_name());
+	keyboard_layout_selector.signal_changed().connect (mem_fun (*this, &OptionEditor::bindings_changed));
+
+	label = manage (new Label (_("Keyboard layout")));
+	label->set_name ("OptionsLabel");
+	label->set_alignment (1.0, 0.5);
+
+	keyboard_mouse_table.attach (*label, 0, 1, 3, 4, Gtk::FILL|Gtk::EXPAND, FILL);
+	keyboard_mouse_table.attach (keyboard_layout_selector, 1, 2, 3, 4, Gtk::FILL|Gtk::EXPAND, FILL);
+}
+
+void
+OptionEditor::bindings_changed ()
+{
+	string txt;
+	
+	txt = keyboard_layout_selector.get_active_text();
+
+	for (std::map<string,string>::iterator i = Keyboard::binding_files.begin(); i != Keyboard::binding_files.end(); ++i) {
+		if (txt == i->first) {
+			if (Keyboard::load_keybindings (i->second)) {
+				Keyboard::save_keybindings ();
+			}
+		}
+	}
 }
 
 void
@@ -1292,5 +1383,7 @@ OptionEditor::parameter_changed (const char* parameter_name)
 
 		save_history_button.set_active (x);
 		saved_history_depth_spinner.set_sensitive (x);
+	} else if (PARAM_IS ("font-scale")) {
+		reset_dpi();
 	}
 }
