@@ -51,6 +51,7 @@
 #include "utils.h"
 #include "gui_thread.h"
 #include "public_editor.h"
+#include "keyboard.h"
 
 #include "i18n.h"
 
@@ -61,11 +62,12 @@ using namespace Gtkmm2ext;
 using namespace Gtk;
 using namespace sigc;
 
-PluginUIWindow::PluginUIWindow (boost::shared_ptr<PluginInsert> insert, bool scrollable)
-	: ArdourDialog ("plugin ui")
+PluginUIWindow::PluginUIWindow (Gtk::Window* win, boost::shared_ptr<PluginInsert> insert, bool scrollable)
+	: parent (win)
 {
 	bool have_gui = false;
 	non_gtk_gui = false;
+	was_visible = false;
 
 	if (insert->plugin()->has_editor()) {
 		switch (insert->type()) {
@@ -99,7 +101,7 @@ PluginUIWindow::PluginUIWindow (boost::shared_ptr<PluginInsert> insert, bool scr
 		GenericPluginUI*  pu  = new GenericPluginUI (insert, scrollable);
 		
 		_pluginui = pu;
-		get_vbox()->add (*pu);
+		add (*pu);
 		
 		set_wmclass (X_("ardour_plugin_editor"), "Ardour");
 
@@ -107,7 +109,7 @@ PluginUIWindow::PluginUIWindow (boost::shared_ptr<PluginInsert> insert, bool scr
 		signal_unmap_event().connect (mem_fun (*pu, &GenericPluginUI::stop_updating));
 	}
 
-	set_position (Gtk::WIN_POS_MOUSE);
+	// set_position (Gtk::WIN_POS_MOUSE);
 	set_name ("PluginEditor");
 	add_events (Gdk::KEY_PRESS_MASK|Gdk::KEY_RELEASE_MASK|Gdk::BUTTON_PRESS_MASK|Gdk::BUTTON_RELEASE_MASK);
 
@@ -133,28 +135,51 @@ PluginUIWindow::~PluginUIWindow ()
 {
 }
 
+void
+PluginUIWindow::set_parent (Gtk::Window* win)
+{
+	parent = win;
+}
+
+void
+PluginUIWindow::on_map ()
+{
+	Window::on_map ();
+	set_keep_above (true);
+}
+
+bool
+PluginUIWindow::on_enter_notify_event (GdkEventCrossing *ev)
+{
+	Keyboard::the_keyboard().enter_window (ev, this);
+	return false;
+}
+
+bool
+PluginUIWindow::on_leave_notify_event (GdkEventCrossing *ev)
+{
+	Keyboard::the_keyboard().leave_window (ev, this);
+	return false;
+}
+
 void
 PluginUIWindow::on_show ()
 {
-	cerr << "PluginWindow shown\n";
-		
-	ArdourDialog::on_show ();
-	Glib::ListHandle<Widget*> kids (get_vbox()->get_children());
-	
-	cerr << "send show to " << kids.size() << " children of this plugin UI\n";
+	if (_pluginui) {
+		_pluginui->update_presets ();
+	}
 
-	for (Glib::ListHandle<Widget*>::iterator x = kids.begin(); x != kids.end(); ++x) {
-		cerr << "\tSend show to " << (*x) << endl;
-		(*x)->show ();
+	Window::on_show ();
+
+	if (parent) {
+		// set_transient_for (*parent);
 	}
-	cerr << "!! send done\n";
 }
 
 void
 PluginUIWindow::on_hide ()
 {
-	cerr << "PluginWindow hidden\n";
-	ArdourDialog::on_hide ();
+	Window::on_hide ();
 }
 
 bool
@@ -174,7 +199,7 @@ PluginUIWindow::create_vst_editor(boost::shared_ptr<PluginInsert> insert)
 		VSTPluginUI* vpu = new VSTPluginUI (insert, vp);
 	
 		_pluginui = vpu;
-		get_vbox()->add (*vpu);
+		add (*vpu);
 		vpu->package (*this);
 	}
 
@@ -191,8 +216,7 @@ PluginUIWindow::create_audiounit_editor (boost::shared_ptr<PluginInsert> insert)
 #else
 	VBox* box;
 	_pluginui = create_au_gui (insert, &box);
-	cerr << "#*#*#*#*#*#*#*#*## PACK " << box << " INTO PLUGIN UI\n";
-	get_vbox()->add (*box);
+	add (*box);
 	non_gtk_gui = true;
 
 	extern sigc::signal<void,bool> ApplicationActivationChanged;
@@ -206,12 +230,20 @@ void
 PluginUIWindow::app_activated (bool yn)
 {
 #if defined (HAVE_AUDIOUNITS) && defined(GTKOSX)
-	if (yn) {
-		if (_pluginui) {
-			_pluginui->activate ();
-		}
-	}
 	cerr << "APP activated ? " << yn << endl;
+	if (_pluginui) {
+		if (yn) {
+			if (was_visible) {
+				_pluginui->activate ();
+				present ();
+				was_visible = true;
+			}
+		} else {
+			was_visible = is_visible();
+			hide ();
+			_pluginui->deactivate ();
+		}
+	} 
 #endif
 }
 
@@ -252,28 +284,37 @@ PlugUIBase::PlugUIBase (boost::shared_ptr<PluginInsert> pi)
 	  save_button(_("Add")),
 	  bypass_button (_("Bypass"))
 {
-        //combo.set_use_arrows_always(true);
-	set_popdown_strings (combo, plugin->get_presets());
-	combo.set_size_request (100, -1);
-	combo.set_active_text ("");
-	combo.signal_changed().connect(mem_fun(*this, &PlugUIBase::setting_selected));
+        //preset_combo.set_use_arrows_always(true);
+	set_popdown_strings (preset_combo, plugin->get_presets());
+	preset_combo.set_size_request (100, -1);
+	preset_combo.set_active_text ("");
+	preset_combo.signal_changed().connect(mem_fun(*this, &PlugUIBase::setting_selected));
 
 	save_button.set_name ("PluginSaveButton");
 	save_button.signal_clicked().connect(mem_fun(*this, &PlugUIBase::save_plugin_setting));
 
+	insert->active_changed.connect (mem_fun(*this, &PlugUIBase::redirect_active_changed));
+	bypass_button.set_active (!pi->active());
+
 	bypass_button.set_name ("PluginBypassButton");
 	bypass_button.signal_toggled().connect (mem_fun(*this, &PlugUIBase::bypass_toggled));
 }
 
+void
+PlugUIBase::redirect_active_changed (Redirect* r, void* src)
+{
+	ENSURE_GUI_THREAD(bind (mem_fun(*this, &PlugUIBase::redirect_active_changed), r, src));
+	bypass_button.set_active (!r->active());
+}
+
 void
 PlugUIBase::setting_selected()
 {
-	if (combo.get_active_text().length() > 0) {
-		if (!plugin->load_preset(combo.get_active_text())) {
-			warning << string_compose(_("Plugin preset %1 not found"), combo.get_active_text()) << endmsg;
+	if (preset_combo.get_active_text().length() > 0) {
+		if (!plugin->load_preset(preset_combo.get_active_text())) {
+			warning << string_compose(_("Plugin preset %1 not found"), preset_combo.get_active_text()) << endmsg;
 		}
 	}
-
 }
 
 void
@@ -295,8 +336,8 @@ PlugUIBase::save_plugin_setting ()
 
 		if (name.length()) {
 			if(plugin->save_preset(name)){
-				set_popdown_strings (combo, plugin->get_presets());
-				combo.set_active_text (name);
+				set_popdown_strings (preset_combo, plugin->get_presets());
+				preset_combo.set_active_text (name);
 			}
 		}
 		break;
@@ -318,3 +359,8 @@ PlugUIBase::bypass_toggled ()
 	}
 }
 
+void
+PlugUIBase::update_presets ()
+{
+	set_popdown_strings (preset_combo, plugin->get_presets());
+}
