@@ -70,29 +70,37 @@ class PlugUIBase : public virtual sigc::trackable
 	virtual ~PlugUIBase() {}
 
 	virtual gint get_preferred_height () = 0;
+	virtual gint get_preferred_width () = 0;
 	virtual bool start_updating(GdkEventAny*) = 0;
 	virtual bool stop_updating(GdkEventAny*) = 0;
+	
+	virtual void activate () {}
+	virtual void deactivate () {}
+
+	virtual void update_presets ();
 
   protected:
 	boost::shared_ptr<ARDOUR::PluginInsert> insert;
 	boost::shared_ptr<ARDOUR::Plugin> plugin;
-	Gtk::ComboBoxText combo;
+	Gtk::ComboBoxText preset_combo;
 	Gtk::Button save_button;
 	Gtk::ToggleButton bypass_button;
 
 	void setting_selected();
 	void save_plugin_setting (void);
 	void bypass_toggled();
+	void redirect_active_changed (ARDOUR::Redirect* r, void* src);
 };
 
-class LadspaPluginUI : public PlugUIBase, public Gtk::VBox 
+class GenericPluginUI : public PlugUIBase, public Gtk::VBox 
 {
   public:
-	LadspaPluginUI (boost::shared_ptr<ARDOUR::PluginInsert> plug, bool scrollable=false);
-	~LadspaPluginUI ();
+	GenericPluginUI (boost::shared_ptr<ARDOUR::PluginInsert> plug, bool scrollable=false);
+	~GenericPluginUI ();
 	
 	gint get_preferred_height () { return prefheight; }
-
+	gint get_preferred_width () { return -1; }
+	
 	bool start_updating(GdkEventAny*);
 	bool stop_updating(GdkEventAny*);
 
@@ -191,22 +199,38 @@ class LadspaPluginUI : public PlugUIBase, public Gtk::VBox
 	void print_parameter (char *buf, uint32_t len, uint32_t param);
 };
 
-class PluginUIWindow : public ArdourDialog
+class PluginUIWindow : public Gtk::Window
 {
   public:
-	PluginUIWindow (boost::shared_ptr<ARDOUR::PluginInsert> insert, bool scrollable=false);
+	PluginUIWindow (Gtk::Window*, boost::shared_ptr<ARDOUR::PluginInsert> insert, bool scrollable=false);
 	~PluginUIWindow ();
 
 	PlugUIBase& pluginui() { return *_pluginui; }
 
 	void resize_preferred();
+	void set_parent (Gtk::Window*);
+
+	bool on_enter_notify_event (GdkEventCrossing*);
+	bool on_leave_notify_event (GdkEventCrossing*);
+	bool on_key_press_event (GdkEventKey*);
+	bool on_key_release_event (GdkEventKey*);
+	void on_show ();
+	void on_hide ();
+	void on_map ();
+
 
-	virtual bool on_key_press_event (GdkEventKey*);
-	virtual bool on_key_release_event (GdkEventKey*);
-	
   private:
 	PlugUIBase* _pluginui;
+	Gtk::Window* parent;
+	Gtk::VBox vbox;
+	bool non_gtk_gui;
+	bool was_visible;
+
+	void app_activated (bool);
 	void plugin_going_away ();
+
+	bool create_vst_editor (boost::shared_ptr<ARDOUR::PluginInsert>);
+	bool create_audiounit_editor (boost::shared_ptr<ARDOUR::PluginInsert>);
 };
 
 #ifdef VST_SUPPORT
@@ -217,6 +241,7 @@ class VSTPluginUI : public PlugUIBase, public Gtk::VBox
 	~VSTPluginUI ();
 
 	gint get_preferred_height ();
+	gint get_preferred_width ();
 	bool start_updating(GdkEventAny*) {return false;}
 	bool stop_updating(GdkEventAny*) {return false;}
 
@@ -233,4 +258,9 @@ class VSTPluginUI : public PlugUIBase, public Gtk::VBox
 };
 #endif // VST_SUPPORT
 
+#ifdef HAVE_AUDIOUNITS
+/* this function has to be in a .mm file */
+extern PlugUIBase* create_au_gui (boost::shared_ptr<ARDOUR::PluginInsert>, Gtk::VBox**);
+#endif
+
 #endif /* __ardour_plugin_ui_h__ */
