@@ -47,6 +47,7 @@
 #include <ardour/ladspa_plugin.h>
 #include <ardour/connection.h>
 #include <ardour/session_connection.h>
+#include <ardour/profile.h>
 
 #include "ardour_ui.h"
 #include "ardour_dialog.h"
@@ -257,6 +258,12 @@ RedirectBox::show_redirect_menu (gint arg)
 		redirect_menu = build_redirect_menu ();
 	}
 
+	Gtk::MenuItem* plugin_menu_item = dynamic_cast<Gtk::MenuItem*>(ActionManager::get_widget("/redirectmenu/newplugin"));
+
+	if (plugin_menu_item) {
+		plugin_menu_item->set_submenu (_plugin_selector.plugin_menu());
+	}
+
 	paste_action->set_sensitive (!_rr_selection.redirects.empty());
 
 	redirect_menu->popup (1, arg);
@@ -384,24 +391,25 @@ RedirectBox::deselect_all_redirects ()
 void
 RedirectBox::choose_plugin ()
 {
-	sigc::connection newplug_connection = _plugin_selector.PluginCreated.connect (mem_fun(*this,&RedirectBox::insert_plugin_chosen));
-	_plugin_selector.show_all();
-	_plugin_selector.run ();
-	newplug_connection.disconnect();
+	_plugin_selector.set_interested_object (*this);
 }
 
 void
-RedirectBox::insert_plugin_chosen (boost::shared_ptr<Plugin> plugin)
+RedirectBox::use_plugins (const SelectedPlugins& plugins)
 {
-	if (plugin) {
+	for (SelectedPlugins::const_iterator p = plugins.begin(); p != plugins.end(); ++p) {
+
+		boost::shared_ptr<Redirect> redirect (new PluginInsert (_session, *p, _placement));
 
-		boost::shared_ptr<Redirect> redirect (new PluginInsert (_session, plugin, _placement));
-		
 		uint32_t err_streams;
-
+		
 		if (_route->add_redirect (redirect, this, &err_streams)) {
-			weird_plugin_dialog (*plugin, err_streams, _route);
+			weird_plugin_dialog (**p, err_streams, _route);
 		} else {
+			
+			if (Profile->get_sae()) {
+				redirect->set_active (true, 0);
+			}
 			redirect->active_changed.connect (bind (mem_fun (*this, &RedirectBox::show_redirect_active_r), boost::weak_ptr<Redirect>(redirect)));
 		}
 	}
@@ -459,6 +467,9 @@ RedirectBox::weird_plugin_dialog (Plugin& p, uint32_t streams, boost::shared_ptr
 					 streams));
 	}
 
+	dialog.set_border_width (PublicEditor::window_border_width);
+
+	label.set_alignment (0.5, 0.5);
 	dialog.get_vbox()->pack_start (label);
 	dialog.add_button (Stock::OK, RESPONSE_ACCEPT);
 
@@ -485,7 +496,12 @@ RedirectBox::choose_send ()
 
 	/* XXX need redirect lock on route */
 
-	send->ensure_io (0, _route->max_redirect_outs(), false, this);
+	try {
+		send->ensure_io (0, _route->max_redirect_outs(), false, this);
+	} catch (AudioEngine::PortRegistrationFailure& err) {
+		error << string_compose (_("Cannot set up new send: %1"), err.what()) << endmsg;
+		return;
+	}
 	
 	IOSelectorWindow *ios = new IOSelectorWindow (_session, send, false, true);
 	
@@ -843,6 +859,13 @@ RedirectBox::idle_delete_redirect (boost::weak_ptr<Redirect> weak_redirect)
 	/* NOT copied to _mixer.selection() */
 
 	no_redirect_redisplay = true;
+
+	void* gui = redirect->get_gui ();
+	
+	if (gui) {
+		static_cast<Gtk::Widget*>(gui)->hide ();
+	}
+	
 	_route->remove_redirect (redirect, this);
 	no_redirect_redisplay = false;
 	redisplay_redirects (this);
@@ -1084,16 +1107,15 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 		if ((plugin_insert = boost::dynamic_pointer_cast<PluginInsert> (insert)) != 0) {
 			
 			PluginUIWindow *plugin_ui;
+
+			/* these are both allowed to be null */
+			
+			Container* toplevel = get_toplevel();
+			Window* win = dynamic_cast<Gtk::Window*>(toplevel);
 			
 			if (plugin_insert->get_gui() == 0) {
-				
-				plugin_ui = new PluginUIWindow (plugin_insert);
-				
-				if (_owner_is_mixer) {
-					ARDOUR_UI::instance()->the_mixer()->ensure_float (*plugin_ui);
-				} else {
-					ARDOUR_UI::instance()->the_editor().ensure_float (*plugin_ui);
-				}
+
+				plugin_ui = new PluginUIWindow (win, plugin_insert);
 				
 				WindowTitle title(Glib::get_application_name());
 				title += generate_redirect_title (plugin_insert);
@@ -1104,9 +1126,9 @@ RedirectBox::edit_redirect (boost::shared_ptr<Redirect> redirect)
 				// change window title when route name is changed
 				_route->name_changed.connect (bind (mem_fun(*this, &RedirectBox::route_name_changed), plugin_ui, boost::weak_ptr<PluginInsert> (plugin_insert)));
 				
-				
 			} else {
 				plugin_ui = reinterpret_cast<PluginUIWindow *> (plugin_insert->get_gui());
+				plugin_ui->set_parent (win);
 			}
 			
 			if (plugin_ui->is_visible()) {
@@ -1168,7 +1190,7 @@ RedirectBox::register_actions ()
 	Glib::RefPtr<Action> act;
 
 	/* new stuff */
-	ActionManager::register_action (popup_act_grp, X_("newplugin"), _("New Plugin ..."),  sigc::ptr_fun (RedirectBox::rb_choose_plugin));
+	ActionManager::register_action (popup_act_grp, X_("newplugin"), _("New Plugin"),  sigc::ptr_fun (RedirectBox::rb_choose_plugin));
 
 	act = ActionManager::register_action (popup_act_grp, X_("newinsert"), _("New Insert"),  sigc::ptr_fun (RedirectBox::rb_choose_insert));
 	ActionManager::jack_sensitive_actions.push_back (act);
