@@ -47,6 +47,7 @@ class RegionView : public TimeAxisViewItem
 	            Gdk::Color&          basic_color);
 
 	RegionView (const RegionView& other);
+	RegionView (const RegionView& other, boost::shared_ptr<ARDOUR::Region> other_region);
 
 	~RegionView ();
 	
@@ -99,6 +100,7 @@ class RegionView : public TimeAxisViewItem
 		boost::shared_ptr<ARDOUR::Region>,
 		double      samples_per_unit,
 		Gdk::Color& basic_color,
+		bool recording,
 		TimeAxisViewItem::Visibility);
 
     virtual void region_resized (ARDOUR::Change);
