@@ -79,9 +79,20 @@
 
 using namespace ARDOUR;
 using namespace PBD;
+using namespace Gtkmm2ext;
 using namespace Gtk;
 using namespace Editing;
+using namespace std;
 
+Glib::RefPtr<Gdk::Pixbuf> RouteTimeAxisView::slider;
+
+void
+RouteTimeAxisView::setup_slider_pix ()
+{
+	if ((slider = ::get_icon ("fader_belt_h")) == 0) {
+		throw failed_constructor ();
+	}
+}
 
 RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::shared_ptr<Route> rt, Canvas& canvas)
 	: AxisView(sess),
@@ -94,11 +105,11 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 	  size_button (_("h")), // height
 	  automation_button (_("a")),
 	  visual_button (_("v")),
-	  lm (rt, sess)
-
+	  gm (rt, sess, slider, true)
 {
-	lm.set_no_show_all();
-	lm.setup_meters(50);
+	gm.get_level_meter().set_no_show_all();
+	gm.get_level_meter().setup_meters(50);
+
 	_has_state = true;
 	playlist_menu = 0;
 	playlist_action_menu = 0;
@@ -150,20 +161,21 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 
 		rec_enable_button->signal_button_press_event().connect (mem_fun(*this, &RouteUI::rec_enable_press), false);
 		rec_enable_button->signal_button_release_event().connect (mem_fun(*this, &RouteUI::rec_enable_release));
-		controls_table.attach (*rec_enable_button, 4, 5, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+		controls_table.attach (*rec_enable_button, 5, 6, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 		ARDOUR_UI::instance()->tooltips().set_tip(*rec_enable_button, _("Record"));
 
 	}
 
-	controls_hbox.pack_start(lm, false, false);
+	controls_hbox.pack_start(gm.get_level_meter(), false, false);
 	_route->meter_change.connect (mem_fun(*this, &RouteTimeAxisView::meter_changed));
 	_route->input_changed.connect (mem_fun(*this, &RouteTimeAxisView::io_changed));
 	_route->output_changed.connect (mem_fun(*this, &RouteTimeAxisView::io_changed));
 
-	controls_table.attach (*mute_button, 5, 6, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
-	controls_table.attach (*solo_button, 6, 7, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+	controls_table.attach (*mute_button, 6, 7, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+	controls_table.attach (*solo_button, 7, 8, 0, 1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 
-	controls_table.attach (edit_group_button, 6, 7, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+	controls_table.attach (edit_group_button, 7, 8, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
+	controls_table.attach (gm.get_gain_slider(), 0, 5, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 0, 0);
 
 	ARDOUR_UI::instance()->tooltips().set_tip(*solo_button,_("Solo"));
 	ARDOUR_UI::instance()->tooltips().set_tip(*mute_button,_("Mute"));
@@ -187,11 +199,11 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 
 	} else {
 
-		controls_table.attach (automation_button, 5, 6, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+		controls_table.attach (automation_button, 6, 7, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
 	}
 
 	if (is_track() && track()->mode() == ARDOUR::Normal) {
-		controls_table.attach (playlist_button, 4, 5, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
+		controls_table.attach (playlist_button, 5, 6, 1, 2, Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND);
 	}
 
 	y_position = -1;
@@ -213,6 +225,9 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 
 	editor.ZoomChanged.connect (mem_fun(*this, &RouteTimeAxisView::reset_samples_per_unit));
 	ColorsChanged.connect (mem_fun (*this, &RouteTimeAxisView::color_handler));
+
+	gm.get_gain_slider().signal_scroll_event().connect(mem_fun(*this, &RouteTimeAxisView::controls_ebox_scroll), false);
+	gm.get_gain_slider().set_name ("TrackGainFader");
 }
 
 RouteTimeAxisView::~RouteTimeAxisView ()
@@ -247,6 +262,7 @@ RouteTimeAxisView::post_construct ()
 	/* map current state of the route */
 
 	update_diskstream_display ();
+	subplugin_menu.items().clear ();
 	_route->foreach_redirect (this, &RouteTimeAxisView::add_redirect_to_subplugin_menu);
 	_route->foreach_redirect (this, &RouteTimeAxisView::add_existing_redirect_automation_curves);
 	reset_redirect_automation_curves ();
@@ -364,7 +380,8 @@ RouteTimeAxisView::playlist_click ()
 	playlist_action_menu->set_name ("ArdourContextMenu");
 	
  	build_playlist_menu (playlist_action_menu);
-	editor.set_selected_track (*this, Selection::Add);
+
+	conditionally_add_to_selection ();
 	playlist_action_menu->popup (1, gtk_get_current_event_time());
 }
 
@@ -377,7 +394,7 @@ RouteTimeAxisView::automation_click ()
 		*/
 		build_display_menu ();
 	}
-	editor.set_selected_track (*this, Selection::Add);
+	conditionally_add_to_selection ();
 	automation_action_menu->popup (1, gtk_get_current_event_time());
 }
 
@@ -488,6 +505,7 @@ RouteTimeAxisView::build_display_menu ()
 	route_active_menu_item->set_active (_route->active());
 
 	items.push_back (SeparatorElem());
+	items.push_back (MenuElem (_("Hide"), mem_fun(*this, &RouteTimeAxisView::hide_click)));
 	items.push_back (MenuElem (_("Remove"), mem_fun(*this, &RouteUI::remove_this_route)));
 }
 
@@ -624,7 +642,7 @@ RouteTimeAxisView::show_timestretch (nframes_t start, nframes_t end)
 
 	x1 = start / editor.get_current_zoom();
 	x2 = (end - 1) / editor.get_current_zoom();
-	y2 = height - 2;
+	y2 = current_height() - 2;
 	
 	timestretch_rect->property_x1() = x1;
 	timestretch_rect->property_y1() = 1.0;
@@ -663,54 +681,30 @@ RouteTimeAxisView::show_selection (TimeSelection& ts)
 }
 
 void
-RouteTimeAxisView::set_height (TrackHeight h)
+RouteTimeAxisView::set_height (uint32_t h)
 {
-	int gmlen = (height_to_pixels (h)) - 5;
-	bool height_changed = (height == 0) || (h != height_style);
-	lm.setup_meters (gmlen);
+	int gmlen = h - 5;
+	bool height_changed = (height == 0) || (h != height);
+	gm.get_level_meter().setup_meters (gmlen);
+
 	TimeAxisView::set_height (h);
 
 	ensure_xml_node ();
 
 	if (_view) {
-		_view->set_height ((double) height);
+		_view->set_height ((double) current_height());
 	}
 
-	switch (height_style) {
-	case Largest:
-		xml_node->add_property ("track_height", "largest");
-		break;
+	char buf[32];
+	snprintf (buf, sizeof (buf), "%u", height);
+	xml_node->add_property ("height", buf);
 
-	case Large:
-		xml_node->add_property ("track_height", "large");
-		break;
-
-	case Larger:
-		xml_node->add_property ("track_height", "larger");
-		break;
-
-	case Normal:
-		xml_node->add_property ("track_height", "normal");
-		break;
-
-	case Smaller:
-		xml_node->add_property ("track_height", "smaller");
-		break;
-
-	case Small:
-		xml_node->add_property ("track_height", "small");
-		break;
-	}
-
-	switch (height_style) {
-	case Largest:
-	case Large:
-	case Larger:
-	case Normal:
+	if (height >= hNormal) {
 		reset_meter();
 		show_name_entry ();
 		hide_name_label ();
 
+		gm.get_gain_slider().show();
 		mute_button->show();
 		solo_button->show();
 		if (rec_enable_button)
@@ -725,13 +719,14 @@ RouteTimeAxisView::set_height (TrackHeight h)
 		if (is_track() && track()->mode() == ARDOUR::Normal) {
 			playlist_button.show();
 		}
-		break;
 
-	case Smaller:
+	} else if (height >= hSmaller) {
+
 		reset_meter();
 		show_name_entry ();
 		hide_name_label ();
 
+		gm.get_gain_slider().hide();
 		mute_button->show();
 		solo_button->show();
 		if (rec_enable_button)
@@ -746,13 +741,14 @@ RouteTimeAxisView::set_height (TrackHeight h)
 		if (is_track() && track()->mode() == ARDOUR::Normal) {
 			playlist_button.hide ();
 		}
-		break;
 
-	case Small:
+	} else {
+
 		hide_meter ();
 		hide_name_entry ();
 		show_name_label ();
-
+		
+		gm.get_gain_slider().hide();
 		mute_button->hide();
 		solo_button->hide();
 		if (rec_enable_button)
@@ -765,7 +761,6 @@ RouteTimeAxisView::set_height (TrackHeight h)
 		automation_button.hide ();
 		playlist_button.hide ();
 		name_label.set_text (_route->name());
-		break;
 	}
 
 	if (height_changed) {
@@ -1080,7 +1075,7 @@ RouteTimeAxisView::get_selectables (nframes_t start, nframes_t end, double top,
 	nframes_t start_adjusted = session_frame_to_track_frame(start, speed);
 	nframes_t end_adjusted   = session_frame_to_track_frame(end, speed);
 
-	if (_view && ((top < 0.0 && bot < 0.0)) || touched (top, bot)) {
+	if ((_view && ((top < 0.0 && bot < 0.0))) || touched (top, bot)) {
 		_view->get_selectables (start_adjusted, end_adjusted, results);
 	}
 
@@ -1427,7 +1422,7 @@ RouteTimeAxisView::show_all_automation ()
 
 	no_redraw = false;
 
-	 _route->gui_changed ("track_height", (void *) 0); /* EMIT_SIGNAL */
+	 _route->gui_changed ("visible_tracks", (void *) 0); /* EMIT_SIGNAL */
 }
 
 void
@@ -1444,8 +1439,8 @@ RouteTimeAxisView::show_existing_automation ()
 	}
 
 	no_redraw = false;
-
-	 _route->gui_changed ("track_height", (void *) 0); /* EMIT_SIGNAL */
+	
+	_route->gui_changed ("visible_tracks", (void *) 0); /* EMIT_SIGNAL */
 }
 
 void
@@ -1460,7 +1455,7 @@ RouteTimeAxisView::hide_all_automation ()
 	}
 
 	no_redraw = false;
-	 _route->gui_changed ("track_height", (void *) 0); /* EMIT_SIGNAL */
+	 _route->gui_changed ("visible_tracks", (void *) 0); /* EMIT_SIGNAL */
 }
 
 
@@ -1609,7 +1604,7 @@ RouteTimeAxisView::redirect_automation_track_hidden (RouteTimeAxisView::Redirect
 
 	r->mark_automation_visible (ran->what, false);
 
-	 _route->gui_changed ("track_height", (void *) 0); /* EMIT_SIGNAL */
+	 _route->gui_changed ("visible_tracks", (void *) 0); /* EMIT_SIGNAL */
 }
 
 void
@@ -1636,7 +1631,7 @@ RouteTimeAxisView::add_redirect_to_subplugin_menu (boost::shared_ptr<Redirect> r
 	using namespace Menu_Helpers;
 	RedirectAutomationInfo *rai;
 	list<RedirectAutomationInfo*>::iterator x;
-	
+
 	const std::set<uint32_t>& automatable = r->what_can_be_automated ();
 	std::set<uint32_t> has_visible_automation;
 
@@ -1745,7 +1740,7 @@ RouteTimeAxisView::redirect_menu_item_toggled (RouteTimeAxisView::RedirectAutoma
 
 		/* now trigger a redisplay */
 		
-		 _route->gui_changed ("track_height", (void *) 0); /* EMIT_SIGNAL */
+		 _route->gui_changed ("visible_tracks", (void *) 0); /* EMIT_SIGNAL */
 
 	}
 }
@@ -1783,7 +1778,7 @@ RouteTimeAxisView::redirects_changed (void *src)
 
 	/* change in visibility was possible */
 
-	_route->gui_changed ("track_height", this);
+	_route->gui_changed ("visible_tracks", this);
 }
 
 RedirectAutomationLine *
@@ -1818,14 +1813,14 @@ RouteTimeAxisView::update_rec_display ()
 void
 RouteTimeAxisView::fast_update ()
 {
-	lm.update_meters ();
+	gm.get_level_meter().update_meters ();
 }
 
 void
 RouteTimeAxisView::hide_meter ()
 {
 	clear_meter ();
-	lm.hide_meters ();
+	gm.get_level_meter().hide_meters ();
 }
 
 void
@@ -1838,7 +1833,7 @@ void
 RouteTimeAxisView::reset_meter ()
 {
 	if (Config->get_show_track_meters()) {
-		lm.setup_meters (height-5);
+		gm.get_level_meter().setup_meters (height-5);
 	} else {
 		hide_meter ();
 	}
@@ -1847,7 +1842,7 @@ RouteTimeAxisView::reset_meter ()
 void
 RouteTimeAxisView::clear_meter ()
 {
-	lm.clear_meters ();
+	gm.get_level_meter().clear_meters ();
 }
 
 void
@@ -1862,3 +1857,4 @@ RouteTimeAxisView::io_changed (IOChange change, void *src)
 {
 	reset_meter ();
 }
+
