@@ -62,7 +62,7 @@ StreamView::StreamView (RouteTimeAxisView& tv)
 	canvas_rect->property_x1() = 0.0;
 	canvas_rect->property_y1() = 0.0;
 	canvas_rect->property_x2() = _trackview.editor.frame_to_pixel (max_frames);
-	canvas_rect->property_y2() = (double) tv.height;
+	canvas_rect->property_y2() = (double) tv.current_height();
 	canvas_rect->property_outline_what() = (guint32) (0x1|0x2|0x8);  // outline ends and bottom 
 	// (Fill/Outline colours set in derived classes)
 
@@ -373,7 +373,6 @@ StreamView::set_selected_regionviews (RegionSelection& regions)
 {
 	bool selected;
 
-	// cerr << _trackview.name() << " (selected = " << regions.size() << ")" << endl;
 	for (list<RegionView*>::iterator i = region_views.begin(); i != region_views.end(); ++i) {
 		
 		selected = false;
@@ -383,8 +382,7 @@ StreamView::set_selected_regionviews (RegionSelection& regions)
 				selected = true;
 			}
 		}
-		
-		// cerr << "\tregion " << (*i)->region().name() << " selected = " << selected << endl;
+
 		(*i)->set_selected (selected);
 	}
 }
