@@ -72,7 +72,7 @@ bool TimeAxisView::need_size_info = true;
 TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisView* rent, Canvas& canvas) 
 	: AxisView (sess), 
 	  editor (ed),
-	  controls_table (2, 7)
+	  controls_table (2, 8)
 {
 	if (need_size_info) {
 		compute_controls_size_info ();
@@ -95,6 +95,7 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	_has_state = false;
 	last_name_entry_key_press_event = 0;
 	name_packing = NamePackingBits (0);
+	resize_drag_start = -1;
 
 	/*
 	  Create the standard LHS Controls
@@ -126,11 +127,27 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 	controls_table.set_col_spacings (0);
 	controls_table.set_homogeneous (true);
 
-	controls_table.attach (name_hbox, 0, 4, 0, 1,  Gtk::FILL|Gtk::EXPAND,  Gtk::FILL|Gtk::EXPAND, 3, 0);
+	controls_table.attach (name_hbox, 0, 5, 0, 1,  Gtk::FILL|Gtk::EXPAND,  Gtk::FILL|Gtk::EXPAND, 3, 0);
 	controls_table.show_all ();
 	controls_table.set_no_show_all ();
 
+	resizer.set_size_request (10, 10);
+	resizer.set_name ("ResizeHandle");
+	resizer.signal_expose_event().connect (mem_fun (*this, &TimeAxisView::resizer_expose));
+	resizer.signal_button_press_event().connect (mem_fun (*this, &TimeAxisView::resizer_button_press));
+	resizer.signal_button_release_event().connect (mem_fun (*this, &TimeAxisView::resizer_button_release));
+	resizer.signal_motion_notify_event().connect (mem_fun (*this, &TimeAxisView::resizer_motion));
+	resizer.set_events (Gdk::BUTTON_PRESS_MASK|
+			    Gdk::BUTTON_RELEASE_MASK|
+			    Gdk::POINTER_MOTION_MASK|
+			    Gdk::SCROLL_MASK);
+
+	resizer_box.pack_start (resizer, false, false);
+	resizer.show ();
+	resizer_box.show();
+
 	controls_vbox.pack_start (controls_table, false, false);
+	controls_vbox.pack_end (resizer_box, false, true);
 	controls_vbox.show ();
 
 	controls_ebox.set_name ("TimeAxisViewControlsBaseUnselected");
@@ -146,6 +163,7 @@ TimeAxisView::TimeAxisView (ARDOUR::Session& sess, PublicEditor& ed, TimeAxisVie
 
 	controls_frame.add (controls_hbox);
 	controls_frame.set_name ("TimeAxisViewControlsBaseUnselected");
+	controls_vbox.set_name ("TimeAxisViewControlsBaseUnselected");
 	controls_frame.set_shadow_type (Gtk::SHADOW_ETCHED_OUT);
 
 	ColorsChanged.connect (mem_fun (*this, &TimeAxisView::color_handler));
@@ -170,6 +188,10 @@ TimeAxisView::~TimeAxisView()
 		delete (*i)->end_trim;
 	}
 
+	for (list<SimpleLine*>::iterator i = feature_lines.begin(); i != feature_lines.end(); ++i) {
+		delete (*i);
+	}
+
 	if (selection_group) {
 		delete selection_group;
 		selection_group = 0;
@@ -221,9 +243,9 @@ TimeAxisView::show_at (double y, int& nth, VBox *parent)
 	_hidden = false;
 	
 	/* height in pixels depends on _order, so update it now we've changed _order */
-	set_height (height_style);
+	set_height (height);
 	
-	effective_height = height;
+	effective_height = current_height();
 
 	/* now show children */
 	
@@ -250,6 +272,9 @@ TimeAxisView::controls_ebox_scroll (GdkEventScroll* ev)
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			step_height (true);
 			return true;
+		} else if (Keyboard::no_modifiers_active (ev->state)) {
+			editor.scroll_tracks_up_line();
+			return true;
 		}
 		break;
 		
@@ -257,6 +282,9 @@ TimeAxisView::controls_ebox_scroll (GdkEventScroll* ev)
 		if (Keyboard::modifier_state_equals (ev->state, Keyboard::TertiaryModifier)) {
 			step_height (false);
 			return true;
+		} else if (Keyboard::no_modifiers_active (ev->state)) {
+			editor.scroll_tracks_down_line();
+			return true;
 		}
 		break;
 
@@ -321,39 +349,21 @@ TimeAxisView::hide ()
 void
 TimeAxisView::step_height (bool bigger)
 {
-  
-       if (height == hLargest) {
-               if (!bigger) set_height (Large);
-               return;
-       }
-       if (height == hLarge) {
-               if (bigger) set_height (Largest);
-                else set_height (Larger);
-               return;
-       }
-       if (height == hLarger) {
-                if (bigger) set_height (Large);
-                else set_height (Normal);
-               return;
-       }
-       if (height == hNormal) {
-                if (bigger) set_height (Larger);
-                else set_height (Smaller);
-               return;
-       }
-       if (height == hSmaller) {
-                if (bigger) set_height (Normal);
-                else set_height (Small);
-               return;
-       }
-       if (height == hSmall) {
-                if (bigger) set_height (Smaller);
-               return;
-       }
+	static const uint32_t step = 20;
+
+	if (bigger) {
+		set_height (height + step);
+	} else {
+		if (height > step) {
+			set_height (std::max (height - step, hSmall));
+		} else if (height != hSmall) {
+			set_height (hSmall);
+		}
+	}
 }
 
 void
-TimeAxisView::set_heights (TrackHeight h)
+TimeAxisView::set_heights (uint32_t h)
 {
 	TrackSelection& ts (editor.get_selection().tracks);
 
@@ -363,23 +373,18 @@ TimeAxisView::set_heights (TrackHeight h)
 }
 
 void
-TimeAxisView::set_height (TrackHeight h)
-{
-	height_style = h;
-	set_height_pixels (height_to_pixels (h));
-}
-
-void
-TimeAxisView::set_height_pixels (uint32_t h)
+TimeAxisView::set_height(uint32_t h)
 {
 	height = h;
-	controls_frame.set_size_request (-1, height + ((order == 0) ? 1 : 0));
-	//cerr << "TimeAxisView::set_height_pixels() called h = " << h << endl;//DEBUG
+
+	controls_frame.set_size_request (-1, current_height() + ((order == 0) ? 1 : 0));
+
  	if (canvas_item_visible (selection_group)) {
 		/* resize the selection rect */
 		show_selection (editor.get_selection().time);
 	}
-	
+
+	reshow_feature_lines ();
 }
 
 bool
@@ -419,8 +424,11 @@ TimeAxisView::name_entry_key_release (GdkEventKey* ev)
 				}
 			}
 
-			if ((*i)->height_style == Small) {
-				(*i)->set_height(Smaller);
+
+			/* resize to show editable name display */
+			
+			if ((*i)->current_height() >= hSmall && (*i)->current_height() < hNormal) {
+				(*i)->set_height (hSmaller);
 			}
 			
 			(*i)->name_entry.grab_focus();
@@ -522,6 +530,17 @@ TimeAxisView::name_entry_button_release (GdkEventButton *ev)
 	return false;
 }
 
+void
+TimeAxisView::conditionally_add_to_selection ()
+{
+	Selection& s (editor.get_selection());
+
+	if (!s.selected (this)) {
+		editor.set_selected_track (*this, Selection::Set);
+	}
+}
+
+
 void
 TimeAxisView::popup_display_menu (guint32 when)
 {
@@ -529,14 +548,14 @@ TimeAxisView::popup_display_menu (guint32 when)
 		build_display_menu ();
 	}
 
-	editor.set_selected_track (*this, Selection::Add);
+	conditionally_add_to_selection ();
 	display_menu->popup (1, when);	
 }
 
 gint
 TimeAxisView::size_click (GdkEventButton *ev)
 {
-	editor.set_selected_track (*this, Selection::Add);
+	conditionally_add_to_selection ();
 	popup_size_menu (ev->time);
 	return TRUE;
 }
@@ -562,7 +581,7 @@ TimeAxisView::set_selected (bool yn)
 	if (_selected) {
 		controls_ebox.set_name (controls_base_selected_name);
 		controls_frame.set_name (controls_base_selected_name);
-		
+		controls_vbox.set_name (controls_base_selected_name);
 		/* propagate any existing selection, if the mode is right */
 
 		if (editor.current_mouse_mode() == Editing::MouseRange && !editor.get_selection().time.empty()) {
@@ -572,7 +591,7 @@ TimeAxisView::set_selected (bool yn)
 	} else {
 		controls_ebox.set_name (controls_base_unselected_name);
 		controls_frame.set_name (controls_base_unselected_name);
-
+		controls_vbox.set_name (controls_base_unselected_name);
 		hide_selection ();
 
 		/* children will be set for the yn=true case. but when deselecting
@@ -584,6 +603,8 @@ TimeAxisView::set_selected (bool yn)
 			(*i)->set_selected (false);
 		}
 	}
+
+	resizer.queue_draw ();
 }
 
 void
@@ -595,12 +616,12 @@ TimeAxisView::build_size_menu ()
 	size_menu->set_name ("ArdourContextMenu");
 	MenuList& items = size_menu->items();
 	
-	items.push_back (MenuElem (_("Largest"), bind (mem_fun (*this, &TimeAxisView::set_heights), Largest)));
-	items.push_back (MenuElem (_("Large"), bind (mem_fun (*this, &TimeAxisView::set_heights), Large)));
-	items.push_back (MenuElem (_("Larger"), bind (mem_fun (*this, &TimeAxisView::set_heights), Larger)));
-	items.push_back (MenuElem (_("Normal"), bind (mem_fun (*this, &TimeAxisView::set_heights), Normal)));
-	items.push_back (MenuElem (_("Smaller"), bind (mem_fun (*this, &TimeAxisView::set_heights),Smaller)));
-	items.push_back (MenuElem (_("Small"), bind (mem_fun (*this, &TimeAxisView::set_heights), Small)));
+	items.push_back (MenuElem (_("Largest"), bind (mem_fun (*this, &TimeAxisView::set_heights), hLargest)));
+	items.push_back (MenuElem (_("Large"), bind (mem_fun (*this, &TimeAxisView::set_heights), hLarge)));
+	items.push_back (MenuElem (_("Larger"), bind (mem_fun (*this, &TimeAxisView::set_heights), hLarger)));
+	items.push_back (MenuElem (_("Normal"), bind (mem_fun (*this, &TimeAxisView::set_heights), hNormal)));
+	items.push_back (MenuElem (_("Smaller"), bind (mem_fun (*this, &TimeAxisView::set_heights),hSmaller)));
+	items.push_back (MenuElem (_("Small"), bind (mem_fun (*this, &TimeAxisView::set_heights), hSmall)));
 }
 
 void
@@ -617,6 +638,14 @@ TimeAxisView::build_display_menu ()
 void
 TimeAxisView::set_samples_per_unit (double spu)
 {
+	AnalysisFeatureList::const_iterator i;
+	list<ArdourCanvas::SimpleLine*>::iterator l;
+
+	for (i = analysis_features.begin(), l = feature_lines.begin(); i != analysis_features.end() && l != feature_lines.end(); ++i, ++l) {
+		(*l)->property_x1() = editor.frame_to_pixel (*i);
+		(*l)->property_x2() = editor.frame_to_pixel (*i);
+	}
+	
 	for (vector<TimeAxisView*>::iterator i = children.begin(); i != children.end(); ++i) {
 		(*i)->set_samples_per_unit (spu);
 	}
@@ -675,7 +704,7 @@ TimeAxisView::show_selection (TimeSelection& ts)
 		
 		x1 = editor.frame_to_unit (start);
 		x2 = editor.frame_to_unit (start + cnt - 1);
-		y2 = height;
+		y2 = current_height();
 
 		rect->rect->property_x1() = x1;
 		rect->rect->property_y1() = 1.0;
@@ -862,7 +891,7 @@ TimeAxisView::touched (double top, double bot)
 	   y_position is the "origin" or "top" of the track.
 	 */
 
-	double mybot = y_position + height;
+	double mybot = y_position + current_height();
 	
 	return ((y_position <= bot && y_position >= top) || 
 		((mybot <= bot) && (top < mybot)) || 
@@ -895,68 +924,69 @@ TimeAxisView::get_parent_with_state ()
 	return parent->get_parent_with_state ();
 }		
 
-void
+
+XMLNode&
+TimeAxisView::get_state ()
+{
+	XMLNode* node = new XMLNode ("TAV-" + name());
+	char buf[32];
+
+	snprintf (buf, sizeof(buf), "%u", height);
+	node->add_property ("height", buf);
+	node->add_property ("marked_for_display", (_marked_for_display ? "1" : "0"));
+	return *node;
+}
+
+int
 TimeAxisView::set_state (const XMLNode& node)
 {
 	const XMLProperty *prop;
 
+	if ((prop = node.property ("marked_for_display")) != 0) {
+		_marked_for_display = (prop->value() == "1");
+	}
+
 	if ((prop = node.property ("track_height")) != 0) {
 
 		if (prop->value() == "largest") {
-			set_height (Largest);
+			set_height (hLargest);
 		} else if (prop->value() == "large") {
-			set_height (Large);
+			set_height (hLarge);
 		} else if (prop->value() == "larger") {
-			set_height (Larger);
+			set_height (hLarger);
 		} else if (prop->value() == "normal") {
-			set_height (Normal);
+			set_height (hNormal);
 		} else if (prop->value() == "smaller") {
-			set_height (Smaller);
+			set_height (hSmaller);
 		} else if (prop->value() == "small") {
-			set_height (Small);
+			set_height (hSmall);
 		} else {
 			error << string_compose(_("unknown track height name \"%1\" in XML GUI information"), prop->value()) << endmsg;
 			set_height (Normal);
 		}
 
+	} else if ((prop = node.property ("height")) != 0) {
+
+		set_height (atoi (prop->value()));
+		
 	} else {
-		set_height (Normal);
+
+		set_height (hNormal);
 	}
+
+	return 0;
 }
 
 void
 TimeAxisView::reset_height()
 {
-	set_height_pixels (height);
+	set_height (height);
 
 	for (vector<TimeAxisView*>::iterator i = children.begin(); i != children.end(); ++i) {
-		(*i)->set_height_pixels ((TrackHeight)(*i)->height);
+		(*i)->set_height ((*i)->height);
 	}
 }
 	
-uint32_t
-TimeAxisView::height_to_pixels (TrackHeight h)
-{
-	switch (h) {
-	case Largest:
-		return hLargest;
-	case Large:
-		return hLarge;
-	case Larger:
-		return hLarger;
-	case Normal:
-		return hNormal;
-	case Smaller:
-		return hSmaller;
-	case Small:
-		return hSmall;
-	}
-	
-	// what is wrong with gcc ?
-	
-	return hNormal;
-}
-			
 void
 TimeAxisView::compute_controls_size_info ()
 {
@@ -965,7 +995,8 @@ TimeAxisView::compute_controls_size_info ()
 	Gtk::Table one_row_table (1, 8);
 	Button* buttons[5];
 	const int border_width = 2;
-	const int extra_height = (2 * border_width) + 2; // 2 pixels for the controls frame
+	const int extra_height = (2 * border_width) + 2 // 2 pixels for the controls frame
+		+ 10; // resizer button
 
 	window.add (one_row_table);
 
@@ -1112,35 +1143,128 @@ TimeAxisView::covers_y_position (double y)
 }
 
 void
-TimeAxisView::show_temporary_lines (const AnalysisFeatureList& pos)
+TimeAxisView::show_feature_lines (const AnalysisFeatureList& pos)
 {
-	while (temp_lines.size()< pos.size()) {
+	analysis_features = pos;
+	reshow_feature_lines ();
+}
+
+
+void
+TimeAxisView::hide_feature_lines ()
+{
+	list<ArdourCanvas::SimpleLine*>::iterator l;
+
+	for (l = feature_lines.begin(); l != feature_lines.end(); ++l) {
+		(*l)->hide();
+	}
+}
+
+void
+TimeAxisView::reshow_feature_lines ()
+{
+	while (feature_lines.size()< analysis_features.size()) {
 		ArdourCanvas::SimpleLine* l = new ArdourCanvas::SimpleLine (*canvas_display);
 		l->property_color_rgba() = (guint) ARDOUR_UI::config()->canvasvar_ZeroLine.get();
-		l->property_y1() = 0;
-		l->property_y2() = height;
-		temp_lines.push_back (l);
+		feature_lines.push_back (l);
 	}
 
-	while (temp_lines.size() > pos.size()) {
-		ArdourCanvas::SimpleLine *line = temp_lines.back();
-		temp_lines.pop_back ();
+	while (feature_lines.size() > analysis_features.size()) {
+		ArdourCanvas::SimpleLine *line = feature_lines.back();
+		feature_lines.pop_back ();
 		delete line;
 	}
 
 	AnalysisFeatureList::const_iterator i;
 	list<ArdourCanvas::SimpleLine*>::iterator l;
 
-	for (i = pos.begin(), l = temp_lines.begin(); i != pos.end() && l != temp_lines.end(); ++i, ++l) {
+	for (i = analysis_features.begin(), l = feature_lines.begin(); i != analysis_features.end() && l != feature_lines.end(); ++i, ++l) {
 		(*l)->property_x1() = editor.frame_to_pixel (*i);
 		(*l)->property_x2() = editor.frame_to_pixel (*i);
+		(*l)->property_y1() = 0;
+		(*l)->property_y2() = current_height();
+		(*l)->show ();
 	}
 }
 
-void
-TimeAxisView::hide_temporary_lines ()
+bool
+TimeAxisView::resizer_button_press (GdkEventButton* event)
 {
-	for (list<ArdourCanvas::SimpleLine*>::iterator l = temp_lines.begin(); l != temp_lines.end(); ++l) {
-		(*l)->hide ();
-	}
+	resize_drag_start = event->y_root;
+	resize_idle_target = current_height();
+	editor.start_resize_line_ops ();
+	return true;
 }
+
+bool
+TimeAxisView::resizer_button_release (GdkEventButton* ev)
+{
+	resize_drag_start = -1;
+	editor.end_resize_line_ops ();
+	return true;
+}
+
+void
+TimeAxisView::idle_resize (uint32_t h)
+{
+	set_height (h);
+}
+
+bool
+TimeAxisView::resizer_motion (GdkEventMotion* ev)
+{
+	if (resize_drag_start < 0) {
+		return true;
+	}
+
+	int32_t delta = (int32_t) floor (resize_drag_start - ev->y_root);
+
+	resize_idle_target = std::max (resize_idle_target - delta, (int) hSmall);
+	editor.add_to_idle_resize (this, resize_idle_target);
+	
+	resize_drag_start = ev->y_root;
+
+	return true;
+}
+
+bool
+TimeAxisView::resizer_expose (GdkEventExpose* event)
+{
+	int w, h, x, y, d;
+	Glib::RefPtr<Gdk::Window> win (resizer.get_window());
+	Glib::RefPtr<Gdk::GC> dark (resizer.get_style()->get_fg_gc (STATE_NORMAL));
+	Glib::RefPtr<Gdk::GC> light (resizer.get_style()->get_bg_gc (STATE_NORMAL));
+
+	win->draw_rectangle (controls_ebox.get_style()->get_bg_gc(STATE_NORMAL),
+			     true,
+			     event->area.x,
+			     event->area.y,
+			     event->area.width,
+			     event->area.height);
+
+	win->get_geometry (x, y, w, h, d);
+
+	/* handle/line #1 */
+	
+	win->draw_line (dark, 0, 0, w - 2, 0);
+	win->draw_point (dark, 0, 1);
+	win->draw_line (light, 1, 1, w - 1, 1);
+	win->draw_point (light, w - 1, 0);
+
+	/* handle/line #2 */
+
+	win->draw_line (dark, 0, 4, w - 2, 4);
+	win->draw_point (dark, 0, 5);
+	win->draw_line (light, 1, 5, w - 1, 5);
+	win->draw_point (light, w - 1, 4);
+
+	/* handle/line #3 */
+
+	win->draw_line (dark, 0, 8, w - 2, 8);
+	win->draw_point (dark, 0, 9);
+	win->draw_line (light, 1, 9, w - 1, 9);
+	win->draw_point (light, w - 1, 8);
+
+	return true;
+}
+
