@@ -25,6 +25,7 @@
 
 #include <gtkmm/box.h>
 #include <gtkmm/frame.h>
+#include <gtkmm/drawingarea.h>
 #include <gtkmm/eventbox.h>
 #include <gtkmm/table.h>
 #include <gtkmm/entry.h>
@@ -32,6 +33,8 @@
 
 #include <gtkmm2ext/focus_entry.h>
 
+#include <pbd/stateful.h>
+
 #include <ardour/types.h>
 #include <ardour/region.h>
 
@@ -68,7 +71,7 @@ class Selectable;
  * extended to create functional time-axis based views.
  *
  */
-class TimeAxisView : public virtual AxisView
+class TimeAxisView : public virtual AxisView, public Stateful
 {
   private:
 	enum NamePackingBits {
@@ -77,15 +80,6 @@ class TimeAxisView : public virtual AxisView
 	};
 
   public:
-	enum TrackHeight { 
-		Largest,
-		Large,
-		Larger,
-		Normal,
-		Smaller,
-		Small
-	};
-	
 	static uint32_t hLargest;
 	static uint32_t hLarge;
 	static uint32_t hLarger;
@@ -93,21 +87,22 @@ class TimeAxisView : public virtual AxisView
 	static uint32_t hSmaller;
 	static uint32_t hSmall;
 
-	static uint32_t height_to_pixels (TrackHeight);
-
 	TimeAxisView(ARDOUR::Session& sess, PublicEditor& ed, TimeAxisView* parent, ArdourCanvas::Canvas& canvas);
 	virtual ~TimeAxisView ();
 
+	XMLNode& get_state (void);
+	int set_state (const XMLNode&);
+
 	/* public data: XXX create accessor/mutators for these ?? */
 
 	PublicEditor& editor;
 	
-	TrackHeight height_style; 
-	uint32_t height;  /* in canvas units */
 	uint32_t effective_height;  /* in canvas units */
-	double  y_position;
-	int     order;
+	double   y_position;
+	int      order;
 	
+	uint32_t current_height() const { return height; }
+
 	ArdourCanvas::Group   *canvas_display;
  	Gtk::VBox       *control_parent;
 
@@ -118,9 +113,20 @@ class TimeAxisView : public virtual AxisView
 	Gtk::Table    controls_table;
 	Gtk::EventBox controls_ebox;
 	Gtk::VBox     controls_vbox;
+	Gtk::DrawingArea resizer;
+	Gtk::HBox     resizer_box;
 	Gtk::HBox     name_hbox;
 	Gtk::Frame    name_frame;
  	Gtkmm2ext::FocusEntry name_entry;
+	
+	bool resizer_button_press (GdkEventButton*);
+	bool resizer_button_release (GdkEventButton*);
+	bool resizer_motion (GdkEventMotion*);
+	bool resizer_expose (GdkEventExpose*);
+
+	double resize_drag_start;
+	int32_t resize_idle_target;
+	void idle_resize (uint32_t);
 
 	void hide_name_label ();
 	void hide_name_entry ();
@@ -154,7 +160,7 @@ class TimeAxisView : public virtual AxisView
 	virtual void entered () {}
 	virtual void exited () {}
 
-	virtual void set_height (TrackHeight h);
+	virtual void set_height (uint32_t h);
 	void reset_height();
 
 	/**
@@ -176,8 +182,8 @@ class TimeAxisView : public virtual AxisView
 	virtual ARDOUR::RouteGroup* edit_group() const { return 0; }
 	virtual boost::shared_ptr<ARDOUR::Playlist> playlist() const { return boost::shared_ptr<ARDOUR::Playlist> (); }
 
-	virtual void show_temporary_lines (const ARDOUR::AnalysisFeatureList&);
-	virtual void hide_temporary_lines ();
+	virtual void show_feature_lines (const ARDOUR::AnalysisFeatureList&);
+	virtual void hide_feature_lines ();
 
 	virtual void set_samples_per_unit (double);
 	virtual void show_selection (TimeSelection&);
@@ -206,19 +212,23 @@ class TimeAxisView : public virtual AxisView
 	virtual void get_selectables (nframes_t start, nframes_t end, double top, double bot, list<Selectable*>& results);
 	virtual void get_inverted_selectables (Selection&, list<Selectable *>& results);
 
+	/* called at load time when first GUI idle occurs. put
+	   expensive data loading/redisplay code in here.
+	*/
+	
+	virtual void first_idle () {}
+
 	/* state/serialization management */
 
 	void set_parent (TimeAxisView& p);
 	bool has_state () const;
 
-	virtual void set_state (const XMLNode&);
-	virtual XMLNode* get_state_node () { return 0; }
-
 	/* call this on the parent */
 
 	virtual XMLNode* get_child_xml_node (const string & childname) { return 0; }
 
   protected:
+	uint32_t height;  /* in canvas units */
 
 	string controls_base_unselected_name;
 	string controls_base_selected_name;
@@ -322,11 +332,14 @@ class TimeAxisView : public virtual AxisView
 	static void compute_controls_size_info ();
 	static bool need_size_info;
 
-	void set_heights (TrackHeight);
-	void set_height_pixels (uint32_t h);
+	void set_heights (uint32_t h);
 	void color_handler ();
 
-	list<ArdourCanvas::SimpleLine*> temp_lines;
+	list<ArdourCanvas::SimpleLine*> feature_lines;
+	ARDOUR::AnalysisFeatureList analysis_features;
+	void reshow_feature_lines ();
+
+	void conditionally_add_to_selection ();
 
 }; /* class TimeAxisView */
 
