@@ -20,6 +20,8 @@
 #include <unistd.h>
 #include <cstdio> /* for snprintf, grrr */
 
+#include <glibmm/miscutils.h>
+
 #include <pbd/failed_constructor.h>
 #include <pbd/xml++.h>
 #include <pbd/error.h>
@@ -57,10 +59,19 @@ int
 UIConfiguration::load_defaults ()
 {
 	int found = 0;
-	std::string rcfile = find_config_file ("ardour2_ui_default.conf");
-	
-	if (rcfile.length())
-	{
+	std::string rcfile;
+	const char* ui_conf;
+
+	ui_conf = getenv ("ARDOUR_UI_CONF");
+
+	if (ui_conf && ui_conf[0] != '\0') {
+		rcfile = find_config_file (ui_conf);
+	} else {
+		rcfile = find_config_file ("ardour2_ui_default.conf");
+	}
+
+	if (rcfile.length()) {
+
 		XMLTree tree;
 		found = 1;
 
@@ -116,8 +127,7 @@ UIConfiguration::save_state()
 	XMLTree tree;
 	string rcfile;
 
-	rcfile = get_user_ardour_path ();
-	rcfile += "ardour2_ui.conf";
+	rcfile = Glib::build_filename (get_user_ardour_path (), "ardour2_ui.conf");
 
 	if (rcfile.length()) {
 		tree.set_root (&get_state());
