@@ -17,6 +17,9 @@
 
 */
 
+#include <pango/pangoft2.h> // for fontmap resolution control for GnomeCanvas
+#include <pango/pangocairo.h> // for fontmap resolution control for GnomeCanvas
+
 #include <cstdlib>
 #include <cctype>
 #include <fstream>
@@ -31,6 +34,7 @@
 
 #include <gtkmm2ext/utils.h>
 #include <ardour/ardour.h>
+#include <ardour/configuration.h>
 
 #include "ardour_ui.h"
 #include "keyboard.h"
@@ -45,6 +49,8 @@ using namespace sigc;
 using namespace Glib;
 using namespace PBD;
 
+sigc::signal<void>  DPIReset;
+
 int
 pixel_width (const ustring& str, Pango::FontDescription& font)
 {
@@ -460,39 +466,10 @@ key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev)
 		   it does allow.
 		*/
 
-		int fakekey = GDK_VoidSymbol;
-		int ret = false;
+		uint32_t fakekey = ev->keyval;
 
-		switch (ev->keyval) {
-		case GDK_Tab:
-		case GDK_ISO_Left_Tab:
-			fakekey = GDK_nabla;
-			break;
-
-		case GDK_Up:
-			fakekey = GDK_uparrow;
-			break;
-
-		case GDK_Down:
-			fakekey = GDK_downarrow;
-			break;
-
-		case GDK_Right:
-			fakekey = GDK_rightarrow;
-			break;
-
-		case GDK_Left:
-			fakekey = GDK_leftarrow;
-			break;
-
-		default:
-			break;
-		}
-
-		if (fakekey != GDK_VoidSymbol) {
-			ret = gtk_accel_groups_activate(G_OBJECT(win), fakekey, GdkModifierType(ev->state));
-			
-			if (ret) {
+		if (possibly_translate_keyval_to_make_legal_accelerator (fakekey)) {
+			if (gtk_accel_groups_activate(G_OBJECT(win), fakekey, GdkModifierType(ev->state))) {
 				return true;
 			}
 
@@ -692,6 +669,68 @@ key_is_legal_for_numeric_entry (guint keyval)
 
 	return false;
 }
+void
+set_pango_fontsize ()
+{
+	long val = ARDOUR::Config->get_font_scale();
 
+	/* FT2 rendering */
 
+	pango_ft2_font_map_set_resolution ((PangoFT2FontMap*) pango_ft2_font_map_for_display(), val/1024, val/1024);
 
+	/* Cairo rendering, in case there is any */
+	
+	// pango_cairo_font_map_set_resolution ((PangoCairoFontMap*) pango_cairo_font_map_get_default(), val/1024);
+}
+
+void
+reset_dpi ()
+{
+	long val = ARDOUR::Config->get_font_scale();
+	set_pango_fontsize ();
+	/* Xft rendering */
+
+	gtk_settings_set_long_property (gtk_settings_get_default(),
+					"gtk-xft-dpi", val, "ardour");
+	DPIReset();//Emit Signal
+}
+
+bool
+possibly_translate_keyval_to_make_legal_accelerator (uint32_t& keyval)
+{
+	int fakekey = GDK_VoidSymbol;
+
+	switch (keyval) {
+	case GDK_Tab:
+	case GDK_ISO_Left_Tab:
+		fakekey = GDK_nabla;
+		break;
+		
+	case GDK_Up:
+		fakekey = GDK_uparrow;
+		break;
+		
+	case GDK_Down:
+		fakekey = GDK_downarrow;
+		break;
+		
+	case GDK_Right:
+		fakekey = GDK_rightarrow;
+		break;
+		
+	case GDK_Left:
+		fakekey = GDK_leftarrow;
+		break;
+		
+	default:
+		break;
+	}
+	
+	if (fakekey != GDK_VoidSymbol) {
+		keyval = fakekey;
+		return true;
+	} 
+
+	return false;
+}
+		
