@@ -21,6 +21,7 @@
 #define __ardour_audio_region_h__
 
 #include <vector>
+#include <list>
 
 #include <pbd/fastlog.h>
 #include <pbd/undo.h>
@@ -59,6 +60,7 @@ class AudioRegion : public Region
 	bool speed_mismatch (float) const;
 
 	boost::shared_ptr<AudioSource> source (uint32_t n=0) const { if (n < sources.size()) return sources[n]; else return sources[0]; } 
+	const SourceList& get_sources() const { return sources; }
 
 	void set_scale_amplitude (gain_t);
 	gain_t scale_amplitude() const { return _scale_amplitude; }
@@ -67,7 +69,9 @@ class AudioRegion : public Region
 
 	uint32_t n_channels() const { return sources.size(); }
 	vector<string> master_source_names();
-	
+	void set_master_sources (const SourceList&);
+	const SourceList& get_master_sources() const { return master_sources; }
+
 	bool envelope_active () const { return _flags & Region::EnvelopeActive; }
 	bool fade_in_active ()  const { return _flags & Region::FadeIn; }
 	bool fade_out_active () const { return _flags & Region::FadeOut; }
@@ -81,11 +85,16 @@ class AudioRegion : public Region
 			nframes_t offset, nframes_t cnt,
 			uint32_t chan_n=0, double samples_per_unit= 1.0) const;
 
+	/* Readable interface */
+	
+	virtual nframes64_t read (Sample*, nframes64_t pos, nframes64_t cnt, int channel) const;
+	virtual nframes64_t readable_length() const { return length(); }
+
 	virtual nframes_t read_at (Sample *buf, Sample *mixdown_buf,
-			float *gain_buf, nframes_t position, nframes_t cnt, 
-			uint32_t       chan_n      = 0,
-			nframes_t read_frames = 0,
-			nframes_t skip_frames = 0) const;
+				   float *gain_buf, nframes_t position, nframes_t cnt, 
+				   uint32_t       chan_n      = 0,
+				   nframes_t read_frames = 0,
+				   nframes_t skip_frames = 0) const;
 
 	nframes_t master_read_at (Sample *buf, Sample *mixdown_buf, 
 			float *gain_buf,
@@ -140,12 +149,14 @@ class AudioRegion : public Region
 
 	void set_playlist (boost::weak_ptr<Playlist>);
 
+	int get_transients (AnalysisFeatureList&, bool force_new = false);
+
   private:
 	friend class RegionFactory;
 
 	AudioRegion (boost::shared_ptr<AudioSource>, nframes_t start, nframes_t length);
 	AudioRegion (boost::shared_ptr<AudioSource>, nframes_t start, nframes_t length, const string& name, layer_t = 0, Region::Flag flags = Region::DefaultFlags);
-	AudioRegion (SourceList &, nframes_t start, nframes_t length, const string& name, layer_t = 0, Region::Flag flags = Region::DefaultFlags);
+	AudioRegion (const SourceList &, nframes_t start, nframes_t length, const string& name, layer_t = 0, Region::Flag flags = Region::DefaultFlags);
 	AudioRegion (boost::shared_ptr<const AudioRegion>, nframes_t start, nframes_t length, const string& name, layer_t = 0, Region::Flag flags = Region::DefaultFlags);
 	AudioRegion (boost::shared_ptr<const AudioRegion>);
 	AudioRegion (boost::shared_ptr<AudioSource>, const XMLNode&);
@@ -159,16 +170,18 @@ class AudioRegion : public Region
 	void recompute_gain_at_end ();
 	void recompute_gain_at_start ();
 
-	nframes_t _read_at (const SourceList&, Sample *buf, Sample *mixdown_buffer, 
-				 float *gain_buffer, nframes_t position, nframes_t cnt, 
-				 uint32_t chan_n = 0,
-				 nframes_t read_frames = 0,
-				 nframes_t skip_frames = 0) const;
+	nframes_t _read_at (const SourceList&, nframes_t limit,
+			    Sample *buf, Sample *mixdown_buffer, 
+			    float *gain_buffer, nframes_t position, nframes_t cnt, 
+			    uint32_t chan_n = 0,
+			    nframes_t read_frames = 0,
+			    nframes_t skip_frames = 0,
+			    bool raw = false) const;
 
 	bool verify_start (nframes_t position);
-	bool verify_length (nframes_t position);
+	bool verify_length (nframes_t& length);
 	bool verify_start_mutable (nframes_t& start);
-	bool verify_start_and_length (nframes_t start, nframes_t length);
+	bool verify_start_and_length (nframes_t start, nframes_t& length);
 	void recompute_at_start ();
 	void recompute_at_end ();
 
@@ -177,6 +190,7 @@ class AudioRegion : public Region
 	void fade_out_changed ();
 	void source_offset_changed ();
 	void listen_to_my_curves ();
+	void listen_to_my_sources ();
 
 	void source_deleted ();
 	
