@@ -74,6 +74,8 @@ class Route : public IO
 	Route (Session&, const XMLNode&, DataType default_type = DataType::AUDIO);
 	virtual ~Route();
 
+	static std::string ensure_track_or_route_name(std::string, Session &);
+
 	std::string comment() { return _comment; }
 	void set_comment (std::string str, void *src);
 
@@ -109,9 +111,6 @@ class Route : public IO
 	void set_gain (gain_t val, void *src);
 	void inc_gain (gain_t delta, void *src);
 
-	bool active() const { return _active; }
-	void set_active (bool yn);
-
 	void set_solo (bool yn, void *src);
 	bool soloed() const { return _soloed; }
 
@@ -193,7 +192,6 @@ class Route : public IO
 	sigc::signal<void,void*> record_enable_changed;
 	sigc::signal<void,void*> edit_group_changed;
 	sigc::signal<void,void*> mix_group_changed;
-	sigc::signal<void>       active_changed;
 	sigc::signal<void,void*> meter_change;
 
 	/* gui's call this for their own purposes. */
@@ -236,7 +234,7 @@ class Route : public IO
 		return _mute_control;
 	}
 	
-	void automation_snapshot (nframes_t now);
+	void automation_snapshot (nframes_t now, bool force);
 	void protect_automation ();
 	
 	void set_remote_control_id (uint32_t id);
@@ -267,7 +265,6 @@ class Route : public IO
 	bool                     _phase_invert : 1;
 	bool                     _denormal_protection : 1;
 	bool                     _recordable : 1;
-	bool                     _active : 1;
 	bool                     _mute_affects_pre_fader : 1;
 	bool                     _mute_affects_post_fader : 1;
 	bool                     _mute_affects_control_outs : 1;
