@@ -162,6 +162,7 @@ class Session : public PBD::StatefulDestructible
 	    union {
 			void*                ptr;
 			bool                 yes_or_no;
+		        nframes_t            target2_frame;
 			SlaveSource slave;
 	    };
 
@@ -248,6 +249,11 @@ class Session : public PBD::StatefulDestructible
 	bool deletion_in_progress() const { return _state_of_the_state & Deletion; }
 	sigc::signal<void> DirtyChanged;
 
+	static sigc::signal<void> AutoBindingOn;
+	static sigc::signal<void> AutoBindingOff;
+
+	static sigc::signal<void,std::string> Dialog;
+	
 	std::string sound_dir (bool with_path = true) const;
 	std::string peak_dir () const;
 	std::string dead_sound_dir () const;
@@ -346,7 +352,7 @@ class Session : public PBD::StatefulDestructible
 	sigc::signal<void> TransportStateChange; /* generic */
 	sigc::signal<void,nframes_t> PositionChanged; /* sent after any non-sequential motion */
 	sigc::signal<void> DurationChanged;
-	sigc::signal<void> HaltOnXrun;
+	sigc::signal<void,nframes_t> Xrun;
 	sigc::signal<void> TransportLooped;
 
 	sigc::signal<void,RouteList&> RouteAdded;
@@ -554,10 +560,13 @@ class Session : public PBD::StatefulDestructible
 	
 	/* region info  */
 
+	void add_regions (std::vector<boost::shared_ptr<Region> >&);
+
 	sigc::signal<void,boost::weak_ptr<AudioRegion> > AudioRegionAdded;
+	sigc::signal<void,std::vector<boost::weak_ptr<AudioRegion> >& > AudioRegionsAdded;
 	sigc::signal<void,boost::weak_ptr<AudioRegion> > AudioRegionRemoved;
 
-	int region_name (string& result, string base = string(""), bool newlevel = false) const;
+	int region_name (string& result, string base = string(""), bool newlevel = false);
 	string new_region_name (string);
 	string path_from_region_name (string name, string identifier);
 
@@ -574,10 +583,10 @@ class Session : public PBD::StatefulDestructible
 	    string doing_what;
 	    
 	    /* control info */
-	    bool sample_convert;
 	    SrcQuality quality;
 	    volatile bool freeze;
 	    std::vector<Glib::ustring> paths;
+	    bool replace_existing_source;
 	    
 	    /* result */
 	    SourceList sources;
@@ -591,9 +600,11 @@ class Session : public PBD::StatefulDestructible
 	SlaveSource post_export_slave;
 	nframes_t post_export_position;
 
-	int start_audio_export (ARDOUR::AudioExportSpecification&);
-	int stop_audio_export (ARDOUR::AudioExportSpecification&);
+	int  pre_export ();
+	int  start_audio_export (ARDOUR::AudioExportSpecification&);
+	int  stop_audio_export (ARDOUR::AudioExportSpecification&);
 	void finalize_audio_export ();
+	static sigc::signal<void, std::string, std::string> Exported;
 
 	void add_source (boost::shared_ptr<Source>);
 	void remove_source (boost::weak_ptr<Source>);
@@ -679,8 +690,8 @@ class Session : public PBD::StatefulDestructible
 
 	/* flattening stuff */
 
-	int write_one_audio_track (AudioTrack&, nframes_t start, nframes_t cnt, bool overwrite, vector<boost::shared_ptr<AudioSource> >&,
-				   InterThreadInfo& wot);
+	boost::shared_ptr<Region> write_one_audio_track (AudioTrack&, nframes_t start, nframes_t end, bool overwrite, vector<boost::shared_ptr<AudioSource> >&,
+							 InterThreadInfo& wot);
 	int freeze (InterThreadInfo&);
 
 	/* session-wide solo/mute/rec-enable */
@@ -879,7 +890,7 @@ class Session : public PBD::StatefulDestructible
 	void reset_playback_load_min ();
 	void reset_capture_load_min ();
 	
-	float read_data_rate () const;
+	float read_data_rate () const; // in usec
 	float write_data_rate () const;
 
 	/* ranges */
@@ -1058,7 +1069,7 @@ class Session : public PBD::StatefulDestructible
 	void set_slave_source (SlaveSource);
 
 	bool _exporting;
-	int prepare_to_export (ARDOUR::AudioExportSpecification&);
+	int  prepare_to_export (ARDOUR::AudioExportSpecification&);
 
 	void prepare_diskstreams ();
 	void commit_diskstreams (nframes_t, bool& session_requires_butler);
@@ -1412,6 +1423,7 @@ class Session : public PBD::StatefulDestructible
 	void set_play_loop (bool yn);
 	void overwrite_some_buffers (Diskstream*);
 	void flush_all_redirects ();
+	int  micro_locate (nframes_t distance);
 	void locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false);
 	void start_locate (nframes_t, bool with_roll, bool with_flush, bool with_loop=false);
 	void force_locate (nframes_t frame, bool with_roll = false);
@@ -1474,6 +1486,9 @@ class Session : public PBD::StatefulDestructible
 
 	/* REGION MANAGEMENT */
 
+	std::map<std::string,uint32_t> region_name_map;
+	void update_region_name_map (boost::shared_ptr<Region>);
+
 	mutable Glib::Mutex region_lock;
 	typedef map<PBD::ID,boost::shared_ptr<AudioRegion> > AudioRegionList;
 	AudioRegionList audio_regions;
@@ -1491,6 +1506,12 @@ class Session : public PBD::StatefulDestructible
 
 	AudioSourceList audio_sources;
 
+  public:
+	AudioSourceList get_audio_sources() { return audio_sources; }
+    
+  private:
+
+
 	int load_sources (const XMLNode& node);
 	XMLNode& get_sources_as_xml ();
 
