@@ -40,27 +40,29 @@ using std::list;
 using std::vector;
 
 namespace ARDOUR {
-
+class Meter;
 class Tempo {
   public:
-	Tempo (double bpm)
-		: _beats_per_minute (bpm) {}
+	Tempo (double bpm, double type=4.0) // defaulting to quarter note
+		: _beats_per_minute (bpm), _note_type(type) {} 
 	Tempo (const Tempo& other) {
 		_beats_per_minute = other._beats_per_minute;
+		_note_type = other._note_type;
 	}
 	void operator= (const Tempo& other) {
 		if (&other != this) {
 			_beats_per_minute = other._beats_per_minute;
+			_note_type = other._note_type;
 		}
 	}
 
-	double beats_per_minute () const { return _beats_per_minute; }
-	double frames_per_beat (nframes_t sr) const {
-		return  ((60.0 * sr) / _beats_per_minute);
-	}
+	double beats_per_minute () const { return _beats_per_minute;}
+	double note_type () const { return _note_type;}
+	double frames_per_beat (nframes_t sr, const Meter& meter) const;
 
   protected:
 	double _beats_per_minute;
+	double _note_type;
 };
 
 class Meter {
@@ -107,6 +109,9 @@ class MetricSection {
   public:
 	MetricSection (const BBT_Time& start)
 		: _start (start), _frame (0), _movable (true) {}
+	MetricSection (nframes_t start)
+		: _frame (start), _movable (true) {}
+
 	virtual ~MetricSection() {}
 
 	const BBT_Time& start() const { return _start; }
@@ -140,6 +145,8 @@ class MeterSection : public MetricSection, public Meter {
   public:
 	MeterSection (const BBT_Time& start, double bpb, double note_type)
 		: MetricSection (start), Meter (bpb, note_type) {}
+	MeterSection (nframes_t start, double bpb, double note_type)
+		: MetricSection (start), Meter (bpb, note_type) {}
 	MeterSection (const XMLNode&);
 
 	static const string xml_state_node_name;
@@ -149,8 +156,10 @@ class MeterSection : public MetricSection, public Meter {
 
 class TempoSection : public MetricSection, public Tempo {
   public:
-	TempoSection (const BBT_Time& start, double qpm)
-		: MetricSection (start), Tempo (qpm) {}
+	TempoSection (const BBT_Time& start, double qpm, double note_type)
+		: MetricSection (start), Tempo (qpm, note_type) {}
+	TempoSection (nframes_t start, double qpm, double note_type)
+		: MetricSection (start), Tempo (qpm, note_type) {}
 	TempoSection (const XMLNode&);
 
 	static const string xml_state_node_name;
@@ -163,7 +172,6 @@ typedef list<MetricSection*> Metrics;
 class TempoMap : public PBD::StatefulDestructible
 {
   public:
-
 	TempoMap (nframes_t frame_rate);
 	~TempoMap();
 
@@ -195,7 +203,7 @@ class TempoMap : public PBD::StatefulDestructible
 
 	BBTPointList *get_points (nframes_t start, nframes_t end) const;
 
-	void           bbt_time (nframes_t when, BBT_Time&) const;
+	void      bbt_time (nframes_t when, BBT_Time&) const;
 	nframes_t frame_time (const BBT_Time&) const;
 	nframes_t bbt_duration_at (nframes_t, const BBT_Time&, int dir) const;
 
@@ -205,9 +213,14 @@ class TempoMap : public PBD::StatefulDestructible
 	const Tempo& tempo_at (nframes_t);
 	const Meter& meter_at (nframes_t);
 
+	const TempoSection& tempo_section_at (nframes_t);
+
 	void add_tempo(const Tempo&, BBT_Time where);
 	void add_meter(const Meter&, BBT_Time where);
 
+	void add_tempo(const Tempo&, nframes_t where);
+	void add_meter(const Meter&, nframes_t where);
+
 	void move_tempo (TempoSection&, const BBT_Time& to);
 	void move_meter (MeterSection&, const BBT_Time& to);
 	
@@ -265,21 +278,26 @@ class TempoMap : public PBD::StatefulDestructible
 	Metric metric_at (nframes_t) const;
         void bbt_time_with_metric (nframes_t, BBT_Time&, const Metric&) const;
 
+	void change_existing_tempo_at (nframes_t, double bpm, double note_type);
+	void change_initial_tempo (double bpm, double note_type);
+
+	int n_tempos () const;
+	int n_meters () const;
+
 	sigc::signal<void,ARDOUR::Change> StateChanged;
 
   private:
 	static Tempo    _default_tempo;
 	static Meter    _default_meter;
 
-	Metrics            *metrics;
-	nframes_t     _frame_rate;
-	nframes_t      last_bbt_when;
-	bool                last_bbt_valid;
-	BBT_Time            last_bbt;
-	mutable Glib::RWLock    lock;
+	Metrics*             metrics;
+	nframes_t           _frame_rate;
+	nframes_t            last_bbt_when;
+	bool                 last_bbt_valid;
+	BBT_Time             last_bbt;
+	mutable Glib::RWLock lock;
 	
-	void timestamp_metrics ();
-
+	void timestamp_metrics (bool use_bbt);
 
 	nframes_t round_to_type (nframes_t fr, int dir, BBTPointType);
 
@@ -296,7 +314,7 @@ class TempoMap : public PBD::StatefulDestructible
 	nframes_t count_frames_between_metrics (const Meter&, const Tempo&, const BBT_Time&, const BBT_Time&) const;
 
 	int move_metric_section (MetricSection&, const BBT_Time& to);
-	void do_insert (MetricSection* section);
+	void do_insert (MetricSection* section, bool with_bbt);
 };
 
 }; /* namespace ARDOUR */
