@@ -477,19 +477,24 @@ AudioTrack::no_roll (nframes_t nframes, nframes_t start_frame, nframes_t end_fra
 		send_silence = true;
 	} else {
 
-		if (Config->get_auto_input()) {
-			if (Config->get_monitoring_model() == SoftwareMonitoring) {
+		if (!Config->get_tape_machine_mode()) {
+			/* 
+			   ADATs work in a strange way.. 
+			   they monitor input always when stopped.and auto-input is engaged. 
+			*/
+			if ((Config->get_monitoring_model() == SoftwareMonitoring) && (Config->get_auto_input () || _diskstream->record_enabled())) {
 				send_silence = false;
 			} else {
 				send_silence = true;
 			}
 		} else {
-			if (_diskstream->record_enabled()) {
-				if (Config->get_monitoring_model() == SoftwareMonitoring) {
-					send_silence = false;
-				} else {
-					send_silence = true;
-				}
+			/* 
+			   Other machines switch to input on stop if the track is record enabled,
+			   regardless of the auto input setting (auto input only changes the 
+			   monitoring state when the transport is rolling) 
+			*/
+			if ((Config->get_monitoring_model() == SoftwareMonitoring) && _diskstream->record_enabled()) {
+				send_silence = false;
 			} else {
 				send_silence = true;
 			}
@@ -539,7 +544,7 @@ AudioTrack::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame,
 		if (lm.locked()) {
 			// automation snapshot can also be called from the non-rt context
 			// and it uses the redirect list, so we take the lock out here
-			automation_snapshot (start_frame);
+			automation_snapshot (start_frame, false);
 		}
 	}
 
@@ -758,19 +763,20 @@ AudioTrack::export_stuff (vector<Sample*>& buffers, uint32_t nbufs, nframes_t st
 	return 0;
 }
 
-void
+boost::shared_ptr<Region>
 AudioTrack::bounce (InterThreadInfo& itt)
 {
 	vector<boost::shared_ptr<AudioSource> > srcs;
-	_session.write_one_audio_track (*this, 0, _session.current_end_frame(), false, srcs, itt);
+	return _session.write_one_audio_track (*this, _session.current_start_frame(), _session.current_end_frame(), 
+					       false, srcs, itt);
 }
 
 
-void
+boost::shared_ptr<Region>
 AudioTrack::bounce_range (nframes_t start, nframes_t end, InterThreadInfo& itt)
 {
 	vector<boost::shared_ptr<AudioSource> > srcs;
-	_session.write_one_audio_track (*this, start, end, false, srcs, itt);
+	return _session.write_one_audio_track (*this, start, end, false, srcs, itt);
 }
 
 void
@@ -811,7 +817,9 @@ AudioTrack::freeze (InterThreadInfo& itt)
 		return;
 	}
 
-	if (_session.write_one_audio_track (*this, _session.current_start_frame(), _session.current_end_frame(), true, srcs, itt)) {
+	boost::shared_ptr<Region> res;
+
+	if ((res = _session.write_one_audio_track (*this, _session.current_start_frame(), _session.current_end_frame(), true, srcs, itt)) == 0) {
 		return;
 	}
 
