@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include <cerrno>
 #include <vector>
+#include <sstream>
 
 #include <glibmm/timer.h>
 #include <pbd/pthread_utils.h>
@@ -62,13 +63,12 @@ AudioEngine::AudioEngine (string client_name)
 	last_monitor_check = 0;
 	monitor_check_interval = max_frames;
 	_processed_frames = 0;
-	_freewheeling = false;
 	_usecs_per_cycle = 0;
 	_jack = 0;
 	_frame_rate = 0;
 	_buffer_size = 0;
-	_freewheeling = false;
 	_freewheel_thread_registered = false;
+	_freewheeling = false;
 
 	m_meter_thread = 0;
 	g_atomic_int_set (&m_meter_exit, 0);
@@ -76,9 +76,6 @@ AudioEngine::AudioEngine (string client_name)
 	if (connect_to_jack (client_name)) {
 		throw NoBackendAvailable ();
 	}
-
-	start_metering_thread();
-
 }
 
 AudioEngine::~AudioEngine ()
@@ -113,6 +110,8 @@ AudioEngine::start ()
 		if (session) {
 			nframes_t blocksize = jack_get_buffer_size (_jack);
 
+			BootMessage (_("Connect session to engine"));
+
 			session->set_block_size (blocksize);
 			session->set_frame_rate (jack_get_sample_rate (_jack));
 
@@ -154,6 +153,8 @@ AudioEngine::start ()
 		} else {
 			// error << _("cannot activate JACK client") << endmsg;
 		}
+
+		start_metering_thread();
 	}
 
 	return _running ? 0 : -1;
@@ -164,11 +165,11 @@ AudioEngine::stop (bool forever)
 {
 	if (_running) {
 		_running = false;
+		stop_metering_thread ();
 		if (forever) {
 			jack_client_t* foo = _jack;
 			_jack = 0;
 			jack_client_close (foo);
-			stop_metering_thread ();
 		} else {
 			jack_deactivate (_jack);
 		}
@@ -293,8 +294,6 @@ AudioEngine::process_callback (nframes_t nframes)
 
 	if (_freewheeling) {
 		if (Freewheel (nframes)) {
-			cerr << "Freewheeling returned non-zero!\n";
-			_freewheeling = false;
 			jack_set_freewheel (_jack, false);
 		}
 		return 0;
@@ -331,6 +330,23 @@ AudioEngine::process_callback (nframes_t nframes)
 		last_monitor_check = next_processed_frames;
 	}
 
+	if (session->silent()) {
+
+		boost::shared_ptr<Ports> p = ports.reader();
+
+		for (Ports::iterator i = p->begin(); i != p->end(); ++i) {
+			
+			Port *port = (*i);
+			
+			if (port->sends_output()) {
+				Sample *buf = port->get_buffer(nframes);
+				memset (buf, 0, sizeof(Sample) * nframes);
+				// this should work but doesn't
+				//port->silence(0, nframes);  //need to implement declicking fade
+			}
+		}
+	}
+
 	_processed_frames = next_processed_frames;
 	return 0;
 }
@@ -401,8 +417,9 @@ void
 AudioEngine::start_metering_thread ()
 {
 	if (m_meter_thread == 0) {
+		g_atomic_int_set (&m_meter_exit, 0);
 		m_meter_thread = Glib::Thread::create (sigc::mem_fun(this, &AudioEngine::meter_thread),
-				500000, true, true, Glib::THREAD_PRIORITY_NORMAL);
+						       500000, true, true, Glib::THREAD_PRIORITY_NORMAL);
 	}
 }
 
@@ -463,6 +480,26 @@ AudioEngine::remove_session ()
 	remove_all_ports ();
 }
 
+void
+AudioEngine::port_registration_failure (const std::string& portname)
+{
+	string full_portname = jack_client_name;
+	full_portname += ':';
+	full_portname += portname;
+	
+	
+	jack_port_t* p = jack_port_by_name (_jack, full_portname.c_str());
+	string reason;
+	
+	if (p) {
+		reason = _("a port with this name already exists: check for duplicated track/bus names");
+	} else {
+		reason = _("unknown error");
+	}
+	
+	throw PortRegistrationFailure (string_compose (_("AudioEngine: cannot register port \"%1\": %2"), portname, reason).c_str());
+}	
+
 Port *
 AudioEngine::register_input_port (DataType type, const string& portname)
 {
@@ -491,7 +528,7 @@ AudioEngine::register_input_port (DataType type, const string& portname)
 		return newport;
 
 	} else {
-		throw PortRegistrationFailure();
+		port_registration_failure (portname);
 	}
 
 	return 0;
@@ -529,7 +566,7 @@ AudioEngine::register_output_port (DataType type, const string& portname)
 		return newport;
 
 	} else {
-		throw PortRegistrationFailure ();
+		port_registration_failure (portname);
 	}
 
 	return 0;
@@ -759,15 +796,39 @@ void
 AudioEngine::halted (void *arg)
 {
 	AudioEngine* ae = static_cast<AudioEngine *> (arg);
+	bool was_running = ae->_running;
+
+	ae->stop_metering_thread ();
 
 	ae->_running = false;
 	ae->_buffer_size = 0;
 	ae->_frame_rate = 0;
 	ae->_jack = 0;
 
-	ae->Halted(); /* EMIT SIGNAL */
+	if (was_running) {
+		ae->Halted(); /* EMIT SIGNAL */
+	}
 }
 
+bool
+AudioEngine::can_request_hardware_monitoring () 
+{
+	const char ** ports;
+
+	if (!_jack) {
+		return 0;
+	}
+
+	if ((ports = jack_get_ports (_jack, NULL, JACK_DEFAULT_AUDIO_TYPE, JackPortCanMonitor)) == 0) {
+		return false;
+	}
+
+	free (ports);
+
+	return true;
+}
+
+
 uint32_t
 AudioEngine::n_physical_audio_outputs () const
 {
@@ -782,10 +843,9 @@ AudioEngine::n_physical_audio_outputs () const
 		return 0;
 	}
 
-	if (ports) {
-		for (i = 0; ports[i]; ++i);
-		free (ports);
-	}
+	for (i = 0; ports[i]; ++i);
+	free (ports);
+
 	return i;
 }
 
@@ -962,11 +1022,18 @@ AudioEngine::freewheel (bool onoff)
 {
 	if (_jack) {
 
-		if (onoff) {
-			_freewheel_thread_registered = false;
-		}
+		if (onoff != _freewheeling) {
 
-		return jack_set_freewheel (_jack, onoff);
+			if (onoff) {
+				_freewheel_thread_registered = false;
+			}
+
+			return jack_set_freewheel (_jack, onoff);
+
+		} else {
+			/* already doing what has been asked for */
+			return 0;
+		}
 
 	} else {
 		return -1;
@@ -1022,9 +1089,8 @@ AudioEngine::connect_to_jack (string client_name)
 	const char *server_name = NULL;
 
 	jack_client_name = client_name; /* might be reset below */
-
 	_jack = jack_client_open (jack_client_name.c_str(), options, &status, server_name);
-	
+
 	if (_jack == NULL) {
 		/* just return without an error message. something else will take care of it */
 		return -1;
@@ -1068,6 +1134,7 @@ AudioEngine::disconnect_from_jack ()
 	_frame_rate = 0;
 
 	if (_running) {
+		stop_metering_thread ();
 		_running = false;
 		Stopped(); /* EMIT SIGNAL */
 	}
@@ -1160,8 +1227,16 @@ AudioEngine::reconnect_to_jack ()
 	for (PortConnections::iterator i = port_connections.begin(); i != port_connections.end(); ++i) {
 		
 		int err;
+		jack_client_t* j = _jack;
+
+		/* JACK could have zombified us. */
+
+		if (!j) {
+			error << _("Disconnected from JACK while reconnecting. You should quit Ardour now.") << endmsg;
+			return -1;
+		}
 		
-		if ((err = jack_connect (_jack, (*i).first.c_str(), (*i).second.c_str())) != 0) {
+		if ((err = jack_connect (j, (*i).first.c_str(), (*i).second.c_str())) != 0) {
 			if (err != EEXIST) {
 				error << string_compose (_("could not reconnect %1 and %2 (err = %3)"),
 						  (*i).first, (*i).second, err)
@@ -1172,6 +1247,8 @@ AudioEngine::reconnect_to_jack ()
 
 	Running (); /* EMIT SIGNAL*/
 
+	start_metering_thread ();
+
 	return 0;
 }
 
