@@ -23,6 +23,7 @@
 
 #include <pbd/failed_constructor.h>
 #include <pbd/xml++.h>
+#include <pbd/stacktrace.h>
 
 #include <ardour/insert.h>
 #include <ardour/plugin.h>
@@ -79,11 +80,6 @@ PluginInsert::PluginInsert (Session& s, boost::shared_ptr<Plugin> plug, Placemen
 	
 	init ();
 
-	if (_plugins[0]->fixed_io()) {
-		Glib::Mutex::Lock em (_session.engine().process_lock());
-		IO::MoreOutputs (output_streams ());
-	}
-
 	RedirectCreated (this); /* EMIT SIGNAL */
 }
 
@@ -94,14 +90,7 @@ PluginInsert::PluginInsert (Session& s, const XMLNode& node)
 		throw failed_constructor();
 	}
 
-	set_automatable ();
-
 	_plugins[0]->ParameterChanged.connect (mem_fun (*this, &PluginInsert::parameter_changed));
-
-	if (_plugins[0]->fixed_io()) {
-		Glib::Mutex::Lock em (_session.engine().process_lock());
-		IO::MoreOutputs (output_streams());
-	}
 }
 
 PluginInsert::PluginInsert (const PluginInsert& other)
@@ -158,8 +147,6 @@ void
 PluginInsert::init ()
 {
 	set_automatable ();
-
-	set<uint32_t>::iterator s;
 }
 
 PluginInsert::~PluginInsert ()
@@ -199,6 +186,7 @@ uint32_t
 PluginInsert::input_streams() const
 {
 	int32_t in = _plugins[0]->get_info()->n_inputs;
+
 	if (in < 0) {
 		return _plugins[0]->input_streams ();
 	} else {
@@ -231,6 +219,10 @@ PluginInsert::is_generator() const
 void
 PluginInsert::set_automatable ()
 {
+	/* fill the parameter automation list with null AutomationLists */
+
+	parameter_automation.assign (_plugins.front()->parameter_count(), (AutomationList*) 0);
+
 	set<uint32_t> a;
 	
 	a = _plugins.front()->automatable ();
@@ -291,21 +283,21 @@ PluginInsert::connect_and_run (vector<Sample*>& bufs, uint32_t nbufs, nframes_t
 
 	if (with_auto) {
 
-		map<uint32_t,AutomationList*>::iterator li;
+		vector<AutomationList*>::iterator li;
 		uint32_t n;
 		
 		for (n = 0, li = parameter_automation.begin(); li != parameter_automation.end(); ++li, ++n) {
 			
-			AutomationList& alist (*((*li).second));
+			AutomationList* alist = *li;
 
-			if (alist.automation_playback()) {
+			if (alist && alist->automation_playback()) {
 				bool valid;
 
-				float val = alist.rt_safe_eval (now, valid);				
+				float val = alist->rt_safe_eval (now, valid);				
 
 				if (valid) {
 					/* set the first plugin, the others will be set via signals */
-					_plugins[0]->set_parameter ((*li).first, val);
+					_plugins[0]->set_parameter (n, val);
 				}
 
 			} 
@@ -320,16 +312,18 @@ PluginInsert::connect_and_run (vector<Sample*>& bufs, uint32_t nbufs, nframes_t
 }
 
 void
-PluginInsert::automation_snapshot (nframes_t now)
+PluginInsert::automation_snapshot (nframes_t now, bool force)
 {
-	map<uint32_t,AutomationList*>::iterator li;
-	
-	for (li = parameter_automation.begin(); li != parameter_automation.end(); ++li) {
+	vector<AutomationList*>::iterator li;
+	uint32_t n;
+
+	for (n = 0, li = parameter_automation.begin(); li != parameter_automation.end(); ++li, ++n) {
 		
-		AutomationList *alist = ((*li).second);
-		if (alist != 0 && alist->automation_write ()) {
+		AutomationList *alist = *li;
+
+		if (alist && alist->automation_write ()) {
 			
-			float val = _plugins[0]->get_parameter ((*li).first);
+			float val = _plugins[0]->get_parameter (n);
 			alist->rt_add (now, val);
 			last_automation_snapshot = now;
 		}
@@ -339,14 +333,18 @@ PluginInsert::automation_snapshot (nframes_t now)
 void
 PluginInsert::transport_stopped (nframes_t now)
 {
-	map<uint32_t,AutomationList*>::iterator li;
+	vector<AutomationList*>::iterator li;
+	uint32_t n;
 
-	for (li = parameter_automation.begin(); li != parameter_automation.end(); ++li) {
-		AutomationList& alist (*(li->second));
-		alist.reposition_for_rt_add (now);
+	for (n = 0, li = parameter_automation.begin(); li != parameter_automation.end(); ++li, ++n) {
 
-		if (alist.automation_state() != Off) {
-			_plugins[0]->set_parameter (li->first, alist.eval (now));
+		AutomationList* alist = *li;
+
+		if (alist) {
+			alist->reposition_for_rt_add (now);
+			if (alist->automation_state() != Off) {
+				_plugins[0]->set_parameter (n, alist->eval (now));
+			}
 		}
 	}
 }
@@ -356,12 +354,11 @@ PluginInsert::silence (nframes_t nframes, nframes_t offset)
 {
 	int32_t in_index = 0;
 	int32_t out_index = 0;
-
-	uint32_t n;
+	int32_t n;
 
 	if (active()) {
 		for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
-			n = (*i) -> get_info()->n_inputs;
+			n = input_streams();
 			(*i)->connect_and_run (_session.get_silent_buffers (n), n, in_index, out_index, nframes, offset);
 		}
 	}
@@ -377,21 +374,20 @@ PluginInsert::run (vector<Sample *>& bufs, uint32_t nbufs, nframes_t nframes, nf
 		} else {
 			connect_and_run (bufs, nbufs, nframes, offset, false);
 		}
+
 	} else {
-		uint32_t in = _plugins[0]->get_info()->n_inputs;
-		uint32_t out = _plugins[0]->get_info()->n_outputs;
 
-		if (in < 0 || out < 0) {
+		uint32_t in = input_streams ();
+		uint32_t out = output_streams ();
+
+		if (out > in) {
 			
-		} else {
-
-			if (out > in) {
-				
-				/* not active, but something has make up for any channel count increase */
-				
-				for (uint32_t n = out - in; n < out; ++n) {
-					memcpy (bufs[n], bufs[in - 1], sizeof (Sample) * nframes);
-				}
+			/* not active, but something has make up for any channel count increase,
+			   so copy the last buffer to the extras.
+			*/
+			
+			for (uint32_t n = out - in; n < out && n < nbufs; ++n) {
+				memcpy (bufs[n], bufs[in - 1], sizeof (Sample) * nframes);
 			}
 		}
 	}
@@ -552,68 +548,29 @@ PluginInsert::plugin_factory (boost::shared_ptr<Plugin> other)
 	return boost::shared_ptr<Plugin> ((Plugin*) 0);
 }
 
-int32_t
-PluginInsert::compute_output_streams (int32_t cnt) const
-{
-	int32_t outputs;
-
-	if ((outputs = _plugins[0]->get_info()->n_outputs) < 0) {
-		// have to ask the plugin itself, because it has flexible I/O
-		return _plugins[0]->compute_output_streams (cnt);
-	}
-
-	return outputs * cnt;
-}
-
 int32_t
 PluginInsert::configure_io (int32_t magic, int32_t in, int32_t out)
 {
-	return set_count (magic);
+	int32_t ret;
+
+	if ((ret = set_count (magic)) < 0) {
+		return ret;
+	}
+
+	/* if we're running replicated plugins, each plugin has
+	   the same i/o configuration and we may need to announce how many
+	   output streams there are.
+
+	   if we running a single plugin, we need to configure it.
+	*/
+
+	return _plugins[0]->configure_io (in, out);
 }
 
 int32_t 
-PluginInsert::can_support_input_configuration (int32_t in) const
+PluginInsert::can_do (int32_t in, int32_t& out)
 {
-	int32_t outputs = _plugins[0]->get_info()->n_outputs;
-	int32_t inputs = _plugins[0]->get_info()->n_inputs;
-
-	if (outputs < 0 || inputs < 0) {
-		/* have to ask the plugin because its got reconfigurable I/O
-		 */
-		return _plugins[0]->can_support_input_configuration (in);
-	}
-
-	if (inputs == 0) {
-
-		/* instrument plugin, always legal, but it throws
-		   away any existing active streams.
-		*/
-
-		return 1;
-	}
-
-	if (outputs == 1 && inputs == 1) {
-		/* mono plugin, replicate as needed */
-		return in;
-	}
-
-	if (inputs == in) {
-		/* exact match */
-		return 1;
-	}
-
-	if ((inputs < in) && (inputs % in == 0)) {
-
-		/* number of inputs is a factor of the requested input
-		   configuration, so we can replicate.
-		*/
-
-		return in/inputs;
-	}
-
-	/* sorry */
-
-	return -1;
+	return _plugins[0]->can_do (in, out);
 }
 
 XMLNode&
@@ -741,7 +698,7 @@ PluginInsert::set_state(const XMLNode& node)
 			_plugins.push_back (plugin_factory (plugin));
 		}
 	}
-	
+
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 		if ((*niter)->name() == plugin->state_node_name()) {
 			for (vector<boost::shared_ptr<Plugin> >::iterator i = _plugins.begin(); i != _plugins.end(); ++i) {
@@ -792,6 +749,8 @@ PluginInsert::set_state(const XMLNode& node)
 		break;
 	}
 		
+	set_automatable ();
+	
 	/* look for port automation node */
 	
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
@@ -926,15 +885,6 @@ PortInsert::PortInsert (const PortInsert& other)
 void
 PortInsert::init ()
 {
-	if (add_input_port ("", this)) {
-		error << _("PortInsert: cannot add input port") << endmsg;
-		throw failed_constructor();
-	}
-	
-	if (add_output_port ("", this)) {
-		error << _("PortInsert: cannot add output port") << endmsg;
-		throw failed_constructor();
-	}
 }
 
 PortInsert::PortInsert (Session& s, const XMLNode& node)
@@ -1061,13 +1011,14 @@ PortInsert::latency()
 }
 
 int32_t
-PortInsert::can_support_input_configuration (int32_t in) const
+PortInsert::can_do (int32_t in, int32_t& out) 
 {
 	if (input_maximum() == -1 && output_maximum() == -1) {
 
 		/* not configured yet */
 
-		return 1; /* we can support anything the first time we're asked */
+		out = in;
+		return 1;
 
 	} else {
 
@@ -1076,6 +1027,7 @@ PortInsert::can_support_input_configuration (int32_t in) const
 		*/
 
 		if (output_maximum() == in) {
+			out = in;
 			return 1;
 		} 
 	}
@@ -1090,8 +1042,12 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 	   the last request config. or something like that.
 	*/
 
+	set_output_maximum (in);
+	set_output_minimum (in);
+	set_input_maximum (out);
+	set_input_minimum (out);
 
-	/* this is a bit odd: 
+	/* this can be momentarily confusing: 
 
 	   the number of inputs we are required to handle corresponds 
 	   to the number of output ports we need.
@@ -1100,11 +1056,6 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 	   to the number of input ports we need.
 	*/
 
-	set_output_maximum (in);
-	set_output_minimum (in);
-	set_input_maximum (out);
-	set_input_minimum (out);
-
 	if (in < 0) {
 		in = n_outputs ();
 	} 
@@ -1116,13 +1067,6 @@ PortInsert::configure_io (int32_t ignored_magic, int32_t in, int32_t out)
 	return ensure_io (out, in, false, this);
 }
 
-int32_t
-PortInsert::compute_output_streams (int32_t cnt) const
-{
-	/* puzzling, eh? think about it ... */
-	return n_inputs ();
-}
-
 uint32_t
 PortInsert::output_streams() const
 {
