@@ -25,6 +25,7 @@
 
 #include <sigc++/bind.h>
 
+#include <glibmm.h>
 #include <glibmm/thread.h>
 
 #include <pbd/xml++.h>
@@ -32,6 +33,7 @@
 
 #include <ardour/audioengine.h>
 #include <ardour/io.h>
+#include <ardour/route.h>
 #include <ardour/port.h>
 #include <ardour/connection.h>
 #include <ardour/session.h>
@@ -113,6 +115,7 @@ IO::IO (Session& s, string name,
 	  _output_maximum (output_max)
 {
 	_panner = new Panner (name, _session);
+	_active = true;
 	_gain = 1.0;
 	_desired_gain = 1.0;
 	_input_connection = 0;
@@ -148,6 +151,7 @@ IO::IO (Session& s, const XMLNode& node, DataType dt)
 	  _gain_automation_curve (0, 0, 0) // all reset in set_state()
 {
 	_panner = 0;
+	_active = true;
 	deferred_state = 0;
 	no_panner_reset = false;
 	_desired_gain = 1.0;
@@ -179,12 +183,16 @@ IO::~IO ()
 	Glib::Mutex::Lock lm (io_lock);
 	vector<Port *>::iterator i;
 
-	for (i = _inputs.begin(); i != _inputs.end(); ++i) {
-		_session.engine().unregister_port (*i);
-	}
-
-	for (i = _outputs.begin(); i != _outputs.end(); ++i) {
-		_session.engine().unregister_port (*i);
+	{
+		BLOCK_PROCESS_CALLBACK ();
+		
+		for (i = _inputs.begin(); i != _inputs.end(); ++i) {
+			_session.engine().unregister_port (*i);
+		}
+		
+		for (i = _outputs.begin(); i != _outputs.end(); ++i) {
+			_session.engine().unregister_port (*i);
+		}
 	}
 
 	m_meter_connection.disconnect();
@@ -830,7 +838,6 @@ int
 IO::add_output_port (string destination, void* src, DataType type)
 {
 	Port* our_port;
-	char name[64];
 
 	if (type == DataType::NIL)
 		type = _default_type;
@@ -848,15 +855,10 @@ IO::add_output_port (string destination, void* src, DataType type)
 		
 			/* Create a new output port */
 			
-			// FIXME: naming scheme for differently typed ports?
-			if (_output_maximum == 1) {
-				snprintf (name, sizeof (name), _("%s/out"), _name.c_str());
-			} else {
-				snprintf (name, sizeof (name), _("%s/out %u"), _name.c_str(), find_output_port_hole());
-			}
+			string portname = build_legal_port_name (false);
 			
-			if ((our_port = _session.engine().register_output_port (type, name)) == 0) {
-				error << string_compose(_("IO: cannot register output port %1"), name) << endmsg;
+			if ((our_port = _session.engine().register_output_port (type, portname)) == 0) {
+				error << string_compose(_("IO: cannot register output port %1"), portname) << endmsg;
 				return -1;
 			}
 			
@@ -944,7 +946,6 @@ int
 IO::add_input_port (string source, void* src, DataType type)
 {
 	Port* our_port;
-	char name[64];
 	
 	if (type == DataType::NIL)
 		type = _default_type;
@@ -954,22 +955,17 @@ IO::add_input_port (string source, void* src, DataType type)
 		
 		{ 
 			Glib::Mutex::Lock lm (io_lock);
-			
+
 			if (_input_maximum >= 0 && (int) _ninputs == _input_maximum) {
 				return -1;
 			}
 
 			/* Create a new input port */
 			
-			// FIXME: naming scheme for differently typed ports?
-			if (_input_maximum == 1) {
-				snprintf (name, sizeof (name), _("%s/in"), _name.c_str());
-			} else {
-				snprintf (name, sizeof (name), _("%s/in %u"), _name.c_str(), find_input_port_hole());
-			}
-			
-			if ((our_port = _session.engine().register_input_port (type, name)) == 0) {
-				error << string_compose(_("IO: cannot register input port %1"), name) << endmsg;
+			string portname = build_legal_port_name (true);
+
+			if ((our_port = _session.engine().register_input_port (type, portname)) == 0) {
+				error << string_compose(_("IO: cannot register input port %1"), portname) << endmsg;
 				return -1;
 			}
 			
@@ -1061,21 +1057,14 @@ IO::ensure_inputs_locked (uint32_t n, bool clear, void* src)
 		
 	while (_ninputs < n) {
 		
-		char buf[64];
-		
 		/* Create a new input port (of the default type) */
 		
-		if (_input_maximum == 1) {
-			snprintf (buf, sizeof (buf), _("%s/in"), _name.c_str());
-		}
-		else {
-			snprintf (buf, sizeof (buf), _("%s/in %u"), _name.c_str(), find_input_port_hole());
-		}
+		string portname = build_legal_port_name (true);
 		
 		try {
 			
-			if ((input_port = _session.engine().register_input_port (_default_type, buf)) == 0) {
-				error << string_compose(_("IO: cannot register input port %1"), buf) << endmsg;
+			if ((input_port = _session.engine().register_input_port (_default_type, portname)) == 0) {
+				error << string_compose(_("IO: cannot register input port %1"), portname) << endmsg;
 				return -1;
 			}
 		}
@@ -1084,7 +1073,7 @@ IO::ensure_inputs_locked (uint32_t n, bool clear, void* src)
 			setup_peak_meters ();
 			reset_panner ();
 			/* pass it on */
-			throw AudioEngine::PortRegistrationFailure();
+			throw;
 		}
 		
 		_inputs.push_back (input_port);
@@ -1167,20 +1156,13 @@ IO::ensure_io (uint32_t nin, uint32_t nout, bool clear, void* src)
 		
 		while (_ninputs < nin) {
 			
-			char buf[64];
-
 			/* Create a new input port */
 			
-			if (_input_maximum == 1) {
-				snprintf (buf, sizeof (buf), _("%s/in"), _name.c_str());
-			}
-			else {
-				snprintf (buf, sizeof (buf), _("%s/in %u"), _name.c_str(), find_input_port_hole());
-			}
-			
+			string portname = build_legal_port_name (true);
+
 			try {
-				if ((port = _session.engine().register_input_port (_default_type, buf)) == 0) {
-					error << string_compose(_("IO: cannot register input port %1"), buf) << endmsg;
+				if ((port = _session.engine().register_input_port (_default_type, portname)) == 0) {
+					error << string_compose(_("IO: cannot register input port %1"), portname) << endmsg;
 					return -1;
 				}
 			}
@@ -1189,7 +1171,7 @@ IO::ensure_io (uint32_t nin, uint32_t nout, bool clear, void* src)
 				setup_peak_meters ();
 				reset_panner ();
 				/* pass it on */
-				throw AudioEngine::PortRegistrationFailure();
+				throw;
 			}
 		
 			_inputs.push_back (port);
@@ -1201,19 +1183,11 @@ IO::ensure_io (uint32_t nin, uint32_t nout, bool clear, void* src)
 		
 		while (_noutputs < nout) {
 			
-			char buf[64];
-			
-			/* Create a new output port */
-			
-			if (_output_maximum == 1) {
-				snprintf (buf, sizeof (buf), _("%s/out"), _name.c_str());
-			} else {
-				snprintf (buf, sizeof (buf), _("%s/out %u"), _name.c_str(), find_output_port_hole());
-			}
-			
+			string portname = build_legal_port_name (false);
+
 			try { 
-				if ((port = _session.engine().register_output_port (_default_type, buf)) == 0) {
-					error << string_compose(_("IO: cannot register output port %1"), buf) << endmsg;
+				if ((port = _session.engine().register_output_port (_default_type, portname)) == 0) {
+					error << string_compose(_("IO: cannot register output port %1"), portname) << endmsg;
 					return -1;
 				}
 			}
@@ -1222,7 +1196,7 @@ IO::ensure_io (uint32_t nin, uint32_t nout, bool clear, void* src)
 				setup_peak_meters ();
 				reset_panner ();
 				/* pass it on */
-				throw AudioEngine::PortRegistrationFailure ();
+				throw;
 			}
 		
 			_outputs.push_back (port);
@@ -1327,18 +1301,12 @@ IO::ensure_outputs_locked (uint32_t n, bool clear, void* src)
 	
 	while (_noutputs < n) {
 		
-		char buf[64];
-		
 		/* Create a new output port */
 		
-		if (_output_maximum == 1) {
-			snprintf (buf, sizeof (buf), _("%s/out"), _name.c_str());
-		} else {
-			snprintf (buf, sizeof (buf), _("%s/out %u"), _name.c_str(), find_output_port_hole());
-		}
+		string portname = build_legal_port_name (false);
 		
-		if ((output_port = _session.engine().register_output_port (_default_type, buf)) == 0) {
-			error << string_compose(_("IO: cannot register output port %1"), buf) << endmsg;
+		if ((output_port = _session.engine().register_output_port (_default_type, portname)) == 0) {
+			error << string_compose(_("IO: cannot register output port %1"), portname) << endmsg;
 			return -1;
 		}
 		
@@ -1465,6 +1433,7 @@ IO::state (bool full_state)
 	node->add_property("name", _name);
 	id().print (buf, sizeof (buf));
 	node->add_property("id", buf);
+	node->add_property("active", _active? "yes" : "no");
 
 	str = "";
 
@@ -1617,6 +1586,10 @@ IO::set_state (const XMLNode& node)
 		/* old school automation handling */
 	}
 
+	if ((prop = node.property (X_("active"))) != 0) {
+		set_active (prop->value() == "yes");
+	}
+
 	for (iter = node.children().begin(); iter != node.children().end(); ++iter) {
 
 		if ((*iter)->name() == "Panner") {
@@ -1697,16 +1670,12 @@ IO::load_automation (string path)
 	float version;
 	LocaleGuard lg (X_("POSIX"));
 
-	fullpath = _session.automation_dir();
-	fullpath += path;
+	fullpath = Glib::build_filename(_session.automation_dir(), path);
 
 	in.open (fullpath.c_str());
 
 	if (!in) {
-		fullpath = _session.automation_dir();
-		fullpath += _session.snap_name();
-		fullpath += '-';
-		fullpath += path;
+		fullpath = Glib::build_filename(_session.automation_dir(), _session.snap_name() + '-' + path);
 
 		in.open (fullpath.c_str());
 
@@ -1810,6 +1779,98 @@ IO::ports_became_legal ()
 	return ret;
 }
 
+Connection *
+IO::find_possible_connection(const string &desired_name, const string &default_name, const string &connection_type_name) 
+{
+	static const string digits = "0123456789";
+
+	Connection* c = _session.connection_by_name (desired_name);
+
+	if (!c) {
+		int connection_number, mask;
+		string possible_name;
+		bool stereo = false;
+		string::size_type last_non_digit_pos;
+
+		error << string_compose(_("Unknown connection \"%1\" listed for %2 of %3"), desired_name, connection_type_name, _name)
+		      << endmsg;
+
+		// find numeric suffix of desired name
+		connection_number = 0;
+		
+		last_non_digit_pos = desired_name.find_last_not_of(digits);
+
+		if (last_non_digit_pos != string::npos) {
+			stringstream s;
+			s << desired_name.substr(last_non_digit_pos);
+			s >> connection_number;
+		}
+	
+		// see if it's a stereo connection e.g. "in 3+4"
+
+		if (last_non_digit_pos > 1 && desired_name[last_non_digit_pos] == '+') {
+			int left_connection_number = 0;
+			string::size_type left_last_non_digit_pos;
+
+			left_last_non_digit_pos = desired_name.find_last_not_of(digits, last_non_digit_pos-1);
+
+			if (left_last_non_digit_pos != string::npos) {
+				stringstream s;
+				s << desired_name.substr(left_last_non_digit_pos, last_non_digit_pos-1);
+				s >> left_connection_number;
+
+				if (left_connection_number > 0 && left_connection_number + 1 == connection_number) {
+					connection_number--;
+					stereo = true;
+				}
+			}
+		}
+
+		// make 0-based
+		if (connection_number)
+			connection_number--;
+
+		// find highest set bit
+		mask = 1;
+		while ((mask <= connection_number) && (mask <<= 1));
+		
+		// "wrap" connection number into largest possible power of 2 
+		// that works...
+
+		while (mask) {
+
+			if (connection_number & mask) {
+				connection_number &= ~mask;
+				
+				stringstream s;
+				s << default_name << " " << connection_number + 1;
+
+				if (stereo) {
+					s << "+" << connection_number + 2;
+				}
+				
+				possible_name = s.str();
+
+				if ((c = _session.connection_by_name (possible_name)) != 0) {
+					break;
+				}
+			}
+			mask >>= 1;
+		}
+		if (c) {
+			info << string_compose (_("Connection %1 was not available - \"%2\" used instead"), desired_name, possible_name)
+			     << endmsg;
+		} else {
+			error << string_compose(_("No %1 connections available as a replacement"), connection_type_name)
+			      << endmsg;
+		}
+
+	}
+
+	return c;
+
+}
+
 int
 IO::create_ports (const XMLNode& node)
 {
@@ -1819,19 +1880,10 @@ IO::create_ports (const XMLNode& node)
 
 	if ((prop = node.property ("input-connection")) != 0) {
 
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection(prop->value(), _("in"), _("input"));
 		
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("in 1"))) == 0) {
-				error << _("No input connections available as a replacement")
-				      << endmsg;
-				return -1;
-			}  else {
-				info << string_compose (_("Connection %1 was not available - \"in 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		num_inputs = c->nports();
@@ -1842,19 +1894,10 @@ IO::create_ports (const XMLNode& node)
 	}
 	
 	if ((prop = node.property ("output-connection")) != 0) {
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection(prop->value(), _("out"), _("output"));
 
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for output of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("out 1"))) == 0) {
-				error << _("No output connections available as a replacement")
-				      << endmsg;
-				return -1;
-			}  else {
-				info << string_compose (_("Connection %1 was not available - \"out 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		num_outputs = c->nports ();
@@ -1885,19 +1928,10 @@ IO::make_connections (const XMLNode& node)
 	const XMLProperty* prop;
 
 	if ((prop = node.property ("input-connection")) != 0) {
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection (prop->value(), _("in"), _("input"));
 		
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for input of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("in 1"))) == 0) {
-				error << _("No input connections available as a replacement")
-				      << endmsg;
-				return -1;
-			} else {
-				info << string_compose (_("Connection %1 was not available - \"in 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		use_input_connection (*c, this);
@@ -1910,19 +1944,10 @@ IO::make_connections (const XMLNode& node)
 	}
 	
 	if ((prop = node.property ("output-connection")) != 0) {
-		Connection* c = _session.connection_by_name (prop->value());
+		Connection* c = find_possible_connection (prop->value(), _("out"), _("output"));
 		
 		if (c == 0) {
-			error << string_compose(_("Unknown connection \"%1\" listed for output of %2"), prop->value(), _name) << endmsg;
-
-			if ((c = _session.connection_by_name (_("out 1"))) == 0) {
-				error << _("No output connections available as a replacement")
-				      << endmsg;
-				return -1;
-			}  else {
-				info << string_compose (_("Connection %1 was not available - \"out 1\" used instead"), prop->value())
-				     << endmsg;
-			}
+			return -1;
 		} 
 
 		use_output_connection (*c, this);
@@ -2085,12 +2110,21 @@ IO::parse_gain_string (const string& str, vector<string>& ports)
 }
 
 int
-IO::set_name (string name, void* src)
+IO::set_name (string requested_name, void* src)
 {
-	if (name == _name) {
+	if (requested_name == _name) {
 		return 0;
 	}
 
+	string name;
+	Route *rt;
+	if ( (rt = dynamic_cast<Route *>(this))) {
+		name = Route::ensure_track_or_route_name(requested_name, _session);
+	} else {
+		name = requested_name;
+	}
+
+
 	/* replace all colons in the name. i wish we didn't have to do this */
 
 	if (replace_all (name, ":", "-")) {
@@ -2465,9 +2499,8 @@ IO::setup_peak_meters ()
 void
 IO::update_meters()
 {
-    Glib::Mutex::Lock guard (m_meter_signal_lock);
-    
-    Meter();
+	Glib::Mutex::Lock guard (m_meter_signal_lock);
+	Meter();
 }
 
 void
@@ -2623,18 +2656,14 @@ IO::end_pan_touch (uint32_t which)
 }
 
 void
-IO::automation_snapshot (nframes_t now)
+IO::automation_snapshot (nframes_t now, bool force)
 {
-	if (last_automation_snapshot > now || (now - last_automation_snapshot) > _automation_interval) {
-
-		if (gain_automation_recording()) {
-			_gain_automation_curve.rt_add (now, gain());
-		}
-		
-		_panner->snapshot (now);
-
-		last_automation_snapshot = now;
+	if (gain_automation_recording()) {
+		_gain_automation_curve.rt_add (now, gain());
 	}
+	
+	_panner->snapshot (now);
+	last_automation_snapshot = now;
 }
 
 void
@@ -2654,8 +2683,54 @@ IO::transport_stopped (nframes_t frame)
 	_panner->transport_stopped (frame);
 }
 
+string
+IO::build_legal_port_name (bool in)
+{
+	const int name_size = jack_port_name_size();
+	int limit;
+	char* suffix;
+	int maxports;
+
+	if (in) {
+		suffix = _("in");
+		maxports = _input_maximum;
+	} else {
+		suffix = _("out");
+		maxports = _output_maximum;
+	}
+	
+	if (maxports == 1) {
+		// allow space for the slash + the suffix
+		limit = name_size - _session.engine().client_name().length() - (strlen (suffix) + 1);
+		char buf[name_size+1];
+		snprintf (buf, name_size+1, ("%.*s/%s"), limit, _name.c_str(), suffix);
+		return string (buf);
+	} 
+	
+	// allow up to 4 digits for the output port number, plus the slash, suffix and extra space
+
+	limit = name_size - _session.engine().client_name().length() - (strlen (suffix) + 5);
+
+	char buf1[name_size+1];
+	char buf2[name_size+1];
+	
+	snprintf (buf1, name_size+1, ("%.*s/%s"), limit, _name.c_str(), suffix);
+	
+	int port_number;
+	
+	if (in) {
+		port_number = find_input_port_hole (buf1);
+	} else {
+		port_number = find_output_port_hole (buf1);
+	}
+	
+	snprintf (buf2, name_size+1, "%s %d", buf1, port_number);
+	
+	return string (buf2);
+}
+
 int32_t
-IO::find_input_port_hole ()
+IO::find_input_port_hole (const char* base)
 {
 	/* CALLER MUST HOLD IO LOCK */
 
@@ -2665,11 +2740,14 @@ IO::find_input_port_hole ()
 		return 1;
 	}
 
-	for (n = 1; n < UINT_MAX; ++n) {
+	/* we only allow up to 4 characters for the port number
+	 */
+
+	for (n = 1; n < 9999; ++n) {
 		char buf[jack_port_name_size()];
 		vector<Port*>::iterator i;
 
-		snprintf (buf, jack_port_name_size(), _("%s/in %u"), _name.c_str(), n);
+		snprintf (buf, jack_port_name_size(), _("%s %u"), base, n);
 
 		for (i = _inputs.begin(); i != _inputs.end(); ++i) {
 			if ((*i)->short_name() == buf) {
@@ -2685,7 +2763,7 @@ IO::find_input_port_hole ()
 }
 
 int32_t
-IO::find_output_port_hole ()
+IO::find_output_port_hole (const char* base)
 {
 	/* CALLER MUST HOLD IO LOCK */
 
@@ -2695,11 +2773,14 @@ IO::find_output_port_hole ()
 		return 1;
 	}
 
-	for (n = 1; n < UINT_MAX; ++n) {
+	/* we only allow up to 4 characters for the port number
+	 */
+
+	for (n = 1; n < 9999; ++n) {
 		char buf[jack_port_name_size()];
 		vector<Port*>::iterator i;
 
-		snprintf (buf, jack_port_name_size(), _("%s/out %u"), _name.c_str(), n);
+		snprintf (buf, jack_port_name_size(), _("%s %u"), base, n);
 
 		for (i = _outputs.begin(); i != _outputs.end(); ++i) {
 			if ((*i)->short_name() == buf) {
@@ -2714,3 +2795,11 @@ IO::find_output_port_hole ()
 	
 	return n;
 }
+
+void
+IO::set_active (bool yn)
+{
+	_active = yn; 
+	 active_changed(); /* EMIT SIGNAL */
+}
+
