@@ -21,6 +21,7 @@
 #include <cmath>
 #include <climits>
 #include <algorithm>
+#include <sstream>
 
 #include <sigc++/bind.h>
 #include <sigc++/class_slot.h>
@@ -28,10 +29,12 @@
 #include <glibmm/thread.h>
 #include <pbd/xml++.h>
 #include <pbd/stacktrace.h>
+#include <pbd/enumwriter.h>
 
 #include <ardour/region.h>
 #include <ardour/playlist.h>
 #include <ardour/session.h>
+#include <ardour/tempo.h>
 #include <ardour/region_factory.h>
 
 #include "i18n.h"
@@ -56,16 +59,23 @@ Region::Region (nframes_t start, nframes_t length, const string& name, layer_t l
 	_read_data_count = 0;
 	_frozen = 0;
 	pending_changed = Change (0);
-
+	valid_transients = false;
 	_name = name;
 	_start = start; 
 	_sync_position = _start;
 	_length = length; 
+	_last_length = length; 
+	_ancestral_start = start; 
+	_ancestral_length = length; 
+	_stretch = 1.0;
+	_shift = 1.0;
+	_last_position = 0; 
 	_position = 0; 
 	_layer = layer;
 	_read_data_count = 0;
 	_first_edit = EditChangesNothing;
 	_last_layer_op = 0;
+	_positional_lock_style = AudioTime;
 }
 
 Region::Region (boost::shared_ptr<const Region> other, nframes_t offset, nframes_t length, const string& name, layer_t layer, Flag flags)
@@ -75,6 +85,7 @@ Region::Region (boost::shared_ptr<const Region> other, nframes_t offset, nframes
 	_frozen = 0;
 	pending_changed = Change (0);
 	_read_data_count = 0;
+	valid_transients = false;
 
 	_start = other->_start + offset; 
 	if (other->_sync_position < offset) {
@@ -83,12 +94,19 @@ Region::Region (boost::shared_ptr<const Region> other, nframes_t offset, nframes
 		_sync_position = _start;
 	}
 	_length = length; 
+	_last_length = length; 
+	_ancestral_start = other->_ancestral_start;
+	_ancestral_length = other->_ancestral_length; 
+	_stretch = other->_stretch;
+	_shift = other->_shift;
 	_name = name;
+	_last_position = 0; 
 	_position = 0; 
 	_layer = layer; 
 	_flags = Flag (flags & ~(Locked|WholeFile|Hidden));
 	_first_edit = EditChangesNothing;
 	_last_layer_op = 0;
+	_positional_lock_style = AudioTime;
 }
 
 Region::Region (boost::shared_ptr<const Region> other)
@@ -98,6 +116,7 @@ Region::Region (boost::shared_ptr<const Region> other)
 	_frozen = 0;
 	pending_changed = Change (0);
 	_read_data_count = 0;
+	valid_transients = false;
 
 	_first_edit = EditChangesID;
 	other->_first_edit = EditChangesName;
@@ -111,26 +130,37 @@ Region::Region (boost::shared_ptr<const Region> other)
 	_start = other->_start;
 	_sync_position = other->_sync_position;
 	_length = other->_length; 
+	_last_length = other->_length; 
+	_ancestral_start = _start; 
+	_ancestral_length = _length; 
+	_stretch = other->_stretch;
+	_shift = other->_shift;
 	_name = other->_name;
+	_last_position = other->_position; 
 	_position = other->_position; 
 	_layer = other->_layer; 
 	_flags = Flag (other->_flags & ~Locked);
 	_last_layer_op = other->_last_layer_op;
+	_positional_lock_style = AudioTime;
 }
 
 Region::Region (const XMLNode& node)
 {
 	_frozen = 0;
 	pending_changed = Change (0);
+	valid_transients = false;
 	_read_data_count = 0;
 	_start = 0; 
 	_sync_position = _start;
 	_length = 0;
+	_last_length = 0;
 	_name = X_("error: XML did not reset this");
+	_last_position = 0; 
 	_position = 0; 
 	_layer = 0;
 	_flags = Flag (0);
 	_first_edit = EditChangesNothing;
+	_positional_lock_style = AudioTime;
 
 	if (set_state (node)) {
 		throw failed_constructor();
@@ -160,6 +190,7 @@ Region::set_name (string str)
 void
 Region::set_length (nframes_t len, void *src)
 {
+	//cerr << "Region::set_length() len = " << len << endl;
 	if (_flags & Locked) {
 		return;
 	}
@@ -178,12 +209,15 @@ Region::set_length (nframes_t len, void *src)
 			return;
 		}
 		
+
+		_last_length = _length;
 		_length = len;
 
 		_flags = Region::Flag (_flags & ~WholeFile);
 
 		first_edit ();
 		maybe_uncopy ();
+		invalidate_transients ();
 
 		if (!_frozen) {
 			recompute_at_end ();
@@ -256,9 +290,41 @@ Region::special_set_position (nframes_t pos)
 	   a way to store its "natural" or "captured" position.
 	*/
 
+	_position = _position;
 	_position = pos;
 }
 
+void
+Region::set_position_lock_style (PositionLockStyle ps)
+{
+	boost::shared_ptr<Playlist> pl (playlist());
+
+	if (!pl) {
+		return;
+	}
+
+	_positional_lock_style = ps;
+
+	if (_positional_lock_style == MusicTime) {
+		pl->session().tempo_map().bbt_time (_position, _bbt_time);
+	}
+	
+}
+
+void
+Region::update_position_after_tempo_map_change ()
+{
+	boost::shared_ptr<Playlist> pl (playlist());
+	
+	if (!pl || _positional_lock_style != MusicTime) {
+		return;
+	}
+
+	TempoMap& map (pl->session().tempo_map());
+	nframes_t pos = map.frame_time (_bbt_time);
+	set_position_internal (pos, false);
+}
+
 void
 Region::set_position (nframes_t pos, void *src)
 {
@@ -266,7 +332,14 @@ Region::set_position (nframes_t pos, void *src)
 		return;
 	}
 
+	set_position_internal (pos, true);
+}
+
+void
+Region::set_position_internal (nframes_t pos, bool allow_bbt_recompute)
+{
 	if (_position != pos) {
+		_last_position = _position;
 		_position = pos;
 
 		/* check that the new _position wouldn't make the current
@@ -276,8 +349,15 @@ Region::set_position (nframes_t pos, void *src)
 		*/
 
 		if (max_frames - _length < _position) {
+			_last_length = _length;
 			_length = max_frames - _position;
 		}
+
+		if (allow_bbt_recompute) {
+			recompute_position_from_lock_style ();
+		}
+
+		invalidate_transients ();
 	}
 
 	/* do this even if the position is the same. this helps out
@@ -295,6 +375,7 @@ Region::set_position_on_top (nframes_t pos, void *src)
 	}
 
 	if (_position != pos) {
+		_last_position = _position;
 		_position = pos;
 	}
 
@@ -312,7 +393,18 @@ Region::set_position_on_top (nframes_t pos, void *src)
 }
 
 void
-Region::nudge_position (long n, void *src)
+Region::recompute_position_from_lock_style ()
+{
+	if (_positional_lock_style == MusicTime) {
+		boost::shared_ptr<Playlist> pl (playlist());
+		if (pl) {
+			pl->session().tempo_map().bbt_time (_position, _bbt_time);
+		}
+	}
+}
+		
+void
+Region::nudge_position (nframes64_t n, void *src)
 {
 	if (_flags & Locked) {
 		return;
@@ -322,6 +414,8 @@ Region::nudge_position (long n, void *src)
 		return;
 	}
 	
+	_last_position = _position;
+
 	if (n > 0) {
 		if (_position > max_frames - n) {
 			_position = max_frames;
@@ -339,6 +433,15 @@ Region::nudge_position (long n, void *src)
 	send_change (PositionChanged);
 }
 
+void
+Region::set_ancestral_data (nframes64_t s, nframes64_t l, float st, float sh)
+{
+	_ancestral_length = l;
+	_ancestral_start = s;
+	_stretch = st;
+	_shift = sh;
+}
+
 void
 Region::set_start (nframes_t pos, void *src)
 {
@@ -359,6 +462,7 @@ Region::set_start (nframes_t pos, void *src)
 		_start = pos;
 		_flags = Region::Flag (_flags & ~WholeFile);
 		first_edit ();
+		invalidate_transients ();
 
 		send_change (StartChanged);
 	}
@@ -528,10 +632,16 @@ Region::trim_to_internal (nframes_t position, nframes_t length, void *src)
 		what_changed = Change (what_changed|StartChanged);
 	}
 	if (_length != length) {
+		if (!_frozen) {
+			_last_length = _length;
+		}
 		_length = length;
 		what_changed = Change (what_changed|LengthChanged);
 	}
 	if (_position != position) {
+		if (!_frozen) {
+			_last_position = _position;
+		}
 		_position = position;
 		what_changed = Change (what_changed|PositionChanged);
 	}
@@ -659,17 +769,19 @@ Region::adjust_to_sync (nframes_t pos)
 {
 	int sync_dir;
 	nframes_t offset = sync_offset (sync_dir);
+
+	// cerr << "adjusting pos = " << pos << " to sync at " << _sync_position << " offset = " << offset << " with dir = " << sync_dir << endl;
 	
 	if (sync_dir > 0) {
-		if (max_frames - pos > offset) {
-			pos += offset;
-		}
-	} else {
 		if (pos > offset) {
 			pos -= offset;
 		} else {
 			pos = 0;
 		}
+	} else {
+		if (max_frames - pos > offset) {
+			pos += offset;
+		}
 	}
 
 	return pos;
@@ -737,7 +849,7 @@ Region::state (bool full_state)
 {
 	XMLNode *node = new XMLNode ("Region");
 	char buf[64];
-	char* fe = NULL;
+	const char* fe = NULL;
 
 	_id.print (buf, sizeof (buf));
 	node->add_property ("id", buf);
@@ -748,6 +860,14 @@ Region::state (bool full_state)
 	node->add_property ("length", buf);
 	snprintf (buf, sizeof (buf), "%u", _position);
 	node->add_property ("position", buf);
+	snprintf (buf, sizeof (buf), "%" PRIi64, _ancestral_start);
+	node->add_property ("ancestral-start", buf);
+	snprintf (buf, sizeof (buf), "%" PRIi64, _ancestral_length);
+	node->add_property ("ancestral-length", buf);
+	snprintf (buf, sizeof (buf), "%.12g", _stretch);
+	node->add_property ("stretch", buf);
+	snprintf (buf, sizeof (buf), "%.12g", _shift);
+	node->add_property ("shift", buf);
 	
 	switch (_first_edit) {
 	case EditChangesNothing:
@@ -760,7 +880,6 @@ Region::state (bool full_state)
 		fe = X_("id");
 		break;
 	default: /* should be unreachable but makes g++ happy */
-		cerr << "Odd region property found\n";
 		fe = X_("nothing");
 		break;
 	}
@@ -774,6 +893,13 @@ Region::state (bool full_state)
 	snprintf (buf, sizeof (buf), "%" PRIu32, _sync_position);
 	node->add_property ("sync-position", buf);
 
+	if (_positional_lock_style != AudioTime) {
+		node->add_property ("positional-lock-style", enum_2_string (_positional_lock_style));
+		stringstream str;
+		str << _bbt_time;
+		node->add_property ("bbt-position", str.str());
+	}
+
 	return *node;
 }
 
@@ -815,9 +941,11 @@ Region::set_live_state (const XMLNode& node, Change& what_changed, bool send)
 		sscanf (prop->value().c_str(), "%" PRIu32, &val);
 		if (val != _length) {
 			what_changed = Change (what_changed|LengthChanged);
+			_last_length = _length;
 			_length = val;
 		}
 	} else {
+		_last_length = _length;
 		_length = 1;
 	}
 
@@ -825,9 +953,11 @@ Region::set_live_state (const XMLNode& node, Change& what_changed, bool send)
 		sscanf (prop->value().c_str(), "%" PRIu32, &val);
 		if (val != _position) {
 			what_changed = Change (what_changed|PositionChanged);
+			_last_position = _position;
 			_position = val;
 		}
 	} else {
+		_last_position = _position;
 		_position = 0;
 	}
 
@@ -852,8 +982,55 @@ Region::set_live_state (const XMLNode& node, Change& what_changed, bool send)
 		_sync_position = _start;
 	}
 
+	if ((prop = node.property ("positional-lock-style")) != 0) {
+		_positional_lock_style = PositionLockStyle (string_2_enum (prop->value(), _positional_lock_style));
+
+		if (_positional_lock_style == MusicTime) {
+			if ((prop = node.property ("bbt-position")) == 0) {
+				/* missing BBT info, revert to audio time locking */
+				_positional_lock_style = AudioTime;
+			} else {
+				if (sscanf (prop->value().c_str(), "%d|%d|%d", 
+					    &_bbt_time.bars,
+					    &_bbt_time.beats,
+					    &_bbt_time.ticks) != 3) {
+					_positional_lock_style = AudioTime;
+				}
+			}
+		}
+			
+	} else {
+		_positional_lock_style = AudioTime;
+	}
+
 	/* XXX FIRST EDIT !!! */
 	
+	/* these 3 properties never change as a result of any editing */
+
+	if ((prop = node.property ("ancestral-start")) != 0) {
+		_ancestral_start = atoi (prop->value());
+	} else {
+		_ancestral_start = _start;
+	}
+
+	if ((prop = node.property ("ancestral-length")) != 0) {
+		_ancestral_length = atoi (prop->value());
+	} else {
+		_ancestral_length = _length;
+	}
+
+	if ((prop = node.property ("stretch")) != 0) {
+		_stretch = atof (prop->value());
+	} else {
+		_stretch = 1.0;
+	}
+
+	if ((prop = node.property ("shift")) != 0) {
+		_shift = atof (prop->value());
+	} else {
+		_shift = 1.0;
+	}
+
 	/* note: derived classes set flags */
 
 	if (_extra_xml) {
@@ -906,6 +1083,8 @@ void
 Region::freeze ()
 {
 	_frozen++;
+	_last_length = _length;
+	_last_position = _position;
 }
 
 void
@@ -986,3 +1165,11 @@ Region::region_list_equivalent (boost::shared_ptr<const Region> other) const
 {
 	return size_equivalent (other) && source_equivalent (other) && _name == other->_name;
 }
+
+void
+Region::invalidate_transients ()
+{
+	valid_transients = false;
+	_transients.clear ();
+}
+
