@@ -24,6 +24,7 @@
 #include <sigc++/bind.h>
 #include <pbd/xml++.h>
 #include <pbd/enumwriter.h>
+#include <pbd/stacktrace.h>
 
 #include <ardour/timestamps.h>
 #include <ardour/buffer.h>
@@ -41,6 +42,7 @@
 #include <ardour/panner.h>
 #include <ardour/dB.h>
 #include <ardour/mix.h>
+#include <ardour/profile.h>
 
 #include "i18n.h"
 
@@ -79,7 +81,6 @@ Route::init ()
 	_phase_invert = false;
 	_denormal_protection = false;
 	order_keys[strdup (N_("signal"))] = order_key_cnt++;
-	_active = true;
 	_silent = false;
 	_meter_point = MeterPostFader;
 	_initial_delay = 0;
@@ -185,6 +186,20 @@ Route::sync_order_keys ()
 	}
 }
 
+string
+Route::ensure_track_or_route_name(string name, Session &session)
+{
+	string newname = name;
+
+	while (session.route_by_name (newname)!=NULL)
+	{
+		newname = bump_name_once (newname);
+	}
+
+	return newname;
+}
+
+
 void
 Route::inc_gain (gain_t fraction, void *src)
 {
@@ -847,10 +862,17 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 		boost::shared_ptr<PluginInsert> pi;
 		boost::shared_ptr<PortInsert> porti;
 
+		_redirects.push_back (redirect);
+
+		if (_reset_plugin_counts (err_streams)) {
+			_redirects.pop_back ();
+			_reset_plugin_counts (0); // it worked before we tried to add it ...
+			return -1;
+		}
+
 		uint32_t potential_max_streams = 0;
 
 		if ((pi = boost::dynamic_pointer_cast<PluginInsert>(redirect)) != 0) {
-			pi->set_count (1);
 
 			if (pi->input_streams() == 0) {
 				/* instrument plugin */
@@ -887,14 +909,6 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 			_max_peak_power.push_back(-INFINITY);
 		}
 
-		_redirects.push_back (redirect);
-
-		if (_reset_plugin_counts (err_streams)) {
-			_redirects.pop_back ();
-			_reset_plugin_counts (0); // it worked before we tried to add it ...
-			return -1;
-		}
-
 		redirect->activate ();
 		redirect->active_changed.connect (mem_fun (*this, &Route::redirect_active_proxy));
 	}
@@ -903,8 +917,8 @@ Route::add_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_t*
 		reset_panner ();
 	}
 
-
 	redirects_changed (src); /* EMIT SIGNAL */
+
 	return 0;
 }
 
@@ -1078,19 +1092,18 @@ Route::remove_redirect (boost::shared_ptr<Redirect> redirect, void *src, uint32_
 			return -1;
 		}
 
-		bool foo = false;
+		_have_internal_generator = false;
 
 		for (i = _redirects.begin(); i != _redirects.end(); ++i) {
 			boost::shared_ptr<PluginInsert> pi;
 			
 			if ((pi = boost::dynamic_pointer_cast<PluginInsert>(*i)) != 0) {
 				if (pi->is_generator()) {
-					foo = true;
+					_have_internal_generator = true;
+					break;
 				}
 			}
 		}
-
-		_have_internal_generator = foo;
 	}
 
 	if (old_rmo != redirect_max_outs) {
@@ -1115,16 +1128,19 @@ int
 Route::_reset_plugin_counts (uint32_t* err_streams)
 {
 	RedirectList::iterator r;
-	uint32_t i_cnt;
-	uint32_t s_cnt;
+	uint32_t insert_cnt = 0;
+	uint32_t send_cnt = 0;
 	map<Placement,list<InsertCount> > insert_map;
-	nframes_t initial_streams;
+	RedirectList::iterator prev;
+	int32_t initial_streams = n_inputs ();;
+	int32_t previous_initial_streams = n_inputs ();
+	int ret = -1;
 
 	redirect_max_outs = 0;
-	i_cnt = 0;
-	s_cnt = 0;
 
-	/* divide inserts up by placement so we get the signal flow
+	/* Step 1: build a map that links each insert to an in/out channel count 
+
+	   Divide inserts up by placement so we get the signal flow
 	   properly modelled. we need to do this because the _redirects
 	   list is not sorted by placement, and because other reasons may 
 	   exist now or in the future for this separate treatment.
@@ -1134,14 +1150,8 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 
 		boost::shared_ptr<Insert> insert;
 
-		/* do this here in case we bomb out before we get to the end of
-		   this function.
-		*/
-
-		redirect_max_outs = max ((*r)->output_streams (), redirect_max_outs);
-
 		if ((insert = boost::dynamic_pointer_cast<Insert>(*r)) != 0) {
-			++i_cnt;
+			++insert_cnt;
 			insert_map[insert->placement()].push_back (InsertCount (insert));
 
 			/* reset plugin counts back to one for now so
@@ -1156,15 +1166,16 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 			}
 
 		} else if (boost::dynamic_pointer_cast<Send> (*r) != 0) {
-			++s_cnt;
+			++send_cnt;
 		}
 	}
 	
-	if (i_cnt == 0) {
-		if (s_cnt) {
+	if (insert_cnt == 0) {
+		if (send_cnt) {
 			goto recompute;
 		} else {
-			return 0;
+			ret = 0;
+			goto streamcount;
 		}
 	}
 
@@ -1175,22 +1186,34 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	/* A: PreFader */
 	
 	if (check_some_plugin_counts (insert_map[PreFader], n_inputs (), err_streams)) {
-		return -1;
+		goto streamcount;
 	}
 
 	/* figure out the streams that will feed into PreFader */
 
 	if (!insert_map[PreFader].empty()) {
-		InsertCount& ic (insert_map[PreFader].back());
-		initial_streams = ic.insert->compute_output_streams (ic.cnt);
-	} else {
-		initial_streams = n_inputs ();
+		previous_initial_streams = n_inputs ();
+		for (list<InsertCount>::iterator i = insert_map[PreFader].begin(); i != insert_map[PreFader].end(); i++) {
+			if (i->insert->can_do (previous_initial_streams, initial_streams) < 0) {
+				goto streamcount;
+			}
+			previous_initial_streams = initial_streams;
+		}
 	}
 
 	/* B: PostFader */
 
 	if (check_some_plugin_counts (insert_map[PostFader], initial_streams, err_streams)) {
-		return -1;
+		goto streamcount;
+	}
+
+	if (!insert_map[PostFader].empty()) {
+		for (list<InsertCount>::iterator i = insert_map[PostFader].begin(); i != insert_map[PostFader].end(); i++) {
+			if (i->insert->can_do (previous_initial_streams, initial_streams) < 0) {
+				goto streamcount;
+			}
+			previous_initial_streams = initial_streams;
+		}
 	}
 
 	/* OK, everything can be set up correctly, so lets do it */
@@ -1203,7 +1226,7 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
   recompute:
 
 	redirect_max_outs = 0;
-	RedirectList::iterator prev = _redirects.end();
+	prev = _redirects.end();
 
 	for (r = _redirects.begin(); r != _redirects.end(); prev = r, ++r) {
 		boost::shared_ptr<Send> s;
@@ -1227,8 +1250,13 @@ Route::_reset_plugin_counts (uint32_t* err_streams)
 	}
 
 	/* we're done */
-
 	return 0;
+
+  streamcount:
+	for (r = _redirects.begin(); r != _redirects.end(); ++r) {
+		redirect_max_outs = max ((*r)->output_streams (), redirect_max_outs);
+	}
+	return ret;
 }				   
 
 int32_t
@@ -1254,8 +1282,8 @@ Route::check_some_plugin_counts (list<InsertCount>& iclist, int32_t required_inp
 	list<InsertCount>::iterator i;
 	
 	for (i = iclist.begin(); i != iclist.end(); ++i) {
-
-		if (((*i).cnt = (*i).insert->can_support_input_configuration (required_inputs)) < 0) {
+		
+		if (((*i).cnt = (*i).insert->can_do (required_inputs, (*i).out)) < 0) {
 			if (err_streams) {
 				*err_streams = required_inputs;
 			}
@@ -1263,8 +1291,6 @@ Route::check_some_plugin_counts (list<InsertCount>& iclist, int32_t required_inp
 		}
 		
 		(*i).in = required_inputs;
-		(*i).out = (*i).insert->compute_output_streams ((*i).cnt);
-
 		required_inputs = (*i).out;
 	}
 
@@ -1449,7 +1475,6 @@ Route::state(bool full_state)
 	
 	node->add_property("default-type", _default_type.to_string());
 
-	node->add_property("active", _active?"yes":"no");
 	node->add_property("muted", _muted?"yes":"no");
 	node->add_property("soloed", _soloed?"yes":"no");
 	node->add_property("phase-invert", _phase_invert?"yes":"no");
@@ -1560,22 +1585,30 @@ Route::add_redirect_from_xml (const XMLNode& node)
 			if ((prop = node.property ("type")) != 0) {
 
 				boost::shared_ptr<Insert> insert;
+				bool have_insert = false;
 
-				if (prop->value() == "ladspa" || prop->value() == "Ladspa" || prop->value() == "vst") {
-
+				if (prop->value() == "ladspa" || prop->value() == "Ladspa" || 
+				    prop->value() == "lv2" ||
+				    prop->value() == "vst" ||
+				    prop->value() == "audiounit") {
+					
 					insert.reset (new PluginInsert(_session, node));
+					have_insert = true;
 					
 				} else if (prop->value() == "port") {
 
 
 					insert.reset (new PortInsert (_session, node));
+					have_insert = true;
 
 				} else {
 
 					error << string_compose(_("unknown Insert type \"%1\"; ignored"), prop->value()) << endmsg;
 				}
 
-				add_redirect (insert, this);
+				if (have_insert) {
+					add_redirect (insert, this);
+				}
 				
 			} else {
 				error << _("Insert XML node has no type property") << endmsg;
@@ -1628,10 +1661,6 @@ Route::_set_state (const XMLNode& node, bool call_base)
 		set_denormal_protection (prop->value()=="yes"?true:false, this);
 	}
 
-	if ((prop = node.property (X_("active"))) != 0) {
-		set_active (prop->value() == "yes");
-	}
-
 	if ((prop = node.property (X_("muted"))) != 0) {
 		bool yn = prop->value()=="yes"?true:false; 
 
@@ -1991,7 +2020,7 @@ Route::set_control_outs (const vector<string>& ports)
 	/* now connect to the named ports */
 
 	for (uint32_t n = 0; n < limit; ++n) {
-		if (_control_outs->connect_output (_control_outs->output (n), ports[n], this)) {
+		if (_control_outs->connect_output (_control_outs->output (n), ports[n % ports.size()], this)) {
 			error << string_compose (_("could not connect %1 to %2"), _control_outs->output(n)->name(), ports[n]) << endmsg;
 			return -1;
 		}
@@ -2163,13 +2192,6 @@ Route::get_mute_config (mute_type t)
 	return onoff;
 }
 
-void
-Route::set_active (bool yn)
-{
-	_active = yn; 
-	 active_changed(); /* EMIT SIGNAL */
-}
-
 void
 Route::handle_transport_stopped (bool abort_ignored, bool did_locate, bool can_flush_redirects)
 {
@@ -2179,7 +2201,7 @@ Route::handle_transport_stopped (bool abort_ignored, bool did_locate, bool can_f
 		Glib::RWLock::ReaderLock lm (redirect_lock);
 
 		if (!did_locate) {
-			automation_snapshot (now);
+			automation_snapshot (now, true);
 		}
 
 		for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
@@ -2286,7 +2308,7 @@ Route::roll (nframes_t nframes, nframes_t start_frame, nframes_t end_frame, nfra
 		if (lm.locked()) {
 			// automation snapshot can also be called from the non-rt context
 			// and it uses the redirect list, so we take the lock out here
-			automation_snapshot (_session.transport_frame());
+			automation_snapshot (_session.transport_frame(), false);
 		}
 	}
 
@@ -2402,6 +2424,11 @@ Route::update_total_latency ()
 		}
 	}
 
+#undef DEBUG_LATENCY
+#ifdef DEBUG_LATENCY
+	cerr << _name << ": internal redirect latency = " << _own_latency << endl;
+#endif
+
 	set_port_latency (_own_latency);
 
 	/* this (virtual) function is used for pure Routes,
@@ -2413,6 +2440,11 @@ Route::update_total_latency ()
 
 	_own_latency += input_latency ();
 
+#ifdef DEBUG_LATENCY
+	cerr << _name << ": input latency = " << input_latency() << " total = "
+	     << _own_latency << endl;
+#endif
+
 	return _own_latency;
 }
 
@@ -2427,12 +2459,16 @@ Route::set_latency_delay (nframes_t longest_session_latency)
 }
 
 void
-Route::automation_snapshot (nframes_t now)
+Route::automation_snapshot (nframes_t now, bool force)
 {
-	IO::automation_snapshot (now);
+	if (!force && !should_snapshot(now)) {
+		return;
+	}
+
+	IO::automation_snapshot (now, force);
 
 	for (RedirectList::iterator i = _redirects.begin(); i != _redirects.end(); ++i) {
-		(*i)->automation_snapshot (now);
+		(*i)->automation_snapshot (now, force);
 	}
 }
 
