@@ -34,6 +34,7 @@
 
 #include <glibmm/thread.h>
 #include <glibmm/miscutils.h>
+#include <glibmm/fileutils.h>
 
 #include <pbd/error.h>
 #include <glibmm/thread.h>
@@ -45,6 +46,7 @@
 #include <ardour/audioengine.h>
 #include <ardour/configuration.h>
 #include <ardour/session.h>
+#include <ardour/analyser.h>
 #include <ardour/audio_diskstream.h>
 #include <ardour/utils.h>
 #include <ardour/audioplaylist.h>
@@ -95,19 +97,29 @@ const char* Session::dead_sound_dir_name = X_("dead_sounds");
 const char* Session::interchange_dir_name = X_("interchange");
 const char* Session::export_dir_name = X_("export");
 
+bool Session::_disable_all_loaded_plugins = false;
+
 Session::compute_peak_t			Session::compute_peak 		= 0;
 Session::find_peaks_t			Session::find_peaks 		= 0;
 Session::apply_gain_to_buffer_t		Session::apply_gain_to_buffer 	= 0;
 Session::mix_buffers_with_gain_t	Session::mix_buffers_with_gain 	= 0;
 Session::mix_buffers_no_gain_t		Session::mix_buffers_no_gain 	= 0;
 
+sigc::signal<void,std::string> Session::Dialog;
 sigc::signal<int> Session::AskAboutPendingState;
+sigc::signal<int,nframes_t,nframes_t> Session::AskAboutSampleRateMismatch;
 sigc::signal<void> Session::SendFeedback;
 
 sigc::signal<void> Session::SMPTEOffsetChanged;
 sigc::signal<void> Session::StartTimeChanged;
 sigc::signal<void> Session::EndTimeChanged;
 
+sigc::signal<void> Session::AutoBindingOn;
+sigc::signal<void> Session::AutoBindingOff;
+
+
+sigc::signal<void, std::string, std::string> Session::Exported;
+
 int
 Session::find_session (string str, string& path, string& snapshot, bool& isnew)
 {
@@ -259,9 +271,9 @@ Session::find_session (string str, string& path, string& snapshot, bool& isnew)
 }
 
 Session::Session (AudioEngine &eng,
-		  string fullpath,
-		  string snapshot_name,
-		  string* mix_template)
+		  const string& fullpath,
+		  const string& snapshot_name,
+		  string mix_template)
 
 	: _engine (eng),
 	  _mmc_port (default_mmc_port),
@@ -272,6 +284,7 @@ Session::Session (AudioEngine &eng,
 	  diskstreams (new DiskstreamList),
 	  routes (new RouteList),
 	  auditioner ((Auditioner*) 0),
+	  _total_free_4k_blocks (0),
 	  _click_io ((IO*) 0),
 	  main_outs (0)
 {
@@ -288,10 +301,10 @@ Session::Session (AudioEngine &eng,
 
 	first_stage_init (fullpath, snapshot_name);
 	
-	new_session = !g_file_test (_path.c_str(), GFileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR));
+	new_session = !Glib::file_test (_path, Glib::FileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR));
+
 	if (new_session) {
 		if (create (new_session, mix_template, compute_initial_length())) {
-			cerr << "create failed\n";
 			destroy ();
 			throw failed_constructor ();
 		}
@@ -334,6 +347,7 @@ Session::Session (AudioEngine &eng,
 	  midi_requests (16),
 	  diskstreams (new DiskstreamList),
 	  routes (new RouteList),
+	  _total_free_4k_blocks (0),
 	  main_outs (0)
 
 {
@@ -361,7 +375,7 @@ Session::Session (AudioEngine &eng,
 	new_session = !g_file_test (_path.c_str(), GFileTest (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR));
 
 	if (new_session) {
-		if (create (new_session, 0, initial_length)) {
+		if (create (new_session, string(), initial_length)) {
 			destroy ();
 			throw failed_constructor ();
 		}
@@ -391,7 +405,7 @@ Session::Session (AudioEngine &eng,
 		}
 		
 		if (!rl.empty()) {
-			add_routes (rl);
+			add_routes (rl, false);
 		}
 		
 	}
@@ -404,17 +418,12 @@ Session::Session (AudioEngine &eng,
 		throw failed_constructor ();
 	}
 	
-	store_recent_sessions(_name, _path);
+	store_recent_sessions (_name, _path);
 	
-	bool was_dirty = dirty ();
-
 	_state_of_the_state = StateOfTheState (_state_of_the_state & ~Dirty);
 
-	Config->ParameterChanged.connect (mem_fun (*this, &Session::config_changed));
 
-	if (was_dirty) {
-		DirtyChanged (); /* EMIT SIGNAL */
-	}
+	Config->ParameterChanged.connect (mem_fun (*this, &Session::config_changed));
 }
 
 Session::~Session ()
@@ -432,6 +441,7 @@ Session::destroy ()
 	remove_pending_capture_state ();
 
 	_state_of_the_state = StateOfTheState (CannotSave|Deletion);
+
 	_engine.remove_session ();
 
 	GoingAway (); /* EMIT SIGNAL */
@@ -576,14 +586,13 @@ Session::destroy ()
 
 		tmp = i;
 		++tmp;
-
+		
 		i->second->drop_references ();
-
+		
 		i = tmp;
 	}
-
 	audio_sources.clear ();
-
+	
 #ifdef TRACK_DESTRUCTION
 	cerr << "delete mix groups\n";
 #endif /* TRACK_DESTRUCTION */
@@ -666,9 +675,13 @@ Session::when_engine_running ()
 
 	/* we don't want to run execute this again */
 
+	BootMessage (_("Set block size and sample rate"));
+
 	set_block_size (_engine.frames_per_cycle());
 	set_frame_rate (_engine.frame_rate());
 
+	BootMessage (_("Using configuration"));
+
 	Config->map_parameters (mem_fun (*this, &Session::config_changed));
 
 	/* every time we reconnect, recompute worst case output latencies */
@@ -724,6 +737,8 @@ Session::when_engine_running ()
 		error << _("cannot setup Click I/O") << endmsg;
 	}
 
+	BootMessage (_("Compute I/O Latencies"));
+
 	set_worst_io_latencies ();
 
 	if (_clicking) {
@@ -734,6 +749,8 @@ Session::when_engine_running ()
 	   to the physical outputs currently available
 	*/
 
+	BootMessage (_("Set up standard connections"));
+
 	/* ONE: MONO */
 
 	for (uint32_t np = 0; np < n_physical_audio_outputs; ++np) {
@@ -838,11 +855,15 @@ Session::when_engine_running ()
 		}
 		add_connection (c);
 	} 
+	
+	BootMessage (_("Setup signal flow and plugins"));
 
 	hookup_io ();
 
 	/* catch up on send+insert cnts */
 
+	BootMessage (_("Catch up with send/insert state"));
+
 	insert_cnt = 0;
 	
 	for (list<PortInsert*>::iterator i = _port_inserts.begin(); i != _port_inserts.end(); ++i) {
@@ -872,17 +893,18 @@ Session::when_engine_running ()
 
 	/* hook us up to the engine */
 
+	BootMessage (_("Connect to engine"));
+
 	_engine.set_session (this);
 
 #ifdef HAVE_LIBLO
 	/* and to OSC */
 
+	BootMessage (_("OSC startup"));
+
 	osc->set_session (*this);
 #endif
     
-	_state_of_the_state = Clean;
-
-	DirtyChanged (); /* EMIT SIGNAL */
 }
 
 void
@@ -894,6 +916,7 @@ Session::hookup_io ()
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state | InitialConnecting);
 
+
 	if (auditioner == 0) {
 		
 		/* we delay creating the auditioner till now because
@@ -963,6 +986,7 @@ Session::hookup_io ()
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state & ~InitialConnecting);
 
+
 	/* now handle the whole enchilada as if it was one
 	   graph reorder event.
 	*/
@@ -1106,7 +1130,6 @@ Session::auto_loop_changed (Location* location)
 	}	
 
 	last_loopend = location->end();
-	
 }
 
 void
@@ -1144,6 +1167,10 @@ Session::set_auto_punch_location (Location* location)
 	auto_punch_changed_connection = location->changed.connect (mem_fun (this, &Session::auto_punch_changed));
 
 	location->set_auto_punch (true, this);
+
+
+	auto_punch_changed (location);
+
 	auto_punch_location_changed (location);
 }
 
@@ -1183,6 +1210,13 @@ Session::set_auto_loop_location (Location* location)
 	auto_loop_changed_connection = location->changed.connect (mem_fun (this, &Session::auto_loop_changed));
 
 	location->set_auto_loop (true, this);
+
+	/* take care of our stuff first */
+
+	auto_loop_changed (location);
+
+	/* now tell everyone else */
+
 	auto_loop_location_changed (location);
 }
 
@@ -1219,6 +1253,12 @@ Session::handle_locations_changed (Locations::LocationList& locations)
 			set_loop = true;
 		}
 		
+		if (location->is_start()) {
+			start_location = location;
+		}
+		if (location->is_end()) {
+			end_location = location;
+		}
 	}
 
 	if (!set_loop) {
@@ -1295,11 +1335,11 @@ Session::step_back_from_record ()
 	if (g_atomic_int_get (&_record_status) == Recording) {
 		g_atomic_int_set (&_record_status, Enabled);
 
-		if (Config->get_monitoring_model() == HardwareMonitoring) {
+		if (Config->get_monitoring_model() == HardwareMonitoring && Config->get_auto_input()) {
 			boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
 			
 			for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
-				if (Config->get_auto_input() && (*i)->record_enabled ()) {
+				if ((*i)->record_enabled ()) {
 					//cerr << "switching from input" << __FILE__ << __LINE__ << endl << endl;
 					(*i)->monitor_input (false);   
 				}
@@ -1823,8 +1863,7 @@ Session::new_audio_track (int input_channels, int output_channels, TrackMode mod
 
   failed:
 	if (!new_routes.empty()) {
-		add_routes (new_routes, false);
-		save_state (_current_snapshot_name);
+		add_routes (new_routes, true);
 	}
 
 	return ret;
@@ -1958,8 +1997,7 @@ Session::new_audio_route (int input_channels, int output_channels, uint32_t how_
 
   failure:
 	if (!ret.empty()) {
-		add_routes (ret, false);
-		save_state (_current_snapshot_name);
+		add_routes (ret, true);
 	}
 
 	return ret;
@@ -2097,14 +2135,14 @@ Session::remove_route (shared_ptr<Route> route)
 
 	find_current_end ();
 	
-	update_latency_compensation (false, false);
-	set_dirty();
-
 	// We need to disconnect the routes inputs and outputs 
 
 	route->disconnect_inputs (0);
 	route->disconnect_outputs (0);
 	
+	update_latency_compensation (false, false);
+	set_dirty();
+
 	/* get rid of it from the dead wood collection in the route list manager */
 
 	/* XXX i think this is unsafe as it currently stands, but i am not sure. (pd, october 2nd, 2006) */
@@ -2409,6 +2447,8 @@ Session::get_maximum_extent () const
 	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
 
 	for (DiskstreamList::const_iterator i = dsl->begin(); i != dsl->end(); ++i) {
+		if ((*i)->destructive())  //ignore tape tracks when getting max extents
+			continue;
 		boost::shared_ptr<Playlist> pl = (*i)->playlist();
 		if ((me = pl->get_maximum_extent()) > max) {
 			max = me;
@@ -2500,7 +2540,7 @@ Session::new_region_name (string old)
 }
 
 int
-Session::region_name (string& result, string base, bool newlevel) const
+Session::region_name (string& result, string base, bool newlevel)
 {
 	char buf[16];
 	string subbase;
@@ -2510,15 +2550,11 @@ Session::region_name (string& result, string base, bool newlevel) const
 		Glib::Mutex::Lock lm (region_lock);
 
 		snprintf (buf, sizeof (buf), "%d", (int)audio_regions.size() + 1);
-
-		
 		result = "region.";
 		result += buf;
 
 	} else {
 
-		/* XXX this is going to be slow. optimize me later */
-		
 		if (newlevel) {
 			subbase = base;
 		} else {
@@ -2531,43 +2567,38 @@ Session::region_name (string& result, string base, bool newlevel) const
 			subbase = base.substr (0, pos);
 
 		}
-
-		bool name_taken = true;
 		
 		{
 			Glib::Mutex::Lock lm (region_lock);
-			
-			for (int n = 1; n < 5000; ++n) {
-				
-				result = subbase;
-				snprintf (buf, sizeof (buf), ".%d", n);
+
+			map<string,uint32_t>::iterator x;
+
+			result = subbase;
+
+			if ((x = region_name_map.find (subbase)) == region_name_map.end()) {
+				result += ".1";
+				region_name_map[subbase] = 1;
+			} else {
+				x->second++;
+				snprintf (buf, sizeof (buf), ".%d", x->second);
 				result += buf;
-				
-				name_taken = false;
-				
-				for (AudioRegionList::const_iterator i = audio_regions.begin(); i != audio_regions.end(); ++i) {
-					if (i->second->name() == result) {
-						name_taken = true;
-						break;
-					}
-				}
-				
-				if (!name_taken) {
-					break;
-				}
 			}
 		}
-			
-		if (name_taken) {
-			fatal << string_compose(_("too many regions with names like %1"), base) << endmsg;
-			/*NOTREACHED*/
-		}
 	}
+
 	return 0;
 }	
 
 void
 Session::add_region (boost::shared_ptr<Region> region)
+{
+	vector<boost::shared_ptr<Region> > v;
+	v.push_back (region);
+	add_regions (v);
+}
+		
+void
+Session::add_regions (vector<boost::shared_ptr<Region> >& new_regions)
 {
 	boost::shared_ptr<AudioRegion> ar;
 	boost::shared_ptr<AudioRegion> oar;
@@ -2576,43 +2607,51 @@ Session::add_region (boost::shared_ptr<Region> region)
 	{ 
 		Glib::Mutex::Lock lm (region_lock);
 
-		if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
+		for (vector<boost::shared_ptr<Region> >::iterator ii = new_regions.begin(); ii != new_regions.end(); ++ii) {
+		
+			boost::shared_ptr<Region> region = *ii;
+			
+			if (region == 0) {
 
-			AudioRegionList::iterator x;
-
-			for (x = audio_regions.begin(); x != audio_regions.end(); ++x) {
-
-				oar = boost::dynamic_pointer_cast<AudioRegion> (x->second);
-
-				if (ar->region_list_equivalent (oar)) {
-					break;
-				}
-			}
-
-			if (x == audio_regions.end()) {
-
-				pair<AudioRegionList::key_type,AudioRegionList::mapped_type> entry;
-
-				entry.first = region->id();
-				entry.second = ar;
-
-				pair<AudioRegionList::iterator,bool> x = audio_regions.insert (entry);
+				error << _("Session::add_region() ignored a null region. Warning: you might have lost a region.") << endmsg;
 
+			} else if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
 				
-				if (!x.second) {
-					return;
+				AudioRegionList::iterator x;
+				
+				for (x = audio_regions.begin(); x != audio_regions.end(); ++x) {
+					
+					oar = boost::dynamic_pointer_cast<AudioRegion> (x->second);
+					
+					if (ar->region_list_equivalent (oar)) {
+						break;
+					}
 				}
+				
+				if (x == audio_regions.end()) {
+					
+					pair<AudioRegionList::key_type,AudioRegionList::mapped_type> entry;
+					
+					entry.first = region->id();
+					entry.second = ar;
+					
+					pair<AudioRegionList::iterator,bool> x = audio_regions.insert (entry);
+					
+					if (!x.second) {
+						return;
+					}
+					
+					added = true;
+				} 
 
-				added = true;
-			} 
-
-		} else {
-
-			fatal << _("programming error: ")
-			      << X_("unknown region type passed to Session::add_region()")
-			      << endmsg;
-			/*NOTREACHED*/
-
+			} else {
+				
+				fatal << _("programming error: ")
+				      << X_("unknown region type passed to Session::add_region()")
+				      << endmsg;
+				/*NOTREACHED*/
+				
+			}
 		}
 	}
 
@@ -2620,12 +2659,58 @@ Session::add_region (boost::shared_ptr<Region> region)
 	   add the region to the region list.
 	*/
 	
-	set_dirty();
+	set_dirty ();
 	
 	if (added) {
-		region->GoingAway.connect (sigc::bind (mem_fun (*this, &Session::remove_region), boost::weak_ptr<Region>(region)));
-		region->StateChanged.connect (sigc::bind (mem_fun (*this, &Session::region_changed), boost::weak_ptr<Region>(region)));
-		AudioRegionAdded (ar); /* EMIT SIGNAL */
+
+		vector<boost::weak_ptr<AudioRegion> > v;
+		boost::shared_ptr<AudioRegion> first_ar;
+
+		for (vector<boost::shared_ptr<Region> >::iterator ii = new_regions.begin(); ii != new_regions.end(); ++ii) {
+
+			boost::shared_ptr<Region> region = *ii;
+			boost::shared_ptr<AudioRegion> ar;
+
+			if (region == 0) {
+
+				error << _("Session::add_region() ignored a null region. Warning: you might have lost a region.") << endmsg;
+
+			} else if ((ar = boost::dynamic_pointer_cast<AudioRegion> (region)) != 0) {
+				v.push_back (ar);
+
+				if (!first_ar) {
+					first_ar = ar;
+				}
+			}
+
+			region->StateChanged.connect (sigc::bind (mem_fun (*this, &Session::region_changed), boost::weak_ptr<Region>(region)));
+			region->GoingAway.connect (sigc::bind (mem_fun (*this, &Session::remove_region), boost::weak_ptr<Region>(region)));
+
+			update_region_name_map (region);
+		}
+
+		if (!v.empty()) {
+			AudioRegionsAdded (v); /* EMIT SIGNAL */
+		}
+	}
+}
+
+void
+Session::update_region_name_map (boost::shared_ptr<Region> region)
+{
+	string::size_type last_period = region->name().find_last_of ('.');
+	
+	if (last_period != string::npos && last_period < region->name().length() - 1) {
+		
+		string base = region->name().substr (0, last_period);
+		string number = region->name().substr (last_period+1);
+		map<string,uint32_t>::iterator x;
+		
+		/* note that if there is no number, we get zero from atoi,
+		   which is just fine
+		*/
+		
+		region_name_map[base] = atoi (number);
 	}
 }
 
@@ -2642,6 +2727,10 @@ Session::region_changed (Change what_changed, boost::weak_ptr<Region> weak_regio
 		/* relay hidden changes */
 		RegionHiddenChange (region);
 	}
+
+	if (what_changed & NameChanged) {
+		update_region_name_map (region);
+	}
 }
 
 void
@@ -2820,6 +2909,9 @@ Session::add_source (boost::shared_ptr<Source> source)
 			set_dirty();
 		}
 
+		if (Config->get_auto_analyse_audio()) {
+			Analyser::queue_source_for_analysis (source, false);
+		}
 	} 
 }
 
@@ -2887,13 +2979,7 @@ Session::source_by_path_and_channel (const Glib::ustring& path, uint16_t chn)
 Glib::ustring
 Session::peak_path (Glib::ustring base) const
 {
-	Glib::ustring res;
-	
-	res = peak_dir ();
-	res += base;
-	res += ".peak";
-
-	return res;
+	return Glib::build_filename(peak_dir (), base + ".peak");
 }
 
 string
@@ -3607,18 +3693,36 @@ void
 Session::tempo_map_changed (Change ignored)
 {
 	clear_clicks ();
+	
+	for (PlaylistList::iterator i = playlists.begin(); i != playlists.end(); ++i) {
+		(*i)->update_after_tempo_map_change ();
+	}
+
+	for (PlaylistList::iterator i = unused_playlists.begin(); i != unused_playlists.end(); ++i) {
+		(*i)->update_after_tempo_map_change ();
+	}
+
 	set_dirty ();
 }
 
 void
 Session::ensure_passthru_buffers (uint32_t howmany)
 {
+	if (current_block_size == 0) {
+		return;
+	}
+
 	while (howmany > _passthru_buffers.size()) {
 		Sample *p;
 #ifdef NO_POSIX_MEMALIGN
 		p =  (Sample *) malloc(current_block_size * sizeof(Sample));
 #else
-		posix_memalign((void **)&p,CPU_CACHE_ALIGN,current_block_size * sizeof(Sample));
+		if (posix_memalign((void **)&p,CPU_CACHE_ALIGN,current_block_size * sizeof(Sample)) != 0) {
+			fatal << string_compose (_("Memory allocation error: posix_memalign (%1 * %2) failed (%3)"),
+						 current_block_size, sizeof (Sample), strerror (errno))
+			      << endmsg;
+			/*NOTREACHED*/
+		}
 #endif			
 		_passthru_buffers.push_back (p);
 
@@ -3627,7 +3731,12 @@ Session::ensure_passthru_buffers (uint32_t howmany)
 #ifdef NO_POSIX_MEMALIGN
 		p =  (Sample *) malloc(current_block_size * sizeof(Sample));
 #else
-		posix_memalign((void **)&p,CPU_CACHE_ALIGN,current_block_size * 4);
+		if (posix_memalign((void **)&p,CPU_CACHE_ALIGN,current_block_size * sizeof(Sample)) != 0) {
+			fatal << string_compose (_("Memory allocation error: posix_memalign (%1 * %2) failed (%3)"),
+						 current_block_size, sizeof (Sample), strerror (errno))
+			      << endmsg;
+			/*NOTREACHED*/
+		}
 #endif			
 		memset (p, 0, sizeof (Sample) * current_block_size);
 		_silent_buffers.push_back (p);
@@ -3840,11 +3949,11 @@ Session::freeze (InterThreadInfo& itt)
 	return 0;
 }
 
-int
-Session::write_one_audio_track (AudioTrack& track, nframes_t start, nframes_t len, 	
+boost::shared_ptr<Region>
+Session::write_one_audio_track (AudioTrack& track, nframes_t start, nframes_t end, 	
 			       bool overwrite, vector<boost::shared_ptr<AudioSource> >& srcs, InterThreadInfo& itt)
 {
-	int ret = -1;
+	boost::shared_ptr<Region> result;
 	boost::shared_ptr<Playlist> playlist;
 	boost::shared_ptr<AudioFileSource> fsource;
 	uint32_t x;
@@ -3854,8 +3963,15 @@ Session::write_one_audio_track (AudioTrack& track, nframes_t start, nframes_t le
 	nframes_t position;
 	nframes_t this_chunk;
 	nframes_t to_do;
+	nframes_t len = end - start;
 	vector<Sample*> buffers;
 
+	if (end <= start) {
+		error << string_compose (_("Cannot write a range where end <= start (e.g. %1 <= %2)"),
+					 end, start) << endmsg;
+		return result;
+	}
+
 	// any bigger than this seems to cause stack overflows in called functions
 	const nframes_t chunk_size = (128 * 1024)/4;
 
@@ -3968,21 +4084,19 @@ Session::write_one_audio_track (AudioTrack& track, nframes_t start, nframes_t le
 		
 		/* construct a region to represent the bounced material */
 
-		boost::shared_ptr<Region> aregion = RegionFactory::create (srcs, 0, srcs.front()->length(), 
-									   region_name_from_path (srcs.front()->name(), true));
-
-		ret = 0;
+		result = RegionFactory::create (srcs, 0, srcs.front()->length(), 
+						region_name_from_path (srcs.front()->name(), true));
 	}
 		
   out:
-	if (ret) {
+	if (!result) {
 		for (vector<boost::shared_ptr<AudioSource> >::iterator src = srcs.begin(); src != srcs.end(); ++src) {
 			boost::shared_ptr<AudioFileSource> afs = boost::dynamic_pointer_cast<AudioFileSource>(*src);
 
 			if (afs) {
 				afs->mark_for_remove ();
 			}
-
+			
 			(*src)->drop_references ();
 		}
 
@@ -3993,22 +4107,50 @@ Session::write_one_audio_track (AudioTrack& track, nframes_t start, nframes_t le
 	}
 
 	for (vector<Sample*>::iterator i = buffers.begin(); i != buffers.end(); ++i) {
-		free(*i);
+		free (*i);
 	}
 
 	g_atomic_int_set (&processing_prohibited, 0);
 
 	itt.done = true;
 
-	return ret;
+	return result;
 }
 
 vector<Sample*>&
 Session::get_silent_buffers (uint32_t howmany)
 {
+	if (howmany > _silent_buffers.size()) {
+
+		error << string_compose (_("Programming error: get_silent_buffers() called for %1 buffers but only %2 exist"),
+					 howmany, _silent_buffers.size()) << endmsg;
+
+		if (howmany > 1000) {
+			cerr << "ABSURD: more than 1000 silent buffers requested!\n";
+			abort ();
+		}
+		
+		while (howmany > _silent_buffers.size()) {
+			Sample *p = 0;
+			
+#ifdef NO_POSIX_MEMALIGN
+			p =  (Sample *) malloc(current_block_size * sizeof(Sample));
+#else
+			if (posix_memalign((void **)&p,CPU_CACHE_ALIGN,current_block_size * sizeof(Sample)) != 0) {
+				fatal << string_compose (_("Memory allocation error: posix_memalign (%1 * %2) failed (%3)"),
+							 current_block_size, sizeof (Sample), strerror (errno))
+				      << endmsg;
+				/*NOTREACHED*/
+			}
+#endif			
+			_silent_buffers.push_back (p);
+		}
+	}
+
 	for (uint32_t i = 0; i < howmany; ++i) {
 		memset (_silent_buffers[i], 0, sizeof (Sample) * current_block_size);
 	}
+
 	return _silent_buffers;
 }
 
