@@ -1,3 +1,4 @@
+
 /*
   Copyright (C) 1999-2002 Paul Davis 
 
@@ -15,7 +16,6 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-  $Id: session_midi.cc,v 1.59 2006/03/29 20:41:24 essej Exp $
 */
 
 #include <string>
@@ -29,14 +29,14 @@
 #include <midi++/port.h>
 #include <midi++/manager.h>
 #include <pbd/error.h>
-#include <pbd/lockmonitor.h>
+#include <glibmm/thread.h>
 #include <pbd/pthread_utils.h>
 
 #include <ardour/configuration.h>
 #include <ardour/audioengine.h>
 #include <ardour/session.h>
 #include <ardour/audio_track.h>
-#include <ardour/diskstream.h>
+#include <ardour/audio_diskstream.h>
 #include <ardour/slave.h>
 #include <ardour/cycles.h>
 
@@ -44,7 +44,7 @@
 
 using namespace std;
 using namespace ARDOUR;
-using namespace SigC;
+using namespace PBD;
 using namespace MIDI;
 
 MachineControl::CommandSignature MMC_CommandSignature;
@@ -83,100 +83,6 @@ Session::use_config_midi_ports ()
  MTC, MMC, etc.
 **********************************************************************/
 
-void
-Session::set_mmc_control (bool yn)
-{
-	if (mmc_control == yn) {
-		return;
-	}
-	
-	mmc_control = yn;
-	set_dirty();
-	poke_midi_thread ();
-	
-	ControlChanged (MMCControl); /* EMIT SIGNAL */
-}
-
-void
-Session::set_midi_control (bool yn)
-{
-	if (midi_control == yn) {
-		return;
-	}
-
-	midi_control = yn;
-	set_dirty();
-	poke_midi_thread ();
-
-	if (_midi_port) {
-		RWLockMonitor lm (route_lock, false, __LINE__, __FILE__);
-		for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
-			(*i)->reset_midi_control (_midi_port, midi_control);
-		}
-	}
-
-	ControlChanged (MidiControl); /* EMIT SIGNAL */
-}
-
-void
-Session::set_send_mtc (bool yn)
-{
-	/* set the persistent option value regardless */
-
-	send_midi_timecode = yn;
-	set_dirty();
-
-	/* only set the internal flag if we have
-	   a port.
-	*/
-
-	if (_mtc_port == 0 || send_mtc == yn) {
-		return;
-	}
-
-	send_mtc = yn;
-	ControlChanged (SendMTC); /* EMIT SIGNAL */
-}
-
-void
-Session::set_send_mmc (bool yn)
-{
-	if (_mmc_port == 0) {
-		return;
-	}
-
-	if (send_midi_machine_control == yn) {
-		return;
-	}
-
-	/* only set the internal flag if we have
-	   a port.
-	*/
-
-	if (_mmc_port) {
-		send_mmc = yn;
-	}
-
-	/* set the persistent option value regardless */
-
-	send_midi_machine_control = yn;
-	set_dirty();
-
-	ControlChanged (SendMMC); /* EMIT SIGNAL */
-}
-
-bool
-Session::get_send_mtc () const
-{
-	return send_mtc;
-}
-
-bool
-Session::get_send_mmc () const
-{
-	return send_mmc;
-}
-
 int
 Session::set_mtc_port (string port_tag)
 {
@@ -200,7 +106,7 @@ Session::set_mtc_port (string port_tag)
 	MIDI::Port* port;
 
 	if ((port = MIDI::Manager::instance()->port (port_tag)) == 0) {
-		error << compose (_("unknown port %1 requested for MTC"), port_tag) << endl;
+		error << string_compose (_("unknown port %1 requested for MTC"), port_tag) << endl;
 		return -1;
 	}
 
@@ -219,9 +125,31 @@ Session::set_mtc_port (string port_tag)
 	return 0;
 }
 
+void
+Session::set_mmc_receive_device_id (uint32_t device_id)
+{
+	if (mmc) {
+		mmc->set_receive_device_id (device_id);
+	}
+}
+
+void
+Session::set_mmc_send_device_id (uint32_t device_id)
+{
+	if (mmc) {
+		mmc->set_send_device_id (device_id);
+		/* reset MMC buffer */
+		mmc_buffer[2] = mmc->send_device_id();
+	}
+}
+
 int
 Session::set_mmc_port (string port_tag)
 {
+	MIDI::byte old_recv_device_id = 0;
+	MIDI::byte old_send_device_id = 0;
+	bool reset_id = false;
+
 	if (port_tag.length() == 0) {
 		if (_mmc_port == 0) {
 			return 0;
@@ -239,6 +167,9 @@ Session::set_mmc_port (string port_tag)
 	_mmc_port = port;
 
 	if (mmc) {
+		old_recv_device_id = mmc->receive_device_id();
+		old_recv_device_id = mmc->send_device_id();
+		reset_id = true;
 		delete mmc;
 	}
 
@@ -246,39 +177,44 @@ Session::set_mmc_port (string port_tag)
 					MMC_CommandSignature,
 					MMC_ResponseSignature);
 
+	if (reset_id) {
+		set_mmc_receive_device_id (old_recv_device_id);
+		set_mmc_send_device_id (old_send_device_id);
+	}
 
 	mmc->Play.connect 
-		(slot (*this, &Session::mmc_deferred_play));
+		(mem_fun (*this, &Session::mmc_deferred_play));
 	mmc->DeferredPlay.connect 
-		(slot (*this, &Session::mmc_deferred_play));
+		(mem_fun (*this, &Session::mmc_deferred_play));
 	mmc->Stop.connect 
-		(slot (*this, &Session::mmc_stop));
+		(mem_fun (*this, &Session::mmc_stop));
 	mmc->FastForward.connect 
-		(slot (*this, &Session::mmc_fast_forward));
+		(mem_fun (*this, &Session::mmc_fast_forward));
 	mmc->Rewind.connect 
-		(slot (*this, &Session::mmc_rewind));
+		(mem_fun (*this, &Session::mmc_rewind));
 	mmc->Pause.connect 
-		(slot (*this, &Session::mmc_pause));
+		(mem_fun (*this, &Session::mmc_pause));
 	mmc->RecordPause.connect 
-		(slot (*this, &Session::mmc_record_pause));
+		(mem_fun (*this, &Session::mmc_record_pause));
 	mmc->RecordStrobe.connect 
-		(slot (*this, &Session::mmc_record_strobe));
+		(mem_fun (*this, &Session::mmc_record_strobe));
 	mmc->RecordExit.connect 
-		(slot (*this, &Session::mmc_record_exit));
+		(mem_fun (*this, &Session::mmc_record_exit));
 	mmc->Locate.connect 
-		(slot (*this, &Session::mmc_locate));
+		(mem_fun (*this, &Session::mmc_locate));
 	mmc->Step.connect 
-		(slot (*this, &Session::mmc_step));
+		(mem_fun (*this, &Session::mmc_step));
 	mmc->Shuttle.connect 
-		(slot (*this, &Session::mmc_shuttle));
+		(mem_fun (*this, &Session::mmc_shuttle));
 	mmc->TrackRecordStatusChange.connect
-		(slot (*this, &Session::mmc_record_enable));
+		(mem_fun (*this, &Session::mmc_record_enable));
+
 
 	/* also handle MIDI SPP because its so common */
 
-	_mmc_port->input()->start.connect (slot (*this, &Session::spp_start));
-	_mmc_port->input()->contineu.connect (slot (*this, &Session::spp_continue));
-	_mmc_port->input()->stop.connect (slot (*this, &Session::spp_stop));
+	_mmc_port->input()->start.connect (mem_fun (*this, &Session::spp_start));
+	_mmc_port->input()->contineu.connect (mem_fun (*this, &Session::spp_continue));
+	_mmc_port->input()->stop.connect (mem_fun (*this, &Session::spp_stop));
 	
 	Config->set_mmc_port_name (port_tag);
 
@@ -307,6 +243,10 @@ Session::set_midi_port (string port_tag)
 	}
 
 	_midi_port = port;
+	
+	/* XXX need something to forward this to control protocols ? or just
+	   use the signal below 
+	*/
 
 	Config->set_midi_port_name (port_tag);
 
@@ -448,42 +388,6 @@ Session::get_trace_midi_output(MIDI::Port *port)
 
 }
 
-
-void
-Session::set_midi_feedback (bool yn)
-{
-	if (_midi_port == 0) {
-		return;
-	}
-	
-	midi_feedback = yn;
-	set_dirty();
-
-	if (yn) {
-		/* make sure the feedback thread is alive */
-		start_feedback ();
-	} else {
-		/* maybe put the feedback thread to sleep */
-		stop_feedback ();
-	}
-
-	ControlChanged (MidiFeedback); /* EMIT SIGNAL */
-
-	send_all_midi_feedback ();
-}
-
-void
-Session::send_all_midi_feedback ()
-{
-	if (midi_feedback) {
-		// send out current state of all routes
-		RWLockMonitor lm (route_lock, false, __LINE__, __FILE__);
-		for (RouteList::iterator i = routes.begin(); i != routes.end(); ++i) {
-			(*i)->send_all_midi_feedback ();
-		}
-	}
-}
-
 void
 Session::setup_midi_control ()
 {
@@ -494,7 +398,7 @@ Session::setup_midi_control ()
 	
 	mmc_buffer[0] = 0xf0; // SysEx
 	mmc_buffer[1] = 0x7f; // Real Time SysEx ID for MMC
-	mmc_buffer[2] = 0x7f; // "broadcast" device ID
+	mmc_buffer[2] = (mmc ? mmc->send_device_id() : 0x7f);
 	mmc_buffer[3] = 0x6;  // MCC
 
 	/* Set up the qtr frame message */
@@ -507,25 +411,6 @@ Session::setup_midi_control ()
 	mtc_msg[10] = 0xf1;
 	mtc_msg[12] = 0xf1;
 	mtc_msg[14] = 0xf1;
-
-	if (_mmc_port != 0) {
-
-		send_mmc = send_midi_machine_control;
-
-	} else {
-
-		mmc = 0;
-		send_mmc = false;
-	}
-
-	if (_mtc_port != 0) {
-
-		send_mtc = send_midi_timecode;
-
-	} else {
-
-		send_mtc = false;
-	}
 }
 
 int
@@ -558,7 +443,7 @@ Session::midi_read (MIDI::Port* port)
 		} else if (errno == EAGAIN) {
 			break;
 		} else {
-			fatal << compose(_("Error reading from MIDI port %1"), port->name()) << endmsg;
+			fatal << string_compose(_("Error reading from MIDI port %1"), port->name()) << endmsg;
 			/*NOTREACHED*/
 		}
 	}
@@ -569,7 +454,7 @@ Session::midi_read (MIDI::Port* port)
 void
 Session::spp_start (Parser& ignored)
 {
-	if (mmc_control && (_slave_type != MTC)) {
+	if (Config->get_mmc_control() && (Config->get_slave_source() != MTC)) {
 		request_transport_speed (1.0);
 	}
 }
@@ -583,7 +468,7 @@ Session::spp_continue (Parser& ignored)
 void
 Session::spp_stop (Parser& ignored)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 		request_stop ();
 	}
 }
@@ -591,7 +476,7 @@ Session::spp_stop (Parser& ignored)
 void
 Session::mmc_deferred_play (MIDI::MachineControl &mmc)
 {
-	if (mmc_control && (_slave_type != MTC)) {
+	if (Config->get_mmc_control() && (Config->get_slave_source() != MTC)) {
 		request_transport_speed (1.0);
 	}
 }
@@ -599,7 +484,7 @@ Session::mmc_deferred_play (MIDI::MachineControl &mmc)
 void
 Session::mmc_record_pause (MIDI::MachineControl &mmc)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 		maybe_enable_record();
 	}
 }
@@ -607,7 +492,7 @@ Session::mmc_record_pause (MIDI::MachineControl &mmc)
 void
 Session::mmc_record_strobe (MIDI::MachineControl &mmc)
 {
-	if (!mmc_control) 
+	if (!Config->get_mmc_control()) 
 		return;
 
 	/* record strobe does an implicit "Play" command */
@@ -621,8 +506,8 @@ Session::mmc_record_strobe (MIDI::MachineControl &mmc)
 		*/
 		
 		save_state ("", true);
-		atomic_set (&_record_status, Enabled);
-		RecordEnabled (); /* EMIT SIGNAL */
+		g_atomic_int_set (&_record_status, Enabled);
+		RecordStateChanged (); /* EMIT SIGNAL */
 		
 		request_transport_speed (1.0);
 
@@ -635,7 +520,7 @@ Session::mmc_record_strobe (MIDI::MachineControl &mmc)
 void
 Session::mmc_record_exit (MIDI::MachineControl &mmc)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 		disable_record (false);
 	}
 }
@@ -643,7 +528,7 @@ Session::mmc_record_exit (MIDI::MachineControl &mmc)
 void
 Session::mmc_stop (MIDI::MachineControl &mmc)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 		request_stop ();
 	}
 }
@@ -651,7 +536,7 @@ Session::mmc_stop (MIDI::MachineControl &mmc)
 void
 Session::mmc_pause (MIDI::MachineControl &mmc)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 
 		/* We support RECORD_PAUSE, so the spec says that
 		   we must interpret PAUSE like RECORD_PAUSE if
@@ -669,10 +554,9 @@ Session::mmc_pause (MIDI::MachineControl &mmc)
 static bool step_queued = false;
 
 void
-
 Session::mmc_step (MIDI::MachineControl &mmc, int steps)
 {
-	if (!mmc_control) {
+	if (!Config->get_mmc_control()) {
 		return;
 	}
 
@@ -691,7 +575,7 @@ Session::mmc_step (MIDI::MachineControl &mmc, int steps)
 	}
 	
 	double diff_secs = diff.tv_sec + (diff.tv_usec / 1000000.0);
-	double cur_speed = (((steps * 0.5) * smpte_frames_per_second) / diff_secs) / smpte_frames_per_second;
+	double cur_speed = (((steps * 0.5) * smpte_frames_per_second()) / diff_secs) / smpte_frames_per_second();
 	
 	if (_transport_speed == 0 || cur_speed * _transport_speed < 0) {
 		/* change direction */
@@ -715,7 +599,7 @@ Session::mmc_step (MIDI::MachineControl &mmc, int steps)
 	last_mmc_step = now;
 
 	if (!step_queued) {
-		midi_timeouts.push_back (slot (*this, &Session::mmc_step_timeout));
+		midi_timeouts.push_back (mem_fun (*this, &Session::mmc_step_timeout));
 		step_queued = true;
 	}
 }
@@ -723,7 +607,7 @@ Session::mmc_step (MIDI::MachineControl &mmc, int steps)
 void
 Session::mmc_rewind (MIDI::MachineControl &mmc)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 		request_transport_speed(-8.0f);
 	}
 }
@@ -731,7 +615,7 @@ Session::mmc_rewind (MIDI::MachineControl &mmc)
 void
 Session::mmc_fast_forward (MIDI::MachineControl &mmc)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 		request_transport_speed(8.0f);
 	}
 }
@@ -739,17 +623,19 @@ Session::mmc_fast_forward (MIDI::MachineControl &mmc)
 void
 Session::mmc_locate (MIDI::MachineControl &mmc, const MIDI::byte* mmc_tc)
 {
-	if (!mmc_control) {
+	if (!Config->get_mmc_control()) {
 		return;
 	}
 
-	jack_nframes_t target_frame;
-	SMPTE_Time smpte;
+	nframes_t target_frame;
+	SMPTE::Time smpte;
 
 	smpte.hours = mmc_tc[0] & 0xf;
 	smpte.minutes = mmc_tc[1];
 	smpte.seconds = mmc_tc[2];
 	smpte.frames = mmc_tc[3];
+	smpte.rate = smpte_frames_per_second();
+	smpte.drop = smpte_drop_frames();
   
 	// Also takes smpte offset into account:
 	smpte_to_sample( smpte, target_frame, true /* use_offset */, false /* use_subframes */ );
@@ -778,18 +664,14 @@ Session::mmc_locate (MIDI::MachineControl &mmc, const MIDI::byte* mmc_tc)
 void
 Session::mmc_shuttle (MIDI::MachineControl &mmc, float speed, bool forw)
 {
-	cerr << "MMC shuttle, speed = " << speed << endl;
-
-	if (!mmc_control) {
+	if (!Config->get_mmc_control()) {
 		return;
 	}
 
-	if (shuttle_speed_threshold >= 0 && speed > shuttle_speed_threshold) {
-		speed *= shuttle_speed_factor;
+	if (Config->get_shuttle_speed_threshold() >= 0 && speed > Config->get_shuttle_speed_threshold()) {
+		speed *= Config->get_shuttle_speed_factor();
 	}
 
-	cerr << "requested MMC control speed = " << speed << endl;
-	
 	if (forw) {
 		request_transport_speed (speed);
 	} else {
@@ -800,21 +682,16 @@ Session::mmc_shuttle (MIDI::MachineControl &mmc, float speed, bool forw)
 void
 Session::mmc_record_enable (MIDI::MachineControl &mmc, size_t trk, bool enabled)
 {
-	if (mmc_control) {
+	if (Config->get_mmc_control()) {
 
-		/* don't take route or diskstream lock: if using dynamic punch,
-		   this could cause a dropout. XXX is that really OK?
-		   or should we queue a rec-enable request?
-		*/
-
-		size_t n;
 		RouteList::iterator i;
-
-		for (n = 0, i = routes.begin(); i != routes.end(); ++i) {
+		boost::shared_ptr<RouteList> r = routes.reader();
+		
+		for (i = r->begin(); i != r->end(); ++i) {
 			AudioTrack *at;
 
-			if ((at = dynamic_cast<AudioTrack*>(*i)) != 0) {
-				if (n++ == trk) {
+			if ((at = dynamic_cast<AudioTrack*>((*i).get())) != 0) {
+				if (trk == at->remote_control_id()) {
 					at->set_record_enable (enabled, &mmc);
 					break;
 				}
@@ -838,8 +715,8 @@ Session::send_midi_time_code_in_another_thread ()
 void
 Session::send_time_code_in_another_thread (bool full)
 {
-	jack_nframes_t two_smpte_frames_duration;
-	jack_nframes_t quarter_frame_duration;
+	nframes_t two_smpte_frames_duration;
+	nframes_t quarter_frame_duration;
 
 	/* Duration of two smpte frames */
 	two_smpte_frames_duration = ((long) _frames_per_smpte_frame) << 1;
@@ -883,9 +760,9 @@ Session::send_full_time_code ()
 
 {
 	MIDI::byte msg[10];
-	SMPTE_Time smpte;
+	SMPTE::Time smpte;
 
-	if (_mtc_port == 0 || !send_mtc) {
+	if (_mtc_port == 0 || !session_send_mtc) {
 		return 0;
 	}
 
@@ -909,8 +786,8 @@ Session::send_full_time_code ()
 		outbound_mtc_smpte_frame = _transport_frame;
 		if (((mtc_smpte_bits >> 5) != MIDI::MTC_25_FPS) && (transmitting_smpte_time.frames % 2)) {
 			// start MTC quarter frame transmission on an even frame
-			smpte_increment( transmitting_smpte_time );
-			outbound_mtc_smpte_frame += (jack_nframes_t) _frames_per_smpte_frame;
+			SMPTE::increment( transmitting_smpte_time );
+			outbound_mtc_smpte_frame += (nframes_t) _frames_per_smpte_frame;
 		}
 	}
 
@@ -933,7 +810,7 @@ Session::send_full_time_code ()
 	msg[8] = smpte.frames;
 
 	{
-		LockMonitor lm (midi_lock, __LINE__, __FILE__);
+		Glib::Mutex::Lock lm (midi_lock);
     
 		if (_mtc_port->midimsg (msg, sizeof (msg))) {
 			error << _("Session: could not send full MIDI time code") << endmsg;
@@ -948,12 +825,12 @@ Session::send_full_time_code ()
 int
 Session::send_midi_time_code ()
 {
-	if (_mtc_port == 0 || !send_mtc || transmitting_smpte_time.negative || (next_quarter_frame_to_send < 0) )  {
+	if (_mtc_port == 0 || !session_send_mtc || transmitting_smpte_time.negative || (next_quarter_frame_to_send < 0) )  {
 		return 0;
 	}
 
-	jack_nframes_t two_smpte_frames_duration;
-	jack_nframes_t quarter_frame_duration;
+	nframes_t two_smpte_frames_duration;
+	nframes_t quarter_frame_duration;
 
 	/* Duration of two smpte frames */
 	two_smpte_frames_duration = ((long) _frames_per_smpte_frame) << 1;
@@ -965,7 +842,7 @@ Session::send_midi_time_code ()
 
 		// Send quarter frames up to current time
 		{
-			LockMonitor lm (midi_lock, __LINE__, __FILE__);
+			Glib::Mutex::Lock lm (midi_lock);
 
 			switch(next_quarter_frame_to_send) {
 			case 0:
@@ -995,7 +872,7 @@ Session::send_midi_time_code ()
 			}			
 			
 			if (_mtc_port->midimsg (mtc_msg, 2)) {
-				error << compose(_("Session: cannot send quarter-frame MTC message (%1)"), strerror (errno)) 
+				error << string_compose(_("Session: cannot send quarter-frame MTC message (%1)"), strerror (errno)) 
 				      << endmsg;
 				
 				return -1;
@@ -1010,8 +887,8 @@ Session::send_midi_time_code ()
 				// Wrap quarter frame counter
 				next_quarter_frame_to_send = 0;
 				// Increment smpte time twice
-				smpte_increment( transmitting_smpte_time );
-				smpte_increment( transmitting_smpte_time );        
+				SMPTE::increment( transmitting_smpte_time );
+				SMPTE::increment( transmitting_smpte_time );        
 				// Re-calculate timing of first quarter frame
 				smpte_to_sample( transmitting_smpte_time, outbound_mtc_smpte_frame, true /* use_offset */, false );
 				// Compensate for audio latency
@@ -1027,11 +904,11 @@ Session::send_midi_time_code ()
 **********************************************************************/
 
 void
-Session::send_mmc_in_another_thread (MIDI::MachineControl::Command cmd, jack_nframes_t target_frame)
+Session::send_mmc_in_another_thread (MIDI::MachineControl::Command cmd, nframes_t target_frame)
 {
 	MIDIRequest* request;
 
-	if (_mtc_port == 0 || !send_mmc) {
+	if (_mtc_port == 0 || !session_send_mmc) {
 		return;
 	}
 
@@ -1045,13 +922,13 @@ Session::send_mmc_in_another_thread (MIDI::MachineControl::Command cmd, jack_nfr
 }
 
 void
-Session::deliver_mmc (MIDI::MachineControl::Command cmd, jack_nframes_t where)
+Session::deliver_mmc (MIDI::MachineControl::Command cmd, nframes_t where)
 {
 	using namespace MIDI;
 	int nbytes = 4;
-	SMPTE_Time smpte;
+	SMPTE::Time smpte;
 
-	if (_mmc_port == 0 || !send_mmc) {
+	if (_mmc_port == 0 || !session_send_mmc) {
 		return;
 	}
 
@@ -1100,10 +977,10 @@ Session::deliver_mmc (MIDI::MachineControl::Command cmd, jack_nframes_t where)
 
 		mmc_buffer[nbytes++] = 0xf7; // terminate SysEx/MMC message
 
-		LockMonitor lm (midi_lock, __LINE__, __FILE__);
+		Glib::Mutex::Lock lm (midi_lock);
 
 		if (_mmc_port->write (mmc_buffer, nbytes) != nbytes) {
-			error << compose(_("MMC: cannot send command %1%2%3"), &hex, cmd, &dec) << endmsg;
+			error << string_compose(_("MMC: cannot send command %1%2%3"), &hex, cmd, &dec) << endmsg;
 		}
 	}
 }
@@ -1205,17 +1082,17 @@ int
 Session::start_midi_thread ()
 {
 	if (pipe (midi_request_pipe)) {
-		error << compose(_("Cannot create transport request signal pipe (%1)"), strerror (errno)) << endmsg;
+		error << string_compose(_("Cannot create transport request signal pipe (%1)"), strerror (errno)) << endmsg;
 		return -1;
 	}
 
 	if (fcntl (midi_request_pipe[0], F_SETFL, O_NONBLOCK)) {
-		error << compose(_("UI: cannot set O_NONBLOCK on "    "signal read pipe (%1)"), strerror (errno)) << endmsg;
+		error << string_compose(_("UI: cannot set O_NONBLOCK on "    "signal read pipe (%1)"), strerror (errno)) << endmsg;
 		return -1;
 	}
 
 	if (fcntl (midi_request_pipe[1], F_SETFL, O_NONBLOCK)) {
-		error << compose(_("UI: cannot set O_NONBLOCK on "    "signal write pipe (%1)"), strerror (errno)) << endmsg;
+		error << string_compose(_("UI: cannot set O_NONBLOCK on "    "signal write pipe (%1)"), strerror (errno)) << endmsg;
 		return -1;
 	}
 
@@ -1232,24 +1109,26 @@ Session::start_midi_thread ()
 void
 Session::terminate_midi_thread ()
 {
-	MIDIRequest* request = new MIDIRequest;
-	void* status;
-
-	request->type = MIDIRequest::Quit;
-
-	midi_requests.write (&request, 1);
-	poke_midi_thread ();
-
-	pthread_join (midi_thread, &status);
+	if (midi_thread) {
+		MIDIRequest* request = new MIDIRequest;
+		void* status;
+		
+		request->type = MIDIRequest::Quit;
+		
+		midi_requests.write (&request, 1);
+		poke_midi_thread ();
+		
+		pthread_join (midi_thread, &status);
+	}
 }
 
 void
 Session::poke_midi_thread ()
 {
-	char c;
+	static char c = 0;
 
 	if (write (midi_request_pipe[1], &c, 1) != 1) {
-		error << compose(_("cannot send signal to midi thread! (%1)"), strerror (errno)) << endmsg;
+		error << string_compose(_("cannot send signal to midi thread! (%1)"), strerror (errno)) << endmsg;
 	}
 }
 
@@ -1276,7 +1155,7 @@ Session::midi_thread_work ()
 	bool restart;
 	vector<MIDI::Port*> ports;
 
-	PBD::ThreadCreated (pthread_self(), X_("MIDI"));
+	PBD::ThreadCreatedWithRequestSize (pthread_self(), X_("MIDI"), 2048);
 
 	memset (&rtparam, 0, sizeof (rtparam));
 	rtparam.sched_priority = 9; /* XXX should be relative to audio (JACK) thread */
@@ -1304,7 +1183,7 @@ Session::midi_thread_work ()
 		   on the appropriate port.
 		*/
 
-		if (mmc_control && _mmc_port && _mmc_port->selectable() >= 0) {
+		if (Config->get_mmc_control() && _mmc_port && _mmc_port->selectable() >= 0) {
 			pfd[nfds].fd = _mmc_port->selectable();
 			pfd[nfds].events = POLLIN|POLLHUP|POLLERR;
 			ports[nfds] = _mmc_port;
@@ -1316,14 +1195,14 @@ Session::midi_thread_work ()
 		   the relevant port.
 		*/
 
-		if (_mtc_port && (_mtc_port != _mmc_port || !mmc_control) && _mtc_port->selectable() >= 0) {
+		if (_mtc_port && (_mtc_port != _mmc_port || !Config->get_mmc_control()) && _mtc_port->selectable() >= 0) {
 			pfd[nfds].fd = _mtc_port->selectable();
 			pfd[nfds].events = POLLIN|POLLHUP|POLLERR;
 			ports[nfds] = _mtc_port;
 			nfds++;
 		}
 
-		if (_midi_port && (_midi_port != _mmc_port || !mmc_control) && (_midi_port != _mtc_port) && _midi_port->selectable() >= 0) {
+		if (_midi_port && (_midi_port != _mmc_port || !Config->get_mmc_control()) && (_midi_port != _mtc_port) && _midi_port->selectable() >= 0) {
 			pfd[nfds].fd = _midi_port->selectable();
 			pfd[nfds].events = POLLIN|POLLHUP|POLLERR;
 			ports[nfds] = _midi_port;
@@ -1344,7 +1223,7 @@ Session::midi_thread_work ()
 				goto again;
 			}
 
-			error << compose(_("MIDI thread poll failed (%1)"), strerror (errno)) << endmsg;
+			error << string_compose(_("MIDI thread poll failed (%1)"), strerror (errno)) << endmsg;
 
 			break;
 		}
@@ -1452,7 +1331,7 @@ Session::midi_thread_work ()
 
 		for (int p = 1; p < nfds; ++p) {
 			if ((pfd[p].revents & ~POLLIN)) {
-				// error << compose(_("Transport: error polling MIDI port %1 (revents =%2%3%4"), p, &hex, pfd[p].revents, &dec) << endmsg;
+				// error << string_compose(_("Transport: error polling MIDI port %1 (revents =%2%3%4"), p, &hex, pfd[p].revents, &dec) << endmsg;
 				break;
 			}
 			
@@ -1472,7 +1351,7 @@ Session::midi_thread_work ()
 				tmp = i;
 				++tmp;
 				
-				if (!(*i).call ()) {
+				if (!(*i)()) {
 					midi_timeouts.erase (i);
 				}
 				
@@ -1482,21 +1361,3 @@ Session::midi_thread_work ()
 	}
 }
 
-bool
-Session::get_mmc_control () const
-{
-	return mmc_control;
-}
-bool
-Session::get_midi_feedback () const
-{
-	/* since this a "write" function we have to check the port as well 
-	   as the control toggle.
-	*/
-	return _midi_port && midi_feedback;
-}
-bool
-Session::get_midi_control () const
-{
-	return midi_control;
-}
