@@ -45,6 +45,8 @@ using namespace std;
 void
 Session::process (nframes_t nframes)
 {
+	_silent = false;
+
 	if (synced_to_jack() && waiting_to_start) {
 		if ( _engine.transport_state() == AudioEngine::TransportRolling) {
 			actually_start_transport ();
@@ -253,7 +255,7 @@ Session::process_with_events (nframes_t nframes)
 	bool session_needs_butler = false;
 	nframes_t stop_limit;
 	long           frames_moved;
-
+	
 	/* make sure the auditioner is silent */
 
 	if (auditioner) {
@@ -278,7 +280,7 @@ Session::process_with_events (nframes_t nframes)
 	}
 
 	if (!process_can_proceed()) {
-		no_roll (nframes, 0);
+		_silent = true;
 		return;
 	}
 		
@@ -293,8 +295,8 @@ Session::process_with_events (nframes_t nframes)
 		Event* this_event;
 		Events::iterator the_next_one;
 		
-		if (post_transport_work & (PostTransportLocate|PostTransportStop)) {
-			no_roll (nframes, 0);
+		if (!process_can_proceed()) {
+			_silent = true;
 			return;
 		}
 		
@@ -475,7 +477,7 @@ Session::follow_slave (nframes_t nframes, nframes_t offset)
 	     << endl;
 #endif	
 
-	if (Config->get_timecode_source_is_synced()) {
+	if (_slave->is_always_synced() || Config->get_timecode_source_is_synced()) {
 
 		/* if the TC source is synced, then we assume that its 
 		   speed is binary: 0.0 or 1.0
@@ -624,7 +626,7 @@ Session::follow_slave (nframes_t nframes, nframes_t offset)
 		slave_state = Stopped;
 	}
 
-	if (slave_state == Running && !Config->get_timecode_source_is_synced()) {
+	if (slave_state == Running && !_slave->is_always_synced() && !Config->get_timecode_source_is_synced()) {
 
 
 		if (_transport_speed != 0.0f) {
@@ -725,65 +727,62 @@ Session::process_without_events (nframes_t nframes)
 	long frames_moved;
 	nframes_t offset = 0;
 
-	{
-		if (post_transport_work & (PostTransportLocate|PostTransportStop)) {
-			no_roll (nframes, 0);
+	if (!process_can_proceed()) {
+		_silent = true;
+		return;
+	}
+
+	if (!_exporting && _slave) {
+		if (!follow_slave (nframes, 0)) {
 			return;
 		}
+	} 
 
-		if (!_exporting && _slave) {
-			if (!follow_slave (nframes, 0)) {
-				return;
-			}
-		} 
-
-		if (_transport_speed == 0) {
-			no_roll (nframes, 0);
-			return;
-		}
+	if (_transport_speed == 0) {
+		no_roll (nframes, 0);
+		return;
+	}
 		
-		if (actively_recording()) {
+	if (actively_recording()) {
+		stop_limit = max_frames;
+	} else {
+		if (Config->get_stop_at_session_end()) {
+			stop_limit = current_end_frame();
+		} else {
 			stop_limit = max_frames;
-		} else {
-			if (Config->get_stop_at_session_end()) {
-				stop_limit = current_end_frame();
-			} else {
-				stop_limit = max_frames;
-			}
 		}
+	}
 		
-		if (maybe_stop (stop_limit)) {
-			no_roll (nframes, 0);
-			return;
-		} 
+	if (maybe_stop (stop_limit)) {
+		no_roll (nframes, 0);
+		return;
+	} 
 
-		if (maybe_sync_start (nframes, offset)) {
-			return;
-		}
+	if (maybe_sync_start (nframes, offset)) {
+		return;
+	}
 
-		click (_transport_frame, nframes, offset);
+	click (_transport_frame, nframes, offset);
 
-		prepare_diskstreams ();
+	prepare_diskstreams ();
 	
-		frames_moved = (long) floor (_transport_speed * nframes);
+	frames_moved = (long) floor (_transport_speed * nframes);
 
-		if (process_routes (nframes, offset)) {
-			no_roll (nframes, offset);
- 			return;
- 		}
+	if (process_routes (nframes, offset)) {
+		no_roll (nframes, offset);
+		return;
+	}
 
-		commit_diskstreams (nframes, session_needs_butler);
+	commit_diskstreams (nframes, session_needs_butler);
 
-		if (frames_moved < 0) {
-			decrement_transport_position (-frames_moved);
-		} else {
-			increment_transport_position (frames_moved);
-		}
+	if (frames_moved < 0) {
+		decrement_transport_position (-frames_moved);
+	} else {
+		increment_transport_position (frames_moved);
+	}
 
-		maybe_stop (stop_limit);
-		check_declick_out ();
-
-	} /* implicit release of route lock */
+	maybe_stop (stop_limit);
+	check_declick_out ();
 
 	if (session_needs_butler) {
 		summon_butler ();
@@ -847,18 +846,38 @@ Session::maybe_sync_start (nframes_t& nframes, nframes_t& offset)
 
 	if (_engine.get_sync_offset (sync_offset) && sync_offset < nframes) {
 
+		/* generate silence up to the sync point, then
+		   adjust nframes + offset to reflect whatever
+		   is left to do.
+		*/
+
 		no_roll (sync_offset, 0);
 		nframes -= sync_offset;
 		offset += sync_offset;
 		waiting_for_sync_offset = false;
 		
 		if (nframes == 0) {
-			return true; // done
+			return true; // done, nothing left to process
 		}
 		
 	} else {
+
+		/* sync offset point is not within this process()
+		   cycle, so just generate silence. and don't bother 
+		   with any fancy stuff here, just the minimal silence.
+		*/
+
+		g_atomic_int_inc (&processing_prohibited);
 		no_roll (nframes, 0);
-		return true; // done
+		g_atomic_int_dec_and_test (&processing_prohibited);
+
+		if (Config->get_locate_while_waiting_for_sync()) {
+			if (micro_locate (nframes)) {
+				/* XXX ERROR !!! XXX */
+			}
+		}
+
+		return true; // done, nothing left to process
 	}
 
 	return false;
