@@ -44,12 +44,12 @@
 #endif
 
 #include <glibmm.h>
+#include <glibmm/thread.h>
 
 #include <midi++/mmc.h>
 #include <midi++/port.h>
-#include <pbd/error.h>
 
-#include <glibmm/thread.h>
+#include <pbd/error.h>
 #include <pbd/pathscanner.h>
 #include <pbd/pthread_utils.h>
 #include <pbd/strsplit.h>
@@ -127,6 +127,9 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	_name = _current_snapshot_name = snapshot_name;
 
 	_current_frame_rate = _engine.frame_rate ();
+	_nominal_frame_rate = _current_frame_rate;
+	_base_frame_rate = _current_frame_rate;
+
 	_tempo_map = new TempoMap (_current_frame_rate);
 	_tempo_map->StateChanged.connect (mem_fun (*this, &Session::tempo_map_changed));
 
@@ -138,6 +141,7 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	transport_sub_state = 0;
 	_transport_frame = 0;
 	last_stop_frame = 0;
+	_requested_return_frame = -1;
 	end_location = new Location (0, 0, _("end"), Location::Flags ((Location::IsMark|Location::IsEnd)));
 	start_location = new Location (0, 0, _("start"), Location::Flags ((Location::IsMark|Location::IsStart)));
 	_end_location_is_free = true;
@@ -162,8 +166,10 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 	_worst_output_latency = 0;
 	_worst_input_latency = 0;
 	_worst_track_latency = 0;
-	_state_of_the_state = StateOfTheState(CannotSave|InitialConnecting|Loading|Deletion);
+	_state_of_the_state = StateOfTheState(CannotSave|InitialConnecting|Loading);
+
 	_slave = 0;
+	_silent = false;
 	butler_mixdown_buffer = 0;
 	butler_gain_buffer = 0;
 	mmc = 0;
@@ -219,9 +225,6 @@ Session::first_stage_init (string fullpath, string snapshot_name)
 		waiting_for_sync_offset = false;
 	}
 
-	_current_frame_rate = 48000;
-	_base_frame_rate = 48000;
-
 	last_smpte_when = 0;
 	_smpte_offset = 0;
 	_smpte_offset_negative = true;
@@ -285,6 +288,7 @@ Session::second_stage_init (bool new_session)
 
 	// set_state() will call setup_raid_path(), but if it's a new session we need
 	// to call setup_raid_path() here.
+
 	if (state_tree) {
 		if (set_state (*state_tree->root())) {
 			return -1;
@@ -305,7 +309,7 @@ Session::second_stage_init (bool new_session)
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state|CannotSave|Loading);
 
- 	// set_auto_input (true);
+
 	_locations.changed.connect (mem_fun (this, &Session::locations_changed));
 	_locations.added.connect (mem_fun (this, &Session::locations_added));
 	setup_click_sounds (0);
@@ -321,22 +325,25 @@ Session::second_stage_init (bool new_session)
 	}
 
 	/* handle this one in a different way than all others, so that its clear what happened */
-	
+
 	catch (AudioEngine::PortRegistrationFailure& err) {
-		error << _("Unable to create all required ports")
-		      << endmsg;
-		return -1;
+		destroy ();
+		throw;
 	}
 
 	catch (...) {
 		return -1;
 	}
 
+	BootMessage (_("Reset Remote Controls"));
+
 	send_full_time_code ();
 	_engine.transport_locate (0);
 	deliver_mmc (MIDI::MachineControl::cmdMmcReset, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, 0);
 
+	BootMessage (_("Reset Control Protocols"));
+
 	ControlProtocolManager::instance().set_session (*this);
 
 	if (new_session) {
@@ -344,7 +351,19 @@ Session::second_stage_init (bool new_session)
 	} else {
 		_end_location_is_free = false;
 	}
+
+	_state_of_the_state = Clean;
 	
+	DirtyChanged (); /* EMIT SIGNAL */
+
+	if (state_was_pending) {
+		save_state (_current_snapshot_name);
+		remove_pending_capture_state ();
+		state_was_pending = false;
+	}
+	
+	BootMessage (_("Session loading complete"));
+
 	return 0;
 }
 
@@ -392,20 +411,9 @@ Session::setup_raid_path (string path)
 		sp.path = path;
 		sp.blocks = 0;
 		session_dirs.push_back (sp);
-
-		string fspath;
-
 		/* sounds dir */
 
-		fspath += sp.path;
-		if (fspath[fspath.length()-1] != '/') {
-			fspath += '/';
-		}
-
-		fspath += sound_dir (false);
-		
-		AudioFileSource::set_search_path (fspath);
-
+		AudioFileSource::set_search_path (Glib::build_filename(sp.path, sound_dir (false)));
 		return;
 	}
 
@@ -419,11 +427,7 @@ Session::setup_raid_path (string path)
 
 		/* add sounds to file search path */
 
-		fspath += sp.path;
-		if (fspath[fspath.length()-1] != '/') {
-			fspath += '/';
-		}
-		fspath += sound_dir (false);
+		fspath += Glib::build_filename(sp.path, sound_dir (false));
 		fspath += ':';
 
 		remaining = remaining.substr (colon+1);
@@ -435,11 +439,7 @@ Session::setup_raid_path (string path)
 		sp.path = remaining;
 
 		fspath += ':';
-		fspath += sp.path;
-		if (fspath[fspath.length()-1] != '/') {
-			fspath += '/';
-		}
-		fspath += sound_dir (false);
+		fspath += Glib::build_filename(sp.path, sound_dir (false));
 		fspath += ':';
 
 		session_dirs.push_back (sp);
@@ -455,19 +455,14 @@ Session::setup_raid_path (string path)
 }
 
 int
-Session::create (bool& new_session, string* mix_template, nframes_t initial_length)
+Session::ensure_subdirs ()
 {
 	string dir;
 
-	if (g_mkdir_with_parents (_path.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session dir \"%1\" (%2)"), _path, strerror (errno)) << endmsg;
-		return -1;
-	}
-
 	dir = peak_dir ();
 
 	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session peakfile dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		error << string_compose(_("Session: cannot create session peakfile folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 		return -1;
 	}
 
@@ -478,7 +473,7 @@ Session::create (bool& new_session, string* mix_template, nframes_t initial_leng
 		dir = sound_dir ();
 		
 		if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-			error << string_compose(_("Session: cannot create session sounds dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+			error << string_compose(_("Session: cannot create session sounds folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 			return -1;
 		}
 	}
@@ -486,22 +481,44 @@ Session::create (bool& new_session, string* mix_template, nframes_t initial_leng
 	dir = dead_sound_dir ();
 
 	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session dead sounds dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		error << string_compose(_("Session: cannot create session dead sounds folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 		return -1;
 	}
 
 	dir = export_dir ();
 
 	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
-		error << string_compose(_("Session: cannot create session export dir \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		error << string_compose(_("Session: cannot create session export folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
 		return -1;
 	}
 
+	dir = analysis_dir ();
+
+	if (g_mkdir_with_parents (dir.c_str(), 0755) < 0) {
+		error << string_compose(_("Session: cannot create session analysis folder \"%1\" (%2)"), dir, strerror (errno)) << endmsg;
+		return -1;
+	}
+
+	return 0;
+}
+
+int
+Session::create (bool& new_session, const string& mix_template, nframes_t initial_length)
+{
+
+	if (g_mkdir_with_parents (_path.c_str(), 0755) < 0) {
+		error << string_compose(_("Session: cannot create session folder \"%1\" (%2)"), _path, strerror (errno)) << endmsg;
+		return -1;
+	}
+
+	if (ensure_subdirs ()) {
+		return -1;
+	}
 
 	/* check new_session so we don't overwrite an existing one */
 
-	if (mix_template) {
-		std::string in_path = *mix_template;
+	if (!mix_template.empty()) {
+		std::string in_path = mix_template;
 
 		ifstream in(in_path.c_str());
 
@@ -545,9 +562,7 @@ Session::create (bool& new_session, string* mix_template, nframes_t initial_leng
 
 	_state_of_the_state = Clean;
 
-	if (save_state (_current_snapshot_name)) {
-		return -1;
-	}
+	save_state ("");
 
 	return 0;
 }
@@ -715,6 +730,7 @@ Session::save_state (string snapshot_name, bool pending)
 		bool was_dirty = dirty();
 
 		_state_of_the_state = StateOfTheState (_state_of_the_state & ~Dirty);
+
 		
 		if (was_dirty) {
 			DirtyChanged (); /* EMIT SIGNAL */
@@ -901,16 +917,16 @@ Session::state(bool full_state)
 
 	// store libardour version, just in case
 	char buf[16];
-	snprintf(buf, sizeof(buf)-1, "%d.%d.%d", 
-		 libardour2_major_version, libardour2_minor_version, libardour2_micro_version);
+	snprintf(buf, sizeof(buf), "%d.%d.%d", libardour2_major_version, libardour2_minor_version, libardour2_micro_version);
 	node->add_property("version", string(buf));
 		
 	/* store configuration settings */
 
 	if (full_state) {
 	
-		/* store the name */
 		node->add_property ("name", _name);
+		snprintf (buf, sizeof (buf), "%" PRId32, _nominal_frame_rate);
+		node->add_property ("sample-rate", buf);
 
 		if (session_dirs.size() > 1) {
 
@@ -1138,6 +1154,17 @@ Session::set_state (const XMLNode& node)
 		_name = prop->value ();
 	}
 
+	if ((prop = node.property (X_("sample-rate"))) != 0) {
+
+		_nominal_frame_rate = atoi (prop->value());
+
+		if (_nominal_frame_rate != _current_frame_rate) {
+			if (AskAboutSampleRateMismatch (_nominal_frame_rate, _current_frame_rate)) {
+				return -1;
+			}
+		}
+	}
+
 	setup_raid_path(_path);
 
 	if ((prop = node.property (X_("id-counter"))) != 0) {
@@ -1313,14 +1340,6 @@ Session::set_state (const XMLNode& node)
 
 	StateReady (); /* EMIT SIGNAL */
 
-	_state_of_the_state = Clean;
-
-	if (state_was_pending) {
-		save_state (_current_snapshot_name);
-		remove_pending_capture_state ();
-		state_was_pending = false;
-	}
-
 	return 0;
 
   out:
@@ -1347,10 +1366,12 @@ Session::load_routes (const XMLNode& node)
 			return -1;
 		}
 
+		BootMessage (string_compose (_("Loaded track/bus %1"), route->name()));
+
 		new_routes.push_back (route);
 	}
 
-	add_routes (new_routes);
+	add_routes (new_routes, false);
 
 	return 0;
 }
@@ -1384,7 +1405,14 @@ Session::load_regions (const XMLNode& node)
 
 	for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 		if ((region = XMLRegionFactory (**niter, false)) == 0) {
-			error << _("Session: cannot create Region from XML description.") << endmsg;
+			error << _("Session: cannot create Region from XML description.");
+			const XMLProperty *name = (**niter).property("name");
+
+			if (name) {
+				error << " " << string_compose (_("Can not load state for region '%1'"), name->value());
+			}
+
+			error << endmsg;
 		}
 	}
 
@@ -1398,6 +1426,7 @@ Session::XMLRegionFactory (const XMLNode& node, bool full)
 	boost::shared_ptr<Source> source;
 	boost::shared_ptr<AudioSource> as;
 	SourceList sources;
+	SourceList master_sources;
 	uint32_t nchans = 1;
 	char buf[128];
 	
@@ -1458,7 +1487,27 @@ Session::XMLRegionFactory (const XMLNode& node, bool full)
 			sources.push_back (as);
 		}
 	}
-	
+
+	for (uint32_t n=0; n < nchans; ++n) {
+		snprintf (buf, sizeof(buf), X_("master-source-%d"), n);
+		if ((prop = node.property (buf)) != 0) {
+		
+			PBD::ID id2 (prop->value());
+			
+			if ((source = source_by_id (id2)) == 0) {
+				error << string_compose(_("Session: XMLNode describing a AudioRegion references an unknown source id =%1"), id2) << endmsg;
+				return boost::shared_ptr<AudioRegion>();
+			}
+			
+			as = boost::dynamic_pointer_cast<AudioSource>(source);
+			if (!as) {
+				error << string_compose(_("Session: XMLNode describing a AudioRegion references a non-audio source id =%1"), id2) << endmsg;
+				return boost::shared_ptr<AudioRegion>();
+			}
+			master_sources.push_back (as);
+		} 
+	}
+
 	try {
 		boost::shared_ptr<AudioRegion> region (boost::dynamic_pointer_cast<AudioRegion> (RegionFactory::create (sources, node)));
 
@@ -1473,6 +1522,13 @@ Session::XMLRegionFactory (const XMLNode& node, bool full)
 			}
 		}
 
+		if (!master_sources.empty()) {
+			if (master_sources.size() != nchans) {
+				error << _("Session: XMLNode describing an AudioRegion is missing some master sources; ignored") << endmsg;
+			} else {
+				region->set_master_sources (master_sources);
+			}
+		}
 
 		return region;
 						       
@@ -1563,7 +1619,8 @@ Session::XMLSourceFactory (const XMLNode& node)
 	}
 
 	try {
-		return SourceFactory::create (*this, node);
+		/* note: do peak building in another thread when loading session state */
+		return SourceFactory::create (*this, node, true);
 	}
 
 	catch (failed_constructor& err) {
@@ -1596,9 +1653,7 @@ Session::save_template (string template_name)
 
 	tree.set_root (&get_template());
 
-	xml_path = dir;
-	xml_path += template_name;
-	xml_path += _template_suffix;
+	xml_path = Glib::build_filename(dir, template_name + _template_suffix);
 
 	ifstream in(xml_path.c_str());
 	
@@ -1620,8 +1675,8 @@ Session::save_template (string template_name)
 int
 Session::rename_template (string old_name, string new_name) 
 {
-	string old_path = template_dir() + old_name + _template_suffix;
-	string new_path = template_dir() + new_name + _template_suffix;
+	string old_path = Glib::build_filename(template_dir(), old_name + _template_suffix);
+	string new_path = Glib::build_filename(template_dir(), new_name + _template_suffix);
 
 	return rename (old_path.c_str(), new_path.c_str());
 }
@@ -1629,9 +1684,7 @@ Session::rename_template (string old_name, string new_name)
 int
 Session::delete_template (string name) 
 {
-	string template_path = template_dir();
-	template_path += name;
-	template_path += _template_suffix;
+	string template_path = Glib::build_filename(template_dir(), name + _template_suffix);
 
 	return remove (template_path.c_str());
 }
@@ -1675,27 +1728,21 @@ Session::ensure_sound_dir (string path, string& result)
 	
 	/* Ensure that the sounds directory exists */
 	
-	result = path;
-	result += '/';
-	result += sound_dir_name;
+	result = Glib::build_filename(path, sound_dir_name);
 	
 	if (g_mkdir_with_parents (result.c_str(), 0775)) {
 		error << string_compose(_("cannot create sounds directory \"%1\"; ignored"), result) << endmsg;
 		return -1;
 	}
 
-	dead = path;
-	dead += '/';
-	dead += dead_sound_dir_name;
+	dead = Glib::build_filename(path, dead_sound_dir_name);
 	
 	if (g_mkdir_with_parents (dead.c_str(), 0775)) {
 		error << string_compose(_("cannot create dead sounds directory \"%1\"; ignored"), dead) << endmsg;
 		return -1;
 	}
 
-	peak = path;
-	peak += '/';
-	peak += peak_dir_name;
+	peak = Glib::build_filename(path, peak_dir_name);
 	
 	if (g_mkdir_with_parents (peak.c_str(), 0775)) {
 		error << string_compose(_("cannot create peak file directory \"%1\"; ignored"), peak) << endmsg;
@@ -1937,6 +1984,7 @@ Session::sound_dir (bool with_path) const
 {
 	string res;
 	string full;
+	vector<string> parts;
 
 	if (with_path) {
 		res = _path;
@@ -1944,12 +1992,11 @@ Session::sound_dir (bool with_path) const
 		full = _path;
 	}
 
-	res += interchange_dir_name;
-	res += '/';
-	res += legalize_for_path (_name);
-	res += '/';
-	res += sound_dir_name;
+	parts.push_back(interchange_dir_name);
+	parts.push_back(legalize_for_path (_name));
+	parts.push_back(sound_dir_name);
 
+	res += Glib::build_filename(parts);
 	if (with_path) {
 		full = res;
 	} else {
@@ -1988,36 +2035,31 @@ Session::sound_dir (bool with_path) const
 string
 Session::peak_dir () const
 {
-	string res = _path;
-	res += peak_dir_name;
-	res += '/';
-	return res;
+	return Glib::build_filename (_path, peak_dir_name);
 }
 	
 string
 Session::automation_dir () const
 {
-	string res = _path;
-	res += "automation/";
-	return res;
+	return Glib::build_filename (_path, "automation");
+}
+
+string
+Session::analysis_dir () const
+{
+	return Glib::build_filename (_path, "analysis");
 }
 
 string
 Session::template_dir ()
 {
-	string path = get_user_ardour_path();
-	path += "templates/";
-
-	return path;
+	return Glib::build_filename (get_user_ardour_path(), "templates");
 }
 
 string
 Session::export_dir () const
 {
-	string res = _path;
-	res += export_dir_name;
-	res += '/';
-	return res;
+	return Glib::build_filename (_path, export_dir_name);
 }
 
 string
@@ -2063,6 +2105,10 @@ Session::template_path ()
 string
 Session::control_protocol_path ()
 {
+	char *p = getenv ("ARDOUR_CONTROL_SURFACE_PATH");
+	if (p && *p) {
+		return p;
+	}
 	return suffixed_search_path (X_("surfaces"), false);
 }
 
@@ -2270,6 +2316,10 @@ Session::commit_reversible_command (Command *cmd)
 		current_trans->add_command (cmd);
 	}
 
+	if (current_trans->empty()) {
+		return;
+	}
+
 	gettimeofday (&now, 0);
 	current_trans->set_timestamp (now);
 
@@ -2423,8 +2473,7 @@ Session::get_template_list (list<string> &template_names)
 int
 Session::read_favorite_dirs (FavoriteDirs & favs)
 {
-	string path = get_user_ardour_path();
-	path += "/favorite_dirs";
+	Glib::ustring path = Glib::build_filename (get_user_ardour_path(), "favorite_dirs");
 
 	ifstream fav (path.c_str());
 
@@ -2458,8 +2507,7 @@ Session::read_favorite_dirs (FavoriteDirs & favs)
 int
 Session::write_favorite_dirs (FavoriteDirs & favs)
 {
-	string path = get_user_ardour_path();
-	path += "/favorite_dirs";
+	Glib::ustring path = Glib::build_filename (get_user_ardour_path(), "favorite_dirs");
 
 	ofstream fav (path.c_str());
 
@@ -2598,6 +2646,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 	int ret = -1;
 		
 	_state_of_the_state = (StateOfTheState) (_state_of_the_state | InCleanup);
+
 	
 	/* step 1: consider deleting all unused playlists */
 
@@ -2761,7 +2810,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 		newpath += dead_sound_dir_name;
 
 		if (g_mkdir_with_parents (newpath.c_str(), 0755) < 0) {
-			error << string_compose(_("Session: cannot create session peakfile dir \"%1\" (%2)"), newpath, strerror (errno)) << endmsg;
+			error << string_compose(_("Session: cannot create session peakfile folder \"%1\" (%2)"), newpath, strerror (errno)) << endmsg;
 			return -1;
 		}
 
@@ -2837,6 +2886,7 @@ Session::cleanup_sources (Session::cleanup_report& rep)
 
   out:
 	_state_of_the_state = (StateOfTheState) (_state_of_the_state & ~InCleanup);
+
 	return ret;
 }
 
@@ -2908,6 +2958,7 @@ Session::set_dirty ()
 
 	_state_of_the_state = StateOfTheState (_state_of_the_state | Dirty);
 
+
 	if (!was_dirty) {
 		DirtyChanged(); /* EMIT SIGNAL */
 	}
@@ -2921,6 +2972,7 @@ Session::set_clean ()
 	
 	_state_of_the_state = Clean;
 
+
 	if (was_dirty) {
 		DirtyChanged(); /* EMIT SIGNAL */
 	}
@@ -2930,6 +2982,7 @@ void
 Session::set_deletion_in_progress ()
 {
 	_state_of_the_state = StateOfTheState (_state_of_the_state | Deletion);
+
 }
 
 void
