@@ -394,6 +394,7 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 				do_locate = true;
 			} else {
 				_transport_frame = last_stop_frame;
+				_requested_return_frame = -1;
 			}
 
 			if (synced_to_jack() && !play_loop) {
@@ -429,9 +430,14 @@ Session::non_realtime_stop (bool abort, int on_entry, bool& finished)
 	}
 #endif
 
-	last_stop_frame = _transport_frame;
+        if (_requested_return_frame < 0) {
+		last_stop_frame = _transport_frame;
+	} else {
+		last_stop_frame = _requested_return_frame;
+		_requested_return_frame = -1;
+	}
 
-	send_full_time_code ();
+        send_full_time_code ();
 	deliver_mmc (MIDI::MachineControl::cmdStop, 0);
 	deliver_mmc (MIDI::MachineControl::cmdLocate, _transport_frame);
 
@@ -634,6 +640,25 @@ Session::start_locate (nframes_t target_frame, bool with_roll, bool with_flush,
 	}
 }
 
+int
+Session::micro_locate (nframes_t distance)
+{
+	boost::shared_ptr<DiskstreamList> dsl = diskstreams.reader();
+	
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+		if (!(*i)->can_internal_playback_seek (distance)) {
+			return -1;
+		}
+	}
+
+	for (DiskstreamList::iterator i = dsl->begin(); i != dsl->end(); ++i) {
+		(*i)->internal_playback_seek (distance);
+	}
+	
+	_transport_frame += distance;
+	return 0;
+}
+
 void
 Session::locate (nframes_t target_frame, bool with_roll, bool with_flush, bool with_loop)
 {
@@ -1188,8 +1213,8 @@ Session::setup_auto_play ()
 void
 Session::request_roll_at_and_return (nframes_t start, nframes_t return_to)
 {
-	request_locate (start, false);
  	Event *ev = new Event (Event::LocateRollLocate, Event::Add, Event::Immediate, return_to, 1.0);
+	ev->target2_frame = start;
 	queue_event (ev);
 }
 
@@ -1229,9 +1254,9 @@ Session::engine_halted ()
 void
 Session::xrun_recovery ()
 {
-	if (Config->get_stop_recording_on_xrun() && actively_recording()) {
+	Xrun (transport_frame()); //EMIT SIGNAL
 
-		 HaltOnXrun (); /* EMIT SIGNAL */
+	if (Config->get_stop_recording_on_xrun() && actively_recording()) {
 
 		/* it didn't actually halt, but we need
 		   to handle things in the same way.
@@ -1252,6 +1277,11 @@ Session::update_latency_compensation (bool with_stop, bool abort)
 
 	_worst_track_latency = 0;
 
+#undef DEBUG_LATENCY
+#ifdef DEBUG_LATENCY
+	cerr << "\n---------------------------------\nUPDATE LATENCY\n";
+#endif
+
 	boost::shared_ptr<RouteList> r = routes.reader ();
 
 	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
@@ -1272,6 +1302,10 @@ Session::update_latency_compensation (bool with_stop, bool abort)
 		}
 	}
 
+#ifdef DEBUG_LATENCY
+	cerr << "\tworst was " << _worst_track_latency << endl;
+#endif
+
 	for (RouteList::iterator i = r->begin(); i != r->end(); ++i) {
 		(*i)->set_latency_delay (_worst_track_latency);
 	}
