@@ -17,6 +17,7 @@
 
 */
 
+#include <cstring>
 #include <cerrno>
 #include <climits>
 
@@ -25,6 +26,7 @@
 #include <sys/stat.h>
 
 #include <glibmm/miscutils.h>
+#include <pbd/stacktrace.h>
 
 #include <ardour/sndfilesource.h>
 #include <ardour/sndfile_helpers.h>
@@ -44,7 +46,6 @@ const AudioFileSource::Flag SndFileSource::default_writable_flags = AudioFileSou
 											   AudioFileSource::Removable|
 											   AudioFileSource::RemovableIfEmpty|
 											   AudioFileSource::CanRename);
-
 SndFileSource::SndFileSource (Session& s, const XMLNode& node)
 	: AudioFileSource (s, node)
 {
@@ -80,7 +81,7 @@ SndFileSource::SndFileSource (Session& s, ustring path, SampleFormat sfmt, Heade
 	*/
 
 	file_is_new = true;
-	
+
 	switch (hf) {
 	case CAF:
 		fmt = SF_FORMAT_CAF;
@@ -174,9 +175,8 @@ SndFileSource::SndFileSource (Session& s, ustring path, SampleFormat sfmt, Heade
 			_flags = Flag (_flags & ~Broadcast);
 			delete _broadcast_info;
 			_broadcast_info = 0;
-		}
-		
-	}
+		} 
+	} 
 }
 
 void 
@@ -186,8 +186,6 @@ SndFileSource::init ()
 
 	// lets try to keep the object initalizations here at the top
 	xfade_buf = 0;
-	interleave_buf = 0;
-	interleave_bufsize = 0;
 	sf = 0;
 	_broadcast_info = 0;
 
@@ -221,13 +219,21 @@ SndFileSource::open ()
 	if ((sf = sf_open (_path.c_str(), (writable() ? SFM_RDWR : SFM_READ), &_info)) == 0) {
 		char errbuf[256];
 		sf_error_str (0, errbuf, sizeof (errbuf) - 1);
+#ifndef HAVE_COREAUDIO
+		/* if we have CoreAudio, we will be falling back to that if libsndfile fails,
+		   so we don't want to see this message.
+		*/
+
 		error << string_compose(_("SndFileSource: cannot open file \"%1\" for %2 (%3)"), 
 					_path, (writable() ? "read+write" : "reading"), errbuf) << endmsg;
+#endif
 		return -1;
 	}
 
 	if (_channel >= _info.channels) {
+#ifndef HAVE_COREAUDIO
 		error << string_compose(_("SndFileSource: file only contains %1 channels; %2 is invalid as a channel number"), _info.channels, _channel) << endmsg;
+#endif
 		sf_close (sf);
 		sf = 0;
 		return -1;
@@ -242,7 +248,7 @@ SndFileSource::open ()
 
 	set_timeline_position (get_timecode_info (sf, _broadcast_info, timecode_info_exists));
 
-	if (!timecode_info_exists) {
+	if (_length != 0 && !timecode_info_exists) {
 		delete _broadcast_info;
 		_broadcast_info = 0;
 		_flags = Flag (_flags & ~Broadcast);
@@ -272,10 +278,6 @@ SndFileSource::~SndFileSource ()
 		touch_peakfile ();
 	}
 
-	if (interleave_buf) {
-		delete [] interleave_buf;
-	}
-
 	if (_broadcast_info) {
 		delete _broadcast_info;
 	}
@@ -318,6 +320,11 @@ SndFileSource::read_unlocked (Sample *dst, nframes_t start, nframes_t cnt) const
 		file_cnt = cnt;
 	}
 	
+	if (file_cnt != cnt) {
+		nframes_t delta = cnt - file_cnt;
+		memset (dst+file_cnt, 0, sizeof (Sample) * delta);
+	}
+
 	if (file_cnt) {
 
 		if (sf_seek (sf, (sf_count_t) start, SEEK_SET|SFM_READ) != (sf_count_t) start) {
@@ -334,21 +341,9 @@ SndFileSource::read_unlocked (Sample *dst, nframes_t start, nframes_t cnt) const
 		}
 	}
 
-	if (file_cnt != cnt) {
-		nframes_t delta = cnt - file_cnt;
-		memset (dst+file_cnt, 0, sizeof (Sample) * delta);
-	}
-
 	real_cnt = cnt * _info.channels;
 
-	if (interleave_bufsize < real_cnt) {
-		
-		if (interleave_buf) {
-			delete [] interleave_buf;
-		}
-		interleave_bufsize = real_cnt;
-		interleave_buf = new float[interleave_bufsize];
-	}
+	Sample* interleave_buf = get_interleave_buffer (real_cnt);
 	
 	nread = sf_read_float (sf, interleave_buf, real_cnt);
 	ptr = interleave_buf + _channel;
@@ -401,7 +396,7 @@ SndFileSource::nondestructive_write_unlocked (Sample *data, nframes_t cnt)
 	update_length (oldlen, cnt);
 
 	if (_build_peakfiles) {
-		compute_and_write_peaks (data, frame_pos, cnt, false);
+		compute_and_write_peaks (data, frame_pos, cnt, false, true);
 	}
 
 	_write_data_count = cnt;
@@ -493,7 +488,7 @@ SndFileSource::destructive_write_unlocked (Sample* data, nframes_t cnt)
 	update_length (file_pos, cnt);
 
 	if (_build_peakfiles) {
-		compute_and_write_peaks (data, file_pos, cnt, false);
+		compute_and_write_peaks (data, file_pos, cnt, false, true);
 	}
 
 	file_pos += cnt;
@@ -864,7 +859,7 @@ SndFileSource::get_soundfile_info (const ustring& path, SoundFileInfo& info, str
 	info.samplerate  = sf_info.samplerate;
 	info.channels    = sf_info.channels;
 	info.length      = sf_info.frames;
-	info.format_name = string_compose("Format: %1, %2",
+	info.format_name = string_compose("%1\n%2",
 					   sndfile_major_format(sf_info.format),
 					   sndfile_minor_format(sf_info.format));
 
@@ -904,3 +899,4 @@ SndFileSource::one_of_several_channels () const
 {
 	return _info.channels > 1;
 }
+
