@@ -30,6 +30,9 @@
 #include <sys/stat.h>
 #include <cerrno>
 
+#include <glibmm/ustring.h>
+#include <glibmm/miscutils.h>
+
 #include <lrdf.h>
 #include <fst.h>
 
@@ -155,12 +158,9 @@ VSTPlugin::get_state()
 
 		/* save it to a file */
 
-		string path;
+		Glib::ustring path = Glib::build_filename (get_user_ardour_path (), "vst");
 		struct stat sbuf;
 
-		path = get_user_ardour_path ();
-		path += "vst";
-
 		if (stat (path.c_str(), &sbuf)) {
 			if (errno == ENOENT) {
 				if (g_mkdir_with_parents (path.c_str(), 0600)) {
@@ -183,7 +183,7 @@ VSTPlugin::get_state()
 			return *root;
 		}
 		
-		path += "something";
+		path = Glib::build_filename (path, "something");
 		
 		/* store information */
 
