@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: gtk_ui.cc 2938 2008-01-18 19:58:34Z paul $
+    $Id: gtk_ui.cc 3254 2008-04-16 00:38:09Z timbyr $
 */
 
 #include <cmath>
@@ -65,7 +65,6 @@ UI::UI (string namestr, int *argc, char ***argv)
 	: AbstractUI<UIRequest> (namestr, true)
 {
 	theMain = new Main (argc, argv);
-	tips = new Tooltips;
 
 	_active = false;
 	_auto_display_errors = true;
@@ -414,7 +413,7 @@ UI::process_error_message (Transmitter::Channel chn, const char *str)
 	RefPtr<Style> style;
 	RefPtr<TextBuffer::Tag> ptag;
 	RefPtr<TextBuffer::Tag> mtag;
-	char *prefix;
+	const char *prefix;
 	size_t prefix_len;
 	bool fatal_received = false;
 #ifndef OLD_STYLE_ERRORS
@@ -572,9 +571,31 @@ UI::popup_error (const char *text)
 	msg.run ();
 }
 
+#ifdef GTKOSX
+extern "C" {
+	int gdk_quartz_in_carbon_menu_event_handler ();
+}
+#endif
+
 void
 UI::flush_pending ()
 {
+#ifdef GTKOSX
+	/* as of february 11th 2008, gtk/osx has a problem in that mac menu events
+	   are handled using Carbon with an "internal" event handling system that 
+	   doesn't pass things back to the glib/gtk main loop. this makes
+	   gtk_main_iteration() block if we call it while in a menu event handler 
+	   because glib gets confused and thinks there are two threads running
+	   g_main_poll_func(). 
+
+	   this hack (relies on code in gtk2_ardour/sync-menu.c) works
+	   around that.
+	*/
+
+	if (gdk_quartz_in_carbon_menu_event_handler()) {
+		return;
+	}
+#endif
 	if (!caller_is_ui_thread()) {
 		error << "non-UI threads cannot call UI::flush_pending()"
 		      << endmsg;
